@@ -2,137 +2,32 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
+
 #include <string.h>
 #include <math.h>
 
 #include "filter.h"
 #include "frame.h"
 
+#include "config.h"
+#include "dsputil.h"
+
 #define ABS(A) ( (A) > 0 ? (A) : -(A) )
 #define CLAMP(A,L,U) ((A)>(U)?(U):((A)<(L)?(L):(A)))
 
-#define MM_MMX    0x0001        /* standard MMX */
-#define MM_3DNOW  0x0004        /* AMD 3DNOW */
-#define MM_MMXEXT 0x0002        /* SSE integer functions or AMD MMX ext */
-#define MM_SSE    0x0008        /* SSE functions */
-#define MM_SSE2   0x0010        /* PIV SSE2 functions */
-
-#ifdef i386
-#include "mmx.h"
+#ifdef MMX
+#include "i386/mmx.h"
 
 static const mmx_t mm_cpool[] =
 {
     { 0x0000000000000000LL },
 };
 
-#define cpuid(index,eax,ebx,ecx,edx)\
-    __asm __volatile\
-        ("movl %%ebx, %%esi\n\t"\
-         "cpuid\n\t"\
-         "xchgl %%ebx, %%esi"\
-         : "=a" (eax), "=S" (ebx),\
-           "=c" (ecx), "=d" (edx)\
-         : "0" (index));
-
-/* Function to test if multimedia instructions are supported...  */
-int
-mm_support (void)
-{
-    int rval;
-    int eax, ebx, ecx, edx;
-
-    __asm__ __volatile__ (
-                             /* See if CPUID instruction is supported ... */
-                             /* ... Get copies of EFLAGS into eax and ecx */
-                             "pushf\n\t" "popl %0\n\t" "movl %0, %1\n\t"
-                             /* ... Toggle the ID bit in one copy and store */
-                             /*     to the EFLAGS reg */
-                             "xorl $0x200000, %0\n\t" "push %0\n\t" "popf\n\t"
-                             /* ... Get the (hopefully modified) EFLAGS */
-                             "pushf\n\t"
-                             "popl %0\n\t":"=a" (eax), "=c" (ecx)::"cc");
-
-    if (eax == ecx)
-        return 0;               /* CPUID not supported */
-
-    cpuid (0, eax, ebx, ecx, edx);
-
-    if (ebx == 0x756e6547 && edx == 0x49656e69 && ecx == 0x6c65746e)
-    {
-
-        /* intel */
-      inteltest:
-        cpuid (1, eax, ebx, ecx, edx);
-        if ((edx & 0x00800000) == 0)
-            return 0;
-        rval = MM_MMX;
-        if (edx & 0x02000000)
-            rval |= MM_MMXEXT | MM_SSE;
-        if (edx & 0x04000000)
-            rval |= MM_SSE2;
-        return rval;
-    }
-    else if (ebx == 0x68747541 && edx == 0x69746e65 && ecx == 0x444d4163)
-    {
-        /* AMD */
-        cpuid (0x80000000, eax, ebx, ecx, edx);
-        if ((unsigned) eax < 0x80000001)
-            goto inteltest;
-        cpuid (0x80000001, eax, ebx, ecx, edx);
-        if ((edx & 0x00800000) == 0)
-            return 0;
-        rval = MM_MMX;
-        if (edx & 0x80000000)
-            rval |= MM_3DNOW;
-        if (edx & 0x00400000)
-            rval |= MM_MMXEXT;
-        return rval;
-    }
-    else if (ebx == 0x746e6543 && edx == 0x48727561 && ecx == 0x736c7561)
-    {                           /*  "CentaurHauls" */
-        /* VIA C3 */
-        cpuid (0x80000000, eax, ebx, ecx, edx);
-        if ((unsigned) eax < 0x80000001)
-            goto inteltest;
-        cpuid (0x80000001, eax, ebx, ecx, edx);
-        rval = 0;
-        if (edx & (1 << 31))
-            rval |= MM_3DNOW;
-        if (edx & (1 << 23))
-            rval |= MM_MMX;
-        if (edx & (1 << 24))
-            rval |= MM_MMXEXT;
-        return rval;
-    }
-    else if (ebx == 0x69727943 && edx == 0x736e4978 && ecx == 0x64616574)
-    {
-        /* Cyrix Section */
-        /* See if extended CPUID level 80000001 is supported */
-        /* The value of CPUID/80000001 for the 6x86MX is undefined
-           according to the Cyrix CPU Detection Guide (Preliminary
-           Rev. 1.01 table 1), so we'll check the value of eax for
-           CPUID/0 to see if standard CPUID level 2 is supported.
-           According to the table, the only CPU which supports level
-           2 is also the only one which supports extended CPUID levels.
-         */
-        if (eax != 2)
-            goto inteltest;
-        cpuid (0x80000001, eax, ebx, ecx, edx);
-        if ((eax & 0x00800000) == 0)
-            return 0;
-        rval = MM_MMX;
-        if (eax & 0x01000000)
-            rval |= MM_MMXEXT;
-        return rval;
-    }
-    else
-    {
-        return 0;
-    }
-}
-#else // i386
-int mm_support(void) { return 0; };
+#else
 #define mmx_t int
 #endif
 
@@ -202,7 +97,7 @@ KDP (uint8_t *Plane, uint8_t *Line, int W, int H, int Threshold)
             LineCur[X] = LineCur1U[X];
 }
 
-#ifdef i386
+#ifdef MMX
 void
 KDP_MMX (uint8_t *Plane, uint8_t *Line, int W, int H, int Threshold)
 {
@@ -352,10 +247,6 @@ KDP_MMX (uint8_t *Plane, uint8_t *Line, int W, int H, int Threshold)
             > Threshold - 1)
             LineCur[X] = LineCur1U[X];
 }
-#else
-void KDP_MMX (uint8_t *Plane, uint8_t *Line, int W, int H, int Threshold) {
-     (void)Plane; (void)Line; (void)W; (void)H; (void)Threshold;
-}
 #endif
 
 static int
@@ -374,8 +265,10 @@ KernelDeint (VideoFilter * f, VideoFrame * frame)
         (filter->filtfunc)(frame->buf + filter->voff, filter->line,
                            filter->cwidth, filter->cheight, filter->threshold);
     }
+#ifdef MMX
     if (filter->mm_flags)
         emms();
+#endif
     TF_END(filter, "KernelDeint: ");
     return 0;
 }
@@ -414,15 +307,20 @@ NewKernelDeintFilter (VideoFrameType inpixfmt, VideoFrameType outpixfmt,
     if (numopts < 1)
         filter->threshold = 12;
 
-    filter->mm_flags = mm_support();
     filter->width = *width;
     filter->height = *height;
     filter->cwidth = *width / 2;
     filter->uoff = *width * *height;
+#ifdef MMX
+    filter->mm_flags = mm_support();
     if (filter->mm_flags & MM_MMX)
         filter->filtfunc = &KDP_MMX;
     else
+#else
+        filter->mm_flags = 0,
+#endif
         filter->filtfunc = &KDP;
+
     switch (inpixfmt)
     {
         case FMT_YUV422P:
