@@ -5,7 +5,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
 
 #include "avcodec.h"
 #include "filter.h"
@@ -50,7 +53,7 @@ int pp(VideoFilter *vf, VideoFrame *frame)
     pp_postprocess( tf->src, tf->srcStride,
                     tf->dst, tf->dstStride,
                     frame->width, frame->height,
-                    frame->qscale_table, frame->qstride,
+                    (signed char *)(frame->qscale_table), frame->qstride,
                     tf->mode, tf->context, PP_FORMAT_420);
 
     TF_END(tf, "PostProcess: ");
