@@ -13,6 +13,7 @@ using namespace std;
 #include "mythcontext.h"
 #include "util.h"
 #include "mythdialogs.h"
+#include "virtualkeyboard.h"
 
 void MythComboBox::keyPressEvent(QKeyEvent *e)
 {
@@ -41,6 +42,20 @@ void MythComboBox::keyPressEvent(QKeyEvent *e)
                 if (count() > 0)
                     setCurrentItem((currentItem() + 1) % count());
             }
+            else if (action == "PAGEDOWN")
+            {
+                if (currentItem() == 0)
+                    setCurrentItem(count() - (step % count()));
+                else if (count() > 0)
+                    setCurrentItem(
+                        (currentItem() + count() - (step % count())) % count());
+            }
+            else if (action == "PAGEUP")
+            {
+                if (count() > 0)
+                    setCurrentItem(
+                        (currentItem() + (step % count())) % count());
+            }
             else if (action == "SELECT" && AcceptOnSelect)
                 emit accepted(currentItem());
             else
@@ -141,6 +156,48 @@ void MythCheckBox::focusOutEvent(QFocusEvent *e)
     QCheckBox::focusOutEvent(e);
 }
 
+void MythRadioButton::keyPressEvent(QKeyEvent* e)
+{
+    bool handled = false;
+    QStringList actions;
+    if (gContext->GetMainWindow()->TranslateKeyPress("qt", e, actions))
+    {
+        for (unsigned int i = 0; i < actions.size() && !handled; i++)
+        {
+            QString action = actions[i];
+            handled = true;
+
+            if (action == "UP")
+                focusNextPrevChild(false);
+            else if (action == "DOWN")
+                focusNextPrevChild(true);
+            else if (action == "LEFT" || action == "RIGHT")
+                toggle();
+            else
+                handled = false;
+        }
+    }
+
+    if (!handled)
+        e->ignore();
+}
+
+void MythRadioButton::focusInEvent(QFocusEvent *e)
+{
+    emit changeHelpText(helptext);
+
+    QColor highlight = colorGroup().highlight();
+
+    this->setPaletteBackgroundColor(highlight);
+    QRadioButton::focusInEvent(e);
+}
+
+void MythRadioButton::focusOutEvent(QFocusEvent *e)
+{
+    this->unsetPalette();
+    QRadioButton::focusOutEvent(e);
+}
+
 bool MythSpinBox::eventFilter(QObject* o, QEvent* e)
 {
     (void)o;
@@ -263,11 +320,24 @@ void MythSlider::focusOutEvent(QFocusEvent *e)
     QSlider::focusOutEvent(e);
 }
 
+MythLineEdit::~MythLineEdit()
+{
+    if (popup)
+        delete popup;
+}
+
+void MythLineEdit::Init()
+{
+    popup = NULL;
+    popupPosition = VK_POSBELOWEDIT;
+}
+
 void MythLineEdit::keyPressEvent(QKeyEvent *e)
 {
     bool handled = false;
     QStringList actions;
-    if (gContext->GetMainWindow()->TranslateKeyPress("qt", e, actions))
+    if ((!popup || !popup->isShown()) &&
+        gContext->GetMainWindow()->TranslateKeyPress("qt", e, actions, false))
     {
         for (unsigned int i = 0; i < actions.size() && !handled; i++)
         {
@@ -278,7 +348,21 @@ void MythLineEdit::keyPressEvent(QKeyEvent *e)
                 focusNextPrevChild(false);
             else if (action == "DOWN")
                 focusNextPrevChild(true);
-            else if (action == "SELECT" && e->text().isNull())
+            else if (action == "SELECT" && 
+                    (e->text().isNull() ||
+                    (e->key() == Qt::Key_Enter) ||
+                    (e->key() == Qt::Key_Return)))
+            {
+                if (gContext->GetNumSetting("UseVirtualKeyboard", 1) == 1)
+                {
+                    popup = new VirtualKeyboard(gContext->GetMainWindow(), this);
+                    gContext->GetMainWindow()->detach(popup);
+                    popup->exec();
+                    delete popup;
+                    popup = NULL;
+                }
+            }
+            else if (action == "SELECT" && e->text().isNull() )
                 e->ignore();
             else
                 handled = false;
@@ -314,21 +398,28 @@ void MythLineEdit::focusInEvent(QFocusEvent *e)
 void MythLineEdit::focusOutEvent(QFocusEvent *e)
 {
     this->unsetPalette();
+    if (popup && popup->isShown() && !popup->hasFocus())
+        popup->hide();
     QLineEdit::focusOutEvent(e);
 }
 
-MythRemoteLineEdit::MythRemoteLineEdit(QFont *a_font, QWidget * parent, 
-                                       const char * name)
-                  : QTextEdit(parent, name)
+void MythLineEdit::hideEvent(QHideEvent *e)
 {
-    my_font = a_font;
-    this->Init();
+    if (popup && popup->isShown())
+        popup->hide();
+    QLineEdit::hideEvent(e);
+}
+
+void MythLineEdit::mouseDoubleClickEvent(QMouseEvent *e)
+{
+    QLineEdit::mouseDoubleClickEvent(e);
 }
 
 MythRemoteLineEdit::MythRemoteLineEdit(QWidget * parent, const char * name)
                   : QTextEdit(parent, name)
 {
     my_font = NULL;
+    m_lines = 1;
     this->Init();
 }
 
@@ -337,10 +428,29 @@ MythRemoteLineEdit::MythRemoteLineEdit(const QString & contents,
                   : QTextEdit(parent, name)
 {
     my_font = NULL;
+    m_lines = 1;
     this->Init();
     setText(contents);
 }
 
+MythRemoteLineEdit::MythRemoteLineEdit(QFont *a_font, QWidget * parent, 
+                                       const char * name)
+                  : QTextEdit(parent, name)
+{
+    my_font = a_font;
+    m_lines = 1;
+    this->Init();
+}
+
+MythRemoteLineEdit::MythRemoteLineEdit(int lines, QWidget * parent, 
+                                       const char * name)
+                  : QTextEdit(parent, name)
+{
+    my_font = NULL;
+    m_lines = lines;
+    this->Init();
+}
+
 void MythRemoteLineEdit::Init()
 {
     //  Bunch of default values
@@ -357,8 +467,6 @@ void MythRemoteLineEdit::Init()
     //  works across most 3.x.x versions of Qt.
     setTextFormat(Qt::PlainText);    
 
-    setCursor(QCursor(Qt::BlankCursor));
-
     cycle_time = 3000;
 
     pre_cycle_text_upto = "";
@@ -383,9 +491,12 @@ void MythRemoteLineEdit::Init()
     QFontMetrics fontsize(font());
     
     setMinimumHeight(fontsize.height() * 5 / 4);
-    setMaximumHeight(fontsize.height() * 5 / 4);
+    setMaximumHeight(fontsize.height() * m_lines * 5 / 4);
 
     connect(cycle_timer, SIGNAL(timeout()), this, SLOT(endCycle()));
+    
+    popup = NULL;
+    popupPosition = VK_POSBELOWEDIT;
 }
 
 void MythRemoteLineEdit::startCycle(QString current_choice, QString set)
@@ -476,15 +587,20 @@ void MythRemoteLineEdit::updateCycle(QString current_choice, QString set)
     {
         set.replace(index, current_choice.length(), bString);
     }    
-    
-        
-    aString = pre_cycle_text_upto;
+
+    QString esc_upto =  pre_cycle_text_upto;
+    QString esc_from =  pre_cycle_text_from;
+
+    esc_upto.replace("<", "&lt;").replace(">", "&gt;").replace("\n", "<br>");
+    esc_from.replace("<", "&lt;").replace(">", "&gt;").replace("\n", "<br>");
+
+    aString = esc_upto;
     aString += "<FONT COLOR=\"#";
     aString += hex_unselected;
     aString += "\">[";
     aString += set;
     aString += "]</FONT>";
-    aString += pre_cycle_text_from;
+    aString += esc_from;
     setTextFormat(Qt::RichText);
     setText(aString);
     setCursorPosition(pre_cycle_para, pre_cycle_pos + set.length());
@@ -585,7 +701,9 @@ void MythRemoteLineEdit::keyPressEvent(QKeyEvent *e)
 {
     bool handled = false;
     QStringList actions;
-    if (gContext->GetMainWindow()->TranslateKeyPress("qt", e, actions))
+
+    if ((!popup || !popup->isShown()) &&
+          gContext->GetMainWindow()->TranslateKeyPress("qt", e, actions, false))
     {
         for (unsigned int i = 0; i < actions.size() && !handled; i++)
         {
@@ -608,6 +726,21 @@ void MythRemoteLineEdit::keyPressEvent(QKeyEvent *e)
                 QWidget::focusNextPrevChild(true);
                 emit tryingToLooseFocus(true);
             }
+            else if (action == "SELECT" &&
+                    (e->text().isNull() ||
+                    (e->key() == Qt::Key_Enter) || 
+                    (e->key() == Qt::Key_Return)))
+            {
+                if (gContext->GetNumSetting("UseVirtualKeyboard", 1) == 1)
+                {
+                    popup = new VirtualKeyboard(gContext->GetMainWindow(), this);
+                    gContext->GetMainWindow()->detach(popup);
+                    popup->exec();
+                    delete popup;
+                    popup = NULL;
+                }
+ 
+            }
             else
                 handled = false;
         }
@@ -616,6 +749,14 @@ void MythRemoteLineEdit::keyPressEvent(QKeyEvent *e)
     if (handled)
         return;
 
+    if (!handled && popup && popup->isShown())
+    {
+        endCycle();
+        QTextEdit::keyPressEvent(e);
+        emit textChanged(this->text());
+        return;
+    }
+
     switch (e->key())
     {
         case Key_Enter:
@@ -789,16 +930,41 @@ void MythRemoteLineEdit::focusOutEvent(QFocusEvent *e)
 {
     this->unsetPalette();
 
+    if (popup && popup->isShown() && !popup->hasFocus())
+        popup->hide();
+
     emit lostFocus();
     QTextEdit::focusOutEvent(e);
 }
 
+
 MythRemoteLineEdit::~MythRemoteLineEdit()
 {
     if (cycle_timer)
     {
         delete cycle_timer;
     }
+
+    if (popup)
+        delete popup;
+}
+
+void MythRemoteLineEdit::insert(QString text)
+{
+    QTextEdit::insert(text);
+    emit textChanged(this->text());
+}
+
+void MythRemoteLineEdit::del()
+{
+    doKeyboardAction(QTextEdit::ActionDelete);
+    emit textChanged(this->text());
+}
+
+void MythRemoteLineEdit::backspace()
+{
+    doKeyboardAction(QTextEdit::ActionBackspace);
+    emit textChanged(this->text());
 }
 
 void MythTable::keyPressEvent(QKeyEvent *e)
@@ -863,9 +1029,13 @@ void MythPushButton::keyPressEvent(QKeyEvent *e)
 {
     bool handled = false;
     QStringList actions;
+    keyPressActions.clear();
+
     if (gContext->GetMainWindow()->TranslateKeyPress("qt", (QKeyEvent *)e,
                                                      actions))
     {
+        keyPressActions = actions;
+
         for (unsigned int i = 0; i < actions.size() && !handled; i++)
         {
             QString action = actions[i];
@@ -899,19 +1069,15 @@ void MythPushButton::keyPressEvent(QKeyEvent *e)
 void MythPushButton::keyReleaseEvent(QKeyEvent *e)
 {
     bool handled = false;
-    QStringList actions;
-    if (gContext->GetMainWindow()->TranslateKeyPress("qt", (QKeyEvent *)e,
-                                                     actions))
+    QStringList actions = keyPressActions;
+    for (unsigned int i = 0; i < actions.size() && !handled; i++)
     {
-        for (unsigned int i = 0; i < actions.size() && !handled; i++)
+        QString action = actions[i];
+        if (action == "SELECT")
         {
-            QString action = actions[i];
-            if (action == "SELECT")
-            {
-                QKeyEvent tempe(QEvent::KeyRelease, Key_Space, ' ', 0, " ");
-                QPushButton::keyReleaseEvent(&tempe);
-                handled = true;
-            }
+            QKeyEvent tempe(QEvent::KeyRelease, Key_Space, ' ', 0, " ");
+            QPushButton::keyReleaseEvent(&tempe);
+            handled = true;
         }
     }
 
@@ -1031,11 +1197,67 @@ void MythListView::focusInEvent(QFocusEvent *e)
     setSelected(currentItem(), true);
 }
 
-void MythListBox::setCurrentItem(const QString& matchText) 
+MythListBox::MythListBox(QWidget* parent): QListBox(parent) 
+{
+}
+
+void MythListBox::polish(void)
+{
+    QListBox::polish();
+    
+    QPalette pal = palette();
+    QColorGroup::ColorRole role = QColorGroup::Highlight;
+    pal.setColor(QPalette::Active, role, pal.active().button()); 
+    pal.setColor(QPalette::Inactive, role, pal.active().button()); 
+    pal.setColor(QPalette::Disabled, role, pal.active().button()); 
+   
+    setPalette(pal);
+}
+
+void MythListBox::setCurrentItem(const QString& matchText, bool caseSensitive, 
+                                 bool partialMatch) 
 {
     for (unsigned i = 0 ; i < count() ; ++i)
-        if (text(i) == matchText)
-            setCurrentItem(i);
+    {
+        if (partialMatch)
+        {
+            if (caseSensitive)
+            {
+                if (text(i).startsWith(matchText))
+                {
+                    setCurrentItem(i);
+                    break;
+                }
+            }
+            else
+            {
+                if (text(i).lower().startsWith(matchText.lower()))
+                {
+                    setCurrentItem(i);
+                    break;
+                }
+            }
+        }
+        else
+        {
+            if (caseSensitive)
+            {
+                if (text(i) == matchText)
+                {
+                    setCurrentItem(i);
+                    break;
+                }
+            }
+            else
+            {    
+                if (text(i).lower() == matchText.lower())
+                {
+                    setCurrentItem(i);
+                    break;
+                }
+            }        
+        }
+    }            
 }
 
 void MythListBox::keyPressEvent(QKeyEvent* e) 
@@ -1048,13 +1270,45 @@ void MythListBox::keyPressEvent(QKeyEvent* e)
         {
             QString action = actions[i];
             if (action == "UP" || action == "DOWN" || action == "PAGEUP" ||
-                action == "PAGEDOWN")
+                action == "PAGEDOWN" || action == "LEFT" || action == "RIGHT")
             {
                 int key;
                 if (action == "UP")
+                {
+                    // Key_Up at top of list allows focus to move to other widgets
+                    if (currentItem() == 0)
+                    {
+                        focusNextPrevChild(false);
+                        handled = true;
+                        continue;
+                    }
+                    
                     key = Key_Up;
+                }
                 else if (action == "DOWN")
+                {
+                    // Key_down at bottom of list allows focus to move to other widgets
+                    if (currentItem() == (int) count() - 1)
+                    {
+                        focusNextPrevChild(true);
+                        handled = true;
+                        continue;
+                    }
+                    
                     key = Key_Down;
+                }
+                else if (action == "LEFT")
+                {
+                    focusNextPrevChild(false);
+                    handled = true;
+                    continue;
+                }
+                else if (action == "RIGHT")
+                {
+                    focusNextPrevChild(true);
+                    handled = true;
+                    continue;
+                }
                 else if (action == "PAGEUP")
                     key = Key_Prior;
                 else if (action == "PAGEDOWN")
@@ -1077,9 +1331,22 @@ void MythListBox::keyPressEvent(QKeyEvent* e)
         e->ignore();
 }
 
+void MythListBox::focusOutEvent(QFocusEvent *e)
+{
+    QPalette pal = palette();
+    QColorGroup::ColorRole role = QColorGroup::Highlight;
+    pal.setColor(QPalette::Active, role, pal.active().button()); 
+    pal.setColor(QPalette::Inactive, role, pal.active().button()); 
+    pal.setColor(QPalette::Disabled, role, pal.active().button()); 
+   
+    setPalette(pal);
+    QListBox::focusOutEvent(e);
+}
+
 void MythListBox::focusInEvent(QFocusEvent *e)
 {
+    this->unsetPalette();
+    
     emit changeHelpText(helptext);
     QListBox::focusInEvent(e);
 }
-
