@@ -16,12 +16,18 @@
 #include <qbuttongroup.h>
 #include <qlistbox.h>
 #include <qcheckbox.h>
+#include <qradiobutton.h>
 #include <qimage.h>
 #include <qlabel.h>
-#include <qtimer.h>
+#include <qtimer.h> 
 
 #include <vector>
 
+using namespace std;
+
+#include "virtualkeyboard.h"
+
+
 // These widgets follow these general navigation rules:
 //
 // - Up and Down shift focus to the previous/next widget in the tab
@@ -35,10 +41,11 @@ class MythComboBox: public QComboBox
     Q_OBJECT
   public:
     MythComboBox(bool rw, QWidget* parent=0, const char* name=0):
-        QComboBox(rw, parent, name) { AcceptOnSelect = false; };
+        QComboBox(rw, parent, name) { AcceptOnSelect = false; step = 1; };
 
     void setHelpText(QString help) { helptext = help; }
     void setAcceptOnSelect(bool Accept) { AcceptOnSelect = Accept; }
+    void setStep(int _step = 1) { step = _step; }
 
   signals:
     void changeHelpText(QString);
@@ -58,6 +65,7 @@ class MythComboBox: public QComboBox
   private:
     QString helptext;
     bool AcceptOnSelect;
+    int step;
 };
 
 class MythSpinBox: public QSpinBox 
@@ -114,15 +122,19 @@ class MythLineEdit : public QLineEdit
     Q_OBJECT
   public:
     MythLineEdit(QWidget *parent=NULL, const char* widgetName=0) :
-      QLineEdit(parent, widgetName) { rw = true; };
+      QLineEdit(parent, widgetName) { rw = true; Init(); };
 
     MythLineEdit(const QString& contents, QWidget *parent=NULL, 
                  const char* widgetName=0) :
-      QLineEdit(contents, parent, widgetName) { rw = true; };
+      QLineEdit(contents, parent, widgetName) { rw = true; Init(); };
+
+    virtual ~MythLineEdit();
 
     void setHelpText(QString help) { helptext = help; };
     void setRW(bool readwrite = true) { rw = readwrite; };
     void setRO() { rw = false; };
+    void setPopupPosition(PopupPosition pos) { popupPosition = pos; }
+    PopupPosition getPopupPosition(void) { return popupPosition; }
 
  public slots:
      virtual void setText(const QString& text);
@@ -134,10 +146,15 @@ class MythLineEdit : public QLineEdit
     virtual void keyPressEvent(QKeyEvent *e);
     virtual void focusInEvent(QFocusEvent *e); 
     virtual void focusOutEvent(QFocusEvent *e);
+    virtual void hideEvent(QHideEvent *e);
+    virtual void mouseDoubleClickEvent(QMouseEvent *e);
+    void Init(void);
 
   private:
+    VirtualKeyboard *popup;
     QString helptext;
     bool rw;
+    PopupPosition popupPosition;
 };
 
 // A Line edit that does special things when you press number keys 
@@ -145,17 +162,23 @@ class MythRemoteLineEdit : public QTextEdit
 {
     Q_OBJECT
   public:
-    
+
     MythRemoteLineEdit( QWidget * parent, const char * name = 0 );
     MythRemoteLineEdit( const QString & contents, QWidget * parent, const char * name = 0 );    
     MythRemoteLineEdit( QFont *a_font, QWidget * parent, const char * name = 0 );    
+    MythRemoteLineEdit( int lines, QWidget * parent, const char * name = 0 );
    ~MythRemoteLineEdit();
     void setHelpText(QString help) { helptext = help; }
     void setCycleTime(float desired_interval); // in seconds
     void setCharacterColors(QColor unselected, QColor selected, QColor special);
+    void insert(QString text);
+    void backspace();
+    void del();
+    void setPopupPosition(PopupPosition pos) { popupPosition = pos; };
+    PopupPosition getPopupPosition(void) { return popupPosition; };
 
   signals:
-    
+
     void    shiftState(bool);
     void    cycleState(QString current_choice, QString set);
     void    changeHelpText(QString);
@@ -204,10 +227,15 @@ class MythRemoteLineEdit : public QTextEdit
     QColor  col_unselected;
     QColor  col_selected;
     QColor  col_special;
-        
+
     QString  hex_unselected;
     QString  hex_selected;
     QString  hex_special;
+
+    int m_lines;
+
+    VirtualKeyboard *popup;
+    PopupPosition popupPosition;
 };
 
 class MythTable : public QTable
@@ -254,6 +282,8 @@ class MythPushButton : public QPushButton
     QColor origcolor;
     QString helptext;
     bool arrowAccel;
+
+    QStringList keyPressActions;
 };
 
 class MythCheckBox: public QCheckBox
@@ -276,6 +306,26 @@ class MythCheckBox: public QCheckBox
     QString helptext;
 };
 
+class MythRadioButton: public QRadioButton
+{
+    Q_OBJECT
+  public:
+    MythRadioButton(QWidget* parent = 0, const char* name = 0):
+        QRadioButton(parent, name) {};
+    void setHelpText(QString help) { helptext = help; }
+
+  signals:
+    void changeHelpText(QString);
+
+  protected:
+    virtual void keyPressEvent(QKeyEvent* e);
+    virtual void focusInEvent(QFocusEvent *e);
+    virtual void focusOutEvent(QFocusEvent *e);
+
+  private:
+    QString helptext;
+};
+
 class MythListView : public QListView
 {
     Q_OBJECT
@@ -292,7 +342,7 @@ class MythListView : public QListView
 class MythListBox: public QListBox {
     Q_OBJECT
   public:
-    MythListBox(QWidget* parent): QListBox(parent) {};
+    MythListBox(QWidget* parent);
 
     virtual void keyPressEvent(QKeyEvent* e);
 
@@ -300,9 +350,12 @@ class MythListBox: public QListBox {
 
   protected:
     void focusInEvent(QFocusEvent *e);
-
+    void focusOutEvent(QFocusEvent *e);
+    virtual void polish(void);
+ 
   public slots:
-    void setCurrentItem(const QString& text);
+    void setCurrentItem(const QString& matchText, bool caseSensitive = true, 
+                        bool partialMatch = false);
     void setCurrentItem(int index) { QListBox::setCurrentItem(index); };
 
   signals:
