@@ -10,27 +10,13 @@ using namespace std;
 #include "util.h"
 #include "mythcontext.h"
 
-RemoteFile::RemoteFile(const QString &url, int lrecordernum)
+RemoteFile::RemoteFile(const QString &url)
 {
-    type = 0;
     path = url;
     readposition = 0;
     filesize = -1;
 
-    if (lrecordernum > 0)
-    {
-        type = 1;
-        query = "QUERY_RECORDER %1";
-        append = "_RINGBUF";
-        recordernum = lrecordernum;
-    }
-    else
-    {
-        type = 0;
-        query = "QUERY_FILETRANSFER %1";
-        append = "";
-        recordernum = -1;
-    }
+    query = "QUERY_FILETRANSFER %1";
 
     controlSock = openSocket(true);
     sock = openSocket(false);
@@ -53,12 +39,16 @@ QSocketDevice *RemoteFile::openSocket(bool control)
     int port = qurl.port();
     QString dir = qurl.path();
 
-    QSocketDevice *sock = new QSocketDevice(QSocketDevice::Stream);
+    QSocketDevice *lsock = new QSocketDevice(QSocketDevice::Stream);
     
-    if (!connectSocket(sock, host, port))
+    if (!connectSocket(lsock, host, port))
     {
-        cerr << "Could not connect to server\n";
-        delete sock;
+        QString stype = (control) ? "control socket" : "file data socket";
+        VERBOSE(VB_IMPORTANT,
+                QString("RemoteFile::openSocket(%1): \n"
+                        "\t\t\tCould not connect to server \"%2\" @ port %3")
+                .arg(stype).arg(host).arg(port));
+        delete lsock;
         return NULL;
     }
     
@@ -69,32 +59,22 @@ QSocketDevice *RemoteFile::openSocket(bool control)
     if (control)
     {
         strlist = QString("ANN Playback %1 %2").arg(hostname).arg(false);
-        WriteStringList(sock, strlist);
-        ReadStringList(sock, strlist, true);
+        WriteStringList(lsock, strlist);
+        ReadStringList(lsock, strlist, true);
     }
     else
     {
-        if (type == 0)
-        {
-            strlist = QString("ANN FileTransfer %1").arg(hostname);
-            strlist << dir;
+        strlist = QString("ANN FileTransfer %1").arg(hostname);
+        strlist << dir;
 
-            WriteStringList(sock, strlist);
-            ReadStringList(sock, strlist, true);
+        WriteStringList(lsock, strlist);
+        ReadStringList(lsock, strlist, true);
 
-            recordernum = strlist[1].toInt();
-            filesize = decodeLongLong(strlist, 2);
-        }
-        else
-        {
-            strlist = QString("ANN RingBuffer %1 %2").arg(hostname)
-                             .arg(recordernum);
-            WriteStringList(sock, strlist);
-            ReadStringList(sock, strlist, true);
-        }
+        recordernum = strlist[1].toInt();
+        filesize = decodeLongLong(strlist, 2);
     }
     
-    return sock;
+    return lsock;
 }    
 
 long long RemoteFile::GetFileSize(void)
@@ -118,17 +98,15 @@ void RemoteFile::Close(void)
         return;
 
     QStringList strlist = QString(query).arg(recordernum);
-    strlist << "DONE" + append;
+    strlist << "DONE";
 
     lock.lock();
     WriteStringList(controlSock, strlist);
     if (!ReadStringList(controlSock, strlist, true))
     {
-        cerr << "Remote file timeout.\n";
+        VERBOSE(VB_IMPORTANT, "Remote file timeout.");
     }
     
-    lock.unlock();
-
     if (sock)
     {
         delete sock;
@@ -138,11 +116,19 @@ void RemoteFile::Close(void)
     {
         delete controlSock;
         controlSock = NULL;
-    }    
+    } 
+
+    lock.unlock();   
 }
 
 void RemoteFile::Reset(void)
 {
+    if (!sock)
+    {
+        VERBOSE(VB_NETWORK, "RemoteFile::Reset(): Called with no socket");
+        return;
+    }
+
     while (sock->bytesAvailable() > 0)
     {
         int avail;
@@ -163,6 +149,12 @@ void RemoteFile::Reset(void)
 
 long long RemoteFile::Seek(long long pos, int whence, long long curpos)
 {
+    if (!sock)
+    {
+        VERBOSE(VB_NETWORK, "RemoteFile::Seek(): Called with no socket");
+        return 0;
+    }
+
     if (!sock->isOpen() || sock->error())
         return 0;
 
@@ -170,7 +162,7 @@ long long RemoteFile::Seek(long long pos, int whence, long long curpos)
         return 0;
 
     QStringList strlist = QString(query).arg(recordernum);
-    strlist << "SEEK" + append;
+    strlist << "SEEK";
     encodeLongLong(strlist, pos);
     strlist << QString::number(whence);
     if (curpos > 0)
@@ -199,6 +191,12 @@ int RemoteFile::Read(void *data, int size)
     bool error = false;
     bool response = false;
    
+    if (!sock)
+    {
+        VERBOSE(VB_NETWORK, "RemoteFile::Read(): Called with no socket");
+        return -1;
+    }
+
     if (!sock->isOpen() || sock->error())
         return -1;
    
@@ -229,7 +227,7 @@ int RemoteFile::Read(void *data, int size)
     }
     
     QStringList strlist = QString(query).arg(recordernum);
-    strlist << "REQUEST_BLOCK" + append;
+    strlist << "REQUEST_BLOCK";
     strlist << QString::number(size);
     WriteStringList(controlSock, strlist);
 
@@ -279,6 +277,9 @@ int RemoteFile::Read(void *data, int size)
     VERBOSE(VB_NETWORK, QString("Read(): reqd=%1, rcvd=%2, rept=%3, error=%4")
                                 .arg(size).arg(recv).arg(sent).arg(error));
 
+    if (sent < 0)
+        return sent;
+
     if (error || sent != recv)
         recv = -1;
 
