@@ -10,21 +10,17 @@ class QSocketDevice;
 class RemoteFile
 {
   public:
-    RemoteFile(const QString &url, bool needevents = false, 
-               int recordernum = -1);
+    RemoteFile(const QString &url);
    ~RemoteFile();
 
     QSocketDevice *getSocket();
     bool isOpen(void);
 
-    void Start(void);
-
     void Close(void);
-    bool RequestBlock(int size);
 
     long long Seek(long long pos, int whence, long long curpos = -1);
 
-    int Read(void *data, int size, bool singlefile = false);
+    int Read(void *data, int size);
     void Reset(void);
 
     bool SaveAs(QByteArray &data);
@@ -41,10 +37,8 @@ class RemoteFile
 
     long long readposition;
     int recordernum;
-    int type;
 
     QString query;
-    QString append;
 
     QMutex lock;
 
