@@ -27,6 +27,18 @@
 
 #include "mythcontext.h"
 #include "mythwizard.h"
+#include "mythdbcon.h"
+#include "DisplayRes.h"
+
+/** \class Configurable
+ *  \brief Configurable is the root of all the database aware widgets.
+ *
+ *   This is an abstract class and several methods must be implemented
+ *   in children. load(), save(), and byName(QString), are all abstract.
+ *   And while configWidget(ConfigurationGroup *, QWidget*, const char*)
+ *   has an implementation, all it does is print an error message and
+ *   return a NULL pointer.
+ */
 
 QWidget* Configurable::configWidget(ConfigurationGroup *cg, QWidget* parent,
                                     const char* widgetName) 
@@ -34,15 +46,40 @@ QWidget* Configurable::configWidget(ConfigurationGroup *cg, QWidget* parent,
     (void)cg;
     (void)parent;
     (void)widgetName;
-    cout << "BUG: Configurable is visible, but has no configWidget\n";
+    VERBOSE(VB_IMPORTANT, "BUG: Configurable is visible, but has no "
+            "configWidget");
     return NULL;
 }
 
-ConfigurationGroup::~ConfigurationGroup()  {
-    for(childList::iterator i = children.begin() ;
-        i != children.end() ;
-        ++i )
-        delete *i;
+/** \fn Configurable::enableOnSet(const QString &val)
+ *  \brief This slot allows you to enable this configurable when a
+ *         binary configurable is set to true.
+ *  \param val signal value, should be "0" to disable, other to disable.
+ */
+void Configurable::enableOnSet(const QString &val)
+{
+    setEnabled( val != "0" );
+}
+
+/** \fn Configurable::enableOnUnset(const QString &val)
+ *  \brief This slot allows you to enable this configurable when a
+ *         binary configurable is set to false.
+ *  \param val signal value, should be "0" to enable, other to disable.
+ */
+void Configurable::enableOnUnset(const QString &val)
+{
+    setEnabled( val == "0" );
+}
+
+ConfigurationGroup::~ConfigurationGroup()
+{
+    childList::iterator it = children.begin();
+    while (it != children.end())
+    {
+        if (*it)
+            delete *it;
+        ++it;
+    }
 }
 
 Setting* ConfigurationGroup::byName(QString name) {
@@ -58,18 +95,25 @@ Setting* ConfigurationGroup::byName(QString name) {
     return NULL;
 }
 
-void ConfigurationGroup::load(QSqlDatabase* db) {
+void ConfigurationGroup::load() {
     for(childList::iterator i = children.begin() ;
         i != children.end() ;
         ++i )
-        (*i)->load(db);
+        (*i)->load();
 }
 
-void ConfigurationGroup::save(QSqlDatabase* db) {
+void ConfigurationGroup::save() {
     for(childList::iterator i = children.begin() ;
         i != children.end() ;
         ++i )
-        (*i)->save(db);
+        (*i)->save();
+}
+
+void ConfigurationGroup::save(QString destination) {
+    for(childList::iterator i = children.begin() ;
+        i != children.end() ;
+        ++i )
+        (*i)->save(destination);
 }
 
 QWidget* VerticalConfigurationGroup::configWidget(ConfigurationGroup *cg, 
@@ -90,17 +134,36 @@ QWidget* VerticalConfigurationGroup::configWidget(ConfigurationGroup *cg,
 
     if (uselabel)
     {
-        layout = new QVBoxLayout(widget, (int)(28 * hmult));
+        int space = -1;
+        int margin = (int)(28 * hmult);
+        if (zeroSpace)
+            space = 4;
+        if (zeroMargin)
+            margin = 4;
+
+        layout = new QVBoxLayout(widget, margin, space);
         // This makes weird and bad things happen in qt -mdz 2002/12/28
         //widget->setInsideMargin(20);
         widget->setTitle(getLabel());
     }
     else
-        layout = new QVBoxLayout(widget, (int)(10 * hmult));
+    {
+        int space = -1;
+        int margin = (int)(10 * hmult);
+        if (zeroSpace)
+            space = 4;
+        if (zeroMargin)
+            margin = 4;
+        layout = new QVBoxLayout(widget, margin, space);
+    }
 
     for(unsigned i = 0 ; i < children.size() ; ++i)
         if (children[i]->isVisible())
-            layout->add(children[i]->configWidget(cg, widget, NULL));
+        {
+            QWidget *child = children[i]->configWidget(cg, widget, NULL);
+            layout->add(child);
+            children[i]->setEnabled(children[i]->isEnabled());
+        }
       
     if (cg)
     {
@@ -129,17 +192,97 @@ QWidget* HorizontalConfigurationGroup::configWidget(ConfigurationGroup *cg,
 
     if (uselabel)
     {
-        layout = new QHBoxLayout(widget, (int)(28 * hmult));
+        int space = -1;
+        int margin = (int)(28 * hmult);
+        if (zeroSpace)
+            space = 4;
+        if (zeroMargin)
+            margin = 4;
+
+        layout = new QHBoxLayout(widget, margin, space);
         // This makes weird and bad things happen in qt -mdz 2002/12/28
         //widget->setInsideMargin(20);
         widget->setTitle(getLabel());
     }
     else
-        layout = new QHBoxLayout(widget, (int)(10 * hmult));
+    {
+        int space = -1;
+        int margin = (int)(10 * hmult);
+        if (zeroSpace)
+            space = 4;
+        if (zeroMargin)
+            margin = 4;
+
+        layout = new QHBoxLayout(widget, margin, space);
+    }
 
     for(unsigned i = 0 ; i < children.size() ; ++i)
         if (children[i]->isVisible())
-            layout->add(children[i]->configWidget(cg, widget, NULL));
+        {
+            QWidget *child = children[i]->configWidget(cg, widget, NULL);
+            layout->add(child);
+            children[i]->setEnabled(children[i]->isEnabled());
+        }
+
+    if (cg)
+    {
+        connect(this, SIGNAL(changeHelpText(QString)), cg,
+                SIGNAL(changeHelpText(QString)));
+    }
+
+    return widget;
+}
+
+QWidget* GridConfigurationGroup::configWidget(ConfigurationGroup *cg, 
+                                              QWidget* parent, 
+                                              const char* widgetName)
+{
+    QGroupBox* widget = new QGroupBox(parent, widgetName);
+    widget->setBackgroundOrigin(QWidget::WindowOrigin);
+
+    if (!useframe)
+        widget->setFrameShape(QFrame::NoFrame);
+
+    QGridLayout *layout = NULL;
+
+    float wmult = 0, hmult = 0;
+
+    gContext->GetScreenSettings(wmult, hmult);
+
+    int rows = (children.size()+columns-1) / columns;
+    if (uselabel)
+    {
+        int space = -1;
+        int margin = (int)(28 * hmult);
+        if (zeroSpace)
+            space = 4;
+        if (zeroMargin)
+            margin = 4;
+
+        layout = new QGridLayout(widget, rows, columns, margin, space);
+        // This makes weird and bad things happen in qt -mdz 2002/12/28
+        //widget->setInsideMargin(20);
+        widget->setTitle(getLabel());
+    }
+    else
+    {
+        int space = -1;
+        int margin = (int)(10 * hmult);
+        if (zeroSpace)
+            space = 4;
+        if (zeroMargin)
+            margin = 4;
+
+        layout = new QGridLayout(widget, rows, columns, margin, space);
+    }
+
+    for (unsigned i = 0 ; i < children.size() ; ++i)
+        if (children[i]->isVisible())
+        {
+            QWidget *child = children[i]->configWidget(cg, widget, NULL);
+            layout->addWidget(child, i / columns, i % columns);
+            children[i]->setEnabled(children[i]->isEnabled());
+        }
 
     if (cg)
     {
@@ -207,11 +350,18 @@ void StackedConfigurationGroup::raise(Configurable* child) {
     cout << "BUG: StackedConfigurationGroup::raise(): unrecognized child " << child << " on setting " << getName() << '/' << getLabel() << endl;
 }
 
-void StackedConfigurationGroup::save(QSqlDatabase* db) {
+void StackedConfigurationGroup::save() {
     if (saveAll)
-        ConfigurationGroup::save(db);
+        ConfigurationGroup::save();
     else if (top < children.size())
-        children[top]->save(db);
+        children[top]->save();
+}
+
+void StackedConfigurationGroup::save(QString destination) {
+    if (saveAll)
+        ConfigurationGroup::save(destination);
+    else if (top < children.size())
+        children[top]->save(destination);
 }
 
 void TriggeredConfigurationGroup::setTrigger(Configurable* _trigger) {
@@ -289,6 +439,18 @@ void SelectSetting::setValue(const QString& newValue)  {
         addSelection(newValue, newValue, true);
 }
 
+void SelectSetting::setValue(int which)
+{
+    if ((unsigned)which > values.size()-1 || which < 0) {
+        VERBOSE(VB_IMPORTANT, 
+                "SelectSetting::setValue(): invalid index " << which);
+        return;
+    }
+    current = which;
+    isSet = true;
+    Setting::setValue(values[current]);
+}
+
 QWidget* LabelSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
                                     const char* widgetName) {
     (void)cg;
@@ -354,6 +516,26 @@ QWidget* LineEditSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
     return widget;
 }
 
+void LineEditSetting::setEnabled(bool b)
+{
+    Configurable::setEnabled(b);
+    if (edit)
+        edit->setEnabled(b);
+}
+
+void LineEditSetting::setVisible(bool b)
+{
+    Configurable::setVisible(b);
+    if (edit)
+    {
+        //QWidget *parent = edit->parentWidget();
+        if (b)
+            edit->show();
+        else
+            edit->hide();
+    }
+}
+
 QWidget* SliderSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
                                      const char* widgetName) {
     QHBox* widget;
@@ -430,8 +612,10 @@ QWidget* SpinBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
     // only set step size if greater than default (1), otherwise
     // this will screw up the single-step/jump behavior of the MythSpinBox
     if (1 < step)
-	spinbox->setLineStep(step);
+        spinbox->setLineStep(step);
     spinbox->setValue(intValue());
+    if (!svtext.isEmpty())
+        spinbox->setSpecialValueText(svtext);
 
     connect(spinbox, SIGNAL(valueChanged(int)), this, SLOT(setValue(int)));
     connect(this, SIGNAL(valueChanged(int)), spinbox, SLOT(setValue(int)));
@@ -510,6 +694,9 @@ QWidget* ComboBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
     if (isSet)
         widget->setCurrentItem(current);
 
+    if (1 < step)
+        widget->setStep(step);
+
     if (rw)
         connect(widget, SIGNAL(highlighted(const QString &)),
                 this, SLOT(setValue(const QString &)));
@@ -529,6 +716,89 @@ QWidget* ComboBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
     return box;
 }
 
+void ComboBoxSetting::setEnabled(bool b)
+{
+    Configurable::setEnabled(b);
+    if (widget)
+        widget->setEnabled(b);
+}
+
+void ComboBoxSetting::setVisible(bool b)
+{
+    Configurable::setVisible(b);
+    if (widget)
+    {
+        //QWidget *parent = widget->parentWidget();
+        if (b)
+            widget->show();
+        else
+            widget->hide();
+    }
+}
+
+void ComboBoxSetting::setValue(QString newValue)
+{
+    if (!rw)
+    {
+        VERBOSE(VB_IMPORTANT, "ComboBoxSetting::setValue(QString): "
+                "BUG: attempted to set value of read-only ComboBox");
+        return;
+    }
+    Setting::setValue(newValue);
+    if (widget)
+        widget->setCurrentItem(current);
+};
+
+void ComboBoxSetting::setValue(int which)
+{
+    if (widget)
+        widget->setCurrentItem(which);
+    SelectSetting::setValue(which);
+};
+
+void HostRefreshRateComboBox::ChangeResolution(const QString& resolution)
+{
+    clearSelections();
+    
+    const vector<short> list = GetRefreshRates(resolution);
+    addSelection(QObject::tr("Any"), "0");
+    int hz50 = -1, hz60 = -1;
+    for (uint i=0; i<list.size(); ++i)
+    {        
+        QString sel = QString::number(list[i]);
+        addSelection(sel+" Hz", sel);
+        hz50 = (50 == list[i]) ? i : hz50;
+        hz60 = (60 == list[i]) ? i : hz60;
+    }
+    
+    setValue(0);
+    if ("640x480" == resolution || "720x480" == resolution)
+        setValue(hz60+1);
+    if ("640x576" == resolution || "720x576" == resolution)
+        setValue(hz50+1);
+    
+    setEnabled(list.size());
+}
+
+const vector<short> HostRefreshRateComboBox::GetRefreshRates(const QString &res)
+{
+    QStringList slist = QStringList::split("x", res);
+    int w = 0, h = 0;
+    bool ok0 = false, ok1 = false;
+    if (2 == slist.size())
+    {
+        w = slist[0].toInt(&ok0);
+        h = slist[1].toInt(&ok1);
+    }
+
+    DisplayRes *display_res = DisplayRes::GetDisplayRes();
+    if (display_res && ok0 && ok1)
+        return display_res->GetRefreshRates(w, h);    
+
+    vector<short> list;
+    return list;
+}
+
 void PathSetting::addSelection(const QString& label,
                                QString value,
                                bool select) {
@@ -597,7 +867,7 @@ QWidget* RadioSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
 
 QWidget* CheckBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
                                        const char* widgetName) {
-    MythCheckBox* widget = new MythCheckBox(parent, widgetName);
+    widget = new MythCheckBox(parent, widgetName);
     widget->setHelpText(getHelpText());
     widget->setBackgroundOrigin(QWidget::WindowOrigin);
     widget->setText(getLabel());
@@ -615,6 +885,13 @@ QWidget* CheckBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
     return widget;
 }
 
+void CheckBoxSetting::setEnabled(bool fEnabled)
+{
+    BooleanSetting::setEnabled(fEnabled);
+    if (widget)
+        widget->setEnabled(fEnabled);
+}
+
 void ConfigurationDialogWidget::keyPressEvent(QKeyEvent* e) 
 {
     bool handled = false;
@@ -630,6 +907,10 @@ void ConfigurationDialogWidget::keyPressEvent(QKeyEvent* e)
                 accept();
             else if (action == "ESCAPE")
                 reject();
+            else if (action == "EDIT")
+                emit editButtonPressed();
+            else if (action == "DELETE")
+                emit deleteButtonPressed();
             else
                 handled = false;
         }
@@ -654,10 +935,10 @@ MythDialog* ConfigurationDialog::dialogWidget(MythMainWindow *parent,
     return dialog;
 }
 
-int ConfigurationDialog::exec(QSqlDatabase* db, bool saveOnAccept, bool doLoad) 
+int ConfigurationDialog::exec(bool saveOnAccept, bool doLoad) 
 {
     if (doLoad)
-        load(db);
+        load();
 
     MythDialog* dialog = dialogWidget(gContext->GetMainWindow());
     dialog->Show();
@@ -665,7 +946,7 @@ int ConfigurationDialog::exec(QSqlDatabase* db, bool saveOnAccept, bool doLoad)
     int ret;
 
     if ((ret = dialog->exec()) == QDialog::Accepted && saveOnAccept)
-        save(db);
+        save();
 
     delete dialog;
 
@@ -695,67 +976,101 @@ MythDialog* ConfigurationWizard::dialogWidget(MythMainWindow *parent,
     return wizard;
 }
 
-void SimpleDBStorage::load(QSqlDatabase* db) {
+void SimpleDBStorage::load() 
+{
     QString querystr = QString("SELECT %1 FROM %2 WHERE %3;")
         .arg(column).arg(table).arg(whereClause());
-    QSqlQuery query = db->exec(querystr);
+    
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.exec(querystr);
 
-    if (query.isActive() && query.numRowsAffected() > 0) {
+    if (query.isActive() && query.size() > 0) 
+    {
         query.next();
         QString result = query.value(0).toString();
-        if (result != QString::null) {
+        if (result != QString::null) 
+        {
           result = QString::fromUtf8(query.value(0).toString());
           setValue(result);
           setUnchanged();
         }
-   }
- }
+    }
+}
 
-void SimpleDBStorage::save(QSqlDatabase* db) {
+void SimpleDBStorage::save(QString table) 
+{
     if (!isChanged())
         return;
 
     QString querystr = QString("SELECT * FROM %1 WHERE %2;")
         .arg(table).arg(whereClause());
-    QSqlQuery query = db->exec(querystr);
 
-    if (query.isActive() && query.numRowsAffected() > 0) {
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.exec(querystr);
+
+    if (query.isActive() && query.size() > 0) {
         // Row already exists
-        querystr = QString("UPDATE %1 SET %2 WHERE %3;")
-            .arg(table).arg(setClause()).arg(whereClause());
-        query = db->exec(querystr);
+        // Don"t change this QString. See the CVS logs rev 1.91.
+        querystr = QString("UPDATE " + table + " SET " + setClause() + 
+                           " WHERE " + whereClause() + ";");
+        // cerr << querystr << endl;
+        query.exec(querystr);
         if (!query.isActive())
             MythContext::DBError("simpledbstorage update", querystr);
     } else {
         // Row does not exist yet
         querystr = QString("INSERT INTO %1 SET %2;")
             .arg(table).arg(setClause());
-        query = db->exec(querystr);
+        query.exec(querystr);
         if (!query.isActive())
             MythContext::DBError("simpledbstorage update", querystr);
     }
 }
 
-void AutoIncrementStorage::save(QSqlDatabase* db) {
-    if (intValue() == 0) {
+void SimpleDBStorage::save() 
+{
+    save(table);
+}
+
+void AutoIncrementStorage::save(QString table) {
+    if (intValue() == 0) 
+    {
         // Generate a new, unique ID
-        QString query = QString("INSERT INTO %1 (%2) VALUES (0);").arg(table).arg(column);
-        QSqlQuery result = db->exec(query);
-        if (!result.isActive() || result.numRowsAffected() < 1) {
-            MythContext::DBError("inserting row", result);
+        QString querystr = QString("INSERT INTO %1 (%2) VALUES (0);")
+                                .arg(table).arg(column);
+
+        MSqlQuery query(MSqlQuery::InitCon());
+        query.exec(querystr);
+
+        if (!query.isActive() || query.numRowsAffected() < 1) 
+        {
+            MythContext::DBError("inserting row", query);
             return;
         }
-        result = db->exec("SELECT LAST_INSERT_ID();");
-        if (!result.isActive() || result.numRowsAffected() < 1) {
-            MythContext::DBError("selecting last insert id", result);
+        query.exec("SELECT LAST_INSERT_ID();");
+        if (!query.isActive() || query.size() < 1) 
+        {
+            MythContext::DBError("selecting last insert id", query);
             return;
         }
 
-        result.next();
-        setValue(result.value(0).toInt());
+        query.next();
+        setValue(query.value(0).toInt());
     }
 }
 
+void AutoIncrementStorage::save() 
+{
+    save(table);
+}
+
+void ListBoxSetting::setEnabled(bool b)
+{
+    Configurable::setEnabled(b);
+    if (widget)
+        widget->setEnabled(b);
+}
+
 QWidget* ListBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent, 
                                       const char* widgetName) {
     QWidget* box = new QVBox(parent, widgetName);
@@ -796,10 +1111,18 @@ QWidget* ListBoxSetting::configWidget(ConfigurationGroup *cg, QWidget* parent,
                 SIGNAL(changeHelpText(QString)));
 
     widget->setFocus();
+    widget->setSelectionMode(selectionMode);
 
     return box;
 }
 
+void ListBoxSetting::setSelectionMode(MythListBox::SelectionMode mode)
+{
+   selectionMode = mode;
+   if (widget)
+       widget->setSelectionMode(selectionMode);
+}
+
 void ListBoxSetting::setValueByLabel(const QString& label) {
     for(unsigned i = 0 ; i < labels.size() ; ++i)
         if (labels[i] == label) {
@@ -884,8 +1207,8 @@ QWidget* ImageSelectSetting::configWidget(ConfigurationGroup *cg,
         widget->setCurrentItem(current);
     else
         current = 0;
-   
-    if (images.size() != 0 && images[current])
+
+    if (images.size() != 0 && current < images.size() && images[current])
     { 
         QImage temp = *(images[current]);
         temp = temp.smoothScale((int)(184 * m_hmult), (int)(138 * m_hmult), 
@@ -921,35 +1244,43 @@ HostnameSetting::HostnameSetting(void)  {
     setValue(gContext->GetHostName());
 }
 
-void ChannelSetting::fillSelections(QSqlDatabase* db, SelectSetting* setting) {
+void ChannelSetting::fillSelections(SelectSetting* setting) {
 
     // this should go somewhere else, in something that knows about
     // channels and how they're stored in the database.  We're just a
     // selector.
 
-    QSqlQuery result = db->exec("SELECT name, chanid FROM channel;");
-    if (result.isActive() && result.numRowsAffected() > 0)
-        while (result.next())
-            setting->addSelection(result.value(0).toString(),
-                                  QString::number(result.value(1).toInt()));
+    MSqlQuery query(MSqlQuery::InitCon()); 
+    query.prepare("SELECT name, chanid FROM channel;");
+    if (query.exec() && query.isActive() && query.size() > 0)
+        while (query.next())
+            setting->addSelection(query.value(0).toString(),
+                                  QString::number(query.value(1).toInt()));
 }
 
 QWidget* ButtonSetting::configWidget(ConfigurationGroup* cg, QWidget* parent,
                                      const char* widgetName)
 {
     (void) cg;
-    MythPushButton* widget = new MythPushButton(parent, widgetName);
+    button = new MythPushButton(parent, widgetName);
 
-    widget->setText(getLabel());
-    //widget->setHelpText(getHelpText());
+    button->setText(getLabel());
+    //button->setHelpText(getHelpText());
 
-    connect(widget, SIGNAL(pressed()), this, SIGNAL(pressed()));
+    connect(button, SIGNAL(pressed()), this, SIGNAL(pressed()));
 
     if (cg)
-        connect(widget, SIGNAL(changeHelpText(QString)),
+        connect(button, SIGNAL(changeHelpText(QString)),
                 cg, SIGNAL(changeHelpText(QString)));
 
-    return widget;
+    return button;
+}
+
+void ButtonSetting::setEnabled(bool fEnabled)
+{
+    Configurable::setEnabled(fEnabled);
+    if (button)
+        button->setEnabled(fEnabled);
 }
 
 QWidget* ProgressSetting::configWidget(ConfigurationGroup* cg, QWidget* parent,
@@ -977,6 +1308,7 @@ void ConfigPopupDialogWidget::keyPressEvent(QKeyEvent* e) {
     switch(e->key()) {
     case Key_Escape:
         reject();
+        emit popupDone();
         break;
     default:
         MythDialog::keyPressEvent(e);
@@ -1012,9 +1344,9 @@ MythDialog* ConfigurationPopupDialog::dialogWidget(MythMainWindow* parent,
     return dialog;
 }
 
-int ConfigurationPopupDialog::exec(QSqlDatabase* db, bool saveOnAccept)
+int ConfigurationPopupDialog::exec(bool saveOnAccept)
 {
-    load(db);
+    load();
 
     dialog = (ConfigPopupDialogWidget*)dialogWidget(gContext->GetMainWindow());
     dialog->ShowPopup(this);
@@ -1022,8 +1354,31 @@ int ConfigurationPopupDialog::exec(QSqlDatabase* db, bool saveOnAccept)
     int ret;
 
     if ((ret = dialog->exec()) == QDialog::Accepted && saveOnAccept)
-        save(db);
+        save();
 
     return ret;
 }
 
+QString HostSetting::whereClause(void)
+{
+    return QString("value = '%1' AND hostname = '%2'")
+                   .arg(getName()).arg(gContext->GetHostName());
+}
+
+QString HostSetting::setClause(void)
+{
+    return QString("value = '%1', data = '%2', hostname = '%3'")
+                   .arg(getName()).arg(getValue().utf8())
+                   .arg(gContext->GetHostName());
+}
+
+QString GlobalSetting::whereClause(void)
+{
+    return QString("value = '%1'").arg(getName().utf8());
+}
+
+QString GlobalSetting::setClause(void)
+{
+    return QString("value = '%1', data = '%2'")
+                   .arg(getName()).arg(getValue());
+}
