@@ -15,13 +15,14 @@
 #include <cmath>
 using namespace std;
 
+#include "exitcodes.h"
 #include "themedmenu.h"
 #include "mythcontext.h"
 #include "util.h"
 #include "mythplugin.h"
 #include "dialogbox.h"
 #include "lcddevice.h"
-
+#include "uitypes.h"
 
 
 extern QMap<QString, fontProp> globalFontMap;
@@ -95,11 +96,11 @@ class ThemedMenuPrivate
    ~ThemedMenuPrivate();
 
     bool keyPressHandler(QKeyEvent *e);
-    void ReloadTheme(void);
+    bool ReloadTheme(void);
 
-    void parseMenu(const QString &menuname, int row = -1, int col = -1);
+    bool parseMenu(const QString &menuname, int row = -1, int col = -1);
 
-    void parseSettings(const QString &dir, const QString &menuname);
+    bool parseSettings(const QString &dir, const QString &menuname);
 
     void parseBackground(const QString &dir, QDomElement &element);
     void parseLogo(const QString &dir, QDomElement &element);
@@ -115,11 +116,12 @@ class ThemedMenuPrivate
     
     void parseFont(QDomElement &element);
 
+    void gotoMainMenu(void);
     void setDefaults(void);
 
     void addButton(const QString &type, const QString &text,
                    const QString &alttext, const QStringList &action);
-    void layoutButtons(void);
+    bool layoutButtons(void);
     void positionButtons(bool resetpos);
     bool makeRowVisible(int newrow, int oldrow, bool forcedraw = true);
 
@@ -163,7 +165,7 @@ class ThemedMenuPrivate
 
     QRect logoRect;
     QPixmap *logo;
-
+    
     QImage *buttonnormal;
     QImage *buttonactive;
 
@@ -264,7 +266,7 @@ ThemedMenuPrivate::~ThemedMenuPrivate()
         delete downarrow;
     if (buttonBackground)
         delete buttonBackground;
-
+        
     QMap<QString, ButtonIcon>::Iterator it;
     for (it = allButtonIcons.begin(); it != allButtonIcons.end(); ++it)
     {
@@ -328,12 +330,14 @@ void ThemedMenuPrivate::parseBackground(const QString &dir,
 
                 buttonArea = tmpArea;
                 hasarea = true;
+                
                 if (info.hasAttribute( "background" ))
                 {
-                    cerr << "Loading " << info.attribute( "background" ) << endl;
+                    cerr << "Loading "  << info.attribute( "background" ) << endl;
                     QString bPath = dir + info.attribute( "background" );
                     buttonBackground = gContext->LoadScalePixmap(bPath);
                 }
+                
             }
             else if (info.tagName() == "buttonspread")
             {
@@ -614,7 +618,7 @@ void ThemedMenuPrivate::parseFont(QDomElement &element)
     
     fontProp newFont;
     
-    if(baseFont)
+    if (baseFont)
         newFont = *baseFont;
 
     if ( haveSizeSmall && fontSizeType == "small")
@@ -622,7 +626,7 @@ void ThemedMenuPrivate::parseFont(QDomElement &element)
         if (sizeSmall > 0)
             size = sizeSmall;
     }
-    else if( haveSizeBig && fontSizeType == "big")
+    else if (haveSizeBig && fontSizeType == "big")
     {
         if (sizeBig > 0)
             size = sizeBig;
@@ -645,7 +649,7 @@ void ThemedMenuPrivate::parseFont(QDomElement &element)
     if (!haveFace && baseFont)
     {
         newFont.face = baseFont->face;
-        if(haveSize)
+        if (haveSize)
             newFont.face.setPointSize(size);
     }
     else
@@ -1246,7 +1250,7 @@ void ThemedMenuPrivate::setDefaults(void)
     watermarkRect = QRect(0, 0, 0, 0);
 }
 
-void ThemedMenuPrivate::parseSettings(const QString &dir, 
+bool ThemedMenuPrivate::parseSettings(const QString &dir, 
                                       const QString &menuname)
 {
     QString filename = dir + menuname;
@@ -1258,7 +1262,7 @@ void ThemedMenuPrivate::parseSettings(const QString &dir,
     {
         VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate::parseSettings(): "
                                       "Can't open: %1").arg(filename));
-        return;
+        return false;
     }
 
     QString errorMsg;
@@ -1278,8 +1282,7 @@ void ThemedMenuPrivate::parseSettings(const QString &dir,
                                               "return to the main menu.")
                                               .arg(menuname)));
         menulevel = 0;
-        parseMenu("mainmenu.xml");
-        return;
+        return parseMenu("mainmenu.xml");
     }
 
     f.close();
@@ -1334,8 +1337,8 @@ void ThemedMenuPrivate::parseSettings(const QString &dir,
             else
             {
                 VERBOSE(VB_GENERAL, QString("ThemedMenuPrivate: Unknown "
-                                            "element %1").arg(e.tagName()));
-                return;
+                                            "element %1. Ignoring.")
+                        .arg(e.tagName()));
             }
         }
         n = n.nextSibling();
@@ -1344,18 +1347,18 @@ void ThemedMenuPrivate::parseSettings(const QString &dir,
     if (!setbackground)
     {
         VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Missing background element");
-        return;
+        return false;
     }
 
     if (!setbuttondef)
     {
         VERBOSE(VB_IMPORTANT,
                 "ThemedMenuPrivate: Missing genericbutton definition");
-        return;
+        return false;
     }
 
     parseFonts = false;
-    return;
+    return true;
 }
 
 void ThemedMenuPrivate::parseThemeButton(QDomElement &element)
@@ -1384,6 +1387,11 @@ void ThemedMenuPrivate::parseThemeButton(QDomElement &element)
                 {
                     text = getFirstText(info);
                 }
+                else if (info.attribute("lang","").lower() == 
+                         gContext->GetLanguageAndVariant())
+                {
+                    text = getFirstText(info);
+                }
                 else if (info.attribute("lang","").lower() == 
                          gContext->GetLanguage())
                 {
@@ -1397,6 +1405,11 @@ void ThemedMenuPrivate::parseThemeButton(QDomElement &element)
                 {
                     alttext = getFirstText(info);
                 }
+                else if (info.attribute("lang","").lower() == 
+                         gContext->GetLanguageAndVariant())
+                {
+                    alttext = getFirstText(info);
+                }
                 else if (info.attribute("lang","").lower() ==
                          gContext->GetLanguage())
                 {
@@ -1444,7 +1457,7 @@ void ThemedMenuPrivate::parseThemeButton(QDomElement &element)
         addButton(type, text, alttext, action);
 }
 
-void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
+bool ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
 {
     QString filename = findMenuFile(menuname);
 
@@ -1457,7 +1470,7 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
                                       "menu file %1").arg(menuname));
         if (menuname == "mainmenu.xml" )
         {
-            exit(-38);
+            return false;
         }
         else
         {
@@ -1467,8 +1480,7 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
                                     "We will now return to the main menu.").
                             arg(menuname)));
             menulevel = 0;
-            parseMenu("mainmenu.xml");
-            return;
+            return parseMenu("mainmenu.xml");
         }
     }
 
@@ -1485,7 +1497,7 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
 
         if (menuname == "mainmenu.xml" )
         {
-            exit(-39);
+            return false;
         }
 
         MythPopupBox::showOkPopup(gContext->GetMainWindow(), 
@@ -1495,8 +1507,7 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
                                               "return to the main menu.")
                                               .arg(menuname)));
         menulevel = 0;
-        parseMenu("mainmenu.xml");
-        return;
+        return parseMenu("mainmenu.xml");
     }
 
     f.close();
@@ -1522,7 +1533,7 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
             {
                 VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate: Unknown "
                                               "element %1").arg(e.tagName()));
-                exit(-40);
+                return false;
             }
         }
         n = n.nextSibling();
@@ -1532,10 +1543,11 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
     {
         VERBOSE(VB_IMPORTANT, QString("ThemedMenuPrivate: No buttons "
                                       "for menu %1").arg(menuname));
-        exit(-41);
+        return false;
     }
 
-    layoutButtons();
+    if (!layoutButtons())
+        return false;
     positionButtons(true);
 
     if (row != -1 && col != -1)
@@ -1591,6 +1603,7 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
 
     selection = "";
     parent->update(menuRect());
+    return true;
 }
 
 void ThemedMenuPrivate::updateLCD()
@@ -1660,7 +1673,7 @@ void ThemedMenuPrivate::addButton(const QString &type, const QString &text,
     buttonList.push_back(newbutton);
 }
 
-void ThemedMenuPrivate::layoutButtons(void)
+bool ThemedMenuPrivate::layoutButtons(void)
 {
     int numbuttons = buttonList.size();
   
@@ -1673,14 +1686,14 @@ void ThemedMenuPrivate::layoutButtons(void)
     {
         VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Must have "
                 "room for at least 1 row of buttons");
-        exit(-43);
+        return false;
     }
     
     if (columns < 1)
     {
         VERBOSE(VB_IMPORTANT, "ThemedMenuPrivate: Must have "
                 "room for at least 1 column of buttons");
-        exit(-44);
+        return false;
     }
 
     if (balancerows)
@@ -1737,6 +1750,7 @@ void ThemedMenuPrivate::layoutButtons(void)
         if (newrow.numitems > 0)
             buttonRows.push_back(newrow);
     }            
+    return true;
 }
 
 void ThemedMenuPrivate::positionButtons(bool resetpos)
@@ -1880,11 +1894,14 @@ void ThemedMenuPrivate::drawInactiveButtons(void)
 
     paintLogo(&tmp);
     paintTitle(&tmp);
-
+    
+    
     if (buttonBackground)
     {
         tmp.drawPixmap(buttonArea.topLeft(), *buttonBackground);
+         
     }
+
     ThemedButton *store = activebutton;
     activebutton = NULL;
     
@@ -2150,7 +2167,7 @@ void ThemedMenuPrivate::drawText(QPainter *p, const QRect &rect,
     }
 }
 
-void ThemedMenuPrivate::ReloadTheme(void)
+bool ThemedMenuPrivate::ReloadTheme(void)
 {
     
     globalFontMap.clear();
@@ -2204,7 +2221,9 @@ void ThemedMenuPrivate::ReloadTheme(void)
     gContext->ThemeWidget(parent);
 
     
-    parseSettings(themedir, "theme.xml");
+    bool ok = parseSettings(themedir, "theme.xml");
+    if (!ok)
+        return ok;
 
     QString file = menufiles.back().name;
     int row = menufiles.back().row;
@@ -2212,7 +2231,7 @@ void ThemedMenuPrivate::ReloadTheme(void)
     menufiles.pop_back();
     menulevel--;
 
-    parseMenu(file, row, col);
+    return parseMenu(file, row, col);
 }
 
 bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
@@ -2239,7 +2258,7 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
 
         if (action == "UP")
         { 
-            if( maxrows > 1)
+            if (maxrows > 1)
             {
                 if (currentrow > 0)
                     currentrow--;
@@ -2271,7 +2290,7 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
         }
         else if (action == "DOWN")
         {
-            if( maxrows > 1)
+            if (maxrows > 1)
             {
                 if (currentrow < (int)buttonRows.size() - 1)
                     currentrow++;
@@ -2305,7 +2324,7 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
         else if (action == "SELECT")
         {
             if (class LCD * lcddev = LCD::Get())
-                lcddev->stopAll();
+                lcddev->switchToTime();
 
             lastbutton = activebutton;
             activebutton = NULL;
@@ -2386,6 +2405,7 @@ QString ThemedMenuPrivate::findMenuFile(const QString &menuname)
 
 bool ThemedMenuPrivate::handleAction(const QString &action)
 {
+    bool ok = true;
     if (action.left(5) == "EXEC ")
     {
         QString rest = action.right(action.length() - 5);
@@ -2442,7 +2462,7 @@ bool ThemedMenuPrivate::handleAction(const QString &action)
             return true;
         }
 
-        parseMenu(rest);
+        ok = parseMenu(rest);
     }
     else if (action.left(6) == "UPMENU")
     {
@@ -2454,7 +2474,7 @@ bool ThemedMenuPrivate::handleAction(const QString &action)
 
         menulevel -= 2;
  
-        parseMenu(file, row, col);
+        ok = parseMenu(file, row, col);
     }
     else if (action.left(12) == "CONFIGPLUGIN")
     {
@@ -2489,6 +2509,11 @@ bool ThemedMenuPrivate::handleAction(const QString &action)
         }
     }
 
+    if (!ok)
+    {
+        exit(FIXME_BUG__LIBRARY_EXIT_NO_THEME);
+    }
+
     return true;
 }
 
@@ -2499,7 +2524,7 @@ bool ThemedMenuPrivate::findDepends(const QString &fileList)
     
     for ( QStringList::Iterator it = files.begin(); it != files.end(); ++it ) {
         filename = findMenuFile(*it);
-        if (filename != "")
+        if (filename != "" && filename.endsWith(".xml"))
             return true;
     
         QString newname = gContext->FindPlugin(*it);
@@ -2565,6 +2590,12 @@ bool ThemedMenuPrivate::checkPinCode(const QString &timestamp_setting,
     return false;
 }
 
+void ThemedMenuPrivate::gotoMainMenu(void)
+{
+    menulevel = 0;
+    parseMenu("mainmenu.xml");
+}
+
 ThemedMenu::ThemedMenu(const char *cdir, const char *menufile,
                        MythMainWindow *parent, bool allowreorder)
           : MythDialog(parent, 0)
@@ -2589,7 +2620,6 @@ void ThemedMenu::Init(const char *cdir, const char *menufile)
     QString dir = QString(cdir) + "/";
     QString filename = dir + "theme.xml";
 
-    d->foundtheme = true;
     QFile filetest(filename);
     if (!filetest.exists())
     {
@@ -2603,9 +2633,8 @@ void ThemedMenu::Init(const char *cdir, const char *menufile)
 
     ReloadExitKey();
 
-    d->parseSettings(dir, "theme.xml");
-
-    d->parseMenu(menufile);
+    if (d->foundtheme = d->parseSettings(dir, "theme.xml"))
+        d->foundtheme = d->parseMenu(menufile);
 }
 
 ThemedMenu::~ThemedMenu(void)
@@ -2669,7 +2698,8 @@ void ThemedMenu::paintEvent(QPaintEvent *e)
 
 void ThemedMenu::ReloadTheme(void)
 {
-    d->ReloadTheme();
+    if (!d->ReloadTheme())
+        d->foundtheme = false;
 }
 
 void ThemedMenu::keyPressEvent(QKeyEvent *e)
@@ -2679,8 +2709,15 @@ void ThemedMenu::keyPressEvent(QKeyEvent *e)
 
     d->ignorekeys = true;
 
-    if (!d->keyPressHandler(e))
+    if ((e->key() == Qt::Key_L) && (e->state() == Qt::ControlButton))
+        gotoMainMenu();
+    else if (!d->keyPressHandler(e))
         MythDialog::keyPressEvent(e);
 
     d->ignorekeys = false;
 }
+
+void ThemedMenu::gotoMainMenu(void)
+{
+    d->gotoMainMenu();
+}
