@@ -19,11 +19,13 @@
 #include "util.h"
 #include "mythdialogs.h"
 #include "generictree.h"
+#include "mythwidgets.h"
 
 using namespace std;
 
 class UIType;
 class MythDialog;
+class MythThemedDialog;
 
 struct fontProp {
     QFont face;
@@ -37,7 +39,7 @@ class LayerSet
   public:
     LayerSet(const QString &name);
    ~LayerSet();
- 
+
     void    Draw(QPainter *, int, int);
     void    DrawRegion(QPainter *, QRect &, int, int);
 
@@ -83,7 +85,7 @@ class LayerSet
 class UIType : public QObject
 {
   Q_OBJECT
-  
+
   public:
     UIType(const QString &name);
     virtual ~UIType();
@@ -105,15 +107,16 @@ class UIType : public QObject
     virtual void DrawRegion(QPainter *, QRect &, int, int);
     virtual void calculateScreenArea();
     QRect   getScreenArea(){return screen_area;}
-    QString cutDown(const QString &data, QFont *font, bool multiline = false, 
+    QString cutDown(const QString &data, QFont *font, bool multiline = false,
                     int overload_width = -1, int overload_height = -1);
     QString getName(){return m_name;}
 
     bool    isShown(){return !hidden;}
     bool    isHidden(){return hidden;}
+    bool    isFocused(){return has_focus;}
     
   public slots:
-  
+
     virtual bool takeFocus();
     virtual void looseFocus();
     virtual void activate(){}
@@ -121,10 +124,10 @@ class UIType : public QObject
     virtual void show();
     virtual void hide();
     virtual bool toggleShow();
-    
+
 
   signals:
-  
+
     //
     //  Ask my container to ask Qt to repaint
     //
@@ -136,7 +139,7 @@ class UIType : public QObject
     void loosingFocus();
 
   protected:
-  
+
     double   m_wmult;
     double   m_hmult;
     int      m_context;
@@ -197,10 +200,10 @@ class AlphaTable
     AlphaTable();
     ~AlphaTable();
     AlphaTable(const QColor &color, int alpha);
-   
+
     unsigned char r[256], g[256], b[256];
-  
-  private:  
+
+  private:
     void maketable(unsigned char* data, int channel, int alpha);
 };
 
@@ -212,7 +215,7 @@ class AlphaBlender
     void init(int alpha = 96, int cacheSize = 30);
     void addColor(const QColor &color);
     void blendImage(const QImage &image, const QColor &color);
-  
+
   private:
     QDict<AlphaTable> alphaTables;
     int alpha;
@@ -237,13 +240,13 @@ class UIGuideType : public UIType
     void SetSelectorType(int type) { seltype = type; }
     void SetNumRows(int numRows) { this->numRows = numRows; }
     void SetWindow(MythDialog *win) { window = win; }
-    
-    void SetRecordingColors(const QString &reccol, const QString &concol) 
+
+    void SetRecordingColors(const QString &reccol, const QString &concol)
                   { reccolor = QColor(reccol); concolor = QColor(concol); }
     void SetSelectorColor(const QString &col) { selcolor = QColor(col); }
     void SetSolidColor(const QString &col) { solidcolor = QColor(col); }
     void SetCategoryColors(const QMap<QString, QString> &catColors);
-    
+
     void SetArea(const QRect &area) { this->area = area; }
     void SetScreenLocation(const QPoint &sl) { screenloc = sl; }
     void SetTextOffset(const QPoint &to) { textoffset = to; }
@@ -254,6 +257,7 @@ class UIGuideType : public UIType
                         int arrow, int recType, int recStat, bool selected);
     void ResetData();
     void ResetRow(int row);
+    void SetProgPast(int ppast);
 
   private:
 
@@ -261,7 +265,7 @@ class UIGuideType : public UIType
     {
       public:
         UIGTCon() { arrow = recType = recStat = 0; };
-        UIGTCon(const QRect &drawArea, const QString &title, 
+        UIGTCon(const QRect &drawArea, const QString &title,
                 const QString &category, int arrow, int recType, int recStat)
         {
             this->drawArea = drawArea;
@@ -281,8 +285,8 @@ class UIGuideType : public UIType
             arrow = o.arrow;
             recType = o.recType;
             recStat = o.recStat;
-        }        
-        
+        }
+
         QRect drawArea;
         QString title;
         QString category;
@@ -291,22 +295,22 @@ class UIGuideType : public UIType
         int recType;
         int recStat;
     };
-    
-    void drawBackground(QPainter *dr, UIGTCon *data); 
+
+    void drawBackground(QPainter *dr, UIGTCon *data);
     void drawBox(QPainter *dr, UIGTCon *data, const QColor &color);
     void drawText(QPainter *dr, UIGTCon *data);
     void drawRecType(QPainter *dr, UIGTCon *data);
     void drawCurrent(QPainter *dr, UIGTCon *data);
-    
+
     QPtrList<UIGTCon> *allData;
     UIGTCon selectedItem;
-    
+
     QPixmap recImages[15];
     QPixmap arrowImages[15];
-    
+
     int maxRows;
     int numRows;
-    
+
     MythDialog *window;
     QRect area;
     QPoint textoffset;
@@ -316,13 +320,13 @@ class UIGuideType : public UIType
     bool multilineText;
     fontProp *font;
     QColor solidcolor;
-    
+
     QColor selcolor;
     int seltype;
-    
+
     QColor reccolor;
     QColor concolor;
-    
+
     int filltype;
     bool cutdown;
     bool drawCategoryColors;
@@ -330,6 +334,8 @@ class UIGuideType : public UIType
 
     QMap<QString, QColor> categoryColors;
     AlphaBlender alphaBlender;
+
+    int prog_past_col;
 };
 
 class UIListType : public UIType
@@ -337,43 +343,55 @@ class UIListType : public UIType
   public:
     UIListType(const QString &, QRect, int);
     ~UIListType();
-  
-    void SetCount(int cnt) { m_count = cnt; 
-                             m_selheight = (int)(m_area.height() / m_count); }
+
+    enum ItemArrows  { ARROW_NONE, ARROW_LEFT, ARROW_RIGHT, ARROW_BOTH };
+
+    void SetCount(int cnt) { m_count = cnt;
+                             if (m_count)
+                                m_selheight = (int)(m_area.height() / m_count); 
+                             else
+                                m_selheight = 0;
+                           }
+                                
 
     void SetItemText(int, int, QString);
     void SetItemText(int, QString);
- 
+    void SetItemArrow(int, int);
+
     void SetActive(bool act) { m_active = act; }
     void SetItemCurrent(int cur) { m_current = cur; }
 
     void SetColumnWidth(int col, int width) { columnWidth[col] = width; }
     void SetColumnContext(int col, int context) { columnContext[col] = context; }
     void SetColumnPad(int pad) { m_pad = pad; }
-
-    void SetImageUpArrow(QPixmap img, QPoint loc) { 
+    void SetImageLeftArrow(QPixmap img, QPoint loc) {
+                           m_leftarrow = img; m_leftarrow_loc = loc; }
+    void SetImageRightArrow(QPixmap img, QPoint loc) {
+                           m_rightarrow = img; m_rightarrow_loc = loc;}
+    void SetImageUpArrow(QPixmap img, QPoint loc) {
                          m_uparrow = img; m_uparrow_loc = loc; }
-    void SetImageDownArrow(QPixmap img, QPoint loc) { 
+    void SetImageDownArrow(QPixmap img, QPoint loc) {
                            m_downarrow = img; m_downarrow_loc = loc; }
-    void SetImageSelection(QPixmap img, QPoint loc) { 
+    void SetImageSelection(QPixmap img, QPoint loc) {
                            m_selection = img; m_selection_loc = loc; }
 
     void SetUpArrow(bool arrow) { m_uarrow = arrow; }
     void SetDownArrow(bool arrow) { m_darrow = arrow; }
- 
+
     void SetFill(QRect area, QColor color, int type) {
-                          m_fill_area = area; m_fill_color = color; 
+                          m_fill_area = area; m_fill_color = color;
                           m_fill_type = type; }
 
-    void SetFonts(QMap<QString, QString> fonts, QMap<QString, fontProp> fontfcn) { 
+    void SetFonts(QMap<QString, QString> fonts, QMap<QString, fontProp> fontfcn) {
                           m_fonts = fonts; m_fontfcns = fontfcn; }
     void EnableForcedFont(int num, QString func) { forceFonts[num] = m_fonts[func]; }
 
+    int GetCurrentItem() { return m_current; }
     int GetItems() { return m_count; }
     QString GetItemText(int, int col = 1);
-    void ResetList() { listData.clear(); forceFonts.clear(); 
+    void ResetList() { listData.clear(); forceFonts.clear(); listArrows.clear();
                        m_current = -1; m_columns = 0; }
- 
+    void ResetArrows() { listArrows.clear(); }
     void Draw(QPainter *, int drawlayer, int);
     bool ShowSelAlways() const { return m_showSelAlways; }
     void ShowSelAlways(bool bnew) { m_showSelAlways = bnew; }
@@ -395,14 +413,19 @@ class UIListType : public UIType
     QPixmap m_selection;
     QPixmap m_downarrow;
     QPixmap m_uparrow;
+    QPixmap m_leftarrow;
+    QPixmap m_rightarrow;
     QPoint m_selection_loc;
     QPoint m_downarrow_loc;
     QPoint m_uparrow_loc;
+    QPoint m_rightarrow_loc;
+    QPoint m_leftarrow_loc;
     QRect m_area;
     QMap<QString, QString> m_fonts;
     QMap<QString, fontProp> m_fontfcns;
     QMap<int, QString> forceFonts;
     QMap<int, QString> listData;
+    QMap<int, int> listArrows;
     QMap<int, int> columnWidth;
     QMap<int, int> columnContext;
 };
@@ -417,7 +440,7 @@ class UIImageType : public UIType
 
     QPoint DisplayPos() { return m_displaypos; }
     void SetPosition(QPoint pos) { m_displaypos = pos; }
-  
+
     void SetFlex(bool flex) { m_flex = flex; }
     void ResetFilename() { m_filename = orig_filename; };
     void SetImage(QString file) { m_filename = file; }
@@ -431,8 +454,10 @@ class UIImageType : public UIType
     int GetSize() { return m_force_x; }
     virtual void Draw(QPainter *, int, int);
 
+    const QString& GetImageFilename() const { return m_filename; }
+
   public slots:
-  
+
     //
     //  We have to redefine this, as pixmaps
     //  are not of a fixed size. And it has to
@@ -450,6 +475,7 @@ class UIImageType : public UIType
     bool    m_isvalid;
     bool    m_flex;
     bool    m_show;
+    bool    m_transparent;
     int     m_drop_x;
     int     m_drop_y;
     int     m_force_x;
@@ -458,35 +484,95 @@ class UIImageType : public UIType
 
 };
 
+class UIAnimatedImageType : public UIType
+{
+    Q_OBJECT
+
+  public:
+    UIAnimatedImageType(const QString &, const QString &, int, int, int, int, QPoint);
+    ~UIAnimatedImageType();
+
+    QPoint DisplayPos() { return m_displaypos; }
+    void SetPosition(QPoint pos) { m_displaypos = pos; }
+
+    void SetFlex(bool flex) { m_flex = flex; }
+    void SetFilename(QString file) { m_filename = file; }
+    void SetSize(int x, int y) { m_force_x = x; m_force_y = y; }
+    void SetSkip(int x, int y) { m_drop_x = x; m_drop_y = y; }
+    void SetImageCount(int count);
+    int  GetImageCount() { return m_imagecount; }
+    void SetInterval(int interval) { m_interval = interval; }
+    int  GetInterval() { return m_interval; }
+    void SetStartInterval(int interval) { m_startinterval = interval; }
+    int  GetStartInterval() { return m_startinterval; }
+    void SetWindow(MythDialog *win) { m_window = win; }
+    void LoadImages();
+    void Pause();
+    void UnPause();
+    bool IsPaused() { return !timer.isActive(); }
+    void NextImage();
+    void PreviousImage();
+    void GotoFirstImage() { m_currentimage = 1; }
+     
+    virtual void Draw(QPainter *, int, int);
+
+  public slots:
+    void refresh();
+    void IntervalTimeout();
+
+  protected:
+    void InitImageCache();
+    void ClearImages();
+    bool LoadImage(int imageNo);
+
+    QPoint  m_displaypos;
+    QString orig_filename;
+    QString m_filename;
+    bool    m_isvalid;
+    bool    m_flex;
+    bool    m_show;
+    int     m_drop_x;
+    int     m_drop_y;
+    int     m_force_x;
+    int     m_force_y;
+    int     m_imagecount;
+    int     m_currentimage;
+    int     m_interval;
+    int     m_startinterval;
+    vector<QPixmap *> *imageList;
+    QTimer  timer;
+    MythDialog* m_window;
+};
+
 class UIRepeatedImageType : public UIImageType
 {
     Q_OBJECT
-    
+
   public:
-  
+
     UIRepeatedImageType(const QString &, const QString &, int, QPoint);
     void setRepeat(int how_many);
     void Draw(QPainter *, int, int);
     void setOrientation(int x);
 
   public slots:
-  
+
     void refresh();
-    
+
   protected:
-  
+
     int m_repeat;
     int m_highest_repeat;
-    
+
   private:
-  
+
     //  0 = left to right
     //  1 = right to left
     //  2 = bottom to top
     //  3 = top to bottom
-    
+
     int m_orientation;
-    
+
 };
 
 
@@ -524,7 +610,6 @@ class UITextType : public UIType
     QRect m_altdisplaysize;
     QString m_message;
     QString m_default_msg;
-    QString m_name;
 
     fontProp *m_font;
 
@@ -532,12 +617,98 @@ class UITextType : public UIType
 
 };
 
+class UIRichTextType : public UIType
+{
+    Q_OBJECT
+
+    public:
+        UIRichTextType(const QString &, fontProp *, const QString &, int,
+                   QRect displayrect, QRect textrect);
+        ~UIRichTextType();
+
+        QString Name() { return m_name; }
+
+        void SetText(const QString &text);
+        QString GetText() { return m_message; }
+
+        QRect DisplayArea() { return m_displayArea; }
+        void SetDisplayArea(const QRect &rect) { m_displayArea = rect; }
+        void SetShowScrollArrows(bool bShowArrows) 
+            { m_showScrollArrows = bShowArrows; }
+
+        QRect TextArea() { return m_textArea; }
+        void SetTextArea(const QRect &rect) { m_textArea = rect; }
+
+        void SetImageUpArrowReg(QPixmap img, QPoint loc) 
+            { m_upArrowReg = img; m_upArrowRegPos = loc; }
+        void SetImageDnArrowReg(QPixmap img, QPoint loc) 
+            { m_dnArrowReg = img; m_dnArrowRegPos = loc; }
+        void SetImageUpArrowSel(QPixmap img, QPoint loc) 
+            { m_upArrowSel = img; m_upArrowSelPos = loc; }
+        void SetImageDnArrowSel(QPixmap img, QPoint loc) 
+            { m_dnArrowSel = img; m_dnArrowSelPos = loc; }
+
+        void SetBackground(QPixmap *background);
+        void SetBackgroundImages(QString bgImageReg, QString bgImageSel);
+
+        virtual void calculateScreenArea();
+
+        virtual void Draw(QPainter *, int, int);
+    public slots:
+        void ScrollDown(void);
+        void ScrollUp(void);
+        void ScrollPageDown(void);
+        void ScrollPageUp(void);
+
+        virtual bool takeFocus();
+        virtual void looseFocus();
+
+    protected:
+        void refreshImage();
+        void updateBackground();
+        void loadBackgroundImg(bool &changed);
+
+        QRect m_displayArea;
+        QRect m_textArea;
+
+        int   m_yPos;        // current scroll position
+        int   m_textHeight;  // total height of rendered text
+
+        QString m_message;
+        QString m_bgImageReg;
+        QString m_bgImageSel;
+
+        fontProp *m_font;
+
+        QPixmap *m_background;    // clipped image of the window background 
+        QPixmap *m_compBackground;// composite of the window background + the
+                                  // widget background image 
+        QPixmap *m_image;         // the completed image including the rich text
+
+        QString m_backgroundFile;   //< current source of background image
+        QImage  *m_backgroundImage; //< current widget background image)
+
+        bool     m_showScrollArrows;
+        bool     m_showUpArrow;
+        bool     m_showDnArrow;
+
+        QPoint   m_upArrowRegPos;
+        QPoint   m_dnArrowRegPos;
+        QPoint   m_upArrowSelPos;
+        QPoint   m_dnArrowSelPos;
+
+        QPixmap  m_upArrowReg;
+        QPixmap  m_dnArrowReg;
+        QPixmap  m_upArrowSel;
+        QPixmap  m_dnArrowSel;
+};
+
 class UIMultiTextType : public UITextType
 {
 
   Q_OBJECT
 
-  public: 
+  public:
 
     enum AnimationStage
     {
@@ -548,32 +719,32 @@ class UIMultiTextType : public UITextType
     };
 
     UIMultiTextType(
-                    const QString &, 
-                    fontProp *, 
+                    const QString &,
+                    fontProp *,
                     int,
-                    QRect displayrect, 
+                    QRect displayrect,
                     QRect altdisplayrect
                    );
 
     void setTexts(QStringList new_messagep);
     void clearTexts();
     void Draw(QPainter*, int, int);
-  
+
     void setDropTimingLength(int x){drop_timing_length = x;}
     void setDropTimingPause(int x){drop_timing_pause = x;}
     void setScrollTimingLength(int x){scroll_timing_length = x;}
     void setScrollTimingPause(int x){scroll_timing_pause = x;}
-    
+
     void setMessageSpacePadding(int x){message_space_padding = x;}
-    
-  
+
+
   private slots:
 
     void        animate();
 
   private:
 
-    QStringList     messages;  
+    QStringList     messages;
     int             current_text_index;
     QTimer          transition_timer;
     AnimationStage  animation_stage;
@@ -584,9 +755,52 @@ class UIMultiTextType : public UITextType
     int             drop_timing_pause;
     int             scroll_timing_length;
     int             scroll_timing_pause;
-    
-    int             message_space_padding;    
 
+    int             message_space_padding;
+
+};
+
+class UIRemoteEditType : public UIType
+{
+    Q_OBJECT
+
+  public:
+
+    UIRemoteEditType(const QString &name, fontProp *font, const QString &text, 
+                     int dorder, QRect displayrect);
+    ~UIRemoteEditType();
+    
+    void    createEdit(MythThemedDialog* parent);
+    QWidget *getEdit(void) { return (QWidget*) edit; };
+    void    Draw(QPainter *, int drawlayer, int context);
+    void    setArea(QRect area){m_displaysize = area;}
+    void    setText(const QString some_text);
+    QString getText();
+    void    setFont(fontProp *font);
+    void    setCharacterColors(QColor unselected, QColor selected, QColor special);
+    void    calculateScreenArea();
+
+  public slots:
+    void takeFocusAwayFromEditor(bool up_or_down);
+    void editorChanged(QString value);
+    virtual bool takeFocus();
+    virtual void looseFocus();
+    virtual void show();
+    virtual void hide();
+     
+  signals:
+    void    textChanged(QString value);
+
+  private:
+    MythRemoteLineEdit *edit;
+    QRect    m_displaysize;
+    QString  m_text;
+    fontProp *m_font;
+    QColor   m_unselected; 
+    QColor   m_selected; 
+    QColor   m_special;
+
+    MythThemedDialog* m_parentDialog;
 };
 
 class UIStatusBarType : public UIType
@@ -616,14 +830,14 @@ class UIStatusBarType : public UIType
     QPixmap m_container;
     QPixmap m_filler;
     QPoint m_location;
-    
+
     // 0 = left to right
     // 1 = left to right
     // 2 = left to right
     // 3 = left to right
-    
+
     int m_orientation;
-    
+
 };
 
 class UIManagedTreeListType : public UIType
@@ -646,7 +860,10 @@ class UIManagedTreeListType : public UIType
 
     UIManagedTreeListType(const QString &name);
     ~UIManagedTreeListType();
-
+    void    setUpArrowOffset(QPoint& pt) { upArrowOffset = pt;}    
+    void    setDownArrowOffset(QPoint& pt) { downArrowOffset = pt;}    
+    void    setSelectPoint(QPoint& pt) { selectPoint = pt;}
+    void    setSelectPadding(int pad) {selectPadding = pad;}
     void    setArea(QRect an_area) { area = an_area; }
     void    setBins(int l_bins) { bins = l_bins; }
     void    setBinAreas(CornerMap some_bin_corners) { bin_corners = some_bin_corners; }
@@ -666,9 +883,10 @@ class UIManagedTreeListType : public UIType
                             left_arrow_image = left;
                             right_arrow_image = right;
                           }
-    void    setFonts(QMap<QString, QString> fonts, QMap<QString, fontProp> fontfcn) { 
+    void    addIcon(int i, QPixmap *img) { iconMap[i] = img; }
+    void    setFonts(QMap<QString, QString> fonts, QMap<QString, fontProp> fontfcn) {
                           m_fonts = fonts; m_fontfcns = fontfcn; }
-    void    drawText(QPainter *p, QString the_text, QString font_name, int x, int y, int bin_number);
+    void    drawText(QPainter *p, QString the_text, QString font_name, int x, int y, int bin_number, int icon_number);
     void    setJustification(int jst) { m_justification = jst; }
     int     getJustification() { return m_justification; }
     void    makeHighlights();
@@ -676,12 +894,13 @@ class UIManagedTreeListType : public UIType
     void    forceLastBin(){active_bin = bins; refresh();}
     void    calculateScreenArea();
     void    setTreeOrdering(int an_int){tree_order = an_int;}
-    void    setVisualOrdering(int an_int){visual_order = an_int;}    
+    void    setVisualOrdering(int an_int){visual_order = an_int;}
+    void    setIconSelector(int an_int){iconAttr = an_int;}
     void    showWholeTree(bool yes_or_no){ show_whole_tree = yes_or_no; }
     void    scrambleParents(bool yes_or_no){ scrambled_parents = yes_or_no; }
     void    colorSelectables(bool yes_or_no){color_selectables = yes_or_no; }
-    void    sortTreeByString(){if(my_tree_data) my_tree_data->sortByString(); }
-    void    sortTreeBySelectable(){if(my_tree_data) my_tree_data->sortBySelectable();}
+    void    sortTreeByString(){if (my_tree_data) my_tree_data->sortByString(); }
+    void    sortTreeBySelectable(){if (my_tree_data) my_tree_data->sortBySelectable();}
     GenericTree *getCurrentNode() { return current_node; }
     void    setCurrentNode(GenericTree *a_node);
     int     getActiveBin();
@@ -701,7 +920,9 @@ class UIManagedTreeListType : public UIType
     void    activate();
     void    enter();
     void    deactivate(){active_node = NULL;}
-    
+    bool    incSearchStart();
+    bool    incSearchNext();
+
   signals:
 
     void    nodeSelected(int, IntVector*); //  emit int and attributes when user selects a node
@@ -710,7 +931,7 @@ class UIManagedTreeListType : public UIType
   private:
 
     int   calculateEntriesInBin(int bin_number);
-    bool  complexInternalNextPrevActive(bool forward_or_reverse, bool wrap_around);    
+    bool  complexInternalNextPrevActive(bool forward_or_reverse, bool wrap_around);
     QRect area;
     int   bins;
     int   active_bin;
@@ -723,7 +944,9 @@ class UIManagedTreeListType : public UIType
     GenericTree *active_node;
     int         tree_order;
     int         visual_order;
-
+    int         iconAttr;
+    int         selectPadding;
+    
     QMap<QString, QString>  m_fonts;
     QMap<QString, fontProp> m_fontfcns;
     int                     m_justification;
@@ -738,48 +961,53 @@ class UIManagedTreeListType : public UIType
     bool                    show_whole_tree;
     bool                    scrambled_parents;
     bool                    color_selectables;
-
+    QMap<int, QPixmap*>     iconMap;
+    QPoint                  selectPoint;
+    QPoint                  upArrowOffset;
+    QPoint                  downArrowOffset;
+    QString                 incSearch;
+    bool                    bIncSearchContains;
 };
 
 class UIPushButtonType : public UIType
 {
     Q_OBJECT
-    
+
   public:
-    
+
     UIPushButtonType(const QString &name, QPixmap on, QPixmap off, QPixmap pushed);
 
     virtual void Draw(QPainter *, int drawlayer, int context);
     void    setPosition(QPoint pos){m_displaypos = pos;}
     virtual void calculateScreenArea();
-    
-  public slots:  
-  
+
+  public slots:
+
     virtual void push();
     virtual void unPush();
     virtual void activate(){push();}
- 
+
   signals:
-  
+
     void    pushed();
-  
+
   protected:
-  
+
     QPoint  m_displaypos;
     QPixmap on_pixmap;
     QPixmap off_pixmap;
     QPixmap pushed_pixmap;
     bool    currently_pushed;
     QTimer  push_timer;
-    
+
 };
 
 class UITextButtonType : public UIType
 {
     Q_OBJECT
-    
+
   public:
-  
+
     UITextButtonType(const QString &name, QPixmap on, QPixmap off, QPixmap pushed);
 
     void    Draw(QPainter *, int drawlayer, int context);
@@ -787,19 +1015,19 @@ class UITextButtonType : public UIType
     void    setText(const QString some_text);
     void    setFont(fontProp *font) { m_font = font; }
     void    calculateScreenArea();
-    
-  public slots:  
-  
+
+  public slots:
+
     void    push();
     void    unPush();
     void    activate(){push();}
- 
+
   signals:
-  
+
     void    pushed();
-  
+
   private:
-  
+
     QPoint   m_displaypos;
     QPixmap  on_pixmap;
     QPixmap  off_pixmap;
@@ -808,7 +1036,7 @@ class UITextButtonType : public UIType
     fontProp *m_font;
     bool     currently_pushed;
     QTimer   push_timer;
-    
+
 };
 
 
@@ -816,36 +1044,37 @@ class UITextButtonType : public UIType
 class UICheckBoxType : public UIType
 {
     Q_OBJECT
-    
+
     //
     //  A simple, theme-able check box
     //
 
-  public: 
-     
-    UICheckBoxType(const QString &name, 
-                   QPixmap checkedp, 
+  public:
+
+    UICheckBoxType(const QString &name,
+                   QPixmap checkedp,
                    QPixmap uncheckedp,
                    QPixmap checked_highp,
                    QPixmap unchecked_highp);
-                   
+
     void    Draw(QPainter *, int drawlayer, int context);
     void    setPosition(QPoint pos){m_displaypos = pos;}
     void    calculateScreenArea();
-  
+    bool    getState(){return checked;}
+     
   public slots:
-  
+
     void    push();
     void    setState(bool checked_or_not);
     void    toggle(){push();}
     void    activate(){push();}
-    
+
   signals:
-  
+
     void    pushed(bool state);
 
   private:
-  
+
     QPoint  m_displaypos;
     QPixmap checked_pixmap;
     QPixmap unchecked_pixmap;
@@ -859,19 +1088,19 @@ class IntStringPair
 {
     //  Miniscule class for holding data
     //  in a UISelectorType (below)
-    
+
   public:
-  
+
     IntStringPair(int an_int, const QString a_string){my_int = an_int; my_string = a_string;}
 
     void    setString(const QString &a_string){my_string = a_string;}
     void    setInt(int an_int){my_int = an_int;}
     QString getString(){return my_string;}
     int     getInt(){return my_int;}
-    
+
 
   private:
-  
+
     int     my_int;
     QString my_string;
 
@@ -881,9 +1110,9 @@ class IntStringPair
 class UISelectorType : public UIPushButtonType
 {
     Q_OBJECT
-    
+
     //
-    //  A theme-able "thingy" (that's the 
+    //  A theme-able "thingy" (that's the
     //  offical term) that can hold a list
     //  of strings and emit an int and/or
     //  the string when the user selects
@@ -891,29 +1120,32 @@ class UISelectorType : public UIPushButtonType
     //
 
   public:
-  
+
     UISelectorType(const QString &name, QPixmap on, QPixmap off, QPixmap pushed, QRect area);
     ~UISelectorType();
-        
+
     void    Draw(QPainter *, int drawlayer, int context);
     void    calculateScreenArea();
     void    addItem(int an_int, const QString &a_string);
     void    setFont(fontProp *font) { m_font = font; }
+    QString getCurrentString();
+    int     getCurrentInt();
     
-  public slots:  
-  
+  public slots:
+
     void push(bool up_or_down);
     void unPush();
     void activate(){push(true);}
     void cleanOut(){current_data = NULL; my_data.clear();}
     void setToItem(int which_item);
- 
+    void setToItem(const QString &which_item);
+     
   signals:
-  
+
     void    pushed(int);
-  
+
   private:
-  
+
     QRect                   m_area;
     fontProp                *m_font;
     QPtrList<IntStringPair> my_data;
@@ -927,7 +1159,7 @@ class UISelectorType : public UIPushButtonType
 class UIBlackHoleType : public UIType
 {
     Q_OBJECT
-    
+
     //
     //  This just holds a blank area of the screen
     //  originally designed to "hold" the place where
@@ -935,14 +1167,14 @@ class UIBlackHoleType : public UIType
     //
 
   public:
-      
+
     UIBlackHoleType(const QString &name);
     void calculateScreenArea();
     void setArea(QRect an_area) { area = an_area; }
     virtual void Draw(QPainter *, int, int){}
 
   protected:
-  
+
     QRect area;
 };
 
@@ -954,36 +1186,83 @@ class UIKeyType : public UIType
     UIKeyType(const QString &);
     ~UIKeyType();
 
-    void SetContainer(LayerSet *container) { m_container = container; }
-    UIType *GetType(const QString &name) { return m_container->GetType(name); }
-
-    QString GetAction() { return m_action; }
-    QString GetChars() { return m_chars; }
     QPoint GetPosition() { return m_pos; }
-    QString GetClicked() { return m_clicked; }
-    QString GetNotClicked() { return m_notclicked; }
-    QString GetSubtitle() { return m_subtitle; }
 
-    void SetAction(QString action) { m_action = action; }
-    void SetClicked(QString filename) { m_clicked = filename; }
-    void SetNotClicked(QString filename) { m_notclicked = filename; }
-    void SetSubtitle(QString filename) { m_subtitle = filename; }
-    void SetChars(QString chars) { m_chars = chars; }
-    void SetFont(QString font) { m_font = font; }
+    void SetArea(QRect &area) { m_area = area; }
     void SetPosition(QPoint pos) { m_pos = pos; }
+    void SetImages(QPixmap *normal, QPixmap *focused, QPixmap *down,
+                   QPixmap *downFocused);
+    void SetDefaultImages(QPixmap *normal, QPixmap *focused, QPixmap *down,
+                          QPixmap *downFocused);
 
-    virtual void Draw(QPainter *, int, int);
+    void SetFonts(fontProp *normal, fontProp *focused, fontProp *down,
+                  fontProp *downFocused);
+    void SetDefaultFonts(fontProp *normal, fontProp *focused, fontProp *down,
+                         fontProp *downFocused);
+
+    void    SetType(QString type) { m_type = type; }
+    QString GetType() { return m_type; }
+
+    void    SetChars(QString normal, QString shift, QString alt, QString shiftAlt);
+    QString GetChar();
+
+    void    SetMoves(QString moveLeft, QString moveRight, QString moveUp, 
+                  QString moveDown);
+    QString GetMove(QString direction);
+
+    void SetShiftState(bool sh, bool ag); 
+    void SetOn(bool bOn) { m_bDown = bOn; refresh(); }
+    bool IsOn(void) { return m_bDown; }
+
+    void SetToggleKey(bool bOn) { m_bToggle = bOn; }
+    bool IsToggleKey(void) { return m_bToggle; }
+
+   virtual void Draw(QPainter *, int, int);
+    virtual void calculateScreenArea();
+
+  public slots:
+    void    push();
+    void    unPush();
+    void    activate(){push();}
+
+  signals:
+    void    pushed();
 
   private:
-    QString m_action;
-    QString m_clicked;
-    QString m_notclicked;
-    QString m_subtitle;
-    QString m_chars;
-    QString m_font;
+    QString decodeChar(QString c);
+
+    QRect   m_area;
+    QString m_type;
+
+    QPixmap *m_normalImg;
+    QPixmap *m_focusedImg;
+    QPixmap *m_downImg;
+    QPixmap *m_downFocusedImg;
+
+    fontProp *m_normalFont;
+    fontProp *m_focusedFont;
+    fontProp *m_downFont;
+    fontProp *m_downFocusedFont;
+
     QPoint m_pos;
 
-    LayerSet *m_container;
+    QString m_normalChar; 
+    QString m_shiftChar; 
+    QString m_altChar; 
+    QString m_shiftAltChar;
+
+    QString m_moveLeft;
+    QString m_moveRight;
+    QString m_moveUp;
+    QString m_moveDown;
+
+    bool    m_bShift;
+    bool    m_bAlt;
+    bool    m_bDown;
+    bool    m_bToggle;
+
+    bool    m_bPushed;
+    QTimer  m_pushTimer;
 };
 
 class UIKeyboardType : public UIType
@@ -994,20 +1273,64 @@ class UIKeyboardType : public UIType
     UIKeyboardType(const QString &, int);
     ~UIKeyboardType();
 
-    typedef QValueList <UIKeyType*> KeyList;
-    void SetContainer(LayerSet *container) { m_container = container; }
-    UIType *GetType(const QString &name) { return m_container->GetType(name); }
-    UIKeyType *GetKey(const QString &action);
-    KeyList GetKeys() { return m_keymap.values(); }
+    typedef QPtrList <UIKeyType> KeyList;
 
-    void AddKey(const QString &action, UIKeyType *key) { m_keymap[action] = key; }
-    bool HasKey(const QString &action) { return m_keymap.keys().contains(action); }
+    void SetContainer(LayerSet *container) { m_container = container; }
+    void SetArea(QRect &area) { m_area = area; }
+    void SetEdit(QWidget* edit) { m_parentEdit = edit; }
+    void SetParentDialog(MythThemedDialog * parentDialog)
+            { m_parentDialog = parentDialog; }
+
+    KeyList GetKeys() { return m_keyList; }
+    void AddKey(UIKeyType *key);
 
     virtual void Draw(QPainter *, int, int);
+    virtual void calculateScreenArea();
+    virtual void keyPressEvent(QKeyEvent *e);
+
+  private slots:
+    void charKey();
+    void lockOnOff();
+    void shiftLOnOff();
+    void shiftROnOff();
+    void shiftOff();
+    void altGrOnOff();
+    void compOnOff();
+    void updateButtons();
+    void leftCursor();
+    void rightCursor();
+    void backspaceKey();
+    void delKey();
+    void close(); 
 
   private:
-    LayerSet *m_container;
-    QMap <QString, UIKeyType*> m_keymap;
+    void init();
+    void insertChar(QString c);
+    void moveUp();
+    void moveDown();
+    void moveLeft();
+    void moveRight();
+    UIKeyType *findKey(QString keyName);
+
+    QRect     m_area;
+
+    bool m_bInitalized;
+
+    bool      m_bCompTrap;
+    QString   m_comp1;
+
+    UIKeyType *m_altKey;
+    UIKeyType *m_lockKey;
+    UIKeyType *m_shiftLKey;
+    UIKeyType *m_shiftRKey;
+    UIKeyType *m_focusedKey;
+    UIKeyType *m_doneKey;
+
+    QWidget          *m_parentEdit;
+    MythThemedDialog *m_parentDialog;
+
+    LayerSet  *m_container;
+    KeyList   m_keyList;
 };
 
 #endif
