@@ -2,31 +2,39 @@
 #ifndef XVMC_SURFACE_TYPES_H_
 #define XVMC_SURFACE_TYPES_H_
 
-#include "../libmyth/mythcontext.h"
+#ifdef USING_XVMC
+
+#include <qwindowdefs.h>
+#include "mythcontext.h"
 #include <X11/extensions/XvMC.h>
+#include "util-x11.h"
 
-extern "C" {
 #include "../libavcodec/xvmc_render.h"
-}
+
+#ifndef XVMC_VLD
+#define XVMC_VLD 0x0020000
+#endif
+
+typedef enum { XvVLD, XvIDCT, XvMC } XvMCAccelID;
 
 class XvMCSurfaceTypes 
 {
   public:
     XvMCSurfaceTypes(Display *dpy, XvPortID port) : num(0) 
     {
-        surfaces = XvMCListSurfaceTypes(dpy, port, &num);
+        X11S(surfaces = XvMCListSurfaceTypes(dpy, port, &num));
     }
         
     ~XvMCSurfaceTypes() 
     {
-        XFree(surfaces);
+        X11S(XFree(surfaces));
     }
 
     /// Find an appropriate surface on the current port.
-    inline int find(int pminWidth, int pminHeight, int chroma, bool idct, 
-                    int mpeg, int pminSubpictureWidth, 
+    inline int find(int pminWidth, int pminHeight, int chroma, bool vld,
+                    bool idct, int mpeg, int pminSubpictureWidth, 
                     int pminSubpictureHeight);
-        
+
     bool hasChroma420(int surface) const 
     {
         return XVMC_CHROMA_FORMAT_420 == surfaces[surface].chroma_format;
@@ -79,6 +87,11 @@ class XvMCSurfaceTypes
         return XVMC_IDCT == (surfaces[surface].mc_type & XVMC_IDCT);
     }
 
+    bool hasVLDAcceleration(int surface) const
+    {
+        return XVMC_VLD == (surfaces[surface].mc_type & XVMC_VLD);
+    }
+
     bool hasMPEG1Support(int surface) const 
     {
         return XVMC_MPEG_1 == (surfaces[surface].mc_type & 0x7);
@@ -132,17 +145,18 @@ class XvMCSurfaceTypes
     int size() const { return num; }
 
     /// Find an appropriate surface on the current display.
-    static void find(int minWidth, int minHeight, int chroma, bool idct,
-                     int mpeg, int minSubpictureWidth, 
+    static void find(int minWidth, int minHeight, int chroma, bool vld,
+                     bool idct, int mpeg, int minSubpictureWidth, 
                      int minSubpictureHeight, Display *dpy, 
                      XvPortID portMin, XvPortID portMax,
                      XvPortID& port, int& surfNum);
 
-    /// Find out if there is an IDCT Acceleration capable surface on any port.
-    static bool hasIDCT(int width, int height,
-                        int chroma = XVMC_CHROMA_FORMAT_420,
-                        Display *disp = 0);
-
+    /// Find out if there is a surface on any port capable of a
+    /// specific acceleration type.
+    static bool has(Display *pdisp,
+                    XvMCAccelID accel_type, uint stream_type, int chroma,
+                    uint width, uint height, uint osd_width, uint osd_height);
+        
     ostream& print(ostream& os, int s) const;
     ostream& print(ostream& os) const;
     ostream& operator<<(ostream& os) const { return print(os); }
@@ -152,4 +166,6 @@ class XvMCSurfaceTypes
     XvMCSurfaceInfo *surfaces;
 };
 
-#endif
+#endif // USING_XVMC
+
+#endif // XVMC_SURFACE_TYPES_H_
