@@ -1,6 +1,7 @@
 #include <qsqldatabase.h>
 #include "settings.h"
 #include "mythcontext.h"
+#include "mythdbcon.h"
 #include "mythwidgets.h"
 #include "channeleditor.h"
 
@@ -17,12 +18,10 @@
 #include "channelsettings.h"
 #include "dvbtransporteditor.h"
 
-#ifdef USING_DVB
 #include "scanwizard.h"
-#endif
 
-ChannelWizard::ChannelWizard(int id, QSqlDatabase* _db)
-             : ConfigurationWizard(), db(_db) {
+ChannelWizard::ChannelWizard(int id)
+             : ConfigurationWizard() {
     setLabel(QObject::tr("Channel Options"));
 
     // Must be first.
@@ -45,30 +44,37 @@ ChannelWizard::ChannelWizard(int id, QSqlDatabase* _db)
 }
 
 QString ChannelWizard::getCardtype() {
-    QSqlQuery query = db->exec(QString("SELECT cardtype"
-        " FROM capturecard, cardinput, channel"
-        " WHERE channel.chanid=%1"
-        " AND channel.sourceid = cardinput.sourceid"
-        " AND cardinput.cardid = capturecard.cardid")
-        .arg(cid->getValue()));
-    if (query.isActive() && query.numRowsAffected() > 0)
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT cardtype"
+                  " FROM capturecard, cardinput, channel"
+                  " WHERE channel.chanid = :CHID "
+                  " AND channel.sourceid = cardinput.sourceid"
+                  " AND cardinput.cardid = capturecard.cardid");
+    query.bindValue(":CHID", cid->getValue()); 
+
+    if (query.exec() && query.isActive() && query.size() > 0)
     {
         query.next();
         return query.value(0).toString();
-    } else
+    }
+    else
+    {
         return "";
+    }
 }
 
 bool ChannelWizard::cardTypesInclude(const QString& thecardtype) {
-    QSqlQuery query = db->exec(QString("SELECT count(cardtype)"
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT count(cardtype)"
         " FROM capturecard, cardinput, channel"
-        " WHERE channel.chanid=%1"
+        " WHERE channel.chanid= :CHID "
         " AND channel.sourceid = cardinput.sourceid"
         " AND cardinput.cardid = capturecard.cardid"
-        " AND capturecard.cardtype=\"%2\"")
-        .arg(cid->getValue())
-        .arg(thecardtype));
-    if (query.isActive() && query.numRowsAffected() > 0)
+        " AND capturecard.cardtype= :CARDTYPE ");
+    query.bindValue(":CHID", cid->getValue());
+    query.bindValue(":CARDTYPE", thecardtype);
+
+    if (query.exec() && query.isActive() && query.size() > 0)
     {
         query.next();
         int count = query.value(0).toInt();
@@ -82,13 +88,15 @@ bool ChannelWizard::cardTypesInclude(const QString& thecardtype) {
 }
 
 int ChannelWizard::countCardtypes() {
-    QSqlQuery query = db->exec(QString("SELECT count(DISTINCT cardtype)"
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT count(DISTINCT cardtype)"
         " FROM capturecard, cardinput, channel"
-        " WHERE channel.chanid=%1"
+        " WHERE channel.chanid = :CHID "
         " AND channel.sourceid = cardinput.sourceid"
-        " AND cardinput.cardid = capturecard.cardid")
-        .arg(cid->getValue()));
-    if (query.isActive() && query.numRowsAffected() > 0)
+        " AND cardinput.cardid = capturecard.cardid");
+    query.bindValue(":CHID", cid->getValue());
+
+    if (query.exec() && query.isActive() && query.size() > 0)
     {
         query.next();
         return query.value(0).toInt();
@@ -101,55 +109,77 @@ void ChannelListSetting::fillSelections(void)
     QString currentValue = getValue();
     clearSelections();
     addSelection(QObject::tr("(New Channel)"));
+    bool fAllSources = true;
+
     QString querystr = "SELECT channel.name,channum,chanid ";
-    if ((currentSourceID == "") ||
-        (currentSourceID == "Unassigned"))
+
+    if ((currentSourceID == "") || (currentSourceID == "Unassigned"))
+    {
         querystr += ",videosource.name FROM channel "
                     "LEFT JOIN videosource ON "
                     "(channel.sourceid = videosource.sourceid) ";
+        fAllSources = true;
+    }
     else
+    {
         querystr += QString("FROM channel WHERE sourceid='%1' ")
                            .arg(currentSourceID);
+        fAllSources = false;
+    }
         
     if (currentSortMode == QObject::tr("Channel Name"))
+    {
         querystr += " ORDER BY channel.name";
+    }
     else if (currentSortMode == QObject::tr("Channel Number"))
+    {
         querystr += " ORDER BY channum + 0";
+    }
 
-    QSqlQuery query = db->exec(querystr);
-    if (query.isActive() && query.numRowsAffected() > 0)
-        while(query.next()) {
+    MSqlQuery query(MSqlQuery::InitCon()); 
+    query.prepare(querystr);
+    
+    if (query.exec() && query.isActive() && query.size() > 0)
+    {
+        while(query.next()) 
+        {
             QString name = QString::fromUtf8(query.value(0).toString());
             QString channum = query.value(1).toString();
             QString chanid = query.value(2).toString();
             QString sourceid = "Unassigned";
-			
-            if (!query.value(3).toString().isNull())
+
+            if (fAllSources && !query.value(3).toString().isNull())
             {
                 sourceid = query.value(3).toString();
                 if (currentSourceID == "Unassigned")
                     continue;
             }
  
-            if (channum == "" && currentHideMode) continue;
+            if (channum == "" && currentHideMode) 
+                continue;
 
-            if (name == "") name = "(Unnamed : " + chanid + ")";
+            if (name == "") 
+                name = "(Unnamed : " + chanid + ")";
 
-            if (currentSortMode == QObject::tr("Channel Name")) {
-                if (channum != "") name += " (" + channum + ")";
-            } else if (currentSortMode == QObject::tr("Channel Number")) {
+            if (currentSortMode == QObject::tr("Channel Name")) 
+            {
+                if (channum != "") 
+                    name += " (" + channum + ")";
+            }
+            else if (currentSortMode == QObject::tr("Channel Number")) 
+            {
                 if (channum != "")
                     name = channum + ". " + name;
                 else
                     name = "???. " + name;
             }
 
-            if ((currentSourceID == "") &&
-                (currentSourceID != "Unassigned"))
+            if ((currentSourceID == "") && (currentSourceID != "Unassigned"))
                 name += " (" + sourceid  + ")";
 
             addSelection(name, chanid, (chanid == currentValue) ? true : false);
         }
+    }
 }
 
 class SourceSetting: public ComboBoxSetting {
@@ -159,17 +189,22 @@ public:
         addSelection(QObject::tr("(All)"),"");
     };
 
-    void save(QSqlDatabase* db) { (void)db; };
-    void load(QSqlDatabase* db) {
-        QSqlQuery query = db->exec(QString(
-            "SELECT name, sourceid"
-            " FROM videosource"));
-        if (query.isActive() && query.numRowsAffected() > 0)
+    void save() {};
+    void load() 
+    {
+        MSqlQuery query(MSqlQuery::InitCon());
+        query.prepare("SELECT name, sourceid FROM videosource");
+
+        if (query.exec() && query.isActive() && query.size() > 0)
+        {
             while(query.next())
+            {
                 addSelection(query.value(0).toString(),
                              query.value(1).toString());
+            } 
+        } 
         addSelection(QObject::tr("(Unassigned)"),"Unassigned");
-    };
+    }
 };
 
 class SortMode: public ComboBoxSetting, public TransientStorage {
@@ -189,36 +224,51 @@ public:
 };
 
 ChannelEditor::ChannelEditor():
-    VerticalConfigurationGroup(), ConfigurationDialog() {
+    VerticalConfigurationGroup(), ConfigurationDialog()
+{
 
     setLabel(tr("Channels"));
     addChild(list = new ChannelListSetting());
 
-    SortMode* sort = new SortMode();
-    SourceSetting* source = new SourceSetting();
-    NoChanNumHide* hide = new NoChanNumHide();
+    SortMode           *sort   = new SortMode();
+    source                     = new SourceSetting();
+    TransButtonSetting *del    = new TransButtonSetting();
+    NoChanNumHide      *hide   = new NoChanNumHide();
+
+    del->setLabel(tr("Delete Channels"));
+    del->setHelpText(
+        tr("Delete all channels on currently selected source[s]."));
+
+    HorizontalConfigurationGroup *src =
+        new HorizontalConfigurationGroup(false, false, true, true);
+    src->addChild(source);
+    src->addChild(del);
 
     sort->setValue(sort->getValueIndex(list->getSortMode()));
     source->setValue(source->getValueIndex(list->getSourceID()));
     hide->setValue(list->getHideMode());
 
     addChild(sort);
-    addChild(source);
+    addChild(src);
     addChild(hide);
 
-#ifdef USING_DVB
     buttonScan = new TransButtonSetting();
-    buttonScan->setLabel(QObject::tr("Scan for channels(s)"));
-    buttonScan->setHelpText(QObject::tr("This button will scan for digital channels."));
-    buttonAdvanced = new TransButtonSetting();
-    buttonAdvanced->setLabel(QObject::tr("Advanced"));
-    buttonAdvanced->setHelpText(QObject::tr("Advanced editing options for digital channels"));
+    buttonScan->setLabel(QObject::tr("Channel Scanner"));
+    buttonScan->setHelpText(QObject::tr("Starts the channel scanner."));
 
-    HorizontalConfigurationGroup *h = new HorizontalConfigurationGroup(false, false);
+    buttonTransportEditor = new TransButtonSetting();
+    buttonTransportEditor->setLabel(QObject::tr("Transport Editor"));
+    buttonTransportEditor->setHelpText(
+        QObject::tr("Allows you to edit the transports directly") + " " +
+        QObject::tr("This is rarely required unless you are using "
+                    "a satelite dish and must enter an initial "
+                    "frequency to for the channel scanner to try."));
+
+    HorizontalConfigurationGroup *h = 
+        new HorizontalConfigurationGroup(false, false);
     h->addChild(buttonScan);
-    h->addChild(buttonAdvanced);
+    h->addChild(buttonTransportEditor);
     addChild(h);
-#endif
 
     connect(source, SIGNAL(valueChanged(const QString&)),
             list, SLOT(setSourceID(const QString&)));
@@ -226,68 +276,106 @@ ChannelEditor::ChannelEditor():
             list, SLOT(setSortMode(const QString&)));
     connect(hide, SIGNAL(valueChanged(bool)),
             list, SLOT(setHideMode(bool)));
-    connect(list, SIGNAL(accepted(int)), this, SLOT(edit(int)));
-    connect(list, SIGNAL(menuButtonPressed(int)), this, SLOT(menu(int)));
-#ifdef USING_DVB
-    connect(buttonScan, SIGNAL(pressed()), this, SLOT(scan()));
-    connect(buttonAdvanced, SIGNAL(pressed()), this, SLOT(advanced()));
-#endif
+    connect(list, SIGNAL(accepted(int)),
+            this, SLOT(edit(int)));
+    connect(list, SIGNAL(menuButtonPressed(int)),
+            this, SLOT(menu(int)));
+    connect(buttonScan, SIGNAL(pressed()),
+            this, SLOT(scan()));
+    connect(buttonTransportEditor, SIGNAL(pressed()),
+            this, SLOT(transportEditor()));
+    connect(del,  SIGNAL(pressed()),
+            this, SLOT(deleteChannels()));
 }
 
-int ChannelEditor::exec(QSqlDatabase* _db) {
-    db = _db;
+void ChannelEditor::deleteChannels(void)
+{
+    const QString currentSourceID = source->getValue();
 
-    //Make sure we have dvb card installed
-#ifdef USING_DVB
-    if (!CardUtil::isCardPresent(db, CardUtil::DVB))
+    int val = MythPopupBox::show2ButtonPopup(
+        gContext->GetMainWindow(), "",
+        tr("Are you sure you would like to delete these channels?"),
+        tr("Yes, delete the channels"),
+        tr("No, don't"), 2);
+
+    if (val != 0) 
+        return;
+
+    MSqlQuery query(MSqlQuery::InitCon());
+    if ((currentSourceID == "") || (currentSourceID == "Unassigned"))
     {
-        buttonScan->setEnabled(false);
-        buttonAdvanced->setEnabled(false);
+        query.prepare("TRUNCATE TABLE channel");
     }
-#endif
+    else
+    {
+        query.prepare("DELETE FROM channel "
+                      "WHERE sourceid = :SOURCEID");
+        query.bindValue(":SOURCEID", currentSourceID);
+    }
+    query.exec();
 
-    while (ConfigurationDialog::exec(db) == QDialog::Accepted)  {}
+    list->fillSelections();
+}
+
+MythDialog* ChannelEditor::dialogWidget(MythMainWindow* parent,
+                                        const char* widgetName)
+{
+    dialog = ConfigurationDialog::dialogWidget(parent, widgetName);
+    connect(dialog, SIGNAL(editButtonPressed()),   this, SLOT(edit()));
+    connect(dialog, SIGNAL(deleteButtonPressed()), this, SLOT(del()));
+    return dialog;
+}
+
+int ChannelEditor::exec()
+{
+    while (ConfigurationDialog::exec() == QDialog::Accepted)  {}
     return QDialog::Rejected;
 }
 
-void ChannelEditor::edit() {
+void ChannelEditor::edit()
+{
     id = list->getValue().toInt();
-    ChannelWizard cw(id,db);
-    cw.exec(db);
+    ChannelWizard cw(id);
+    cw.exec();
 
     list->fillSelections();
     list->setFocus();
 }
 
-void ChannelEditor::edit(int /*iSelected*/) {
-    id = list->getValue().toInt();
+void ChannelEditor::edit(int /*iSelected*/)
+{ 
     edit();
 }
 
-void ChannelEditor::del() {
-    int val = MythPopupBox::show2ButtonPopup(gContext->GetMainWindow(), "",
-                                             tr("Are you sure you would like to"
-                                             " delete this channel?"), 
-                                             tr("Yes, delete the channel"), 
-                                             tr("No, don't"), 2);
+void ChannelEditor::del()
+{
+    id = list->getValue().toInt();
 
-    if (val == 0) {
-        QSqlQuery query;
-    
-        query = db->exec(QString("DELETE FROM channel "
-                                 "WHERE chanid ='%1'").arg(id));
-        if (!query.isActive())
+    int val = MythPopupBox::show2ButtonPopup(
+        gContext->GetMainWindow(), "",
+        tr("Are you sure you would like to delete this channel?"),
+        tr("Yes, delete the channel"),
+        tr("No, don't"), 2);
+
+    if (val == 0) 
+    {
+        MSqlQuery query(MSqlQuery::InitCon());
+        query.prepare("DELETE FROM channel WHERE chanid = :CHID ;");
+        query.bindValue(":CHID", id);
+        if (!query.exec() || !query.isActive())
             MythContext::DBError("ChannelEditor Delete Channel", query);
 
         list->fillSelections();
     }
 }
 
-void ChannelEditor::menu(int /*iSelected*/) {
+void ChannelEditor::menu(int /*iSelected*/)
+{
     id = list->getValue().toInt();
-    if (id == 0) {
+    if (id == 0)
        edit();
-    } else {
+    else
+    {
         int val = MythPopupBox::show2ButtonPopup(gContext->GetMainWindow(),
                                                  "",
                                                  tr("Channel Menu"),
@@ -305,20 +393,23 @@ void ChannelEditor::menu(int /*iSelected*/) {
 
 void ChannelEditor::scan()
 {
-#ifdef USING_DVB
-    ScanWizard scanwizard(db);
-    scanwizard.exec(db,false,true);
+#ifdef USING_BACKEND
+    ScanWizard scanwizard;
+    scanwizard.exec(false,true);
 
     list->fillSelections();
     list->setFocus();
+#else
+    VERBOSE(VB_IMPORTANT,  "You must compile the backend "
+            "to be able to scan for channels");
 #endif
 }
 
-void ChannelEditor::advanced()
+void ChannelEditor::transportEditor()
 {
-#ifdef USING_DVB
     DVBTransportsEditor advancedDialog;
-    advancedDialog.exec(db);
-#endif
+    advancedDialog.exec();
+    list->fillSelections();
+    list->setFocus();
 }
 
