@@ -87,6 +87,12 @@ public:
     Rank(const ChannelID& id):
         SpinBoxSetting(-99,99,1), CSetting(id, "rank") {
         setLabel(QObject::tr("Rank"));
+        setHelpText(
+            QObject::tr("Number of bonus points to be added to any "
+                        "recording on this channel during scheduling.")+" "+
+            QObject::tr("Use a positive number as the rank if you "
+                        "want this to be a preferred channel, a "
+                        "negative one to deprecate this channel."));
     };
 };
 
@@ -95,6 +101,8 @@ public:
     Icon(const ChannelID& id):
         LineEditSetting(), CSetting(id, "icon") {
         setLabel(QObject::tr("Icon"));
+        setHelpText(QObject::tr("Image file to use as the icon for this "
+                                "channel on various MythTV displays."));
     };
 };
 
@@ -129,6 +137,11 @@ public:
     XmltvID(const ChannelID& id):
         LineEditSetting(), CSetting(id, "xmltvid") {
         setLabel(QObject::tr("XMLTV ID"));
+        setHelpText(QObject::tr(
+                        "ID used by listing services to get an exact "
+                        "correspondance between a channel in your line-up "
+                        "and a channel in their database. Normally this is "
+                        "set automatically when 'mythfilldatabase' is run."));
     };
 };
 
@@ -168,19 +181,29 @@ public:
         Channel Options - Video 4 Linux
  *****************************************************************************/
 
-class Freqid: public LineEditSetting, public CSetting {
-public:
-    Freqid(const ChannelID& id):
-        LineEditSetting(), CSetting(id, "freqid") {
-        setLabel(QObject::tr("Frequency ID"));
+class Freqid: public LineEditSetting, public CSetting
+{
+  public:
+    Freqid(const ChannelID& id)
+        : LineEditSetting(), CSetting(id, "freqid")
+    {
+        setLabel(QObject::tr("Frequency")+" "+QObject::tr("or")+" "+
+                 QObject::tr("Channel"));
+        setHelpText(QObject::tr(
+                        "Specify either the exact frequency in kHz or "
+                        "a valid channel for your 'TV Format'."));
     };
 };
 
-class Finetune: public SliderSetting, public CSetting {
-public:
-    Finetune(const ChannelID& id):
-        SliderSetting(-200,200,1), CSetting(id, "finetune") {
-        setLabel(QObject::tr("Finetune"));
+class Finetune: public SliderSetting, public CSetting
+{
+  public:
+    Finetune(const ChannelID& id)
+        : SliderSetting(-300,300,1), CSetting(id, "finetune")
+    {
+        setLabel(QObject::tr("Finetune")+" (kHz)");
+        setHelpText(QObject::tr("Value to be added to your desired frequency "
+                                "in kHz, for 'fine tuning'."));
     };
 };
 
@@ -277,41 +300,39 @@ void ChannelOptionsCommon::onAirGuideChanged(bool fValue)
 #endif
 }
 
-void ChannelOptionsCommon::sourceChanged(const QString& str)
+void ChannelOptionsCommon::sourceChanged(const QString& sourceid)
 {
-    (void)str;
-#ifdef USING_DVB
-
-    bool fDVB =false;
+    bool supports_eit  = true;
+    bool uses_eit_only = false;
 
     MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("SELECT SUM(1) "
+                  "FROM capturecard, videosource, cardinput "
+                  "WHERE cardinput.sourceid   = videosource.sourceid AND "
+                  "      cardinput.cardid     = capturecard.cardid   AND "
+                  "      videosource.sourceid = :SOURCEID            AND "
+                  "      cardtype IN ('DVB','HDTV')");
+    query.bindValue(":SOURCEID", sourceid);
 
-    QString queryStr = QString("SELECT count(cardtype) "
-       "FROM capturecard,videosource,cardinput "
-       "WHERE cardinput.sourceid=videosource.sourceid "
-       "AND cardinput.cardid=capturecard.cardid "
-       "AND capturecard.cardtype in (\"DVB\") "
-       "AND videosource.sourceid=\"%1\" "
-       "AND capturecard.hostname=\"%2\"").arg(str).arg(gContext->GetHostName());
-
-    query.prepare(queryStr);
-    if (query.exec() && query.isActive() && query.size() > 0)
-    {
-        query.next();
-        if (query.value(0).toInt())
-           fDVB = true;
-    }
-    if (fDVB)
-    {
-       onairguide->setEnabled(true);
-       xmltvID->setEnabled(!onairguide->boolValue());
-    }
+    if (!query.exec() || !query.isActive())
+        MythContext::DBError("sourceChanged -- supports eit", query);
     else
     {
-       onairguide->setEnabled(false);
-       xmltvID->setEnabled(true);
+        supports_eit = query.next() && query.value(0).toInt();
+
+        query.prepare("SELECT xmltvgrabber "
+                      "FROM videosource "
+                      "WHERE sourceid = :SOURCEID");
+        query.bindValue(":SOURCEID", sourceid);
+
+        if (!query.exec() || !query.isActive())
+            MythContext::DBError("sourceChanged -- eit only", query);
+        else if (query.next())
+            uses_eit_only = (query.value(0).toString() == "eitonly");
     }
-#endif
+
+    onairguide->setEnabled(supports_eit);
+    xmltvID->setEnabled(!uses_eit_only);
 }
 
 ChannelOptionsV4L::ChannelOptionsV4L(const ChannelID& id)
