@@ -4,10 +4,20 @@
 #include "mythdbcon.h"
 #include "util.h"
 
+#include <unistd.h>
 #include <qfile.h>
 #include <qstring.h>
 #include <cerrno>
 
+
+#define LOC QString("DataDirect: ")
+#define LOC_ERR QString("DataDirect, Error: ")
+
+const QString DataDirectURLS[] =
+{
+    "http://datadirect.webservices.zap2it.com/tvlistings/xtvdService"
+};
+
 // XXX Program duration should be stored as seconds, not as a QTime.
 //     limited to 24 hours this way.
 
@@ -169,19 +179,19 @@ bool DDStructureParser::endElement(const QString &pnamespaceuri,
     else if (pqname == "schedule") 
     {
         QDateTime endtime = curr_schedule.time.addSecs(
-                                        QTime().secsTo(curr_schedule.duration));
+            QTime().secsTo(curr_schedule.duration));
 
         query.prepare("INSERT INTO dd_schedule (programid,stationid,"
-                      "scheduletime,duration,repeat,stereo,subtitled,hdtv,"
+                      "scheduletime,duration,isrepeat,stereo,subtitled,hdtv,"
                       "closecaptioned,tvrating,partnumber,parttotal,endtime) "
                       "VALUES(:PROGRAMID,:STATIONID,:TIME,:DURATION,"
-                      ":REPEAT,:STEREO,:SUBTITLED,:HDTV,:CLOSECAPTIONED,"
+                      ":ISREPEAT,:STEREO,:SUBTITLED,:HDTV,:CLOSECAPTIONED,"
                       ":TVRATING,:PARTNUMBER,:PARTTOTAL,:ENDTIME);");
         query.bindValue(":PROGRAMID", curr_schedule.programid);
         query.bindValue(":STATIONID", curr_schedule.stationid);
         query.bindValue(":TIME", curr_schedule.time);
         query.bindValue(":DURATION", curr_schedule.duration);
-        query.bindValue(":REPEAT", curr_schedule.repeat);
+        query.bindValue(":ISREPEAT", curr_schedule.repeat);
         query.bindValue(":STEREO", curr_schedule.stereo);
         query.bindValue(":SUBTITLED", curr_schedule.subtitled);
         query.bindValue(":HDTV", curr_schedule.hdtv);
@@ -305,7 +315,12 @@ bool DDStructureParser::characters(const QString& pchars)
            QDateTime EDFM = QDateTime::fromString(ExtractDateFromMessage, Qt::ISODate);
            QString ExpirationDate = EDFM.toString(Qt::LocalDate);
            QString ExpirationDateMessage = "Your subscription expires on " + ExpirationDate;
-           cout << ExpirationDateMessage << endl;
+
+           QDateTime curTime = QDateTime::currentDateTime();
+           if (curTime.daysTo(EDFM) <= 5)
+               VERBOSE(VB_IMPORTANT, LOC + QString("WARNING: ") + ExpirationDateMessage);
+           else
+               VERBOSE(VB_IMPORTANT, LOC + ExpirationDateMessage);
 
            MSqlQuery query(MSqlQuery::DDCon());
 
@@ -376,21 +391,19 @@ bool DDStructureParser::characters(const QString& pchars)
 void DataDirectProcessor::updateStationViewTable() 
 {
     MSqlQuery query(MSqlQuery::DDCon());
-    QString querystr;
    
     if (!query.exec("TRUNCATE TABLE dd_v_station;")) 
         MythContext::DBError("Truncating temporary table dd_v_station", query);
 
-    querystr = QString("INSERT INTO dd_v_station (stationid, callsign, "
-                       "stationname, affiliate, fccchannelnumber, channel, "
-                       "channelMinor) "
-                       "SELECT dd_station.stationid, callsign, stationname, "
-                       "affiliate, fccchannelnumber, channel, channelMinor "
-                       "FROM dd_station, dd_lineupmap WHERE "
-                       " ( (dd_station.stationid = dd_lineupmap.stationid) AND "
-                       "   ( dd_lineupmap.lineupid = \"%0\") );")
-                      .arg(getLineup());
-    query.prepare(querystr);
+    query.prepare("INSERT INTO dd_v_station (stationid, callsign, "
+                  "    stationname, affiliate, fccchannelnumber, channel, "
+                  "    channelMinor) "
+                  "SELECT dd_station.stationid, callsign, stationname, "
+                  "    affiliate, fccchannelnumber, channel, channelMinor "
+                  "FROM dd_station, dd_lineupmap WHERE "
+                  "    ( (dd_station.stationid = dd_lineupmap.stationid) AND "
+                  "    ( dd_lineupmap.lineupid = :LINEUP) );");
+    query.bindValue(":LINEUP", getLineup());
 
     if (!query.exec())
         MythContext::DBError("Populating temporary table dd_v_station", query);
@@ -399,29 +412,28 @@ void DataDirectProcessor::updateStationViewTable()
 void DataDirectProcessor::updateProgramViewTable(int sourceid) 
 {
     MSqlQuery query(MSqlQuery::DDCon());
-    QString querystr;
    
     if (!query.exec("TRUNCATE TABLE dd_v_program;"))
         MythContext::DBError("Truncating temporary table dd_v_program", query);
 
-    querystr = QString("INSERT INTO dd_v_program (chanid, starttime, endtime, "
-                       "title, subtitle, description, airdate, stars, "
-                       "previouslyshown, stereo, subtitled, hdtv, "
-                       "closecaptioned, partnumber, parttotal, seriesid, "
-                       "originalairdate, showtype, category_type, colorcode, "
-                       "syndicatedepisodenumber, tvrating, mpaarating, "
-                       "programid) "
-                       "SELECT chanid, scheduletime, endtime, title, "
-                       "subtitle, description, year, stars, repeat, stereo, "
-                       "subtitled, hdtv, closecaptioned, partnumber, "
-                       "parttotal, seriesid, originalairdate, showtype, "
-                       "category_type, colorcode, syndicatedepisodenumber, "
-                       "tvrating, mpaarating, dd_program.programid "
-                       "FROM channel, dd_schedule, dd_program WHERE "
-                       " ( (dd_schedule.programid = dd_program.programid) AND "
-                       "   (channel.xmltvid = dd_schedule.stationid) AND "
-                       "   (channel.sourceid = %0 ));").arg(sourceid);
-    query.prepare(querystr);
+    query.prepare("INSERT INTO dd_v_program (chanid, starttime, endtime, "
+                  "title, subtitle, description, airdate, stars, "
+                  "previouslyshown, stereo, subtitled, hdtv, "
+                  "closecaptioned, partnumber, parttotal, seriesid, "
+                  "originalairdate, showtype, category_type, colorcode, "
+                  "syndicatedepisodenumber, tvrating, mpaarating, "
+                  "programid) "
+                  "SELECT chanid, scheduletime, endtime, title, "
+                  "subtitle, description, year, stars, isrepeat, stereo, "
+                  "subtitled, hdtv, closecaptioned, partnumber, "
+                  "parttotal, seriesid, originalairdate, showtype, "
+                  "category_type, colorcode, syndicatedepisodenumber, "
+                  "tvrating, mpaarating, dd_program.programid "
+                  "FROM channel, dd_schedule, dd_program WHERE "
+                  " ( (dd_schedule.programid = dd_program.programid) AND "
+                  "   (channel.xmltvid = dd_schedule.stationid) AND "
+                  "   (channel.sourceid = :SOURCEID ));");
+    query.bindValue(":SOURCEID", sourceid);
 
     if (!query.exec())
         MythContext::DBError("Populating temporary table dd_v_program", query);
@@ -443,9 +455,10 @@ FILE *DataDirectProcessor::getInputFile(bool plineupsOnly, QDateTime pstartDate,
 {
     FILE *ret = NULL;
     if (inputfilename.isNull())
-    {
-        QString ddurl("http://datadirect.webservices.zap2it.com/tvlistings/xtvdService");
-
+    {   
+        //QString ddurl("http://datadirect.webservices.zap2it.com/tvlistings/xtvdService");
+        QString ddurl(DataDirectURLS[source]);
+         
         if (plineupsOnly) 
         {
             pstartDate = QDateTime(QDate::currentDate().addDays(2),
@@ -460,8 +473,8 @@ FILE *DataDirectProcessor::getInputFile(bool plineupsOnly, QDateTime pstartDate,
         char ctempfilename[] = "/tmp/mythpostXXXXXX";
         if (mkstemp(ctempfilename) == -1) 
         {
-            VERBOSE(VB_IMPORTANT,
-                    QString("DDP: error creating temp files -- %1").
+            VERBOSE(VB_IMPORTANT, LOC_ERR +
+                    QString("Creating temp files -- %1").
                     arg(strerror(errno)));
             return NULL;
         }
@@ -498,12 +511,17 @@ FILE *DataDirectProcessor::getInputFile(bool plineupsOnly, QDateTime pstartDate,
 
         QString command = QString("wget --http-user='%1' --http-passwd='%2' "
                                   "--post-file='%3' --header='Accept-Encoding:gzip'"
-                                  " %4 --output-document=- | gzip -df")
+                                  " %4 --output-document=- ")
                                  .arg(getUserID())
                                  .arg(getPassword())
                                  .arg(tmpfilename)
                                  .arg(ddurl);
 
+        if ((print_verbose_messages & VB_GENERAL) == 0)
+            command += " 2> /dev/null ";
+
+        command += " | gzip -df";
+
         err_txt = command;
 
         ret = popen(command.ascii(), "r");
@@ -517,6 +535,142 @@ FILE *DataDirectProcessor::getInputFile(bool plineupsOnly, QDateTime pstartDate,
     return ret;
 }
 
+bool DataDirectProcessor::getNextSuggestedTime(void)
+{
+    QString ddurl(DataDirectURLS[source]);
+         
+    char ctempfilenamesend[] = "/tmp/mythpostXXXXXX";
+    if (mkstemp(ctempfilenamesend) == -1) 
+    {
+        VERBOSE(VB_IMPORTANT, LOC_ERR +
+                QString("Creating temp file for sending -- %1").
+                arg(strerror(errno)));
+        return FALSE;
+    }
+    QString tmpfilenamesend = QString(ctempfilenamesend);
+
+    char ctempfilenamerecv[] = "/tmp/mythpostXXXXXX";
+    if (mkstemp(ctempfilenamerecv) == -1) 
+    {
+        VERBOSE(VB_IMPORTANT, LOC_ERR +
+                QString("Creating temp file for receiving -- %1").
+                arg(strerror(errno)));
+        return FALSE;
+    }
+    QString tmpfilenamereceive = QString(ctempfilenamerecv);
+
+    QFile postfile(tmpfilenamesend);
+    if (postfile.open(IO_WriteOnly)) 
+    {
+        QTextStream poststream(&postfile);
+        poststream << "<?xml version='1.0' encoding='utf-8'?>\n";
+        poststream << "<SOAP-ENV:Envelope\n";
+        poststream << "xmlns:SOAP-ENV='http://schemas.xmlsoap.org/soap/envelope/'\n";
+        poststream << "xmlns:xsd='http://www.w3.org/2001/XMLSchema'\n";
+        poststream << "xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'\n";
+        poststream << "xmlns:SOAP-ENC='http://schemas.xmlsoap.org/soap/encoding/'>\n";
+        poststream << "<SOAP-ENV:Body>\n";
+        poststream << "<tms:acknowledge xmlns:tms='urn:TMSWebServices'>\n";
+        poststream << "</SOAP-ENV:Body>\n";
+        poststream << "</SOAP-ENV:Envelope>\n";
+    }
+    else
+    {
+        VERBOSE(VB_IMPORTANT, LOC_ERR +
+                QString("Creating tmpfilesend -- %1").
+                arg(strerror(errno)));
+        return FALSE;
+    }
+    postfile.close();
+
+    QString command = QString("wget --http-user='%1' --http-passwd='%2' "
+                              "--post-file='%3' "
+                              " %4 --output-document='%5'")
+                              .arg(getUserID())
+                              .arg(getPassword())
+                              .arg(tmpfilenamesend)
+                              .arg(ddurl)
+                              .arg(tmpfilenamereceive);
+
+    myth_system(command.ascii());
+
+    QDateTime NextSuggestedTime;
+
+    QFile file(tmpfilenamereceive);
+
+    bool GotNextSuggestedTime = FALSE;
+
+    if (file.open(IO_ReadOnly)) 
+    {
+        QTextStream stream(&file);
+        QString line;
+        while (!stream.atEnd()) 
+        {
+            line = stream.readLine();
+            if (line.startsWith("<suggestedTime>"))
+            {
+                GotNextSuggestedTime = TRUE;
+                QDateTime UTCdt = QDateTime::fromString(line.mid(15,20),
+                                                        Qt::ISODate);
+                NextSuggestedTime = MythUTCToLocal(UTCdt);
+                VERBOSE(VB_GENERAL, LOC + QString("NextSuggestedTime is: ") 
+                        + NextSuggestedTime.toString(Qt::ISODate));
+            }
+        }
+        file.close();
+    }
+    unlink(tmpfilenamesend.ascii());
+    unlink(tmpfilenamereceive.ascii());
+
+    if (GotNextSuggestedTime)
+    {
+        int daysToSuggested =
+                QDateTime::currentDateTime().daysTo(NextSuggestedTime);
+        int desiredPeriod = gContext->GetNumSetting("MythFillPeriod", 1);
+
+
+        if (daysToSuggested > desiredPeriod)
+        {
+            QDateTime newTime =
+                NextSuggestedTime.addDays(desiredPeriod - daysToSuggested);
+            VERBOSE(VB_IMPORTANT, LOC + QString("Provider suggested running "
+                    "again at %1, but MythFillPeriod is %2.  Next run time "
+                    "will be adjusted to be %3.")
+                    .arg(NextSuggestedTime.toString(Qt::ISODate))
+                    .arg(desiredPeriod)
+                    .arg(newTime.toString(Qt::ISODate)));
+            NextSuggestedTime = newTime;
+        }
+
+        int minhr = NextSuggestedTime.toString("h").toInt();
+        int maxhr = NextSuggestedTime.addSecs(7200).toString("h").toInt();
+
+        if (maxhr < minhr)
+        {
+            minhr = 22;
+            maxhr = 24;
+        }
+
+        MSqlQuery query(MSqlQuery::DDCon());
+        QString querystr =
+            QString("UPDATE settings SET data = '%1' WHERE value = '%2';");
+
+        query.prepare(querystr.arg(minhr).arg("MythFillMinHour"));
+        if (!query.exec())
+            MythContext::DBError("Updating DataDirect MythFillMinHour", query);
+
+        query.prepare(querystr.arg(maxhr).arg("MythFillMaxHour"));
+        if (!query.exec())
+            MythContext::DBError("Updating DataDirect MythFillMaxHour", query);
+
+        query.prepare(querystr.arg(NextSuggestedTime.toString(Qt::ISODate))
+                              .arg("MythFillSuggestedRunTime"));
+        if (!query.exec())
+            MythContext::DBError("Updating DataDirect Suggested RunTime", query);
+    }
+    return GotNextSuggestedTime;
+}
+
 bool DataDirectProcessor::grabData(bool plineupsOnly, QDateTime pstartDate, 
                                    QDateTime pendDate) 
 {
@@ -527,7 +681,8 @@ bool DataDirectProcessor::grabData(bool plineupsOnly, QDateTime pstartDate,
 
     if (fp == NULL) 
     {
-        VERBOSE(VB_IMPORTANT, QString("DDP: Failed to get data (%1) -- %2").
+        VERBOSE(VB_IMPORTANT, LOC_ERR +
+                QString("Failed to get data (%1) -- %2").
                 arg(ferror.ascii()).arg(strerror(errno)));
         return false;
     }
@@ -536,7 +691,7 @@ bool DataDirectProcessor::grabData(bool plineupsOnly, QDateTime pstartDate,
 
     if (!f.open(IO_ReadOnly, fp)) 
     {
-       VERBOSE(VB_GENERAL,"Error opening DataDirect file");
+       VERBOSE(VB_GENERAL, LOC_ERR + "Error opening DataDirect file");
        return false;
     }
     
@@ -614,7 +769,7 @@ void DataDirectProcessor::createTempTables()
     createATempTable ("dd_v_station", table);
 
     table = "( programid char(12), stationid char(12), scheduletime datetime, "
-            "duration time, repeat bool, stereo bool, subtitled bool, "
+            "duration time, isrepeat bool, stereo bool, subtitled bool, "
             "hdtv bool, closecaptioned bool, tvrating char(5), partnumber int, "
             "parttotal int, endtime datetime, INDEX progidx (programid) )";
     createATempTable("dd_schedule", table); 
@@ -632,7 +787,7 @@ void DataDirectProcessor::createTempTables()
             "endtime datetime, title varchar(128), subtitle varchar(128), "
             "description text, category varchar(64), "
             "category_type varchar(64), airdate year, stars float unsigned, "
-            "previouslyshown tinyint, repeat bool, stereo bool, "
+            "previouslyshown tinyint, isrepeat bool, stereo bool, "
             "subtitled bool, hdtv bool,  closecaptioned bool, partnumber int, "
             "parttotal int, seriesid char(12), originalairdate date, "
             "showtype varchar(30), colorcode varchar(20), "
@@ -649,3 +804,5 @@ void DataDirectProcessor::createTempTables()
             "relevance char(1), INDEX progidx (programid))";
     createATempTable("dd_genre", table);
 }
+
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
