@@ -30,6 +30,8 @@
  */
 
 #include <iostream>
+#include <cmath>
+
 #include "pthread.h"
 #include "qsqldatabase.h"
 
@@ -39,6 +41,12 @@
 #include "dvbtypes.h"
 #include "dvbdiseqc.h"
 
+#define TO_RADS (M_PI / 180.0)
+#define TO_DEC (180.0 / M_PI)
+
+#define LOC QString("DiSEqC(%1): ").arg(cardnum)
+#define LOC_ERR QString("DiSEqC(%1) Error: ").arg(cardnum)
+
 DVBDiSEqC::DVBDiSEqC(int _cardnum, int _fd_frontend):
     cardnum(_cardnum), fd_frontend(_fd_frontend)
 {
@@ -51,7 +59,7 @@ DVBDiSEqC::~DVBDiSEqC()
 {
 }
 
-bool DVBDiSEqC::Set(dvb_tuning_t& tuning, bool reset, bool& havetuned)
+bool DVBDiSEqC::Set(DVBTuning& tuning, bool reset, bool& havetuned)
 {
 
     switch(tuning.diseqc_type)
@@ -66,9 +74,12 @@ bool DVBDiSEqC::Set(dvb_tuning_t& tuning, bool reset, bool& havetuned)
             break;
         case 2: // v1.0 2 Way
         case 3: // v1.1 2 Way
+            if (!Diseqc1xSwitch(tuning, reset, havetuned, 2))
+                return false;
+            break;
         case 4: // v1.0 4 Way
         case 5: // v1.1 4 Way
-            if (!Diseqc1xSwitch(tuning, reset, havetuned))
+            if (!Diseqc1xSwitch(tuning, reset, havetuned, 4))
                 return false;
             break;
         case 6: // 1.2 Positioner (HH Motor)
@@ -79,9 +90,14 @@ bool DVBDiSEqC::Set(dvb_tuning_t& tuning, bool reset, bool& havetuned)
             if (!PositionerGotoAngular(tuning,reset,havetuned))
 		return false;
             break;
+        case 8: // v1.1 10 Way
+            if (!Diseqc1xSwitch(tuning, reset, havetuned, 10))
+                return false;
+            break;
  
         default:
-            ERRNO("Unsupported DiSEqC type.");
+            VERBOSE(VB_IMPORTANT, LOC_ERR + "Unsupported DiSEqC type("
+                    <<tuning.diseqc_type<<")");
     }
     
     return true;
@@ -91,9 +107,9 @@ bool DVBDiSEqC::Set(dvb_tuning_t& tuning, bool reset, bool& havetuned)
                         Backward Compatible Methods
  ****************************************************************************/
 
-bool DVBDiSEqC::ToneVoltageLnb(dvb_tuning_t& tuning, bool reset, bool& havetuned)
+bool DVBDiSEqC::ToneVoltageLnb(DVBTuning& tuning, bool reset, bool& havetuned)
 {
-    CHANNEL(QString("Setting LNB: %1 %2")
+    VERBOSE(VB_CHANNEL, LOC + QString("Setting LNB: %1 %2")
             .arg(tuning.tone==SEC_TONE_ON?"Tone ON":"Tone OFF")
             .arg(tuning.voltage==SEC_VOLTAGE_13?"13V":"18V"));
 
@@ -101,12 +117,12 @@ bool DVBDiSEqC::ToneVoltageLnb(dvb_tuning_t& tuning, bool reset, bool& havetuned
     {
         if (ioctl(fd_frontend, FE_SET_TONE, tuning.tone) < 0)
         {
-            ERRNO("Setting Tone mode failed.");
+            VERBOSE(VB_IMPORTANT, LOC_ERR +
+                    "Setting Tone mode failed." + ENO);
             return false;
         }
 
         prev_tuning.tone = tuning.tone;
-        havetuned = true;
     }
 
     usleep(DISEQC_SHORT_WAIT);
@@ -115,37 +131,44 @@ bool DVBDiSEqC::ToneVoltageLnb(dvb_tuning_t& tuning, bool reset, bool& havetuned
     {
         if (ioctl(fd_frontend, FE_SET_VOLTAGE, tuning.voltage) < 0)
         {
-            ERRNO("Setting Polarization failed.");
+            VERBOSE(VB_IMPORTANT, LOC_ERR +
+                    "Setting Polarization failed." + ENO);
             return false;
         }
 
         prev_tuning.voltage = tuning.voltage;
-        havetuned = true;
     }
 
+    havetuned |= ((prev_tuning.voltage == tuning.voltage) &&
+                  (prev_tuning.tone == tuning.tone));
+
     return true;
 }
 
-bool DVBDiSEqC::ToneSwitch(dvb_tuning_t& tuning, bool reset, bool& havetuned)
+bool DVBDiSEqC::ToneSwitch(DVBTuning& tuning, bool reset, bool& havetuned)
 {
-    CHANNEL(QString("DiSEqC Tone Switch - Port %1/2").arg(tuning.diseqc_port));
+    VERBOSE(VB_CHANNEL, LOC + QString("Tone Switch - Port %1/2")
+            .arg(tuning.diseqc_port));
 
     if (prev_tuning.diseqc_port != tuning.diseqc_port || reset)
     {
         if (tuning.diseqc_port > 2)
-            ERRNO("Tone Switches only supports two ports.");
+            VERBOSE(VB_IMPORTANT, LOC_ERR +
+                    "Tone Switches only support two ports.");
 
         if (ioctl(fd_frontend, FE_DISEQC_SEND_BURST,
-            (tuning.diseqc_port == 1 ? SEC_MINI_A : SEC_MINI_B )) < 0)
+                  (tuning.diseqc_port == 1 ? SEC_MINI_A : SEC_MINI_B )) < 0)
         {
-            ERRNO("Setting Tone Switch failed.");
+            VERBOSE(VB_IMPORTANT, LOC_ERR +
+                    "Setting Tone Switch failed." + ENO);
             return false;
         }
 
         prev_tuning.diseqc_port = tuning.diseqc_port;
-        havetuned = true;
     }
 
+    havetuned |= (prev_tuning.diseqc_port == tuning.diseqc_port);
+
     return true;
 }
 
@@ -153,38 +176,40 @@ bool DVBDiSEqC::ToneSwitch(dvb_tuning_t& tuning, bool reset, bool& havetuned)
                     Diseqc 1.x Compatible Methods
  ****************************************************************************/
 
-bool DVBDiSEqC::SendDiSEqCMessage(dvb_tuning_t& tuning, dvb_diseqc_master_cmd &cmd)
+bool DVBDiSEqC::SendDiSEqCMessage(DVBTuning& tuning, dvb_diseqc_master_cmd &cmd)
 {
     // Turn off tone burst
     if (ioctl(fd_frontend, FE_SET_TONE, SEC_TONE_OFF) == -1) 
     {
-        ERRNO("FE_SET_TONE failed");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "FE_SET_TONE failed" + ENO);
         return false;
     }
 
 /* 
-   Old version of the code set the voltage to 13V everytime.  After looking at the EutelSat
-   specs I saw no reason that this was done. I have tested this with my DiSEqC switch and all
-   is fine. 
+   Old version of the code set the voltage to 13V everytime.
+   After looking at the EutelSat specs I saw no reason that
+   this was done. I have tested this with my DiSEqC switch
+   and all is fine. 
 */
 
     if (ioctl(fd_frontend, FE_SET_VOLTAGE, tuning.voltage) == -1) 
     {
-        ERRNO("FE_SET_VOLTAGE failed");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "FE_SET_VOLTAGE failed" + ENO);
         return false;
     }   
 
     usleep(DISEQC_SHORT_WAIT);
 
-    GENERAL(QString("DiSEqC Sending 1.0 Command: %1 %2 %3 %4")
-                   .arg(cmd.msg[0], 2, 16)
-                   .arg(cmd.msg[1], 2, 16)
-                   .arg(cmd.msg[2], 2, 16)
-                   .arg(cmd.msg[3], 2, 16));
+    VERBOSE(VB_CHANNEL, LOC + QString("Sending 1.0 Command: %1 %2 %3 %4")
+            .arg(cmd.msg[0], 2, 16)
+            .arg(cmd.msg[1], 2, 16)
+            .arg(cmd.msg[2], 2, 16)
+            .arg(cmd.msg[3], 2, 16));
 
     if (ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd) == -1) 
     {
-        ERRNO("FE_DISEQC_SEND_MASTER_CMD failed");
+        VERBOSE(VB_IMPORTANT, LOC_ERR +
+                "FE_DISEQC_SEND_MASTER_CMD failed" + ENO);
         return false;
     }
 
@@ -199,16 +224,31 @@ bool DVBDiSEqC::SendDiSEqCMessage(dvb_tuning_t& tuning, dvb_diseqc_master_cmd &c
         while (repeats--) 
         {
 
-            GENERAL(QString("DiSEqC Sending 1.1/1.2/1.3 Repeat Command: %1 %2 %3 %4")
-                           .arg(cmd.msg[0],2,16)
-                           .arg(cmd.msg[1],2,16)
-                           .arg(cmd.msg[2],2,16)
-                           .arg(cmd.msg[3],2,16));
+            if (tuning.diseqc_type == 7)
+            {
+                VERBOSE(VB_CHANNEL, LOC +
+                        QString("Sending 1.3 Repeat Command: %1 %2 %3 %4 %5")
+                        .arg(cmd.msg[0],2,16)
+                        .arg(cmd.msg[1],2,16)
+                        .arg(cmd.msg[2],2,16)
+                        .arg(cmd.msg[3],2,16)
+                        .arg(cmd.msg[4],2,16));
+            }
+            else
+            {
+                VERBOSE(VB_CHANNEL, LOC +
+                        QString("Sending 1.1/1.2 Repeat Command: %1 %2 %3 %4")
+                        .arg(cmd.msg[0],2,16)
+                        .arg(cmd.msg[1],2,16)
+                        .arg(cmd.msg[2],2,16)
+                        .arg(cmd.msg[3],2,16));
+            }
 
             cmd.msg[0] = CMD_REPEAT;      
             if (ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd) == -1) 
             {
-                ERRNO("FE_DISEQC_SEND_MASTER_CMD failed");
+                VERBOSE(VB_IMPORTANT, LOC_ERR +
+                        "FE_DISEQC_SEND_MASTER_CMD failed" + ENO);
                 return false;
             }
             usleep(DISEQC_SHORT_WAIT);
@@ -216,7 +256,8 @@ bool DVBDiSEqC::SendDiSEqCMessage(dvb_tuning_t& tuning, dvb_diseqc_master_cmd &c
             cmd.msg[0] = CMD_FIRST;      
             if (ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd) == -1) 
             {
-                ERRNO("FE_DISEQC_SEND_MASTER_CMD failed");
+                VERBOSE(VB_IMPORTANT, LOC_ERR +
+                        "FE_DISEQC_SEND_MASTER_CMD failed" + ENO);
                 return false;
             }
             usleep(DISEQC_SHORT_WAIT);
@@ -225,7 +266,8 @@ bool DVBDiSEqC::SendDiSEqCMessage(dvb_tuning_t& tuning, dvb_diseqc_master_cmd &c
 
     if (ioctl(fd_frontend, FE_DISEQC_SEND_BURST, SEC_MINI_A ) == -1) 
     {
-        ERRNO("FE_DISEQC_SEND_BURST failed");
+        VERBOSE(VB_IMPORTANT, LOC_ERR +
+                "FE_DISEQC_SEND_BURST failed" + ENO);
         return false;
     }
 
@@ -233,7 +275,7 @@ bool DVBDiSEqC::SendDiSEqCMessage(dvb_tuning_t& tuning, dvb_diseqc_master_cmd &c
 
     if (ioctl(fd_frontend, FE_SET_TONE, tuning.tone) == -1) 
     {
-        ERRNO("FE_SET_TONE failed");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "FE_SET_TONE failed" + ENO);
         return false;
     }
 
@@ -246,21 +288,22 @@ bool DVBDiSEqC::SendDiSEqCMessage(dvb_diseqc_master_cmd &cmd)
     // Turn off tone burst
     if (ioctl(fd_frontend, FE_SET_TONE, SEC_TONE_OFF) == -1) 
     {
-        ERRNO("FE_SET_TONE failed");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "FE_SET_TONE failed" + ENO);
         return false;
     }
 
     usleep(DISEQC_SHORT_WAIT);
 
-    GENERAL(QString("DiSEqC Sending 1.0 Command: %1 %2 %3 %4")
-                   .arg(cmd.msg[0], 2, 16)
-                   .arg(cmd.msg[1], 2, 16)
-                   .arg(cmd.msg[2], 2, 16)
-                   .arg(cmd.msg[3], 2, 16));
+    VERBOSE(VB_CHANNEL, LOC + QString("Sending 1.0 Command: %1 %2 %3 %4")
+            .arg(cmd.msg[0], 2, 16)
+            .arg(cmd.msg[1], 2, 16)
+            .arg(cmd.msg[2], 2, 16)
+            .arg(cmd.msg[3], 2, 16));
 
     if (ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd) == -1) 
     {
-        ERRNO("FE_DISEQC_SEND_MASTER_CMD failed");
+        VERBOSE(VB_IMPORTANT, LOC_ERR +
+                "FE_DISEQC_SEND_MASTER_CMD failed" + ENO);
         return false;
     }
 
@@ -269,16 +312,18 @@ bool DVBDiSEqC::SendDiSEqCMessage(dvb_diseqc_master_cmd &cmd)
     int repeats = repeat;
     while (repeats--) 
     {
-        GENERAL(QString("DiSEqC Sending 1.1/1.2/1.3 Repeat Command: %1 %2 %3 %4")
-                       .arg(cmd.msg[0], 2, 16)
-                       .arg(cmd.msg[1], 2, 16)
-                       .arg(cmd.msg[2], 2, 16)
-                       .arg(cmd.msg[3], 2, 16));
+        VERBOSE(VB_CHANNEL, LOC +
+                QString("Sending 1.1/1.2/1.3 Repeat Command: %1 %2 %3 %4")
+                .arg(cmd.msg[0], 2, 16)
+                .arg(cmd.msg[1], 2, 16)
+                .arg(cmd.msg[2], 2, 16)
+                .arg(cmd.msg[3], 2, 16));
 
         cmd.msg[0] = CMD_REPEAT;      
         if (ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd) == -1) 
         {
-            ERRNO("FE_DISEQC_SEND_MASTER_CMD failed");
+            VERBOSE(VB_IMPORTANT, LOC_ERR +
+                    "FE_DISEQC_SEND_MASTER_CMD failed" + ENO);
             return false;
         }
         usleep(DISEQC_SHORT_WAIT);
@@ -286,7 +331,8 @@ bool DVBDiSEqC::SendDiSEqCMessage(dvb_diseqc_master_cmd &cmd)
         cmd.msg[0] = CMD_FIRST;      
         if (ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd) == -1) 
         {
-            ERRNO("FE_DISEQC_SEND_MASTER_CMD failed");
+            VERBOSE(VB_IMPORTANT, LOC_ERR +
+                    "FE_DISEQC_SEND_MASTER_CMD failed" + ENO);
             return false;
         }
         usleep(DISEQC_SHORT_WAIT); 
@@ -294,48 +340,68 @@ bool DVBDiSEqC::SendDiSEqCMessage(dvb_diseqc_master_cmd &cmd)
 
     if (ioctl(fd_frontend, FE_DISEQC_SEND_BURST, SEC_MINI_A ) == -1) 
     {
-        ERRNO("FE_DISEQC_SEND_BURST failed");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "FE_DISEQC_SEND_BURST failed" + ENO);
         return false;
     }
 
     return true;
 }
 
-bool DVBDiSEqC::Diseqc1xSwitch(dvb_tuning_t& tuning, bool reset, 
-                               bool& havetuned)
+bool DVBDiSEqC::Diseqc1xSwitch(DVBTuning& tuning, bool reset, 
+                               bool& havetuned, uint ports)
 {
     if (reset) 
     {
       	if (!DiseqcReset()) 
         {
-      	    ERRNO("DiseqcReset() failed");
+      	    VERBOSE(VB_IMPORTANT, LOC_ERR + "DiseqcReset() failed");
       	    return false;
       	}
     }
 
-    GENERAL(QString("DiSEqC 1.0 Switch - Port %1").arg(tuning.diseqc_port));
+    VERBOSE(VB_CHANNEL, LOC +
+            QString("1.1 Switch (%1 ports) - Port %2 - %3 %4")
+            .arg(ports)
+            .arg(tuning.diseqc_port)
+            .arg(tuning.tone==SEC_TONE_ON?"Tone ON":"Tone OFF")
+            .arg(tuning.voltage==SEC_VOLTAGE_13?"13V":"18V"));
 
-    if ((prev_tuning.diseqc_port != tuning.diseqc_port ||
-        prev_tuning.tone != tuning.tone ||
-        prev_tuning.voltage != tuning.voltage) || reset)
+    if ((prev_tuning.diseqc_port != tuning.diseqc_port  ||
+         prev_tuning.tone != tuning.tone                ||
+         prev_tuning.voltage != tuning.voltage        ) || reset)
     {
-        if (tuning.diseqc_port > 3)
+        dvb_diseqc_master_cmd cmd =
+            {{CMD_FIRST, MASTER_TO_LSS, WRITE_N1, 0xf0, 0x00, 0x00}, 4};
+
+        if (tuning.diseqc_port >= ports )
         {
-            ERRNO("Supports only up to 4-way switches.");
+            VERBOSE(VB_IMPORTANT, LOC_ERR + "Unsupported switch");
             return false;
         }
 
-        dvb_diseqc_master_cmd cmd = 
-            {{CMD_FIRST, MASTER_TO_LSS, WRITE_N0, 0xf0, 0x00, 0x00}, 4};
-
-        cmd.msg[DATA_1] = 0xF0 
-                          | (((tuning.diseqc_port) * 4) & 0x0F) 
-                          | ((tuning.voltage == SEC_VOLTAGE_18) ? 2 : 0) 
-                          | ((tuning.tone == SEC_TONE_ON) ? 1 : 0);
+        switch (ports)
+        {
+            case 10:
+                cmd.msg[COMMAND] = WRITE_N1;
+                cmd.msg[DATA_1] = 0xF0 | (tuning.diseqc_port & 0x0F);
+                break;
+            case 4:
+            case 2: 
+                cmd.msg[COMMAND] = WRITE_N0;
+                cmd.msg[DATA_1] =
+                    0xF0 |
+                    (((tuning.diseqc_port) * 4) & 0x0F)          |
+                    ((tuning.voltage == SEC_VOLTAGE_18) ? 2 : 0) |
+                    ((tuning.tone == SEC_TONE_ON) ? 1 : 0);
+                break;
+            default:
+                VERBOSE(VB_IMPORTANT, LOC_ERR +
+                        "Unsupported number of ports for DiSEqC 1.1 Switch");
+        }
 
         if (!SendDiSEqCMessage(tuning,cmd)) 
         {
-            ERRNO("Setting DiSEqC failed.\n");
+            VERBOSE(VB_IMPORTANT, LOC_ERR + "Setting DiSEqC failed.");
             return false;
         }
 
@@ -346,34 +412,42 @@ bool DVBDiSEqC::Diseqc1xSwitch(dvb_tuning_t& tuning, bool reset,
 
     }
 
+    havetuned |=
+        (prev_tuning.diseqc_port == tuning.diseqc_port) &&
+        (prev_tuning.voltage     == tuning.voltage)     &&
+        (prev_tuning.tone        == tuning.tone);
+
     return true;
 }
 
 bool DVBDiSEqC::DiseqcReset()
 {
     struct dvb_diseqc_master_cmd reset_cmd =
-       {{CMD_FIRST, MASTER_TO_LSS, RESET, 0x00, 0x00}, 3};
+        {{CMD_FIRST, MASTER_TO_LSS, RESET, 0x00, 0x00}, 3};
  
     struct dvb_diseqc_master_cmd init_cmd =
-       {{CMD_FIRST, MASTER_TO_LSS, POWERON, 0x00, 0x00}, 3};
+        {{CMD_FIRST, MASTER_TO_LSS, POWERON, 0x00, 0x00}, 3};
 
     if (ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &init_cmd) <0)
     {
-        ERRNO("Setup: Sending init command failed.");
+        VERBOSE(VB_IMPORTANT, LOC_ERR +
+                "Setup: Sending init command failed." + ENO);
         return false;
     }
     usleep(DISEQC_LONG_WAIT);
 
     if (ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &reset_cmd) <0)
     {
-        ERRNO("Setup: Sending reset command failed.");
+        VERBOSE(VB_IMPORTANT, LOC_ERR +
+                "Setup: Sending reset command failed." + ENO);
         return false;
     }
     usleep(DISEQC_LONG_WAIT);
 
     if (ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &init_cmd) <0)
     {
-        ERRNO("Setup: Sending init command failed.");
+        VERBOSE(VB_IMPORTANT, LOC_ERR +
+                "Setup: Sending init command failed." + ENO);
         return false;
     }
     usleep(DISEQC_LONG_WAIT);
@@ -389,7 +463,7 @@ bool DVBDiSEqC::PositionerDriveEast(int timestep)
 {
     if (!DiseqcReset()) 
     {
-      	ERRNO("DiseqcReset() failed");
+      	VERBOSE(VB_IMPORTANT, LOC_ERR + "DiseqcReset() failed");
     	return false;
     }
 
@@ -398,7 +472,7 @@ bool DVBDiSEqC::PositionerDriveEast(int timestep)
 
     if (!SendDiSEqCMessage(cmd)) 
     {
-        ERRNO("Setting DiSEqC failed.\n");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Setting DiSEqC failed.");
         return false;
     }
     
@@ -409,7 +483,7 @@ bool DVBDiSEqC::PositionerDriveWest(int timestep)
 {
     if (!DiseqcReset()) 
     {
-      	ERRNO("DiseqcReset() failed");
+      	VERBOSE(VB_IMPORTANT, LOC_ERR + "DiseqcReset() failed");
     	return false;
     }
 
@@ -418,18 +492,18 @@ bool DVBDiSEqC::PositionerDriveWest(int timestep)
 
     if (!SendDiSEqCMessage(cmd)) 
     {
-        ERRNO("Setting DiSEqC failed.\n");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Setting DiSEqC failed.");
         return false;
     }
 
     return true;
 }
 
-bool DVBDiSEqC::PositionerGoto(dvb_tuning_t& tuning, bool reset, bool& havetuned)
+bool DVBDiSEqC::PositionerGoto(DVBTuning& tuning, bool reset, bool& havetuned)
 {
     // A reset seems to be required for my positioner to work consistently
-    GENERAL(QString("DiSEqC 1.2 Motor - Goto Stored Position %1")
-                    .arg(tuning.diseqc_port));
+    VERBOSE(VB_CHANNEL, LOC + QString("1.2 Motor - Goto Stored Position %1")
+            .arg(tuning.diseqc_port));
 
     if ((prev_tuning.diseqc_port != tuning.diseqc_port ||
          prev_tuning.tone != tuning.tone ||
@@ -437,7 +511,7 @@ bool DVBDiSEqC::PositionerGoto(dvb_tuning_t& tuning, bool reset, bool& havetuned
     {
         if (!DiseqcReset()) 
         {
-      	    ERRNO("DiseqcReset() failed");
+      	    VERBOSE(VB_IMPORTANT, LOC_ERR + "DiseqcReset() failed");
     	    return false;
         }
         
@@ -447,29 +521,33 @@ bool DVBDiSEqC::PositionerGoto(dvb_tuning_t& tuning, bool reset, bool& havetuned
 
         if (!SendDiSEqCMessage(tuning,cmd)) 
         {
-            ERRNO("Setting DiSEqC failed.\n");
+            VERBOSE(VB_IMPORTANT, LOC_ERR + "Setting DiSEqC failed.");
             return false;
         }
 
         prev_tuning.diseqc_port = tuning.diseqc_port;
         prev_tuning.tone = tuning.tone;
         prev_tuning.voltage = tuning.voltage;
-        havetuned = true;
     }
 
+    havetuned |=
+        (prev_tuning.diseqc_port == tuning.diseqc_port) &&
+        (prev_tuning.voltage     == tuning.voltage)     &&
+        (prev_tuning.tone        == tuning.tone);
+
     return true;
 }
 
-bool DVBDiSEqC::PositionerStore(dvb_tuning_t& tuning)
+bool DVBDiSEqC::PositionerStore(DVBTuning& tuning)
 {
     if (!DiseqcReset()) 
     {
-      	ERRNO("DiseqcReset() failed");
+      	VERBOSE(VB_IMPORTANT, LOC_ERR + "DiseqcReset() failed");
     	return false;
     }
 
-    GENERAL(QString("DiSEqC 1.2 Motor - Store Stored Position %1")
-        .arg(tuning.diseqc_port));
+    VERBOSE(VB_CHANNEL, LOC + QString("1.2 Motor - Store Stored Position %1")
+            .arg(tuning.diseqc_port));
 
     dvb_diseqc_master_cmd cmd = 
         {{CMD_FIRST, MASTER_TO_POSITIONER, STORE, tuning.diseqc_port , 0x00, 
@@ -477,7 +555,7 @@ bool DVBDiSEqC::PositionerStore(dvb_tuning_t& tuning)
 
     if (!SendDiSEqCMessage(cmd)) 
     {
-        ERRNO("Setting DiSEqC failed.\n");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Setting DiSEqC failed.");
         return false;
     }
 
@@ -488,7 +566,7 @@ bool DVBDiSEqC::PositionerStoreEastLimit()
 {
     if (!DiseqcReset()) 
     {
-      	ERRNO("DiseqcReset() failed");
+      	VERBOSE(VB_IMPORTANT, LOC_ERR + "DiseqcReset() failed");
     	return false;
     }
 
@@ -497,7 +575,7 @@ bool DVBDiSEqC::PositionerStoreEastLimit()
 
     if (!SendDiSEqCMessage(cmd)) 
     {
-        ERRNO("Setting DiSEqC failed.\n");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Setting DiSEqC failed.");
         return false;
     }
 
@@ -508,7 +586,7 @@ bool DVBDiSEqC::PositionerStoreWestLimit()
 {
     if (!DiseqcReset()) 
     {
-      	ERRNO("DiseqcReset() failed");
+      	VERBOSE(VB_IMPORTANT, LOC_ERR + "DiseqcReset() failed");
     	return false;
     }
 
@@ -517,7 +595,7 @@ bool DVBDiSEqC::PositionerStoreWestLimit()
 
     if (!SendDiSEqCMessage(cmd)) 
     {
-        ERRNO("Setting DiSEqC failed.\n");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Setting DiSEqC failed.");
         return false;
     }
 
@@ -528,7 +606,7 @@ bool DVBDiSEqC::PositionerStopMovement()
 {
     if (!DiseqcReset()) 
     {
-      	ERRNO("DiseqcReset() failed");
+      	VERBOSE(VB_IMPORTANT, LOC_ERR + "DiseqcReset() failed");
     	return false;
     }
 
@@ -537,7 +615,7 @@ bool DVBDiSEqC::PositionerStopMovement()
 
     if (!SendDiSEqCMessage(cmd)) 
     {
-        ERRNO("Setting DiSEqC failed.\n");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Setting DiSEqC failed.");
         return false;
     }
     return true;
@@ -556,7 +634,7 @@ bool DVBDiSEqC::PositionerDisableLimits()
 {
     if (!DiseqcReset()) 
     {
-      	ERRNO("DiseqcReset() failed");
+      	VERBOSE(VB_IMPORTANT, LOC_ERR + "DiseqcReset() failed");
     	return false;
     }
 
@@ -565,7 +643,7 @@ bool DVBDiSEqC::PositionerDisableLimits()
 
     if (!SendDiSEqCMessage(cmd)) 
     {
-        ERRNO("Setting DiSEqC failed.\n");
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "Setting DiSEqC failed.");
         return false;
     }
 
@@ -573,49 +651,43 @@ bool DVBDiSEqC::PositionerDisableLimits()
 }
 
 /*****************************************************************************
-                                Diseqc v1.3 (AKA USALS)
+                                Diseqc v1.3 (Goto X)
  ****************************************************************************/
 
-bool DVBDiSEqC::PositionerGotoAngular(dvb_tuning_t& tuning, bool reset, 
+bool DVBDiSEqC::PositionerGotoAngular(DVBTuning& tuning, bool reset, 
                                       bool& havetuned) 
 {
-    // Some of this code taken from satpos.c from linuxtv.org by J.Mourer
-   
-    // I have not verified this because my dish is not easily accesable and is 
-    // slightly out of alignment so I use DiSEqC 1.2 Stored Positions
+    // TODO: Send information here to FE saying motor is moving and
+    //       to expect a longer than average tuning delay
+    if (prev_tuning.diseqc_pos != tuning.diseqc_pos)
+        VERBOSE(VB_CHANNEL, LOC + "DiSEqC Motor Moving");
 
-    double H = 180.0 / M_PI;		// Cheat on Radians
-    double B, GSLon, GSLat, SatLon, Azimuth;
-    int CMD1=0x00,CMD2=0x00;		// Bytes sent to motor
+    int CMD1=0x00 , CMD2=0x00;        // Bytes sent to motor
     double USALS=0.0;
-    int DecimalLookup[11] = 
-       { 0x00, 0x02, 0x03, 0x05, 0x06, 0x08, 0x0A, 0x0B, 0x0D, 0x0E, 0x00 };
-  
-    // Hard Coded Test Values 
-    // These vaules should be stored in the settings table when implimentation 
-    // is complete
-    GSLat = 35.0;
-    GSLon = -78.0;
-    SatLon = -95.0;
+    int DecimalLookup[10] =
+        { 0x00, 0x02, 0x03, 0x05, 0x06, 0x08, 0x0A, 0x0B, 0x0D, 0x0E };
 
-    // Calcualte Azimuth required to aim satellite antenna
-    B = GSLon - SatLon;
-    Azimuth = 180.0 + (H * atan2( tan(B / H), sin(GSLat / H) ) );
-    Azimuth += .5;
+    // Equation lifted from VDR rotor plugin by
+    // Thomas Bergwinkl <Thomas.Bergwinkl@t-online.de>
 
-    if (Azimuth < 180) 
-    {
-        // Bird is east of ground station
-   	USALS=180.0-Azimuth;
-        CMD1=0xE0;
-    } 
+    double P = gContext->GetSetting("Latitude", "").toFloat() * TO_RADS;           // Earth Station Latitude
+    double Ue = gContext->GetSetting("Longitude", "").toFloat() * TO_RADS;           // Earth Station Longitude
+    double Us = tuning.diseqc_pos * TO_RADS;          // Satellite Longitude
+
+    double az = M_PI + atan( tan (Us-Ue) / sin(P) );
+    double x = acos( cos(Us-Ue) * cos(P) );
+    double el = atan( (cos(x) - 0.1513 ) /sin(x) );
+    double Azimuth = atan((-cos(el)*sin(az))/(sin(el)*cos(P)-cos(el)*sin(P)*cos(az)))* TO_DEC;
+
+//    printf("Offset = %f\n",Azimuth);
+
+    if (Azimuth > 0.0)
+        CMD1=0xE0;    // East
     else 
-    {
-        // Bird is west of ground station
-        USALS=Azimuth-180.0;
-        CMD1=0xD0;
-    }
-   
+        CMD1=0xD0;      // West
+
+    USALS = fabs(Azimuth);
+ 
     while (USALS > 16) 
     {
         CMD1++;
@@ -632,7 +704,7 @@ bool DVBDiSEqC::PositionerGotoAngular(dvb_tuning_t& tuning, bool reset,
   
     if (!DiseqcReset()) 
     {
-      	ERRNO("DiseqcReset() failed");
+      	VERBOSE(VB_IMPORTANT, LOC_ERR + "DiseqcReset() failed");
     	return false;
     }
 
@@ -640,11 +712,13 @@ bool DVBDiSEqC::PositionerGotoAngular(dvb_tuning_t& tuning, bool reset,
     // and added to tuning
     // sat_pos be passed into tuning, and be a float not an int./
 
-    GENERAL(QString("DiSEqC 1.3 Motor - Goto Angular Position %1").arg(SatLon));
+    VERBOSE(VB_CHANNEL, LOC + QString("1.3 Motor - Goto Angular Position %1")
+            .arg(tuning.diseqc_pos));
 
-    if ((prev_tuning.diseqc_port != tuning.diseqc_port ||
-          prev_tuning.tone != tuning.tone ||
-          prev_tuning.voltage != tuning.voltage) || reset)
+    if ((prev_tuning.diseqc_port != tuning.diseqc_port  ||
+         prev_tuning.tone        != tuning.tone         ||
+         prev_tuning.diseqc_pos  != tuning.diseqc_pos   ||
+         prev_tuning.voltage     != tuning.voltage    ) || reset)
     {
 
         dvb_diseqc_master_cmd cmd = 
@@ -653,15 +727,21 @@ bool DVBDiSEqC::PositionerGotoAngular(dvb_tuning_t& tuning, bool reset,
 
         if (!SendDiSEqCMessage(tuning,cmd)) 
         {
-            ERRNO("Setting DiSEqC failed.\n");
+            VERBOSE(VB_IMPORTANT, LOC_ERR + "Setting DiSEqC failed.");
             return false;
         }
 
         prev_tuning.diseqc_port = tuning.diseqc_port;
+        prev_tuning.diseqc_pos = tuning.diseqc_pos;
         prev_tuning.tone = tuning.tone;
         prev_tuning.voltage = tuning.voltage;
-        havetuned = true;
     }
 
+    havetuned |=
+        (prev_tuning.diseqc_port == tuning.diseqc_port) &&
+        (prev_tuning.diseqc_pos  == tuning.diseqc_pos)  &&
+        (prev_tuning.voltage     == tuning.voltage)     &&
+        (prev_tuning.tone        == tuning.tone);
+
     return true;
 }
