@@ -4,65 +4,55 @@
  *  Distributed as part of MythTV under GPL v2 and later.
  */
 
+// C includes
+#include <pthread.h>
+#include <sys/select.h>
+
+// C++ includes
 #include <iostream>
 using namespace std;
 
-#include <pthread.h>
-#include "RingBuffer.h"
-#include "mythcontext.h"
+// MythTV includes
 #include "firewirerecorder.h"
-#include "mpeg/tspacket.h"
-#include <sys/select.h>
+#include "mythcontext.h"
+#include "tspacket.h"
+#include "tv_rec.h"
 
 // callback function for libiec61883
-int read_tspacket (unsigned char *tspacket, int len, unsigned int dropped, void *callback_data) {
+int read_tspacket (unsigned char *tspacket, int /*len*/,
+                   uint dropped, void *callback_data)
+{
+    FirewireRecorder *fw = (FirewireRecorder*) callback_data;
 
-     FirewireRecorder *fw = (FirewireRecorder*)callback_data;
-     if(!fw) return 0;
+    if (!fw)
+        return 0;
 
-     if(dropped) {
-         VERBOSE(VB_GENERAL,QString("Firewire: %1 packet(s) dropped.").arg(dropped));
-     }
-     fw->ProcessTSPacket(tspacket,len);
-     return 1;
+    if (dropped)
+    {
+        VERBOSE(VB_RECORD,
+                QString("Firewire: %1 packet(s) dropped.").arg(dropped));
+    }
+
+    if (SYNC_BYTE != tspacket[0])
+    {
+        VERBOSE(VB_IMPORTANT, "Firewire: Got out of sync TS Packet");
+        return 1;
+    }
+
+    fw->ProcessTSPacket(*(reinterpret_cast<TSPacket*>(tspacket)));
+
+    return 1;
 }
 
-
-FirewireRecorder::FirewireRecorder() {
-
-    isopen = false;
-    fwchannel = -1;
-    fwbandwidth = -1;
-    fwport = -1;
-    fwnode = 0;
-    fwspeed = -1;
-    fwhandle = NULL;
-    fwmpeg = NULL;
-    fwfd = -1;
-    fwoplug = -1;
-    fwiplug = -1;
-    fwconnection = FIREWIRE_CONNECTION_P2P;
-}
-
-FirewireRecorder::~FirewireRecorder() {
-
-    if(isopen) {
-        VERBOSE(VB_GENERAL,QString("Firewire: releasing iec61883_mpeg2 object"));
-        iec61883_mpeg2_close(fwmpeg);
-        if(fwconnection == FIREWIRE_CONNECTION_P2P && fwchannel > -1) {
-              VERBOSE(VB_GENERAL,QString("Firewire: disconnecting channel %1").arg(fwchannel));
-	      iec61883_cmp_disconnect (fwhandle, fwnode | 0xffc0, fwoplug,
-                   raw1394_get_local_id (fwhandle), fwiplug, fwchannel, fwbandwidth);
-        }
-        VERBOSE(VB_GENERAL,QString("Firewire: releasing raw1394 handle"));
-        raw1394_destroy_handle(fwhandle);
-    }    
-    isopen = false;
+void FirewireRecorder::deleteLater(void)
+{
+    Close();
+    DTVRecorder::deleteLater();
 }
 
 bool FirewireRecorder::Open() {
 
-     if(isopen)
+     if (isopen)
          return true;
     
      VERBOSE(VB_GENERAL,QString("Firewire: Initializing Port: %1, Node: %2, Speed: %3")
@@ -70,22 +60,22 @@ bool FirewireRecorder::Open() {
                                .arg(fwnode)
                                .arg(FirewireSpeedString(fwspeed)));
 
-     if((fwhandle = raw1394_new_handle_on_port(fwport)) == NULL) {
+     if ((fwhandle = raw1394_new_handle_on_port(fwport)) == NULL) {
          VERBOSE(VB_IMPORTANT, QString("Firewire: unable to get handle for port: %1, bailing").arg(fwport));
          perror("firewire port");
          return false;
      }
 
-     if(fwconnection == FIREWIRE_CONNECTION_P2P) {
+     if (fwconnection == FIREWIRE_CONNECTION_P2P) {
           VERBOSE(VB_GENERAL,QString("Firewire: Creating P2P Connection with Node: %1").arg(fwnode));
           fwchannel = iec61883_cmp_connect (fwhandle, fwnode | 0xffc0, &fwoplug, 
                         raw1394_get_local_id (fwhandle), &fwiplug, &fwbandwidth);
-          if(fwchannel > -1) {
+          if (fwchannel > -1) {
 	      VERBOSE(VB_GENERAL,QString("Firewire: Created Channel: %1, Bandwidth Allocation: %2").arg(fwchannel).arg(fwbandwidth));
           }
      } else {
           VERBOSE(VB_GENERAL,QString("Firewire: Creating Broadcast Connection with Node: %1").arg(fwnode));
-          if(iec61883_cmp_create_bcast_output(fwhandle, fwnode | 0xffc0, 0, FIREWIRE_CHANNEL_BROADCAST, fwspeed) != 0) {
+          if (iec61883_cmp_create_bcast_output(fwhandle, fwnode | 0xffc0, 0, FIREWIRE_CHANNEL_BROADCAST, fwspeed) != 0) {
 	      VERBOSE(VB_IMPORTANT, QString("Firewire: Failed to create connection"));
 	      // release raw1394 object;
               raw1394_destroy_handle(fwhandle);
@@ -96,7 +86,7 @@ bool FirewireRecorder::Open() {
      }
 
 
-     if((fwmpeg = iec61883_mpeg2_recv_init (fwhandle, read_tspacket, this)) == NULL) {
+     if ((fwmpeg = iec61883_mpeg2_recv_init (fwhandle, read_tspacket, this)) == NULL) {
          VERBOSE(VB_IMPORTANT, QString("Firewire: unable to init iec61883_mpeg2 object, bailing"));
          perror("iec61883_mpeg2 object");
  
@@ -108,12 +98,12 @@ bool FirewireRecorder::Open() {
      // set speed if needed
      // probably shouldnt even allow user to set, 100Mbps should be more the enough
      int curspeed = iec61883_mpeg2_get_speed(fwmpeg);
-     if(curspeed != fwspeed) {
+     if (curspeed != fwspeed) {
          VERBOSE(VB_GENERAL,QString("Firewire: Changing Speed %1 -> %2")
                                     .arg(FirewireSpeedString(curspeed))
                                     .arg(FirewireSpeedString(fwspeed)));
          iec61883_mpeg2_set_speed(fwmpeg, fwspeed);
-         if(fwspeed != iec61883_mpeg2_get_speed(fwmpeg)) {
+         if (fwspeed != iec61883_mpeg2_get_speed(fwmpeg)) {
               VERBOSE(VB_IMPORTANT, QString("Firewire: unable to set firewire speed, continuing"));
          }
      }
@@ -123,6 +113,28 @@ bool FirewireRecorder::Open() {
      return true;
 }
 
+void FirewireRecorder::Close(void)
+{
+    if (!isopen)
+        return;
+
+    isopen = false;
+
+    VERBOSE(VB_RECORD, "Firewire: releasing iec61883_mpeg2 object");
+    iec61883_mpeg2_close(fwmpeg);
+
+    if (fwconnection == FIREWIRE_CONNECTION_P2P && fwchannel > -1)
+    {
+        VERBOSE(VB_RECORD, "Firewire: disconnecting channel " << fwchannel);
+        iec61883_cmp_disconnect(fwhandle, fwnode | 0xffc0, fwoplug,
+                                raw1394_get_local_id (fwhandle),
+                                fwiplug, fwchannel, fwbandwidth);
+    }
+
+    VERBOSE(VB_RECORD, "Firewire: releasing raw1394 handle");
+    raw1394_destroy_handle(fwhandle);
+}
+
 void FirewireRecorder::StartRecording(void) {
 
     struct timeval tv;
@@ -141,24 +153,10 @@ void FirewireRecorder::StartRecording(void) {
     iec61883_mpeg2_recv_start(fwmpeg,fwchannel);
     lastpacket = time(NULL);
     while(_request_recording) {
-       if(_request_pause) {
-            if(!_paused) {
-                 // stop mpeg
-                 iec61883_mpeg2_recv_stop(fwmpeg);
-                  _paused = true;
-            }
-            pauseWait.wakeAll();
+       if (PauseAndWait())
+           continue;
 
-            usleep(1000);
-            continue;
-
-       } else if(!_request_pause && _paused) {
-            _paused = false;
-            iec61883_mpeg2_recv_start(fwmpeg,fwchannel);
-            lastpacket = time(NULL);
-       }
-
-       if(time(NULL) - lastpacket > FIREWIRE_TIMEOUT) {
+       if (time(NULL) - lastpacket > FIREWIRE_TIMEOUT) {
             VERBOSE(VB_IMPORTANT, QString("Firewire: No Input in %1 seconds [P:%2 N:%3] (time)").arg(FIREWIRE_TIMEOUT).arg(fwport).arg(fwnode));
             iec61883_mpeg2_recv_stop(fwmpeg);
             _error = true;
@@ -170,8 +168,8 @@ void FirewireRecorder::StartRecording(void) {
        tv.tv_sec = FIREWIRE_TIMEOUT;
        tv.tv_usec = 0;
 
-       if(select (fwfd + 1, &rfds, NULL, NULL, &tv) > 0) {
-            if(raw1394_loop_iterate(fwhandle) != 0) {
+       if (select (fwfd + 1, &rfds, NULL, NULL, &tv) > 0) {
+            if (raw1394_loop_iterate(fwhandle) != 0) {
                  VERBOSE(VB_IMPORTANT, "Firewire: libraw1394_loop_iterate() returned error.");
                  iec61883_mpeg2_recv_stop(fwmpeg);
                  _error = true;
@@ -191,47 +189,46 @@ void FirewireRecorder::StartRecording(void) {
     _recording = false;
 }  
 
-void FirewireRecorder::ProcessTSPacket(unsigned char *tspacket, int len) {
-    tpkt.InitHeader(tspacket);
-    tpkt.InitPayload(tspacket+4);
-    FindKeyframes(&tpkt);
+void FirewireRecorder::ProcessTSPacket(const TSPacket &tspacket)
+{
     lastpacket = time(NULL);
-    ringBuffer->Write(tspacket,len);
+    _buffer_packets = !FindKeyframes(&tspacket);
+    BufferedWrite(tspacket);
 }
 
 void FirewireRecorder::SetOptionsFromProfile(RecordingProfile *profile,
-                                         const QString &videodev,
-                                         const QString &audiodev,
-                                         const QString &vbidev, int ispip) {
+                                             const QString &videodev,
+                                             const QString &audiodev,
+                                             const QString &vbidev)
+{
     (void)videodev;
     (void)audiodev;
     (void)vbidev;
     (void)profile;
-    (void)ispip;
 }
 
 void FirewireRecorder::SetOption(const QString &name, const QString &value) {
 
-    if(name == "model") {
+    if (name == "model") {
         fwmodel = value;
     }
 }
 
 void FirewireRecorder::SetOption(const QString &name, int value) {
 
-    if(name == "port") {
+    if (name == "port") {
 	fwport = value;
-    } else if(name == "node") {
+    } else if (name == "node") {
         fwnode = value;
-    } else if(name == "speed") {
+    } else if (name == "speed") {
         fwspeed = value;
-        if(fwspeed != RAW1394_ISO_SPEED_100 && fwspeed != RAW1394_ISO_SPEED_200 && fwspeed != RAW1394_ISO_SPEED_400) {
+        if (fwspeed != RAW1394_ISO_SPEED_100 && fwspeed != RAW1394_ISO_SPEED_200 && fwspeed != RAW1394_ISO_SPEED_400) {
             VERBOSE(VB_IMPORTANT, QString("Firewire: Invalid speed '%1', assuming 0 (100Mbps)").arg(fwspeed));
             fwspeed = 0;
         }
-    } else if(name == "connection") {
+    } else if (name == "connection") {
 	fwconnection = value;
-	if(fwconnection != FIREWIRE_CONNECTION_P2P && fwconnection != FIREWIRE_CONNECTION_BROADCAST) {
+	if (fwconnection != FIREWIRE_CONNECTION_P2P && fwconnection != FIREWIRE_CONNECTION_BROADCAST) {
 	    VERBOSE(VB_IMPORTANT, QString("Firewire: Invalid Connection type '%1', assuming P2P").arg(fwconnection));
             fwconnection = FIREWIRE_CONNECTION_P2P;
         }
@@ -250,3 +247,26 @@ QString FirewireRecorder::FirewireSpeedString (int speed) {
                return(QString("Invalid (%1)").arg(speed));
      }
 }
+
+// documented in recorderbase.cpp
+bool FirewireRecorder::PauseAndWait(int timeout)
+{
+    if (request_pause)
+    {
+        if (!paused)
+        {
+            iec61883_mpeg2_recv_stop(fwmpeg);
+            paused = true;
+            pauseWait.wakeAll();
+            if (tvrec)
+                tvrec->RecorderPaused();
+        }
+        unpauseWait.wait(timeout);
+    }
+    if (!request_pause && paused)
+    {
+        iec61883_mpeg2_recv_start(fwmpeg, fwchannel);
+        paused = false;
+    }
+    return paused;
+}
