@@ -237,18 +237,20 @@ static struct CHANLIST ntsc_cable[] = {
 /* US HRC */
 static struct CHANLIST ntsc_hrc[] = {
     { "1",	  72000 },
+
     { "2",	  54000 }, 
     { "3",	  60000 }, 
     { "4",	  66000 }, 
+
     { "5",	  78000 }, 
     { "6",	  84000 }, 
+
     { "7",	 174000 },
     { "8",	 180000 },
     { "9",	 186000 },
     { "10",	 192000 },
     { "11",	 198000 },
     { "12",	 204000 },
-
     { "13",	 210000 },
     { "14",	 120000 },
     { "15",	 126000 },
@@ -257,7 +259,6 @@ static struct CHANLIST ntsc_hrc[] = {
     { "18",	 144000 },
     { "19",	 150000 },
     { "20",	 156000 },
- 
     { "21",	 162000 },
     { "22",	 168000 },
     { "23",	 216000 },
@@ -307,7 +308,6 @@ static struct CHANLIST ntsc_hrc[] = {
     { "67",	 480000 },
     { "68",	 486000 },
     { "69",	 492000 },
- 
     { "70",	 498000 },
     { "71",	 504000 },
     { "72",	 510000 },
@@ -333,11 +333,13 @@ static struct CHANLIST ntsc_hrc[] = {
     { "92",	 630000 },
     { "93",	 636000 },
     { "94",	 642000 },
+
     { "95",	  90000 },
     { "96",	  96000 },
     { "97",	 102000 },
     { "98",	 108000 },
     { "99",	 114000 },
+
     { "100",	 648000 },
     { "101",	 654000 },
     { "102",	 660000 },
@@ -375,6 +377,147 @@ static struct CHANLIST ntsc_hrc[] = {
     { "T14",	  49000 }, 
 };
 
+/** US IRC http://www.jneuhaus.com/fccindex/cablech.html */
+static struct CHANLIST ntsc_irc[] = {
+    { "1",	 73250 },
+    { "2",	 55250 },
+    { "3",	 61250 },
+    { "4",	 67250 },
+    { "5",	 79250 },
+    { "6",	 85250 },
+    { "7",	175250 },
+    { "8",	181250 },
+    { "9",	187250 },
+    { "10",	193250 },
+    { "11",	199250 },
+    { "12",	205250 },
+
+    { "13",	211250 },
+    { "14",	121250 },
+    { "15",	127250 },
+    { "16",	133250 },
+    { "17",	139250 },
+    { "18",	145250 },
+    { "19",	151250 },
+    { "20",	157250 },
+ 
+    { "21",	163250 },
+    { "22",	169250 },
+    { "23",	217250 },
+    { "24",	223250 },
+    { "25",	229250 },
+    { "26",	235250 },
+    { "27",	241250 },
+    { "28",	247250 },
+    { "29",	253250 },
+    { "30",	259250 },
+    { "31",	265250 },
+    { "32",	271250 },
+    { "33",	277250 },
+    { "34",	283250 },
+    { "35",	289250 },
+    { "36",	295250 },
+    { "37",	301250 },
+    { "38",	307250 },
+    { "39",	313250 },
+    { "40",	319250 },
+    { "41",	325250 },
+    { "42",	331250 },
+    { "43",	337250 },
+    { "44",	343250 },
+    { "45",	349250 },
+    { "46",	355250 },
+    { "47",	361250 },
+    { "48",	367250 },
+    { "49",	373250 },
+    { "50",	379250 },
+    { "51",	385250 },
+    { "52",	391250 },
+    { "53",	397250 },
+    { "54",	403250 },
+    { "55",	409250 },
+    { "56",	415250 },
+    { "57",	421250 },
+    { "58",	427250 },
+    { "59",	433250 },
+    { "60",	439250 },
+    { "61",	445250 },
+    { "62",	451250 },
+    { "63",	457250 },
+    { "64",	463250 },
+    { "65",	469250 },
+    { "66",	475250 },
+    { "67",	481250 },
+    { "68",	487250 },
+    { "69",	493250 },
+ 
+    { "70",	499250 },
+    { "71",	505250 },
+    { "72",	511250 },
+    { "73",	517250 },
+    { "74",	523250 },
+    { "75",	529250 },
+    { "76",	535250 },
+    { "77",	541250 },
+    { "78",	547250 },
+    { "79",	553250 },
+    { "80",	559250 },
+    { "81",	565250 },
+    { "82",	571250 },
+    { "83",	577250 },
+    { "84",	583250 },
+    { "85",	589250 },
+    { "86",	595250 },
+    { "87",	601250 },
+    { "88",	607250 },
+    { "89",	613250 },
+    { "90",	619250 },
+    { "91",	625250 },
+    { "92",	631250 },
+    { "93",	637250 },
+    { "94",	643250 },
+    { "95",	 91250 },
+    { "96",	 97250 },
+    { "97",	103250 },
+    { "98",	109250 },
+    { "99",	115250 },
+    { "100",	649250 },
+    { "101",	655250 },
+    { "102",	661250 },
+    { "103",	667250 },
+    { "104",	673250 },
+    { "105",	679250 },
+    { "106",	685250 },
+    { "107",	691250 },
+    { "108",	697250 },
+    { "109",	703250 },
+    { "110",	709250 },
+    { "111",	715250 },
+    { "112",	721250 },
+    { "113",	727250 },
+    { "114",	733250 },
+    { "115",	739250 },
+    { "116",	745250 },
+    { "117",	751250 },
+    { "118",	757250 },
+    { "119",	763250 },
+    { "120",	769250 },
+    { "121",	775250 },
+    { "122",	781250 },
+    { "123",	787250 },
+    { "124",	793250 },
+    { "125",	799250 },
+ 
+    { "T7", 	  8250 },
+    { "T8",	 14250 },
+    { "T9",	 20250 },
+    { "T10",	 26250 },
+    { "T11",	 32250 },
+    { "T12",	 38250 },
+    { "T13",	 44250 },
+    { "T14",	 50250 }
+};
+
 /* --------------------------------------------------------------------- */
 
 /* JP broadcast */
@@ -688,6 +831,14 @@ static struct CHANLIST pal_australia_optus[] = {
     { "S40", 455250 },	\
     { "S41", 463250 }
 
+#define FREQ_CCIR_H2          \
+    { "E21",   471250 },      \
+    { "E22",   479250 },      \
+    { "E23",   487250 },      \
+    { "E24",   495250 },      \
+    { "E25",   503250 },      \
+    { "E26",   511250 }
+
 /* OIRT frequencies */
 
 #define FREQ_OIRT_I_III		\
@@ -781,6 +932,7 @@ static struct CHANLIST europe_west[] = {
     FREQ_CCIR_I_III,
     FREQ_CCIR_SL_SH,
     FREQ_CCIR_H,
+    FREQ_CCIR_H2,
     FREQ_UHF
 };
 
@@ -790,6 +942,7 @@ static struct CHANLIST europe_east[] = {
     FREQ_CCIR_I_III,
     FREQ_CCIR_SL_SH,
     FREQ_CCIR_H,
+    FREQ_CCIR_H2,
     FREQ_UHF
 };
 
@@ -1138,6 +1291,7 @@ struct CHANLISTS chanlists[] = {
     { "us-bcast",         ntsc_bcast,        CHAN_COUNT(ntsc_bcast)        },
     { "us-cable",         ntsc_cable,        CHAN_COUNT(ntsc_cable)        },
     { "us-cable-hrc",     ntsc_hrc,          CHAN_COUNT(ntsc_hrc)          },
+    { "us-cable-irc",     ntsc_irc,          CHAN_COUNT(ntsc_irc)          },
     { "japan-bcast",      ntsc_bcast_jp,     CHAN_COUNT(ntsc_bcast_jp)     },
     { "japan-cable",      ntsc_cable_jp,     CHAN_COUNT(ntsc_cable_jp)     },
     { "europe-west",      europe_west,       CHAN_COUNT(europe_west)       },
