@@ -30,18 +30,19 @@ using namespace std;
 #include "util.h"
 #include "remoteutil.h"
 
-QString RunProgramGuide(QString startchannel, bool thread, TV *player,
-                        bool allowsecondaryepg)
+bool RunProgramGuide(uint &chanid, QString &channum,
+                     bool thread, TV *player,
+                     bool allowsecondaryepg)
 {
-    QString chanstr;
-   
+    bool channel_changed = false;
+
     if (thread)
         qApp->lock();
 
-    if (startchannel == QString::null)
-        startchannel = "";
+    gContext->addCurrentLocation("GuideGrid");
 
-    GuideGrid *gg = new GuideGrid(gContext->GetMainWindow(), startchannel, 
+    GuideGrid *gg = new GuideGrid(gContext->GetMainWindow(),
+                                  chanid, channum,
                                   player, allowsecondaryepg, "guidegrid");
 
     gg->Show();
@@ -56,21 +57,30 @@ QString RunProgramGuide(QString startchannel, bool thread, TV *player,
     else
         gg->exec();
 
-    chanstr = gg->getLastChannel();
+    if (chanid != gg->GetChanID())
+    {
+        chanid  = gg->GetChanID();
+        channum = gg->GetChanNum();
+        channel_changed = true;
+    }
 
     if (thread)
         qApp->lock();
 
     delete gg;
 
+    gContext->removeCurrentLocation();
+
     if (thread)
         qApp->unlock();
 
-    return chanstr;
+    return channel_changed;
 }
 
-GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
-                     bool allowsecondaryepg, const char *name)
+GuideGrid::GuideGrid(MythMainWindow *parent,
+                     uint chanid, QString channum,
+                     TV *player, bool allowsecondaryepg,
+                     const char *name)
          : MythDialog(parent, name)
 {
     desiredDisplayChans = DISPLAY_CHANS = 6;
@@ -212,7 +222,8 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     int secsoffset = -((m_originalStartTime.time().minute() % 30) * 60 +
                         m_originalStartTime.time().second());
     m_currentStartTime = m_originalStartTime.addSecs(secsoffset);
-    m_startChanStr = channel;
+    startChanID  = chanid;
+    startChanNum = channum;
 
     m_currentRow = (int)(desiredDisplayChans / 2);
     m_currentCol = 0;
@@ -227,12 +238,13 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
             m_programInfos[y][x] = NULL;
     }
 
-    //QTime clock = QTime::currentTime();
+    //MythTimer clock = QTime::currentTime();
     //clock.start();
     fillTimeInfos();
     //int filltime = clock.elapsed();
     //clock.restart();
-    fillChannelInfos(maxchannel);
+    fillChannelInfos();
+    maxchannel = max((int)m_channelInfos.size() - 1, 0);
     setStartChannel((int)(m_currentStartChannel) - 
                     (int)(desiredDisplayChans / 2));
     if (DISPLAY_CHANS > maxchannel)
@@ -496,15 +508,22 @@ void GuideGrid::parseContainer(QDomElement &element)
         videoRect = area;
 }
 
-QString GuideGrid::getLastChannel(void)
+QString GuideGrid::GetChanNum(void)
 {
-    unsigned int chanNum = m_currentRow + m_currentStartChannel;
-    if (chanNum >= m_channelInfos.size())
-        chanNum -= m_channelInfos.size();
+    if (m_channelInfos.empty() || !selectState)
+        return "";
 
-    if (selectState)
-        return m_channelInfos[chanNum].chanstr;
-    return 0;
+    uint idx = (m_currentRow + m_currentStartChannel) % m_channelInfos.size();
+    return m_channelInfos[idx].chanstr;
+}
+
+uint GuideGrid::GetChanID(void)
+{
+    if (m_channelInfos.empty() || !selectState)
+        return 0;
+
+    uint idx = (m_currentRow + m_currentStartChannel) % m_channelInfos.size();
+    return m_channelInfos[idx].chanid;
 }
 
 void GuideGrid::timeout()
@@ -535,80 +554,94 @@ void GuideGrid::timeout()
     repaint(curInfoRect, false);
 }
 
-void GuideGrid::fillChannelInfos(int &maxchannel, bool gotostartchannel)
+void GuideGrid::fillChannelInfos(bool gotostartchannel)
 {
-    m_channelInfos.clear();
-
-    QString queryfav;
-   
     MSqlQuery query(MSqlQuery::InitCon());
 
-    QString queryall = "SELECT channel.channum, channel.callsign, "
-                       "channel.icon, channel.chanid, favorites.favid, "
-                       "channel.name FROM channel LEFT JOIN favorites ON "
-                       "favorites.chanid = channel.chanid WHERE visible = 1 "
-                       "GROUP BY channum, callsign "
-                       "ORDER BY " + channelOrdering + ";";
+    m_channelInfos.clear();
 
     if (showFavorites)
     {
-        queryfav = "SELECT channel.channum, channel.callsign, "
-                   "channel.icon, channel.chanid, favorites.favid, "
-                   "channel.name FROM favorites, channel WHERE "
-                   "channel.chanid = favorites.chanid and visible = 1 "
-                   "ORDER BY " + channelOrdering + ";";
+        query.prepare(
+            "SELECT channel.channum, channel.callsign, "
+            "       channel.icon,    channel.chanid, "
+            "       favorites.favid, channel.name "
+            "FROM videosource, cardinput, favorites, channel "
+            "WHERE channel.chanid       = favorites.chanid     AND "
+            "      visible              = 1                    AND "
+            "      channel.sourceid     = videosource.sourceid AND "
+            "      videosource.sourceid = cardinput.sourceid "
+            "GROUP BY channum, callsign "
+            "ORDER BY " + channelOrdering);
 
-        query.prepare(queryfav);
-        query.exec();   
-
-        // If we don't have any favorites, then just show regular listings.
-        if (!query.isActive() || query.size() == 0)
-        {
-            showFavorites = (!showFavorites);
-            query.prepare(queryall);
-            query.exec();
-        }
+        if (query.exec() && query.isActive())
+            showFavorites = query.size();
+        else
+            MythContext::DBError("fillChannelInfos -- favorites", query);
     }
-    else
+
+    if (!showFavorites)
     {
-        query.prepare(queryall);
-        query.exec();
+        query.prepare(
+            "SELECT channel.channum, channel.callsign, "
+            "       channel.icon,    channel.chanid, "
+            "       favorites.favid, channel.name "
+            "FROM videosource, cardinput, "
+            "       channel LEFT JOIN favorites "
+            "       ON favorites.chanid = channel.chanid "
+            "WHERE visible              = 1                    AND "
+            "      channel.sourceid     = videosource.sourceid AND "
+            "      videosource.sourceid = cardinput.sourceid "
+            "GROUP BY channum, callsign "
+            "ORDER BY " + channelOrdering);
+
+        if (!query.exec() || !query.isActive())
+        {
+            MythContext::DBError("fillChannelInfos -- all connected", query);
+            return;
+        }
     }
  
-    bool set = false;
-    maxchannel = 0;
-    
-    if (query.isActive() && query.size() > 0)
+    bool startingset = false;
+    while (query.next())
     {
-        while (query.next())
+        ChannelInfo val;
+
+        val.chanstr  = query.value(0).toString();
+        val.chanid   = query.value(3).toInt();
+
+        // validate the channum and chanid, skip channel if these are invalid
+        if (val.chanstr == "" || !val.chanid)
+            continue;
+
+        // fill in the rest of the data...
+        val.callsign = QString::fromUtf8(query.value(1).toString());
+        val.iconpath = query.value(2).toString();
+        val.favid    = query.value(4).toInt();
+        val.channame = QString::fromUtf8(query.value(5).toString());
+        val.icon     = NULL;
+
+        // set starting channel index if it hasn't been set
+        bool match   = gotostartchannel && !startingset;
+        match &= (startChanID)  ? val.chanid  == (int)startChanID : true;
+        match &= (!startChanID) ? val.chanstr == startChanNum     : true;
+        if (match)
         {
-            ChannelInfo val;
-            val.chanstr = query.value(0).toString();
-            if (val.chanstr != "")
-            {
-                val.callsign = QString::fromUtf8(query.value(1).toString());
-                val.iconpath = query.value(2).toString();
-                val.chanstr = query.value(0).toString();
-                val.chanid = query.value(3).toInt();
-                val.favid = query.value(4).toInt();
-                val.channame = QString::fromUtf8(query.value(5).toString());
-                val.icon = NULL;
-        
-                if (gotostartchannel && val.chanstr == m_startChanStr && !set)
-                {
-                    m_currentStartChannel = m_channelInfos.size();
-                    set = true;
-                }
-                
-                m_channelInfos.push_back(val);
-                maxchannel++;
-            }
+            m_currentStartChannel = m_channelInfos.size();
+            startingset = true;
         }
+                
+        // add the new channel to the list
+        m_channelInfos.push_back(val);
     }
-    else
+    // set starting channel index to 0 if it hasn't been set
+    m_currentStartChannel = (startingset) ? m_currentStartChannel : 0;
+
+    if (m_channelInfos.empty())
     {
-        cerr << "You don't have any channels defined in the database\n";
-        cerr << "This isn't going to work.\n";
+        VERBOSE(VB_IMPORTANT, "GuideGrid: "
+                "\n\t\t\tYou don't have any channels defined in the database."
+                "\n\t\t\tGuide grid will have nothing to show you.");
     }
 }
 
@@ -706,21 +739,26 @@ void GuideGrid::fillProgramRowInfos(unsigned int row)
     }
 
     int chanNum = row + m_currentStartChannel;
-    if (chanNum >= (int)m_channelInfos.size())
-        chanNum -= (int)m_channelInfos.size();
+    if (chanNum >= (int) m_channelInfos.size())
+        chanNum -= (int) m_channelInfos.size();
+    if (chanNum >= (int) m_channelInfos.size())
+        return;
+
     if (chanNum < 0)
         chanNum = 0;
 
     m_programs[row] = proglist = new ProgramList();
 
-    QString querystr = QString(
-        "WHERE program.chanid = %1 AND program.endtime >= %2 "
-        "    AND program.starttime <= %3 AND program.manualid = 0 ")
-        .arg(m_channelInfos[chanNum].chanid)
-        .arg(m_currentStartTime.toString("yyyyMMddhhmm00"))
-        .arg(m_currentEndTime.toString("yyyyMMddhhmm00"));
+    MSqlBindings bindings;
+    QString querystr = "WHERE program.chanid = :CHANID "
+                       "  AND program.endtime >= :STARTTS "
+                       "  AND program.starttime <= :ENDTS "
+                       "  AND program.manualid = 0 ";
+    bindings[":CHANID"] = m_channelInfos[chanNum].chanid;
+    bindings[":STARTTS"] = m_currentStartTime.toString("yyyy-MM-ddThh:mm:00");
+    bindings[":ENDTS"] = m_currentEndTime.toString("yyyy-MM-ddThh:mm:00");
 
-    proglist->FromProgram(querystr, m_recList);
+    proglist->FromProgram(querystr, bindings, m_recList);
 
     QDateTime ts = m_currentStartTime;
 
@@ -1085,7 +1123,9 @@ void GuideGrid::paintChannels(QPainter *p)
         unsigned int chanNumber = y + m_currentStartChannel;
         if (chanNumber >= m_channelInfos.size())
             chanNumber -= m_channelInfos.size();
-  
+        if (chanNumber >= m_channelInfos.size())
+            break;  
+
         chinfo = &(m_channelInfos[chanNumber]);
         if ((y == (unsigned int)2 && scrolltype != 1) || 
             ((signed int)y == m_currentRow && scrolltype == 1))
@@ -1113,7 +1153,7 @@ void GuideGrid::paintChannels(QPainter *p)
 
         if (type)
         {
-            if (chinfo->iconpath != "none" && chinfo->iconpath != "")
+            if (!chinfo->iconpath.isEmpty() && chinfo->iconpath != "none")
             {
                 int iconsize = 0;
                 iconsize = type->GetSize();
@@ -1208,6 +1248,8 @@ void GuideGrid::paintInfo(QPainter *p)
     int chanNum = m_currentRow + m_currentStartChannel;
     if (chanNum >= (int)m_channelInfos.size())
         chanNum -= (int)m_channelInfos.size();
+    if (chanNum >= (int)m_channelInfos.size())
+        return;
     if (chanNum < 0)
         chanNum = 0;
 
@@ -1261,7 +1303,8 @@ void GuideGrid::generateListings()
 
     int maxchannel = 0;
     DISPLAY_CHANS = desiredDisplayChans;
-    fillChannelInfos(maxchannel);
+    fillChannelInfos();
+    maxchannel = max((int)m_channelInfos.size() - 1, 0);
     if (DISPLAY_CHANS > maxchannel)
         DISPLAY_CHANS = maxchannel;
 
@@ -1272,13 +1315,14 @@ void GuideGrid::generateListings()
 
 void GuideGrid::toggleChannelFavorite()
 {
-    QString thequery;
     MSqlQuery query(MSqlQuery::InitCon());
 
     // Get current channel id, and make sure it exists...
     int chanNum = m_currentRow + m_currentStartChannel;
     if (chanNum >= (int)m_channelInfos.size())
         chanNum -= (int)m_channelInfos.size();
+    if (chanNum >= (int)m_channelInfos.size())
+        return;
     if (chanNum < 0)
         chanNum = 0;
 
@@ -1287,19 +1331,15 @@ void GuideGrid::toggleChannelFavorite()
 
     if (favid > 0) 
     {
-        thequery = QString("DELETE FROM favorites WHERE favid = '%1'")
-                           .arg(favid);
-
-        query.prepare(thequery);
+        query.prepare("DELETE FROM favorites WHERE favid = :FAVID ;");
+        query.bindValue(":FAVID", favid);
         query.exec(); 
     }
     else
     {
         // We have no favorites record...Add one to toggle...
-        thequery = QString("INSERT INTO favorites (chanid) VALUES ('%1')")
-                           .arg(chanid);
-
-        query.prepare(thequery);
+        query.prepare("INSERT INTO favorites (chanid) VALUES (:FAVID);");
+        query.bindValue(":FAVID", chanid);
         query.exec(); 
     }
 
@@ -1309,7 +1349,8 @@ void GuideGrid::toggleChannelFavorite()
     {
         int maxchannel = 0;
         DISPLAY_CHANS = desiredDisplayChans;
-        fillChannelInfos(maxchannel, false);
+        fillChannelInfos(false);
+        maxchannel = max((int)m_channelInfos.size() - 1, 0);
         if (DISPLAY_CHANS > maxchannel)
             DISPLAY_CHANS = maxchannel;
 
@@ -1692,7 +1733,7 @@ void GuideGrid::upcoming()
     if (pginfo->title == unknownTitle)
         return;
 
-    ProgLister *pl = new ProgLister(plTitle, pginfo->title,
+    ProgLister *pl = new ProgLister(plTitle, pginfo->title, "",
                                    gContext->GetMainWindow(), "proglist");
     pl->exec();
     delete pl;
@@ -1713,16 +1754,14 @@ void GuideGrid::details()
 
 void GuideGrid::channelUpdate(void)
 {
-    if (!m_player)
+    if (m_channelInfos.empty())
         return;
 
-    int chanNum = m_currentRow + m_currentStartChannel;
-    if (chanNum >= (int)m_channelInfos.size())
-        chanNum -= (int)m_channelInfos.size();
-    if (chanNum < 0)
-        chanNum = 0;
+    uint idx = (m_currentRow + m_currentStartChannel) % m_channelInfos.size();
+    ChannelInfo info = m_channelInfos[idx];
 
-    m_player->EPGChannelUpdate(m_channelInfos[chanNum].chanstr);
+    if (m_player)
+        m_player->EPGChannelUpdate(info.chanid, info.chanstr);
 }
 
 //
