@@ -2,6 +2,7 @@
 #include <pthread.h>
 #include <cstdio>
 #include <cstdlib>
+#include <cerrno>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -9,11 +10,16 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include <ctime>
+#include <qregexp.h>
 #include "videodev_myth.h"
 
 extern "C" {
 #include <inttypes.h>
-#include "ivtv-ext-api.h"
+#ifdef USING_IVTV_HEADER
+#include <linux/ivtv.h>
+#else
+#include "ivtv_myth.h"
+#endif
 }
 
 using namespace std;
@@ -23,10 +29,10 @@ using namespace std;
 #include "mythcontext.h"
 #include "programinfo.h"
 #include "recordingprofile.h"
+#include "tv_rec.h"
 
 extern "C" {
 #include "../libavcodec/avcodec.h"
-#include "../libavformat/avformat.h"
 }
 
 const int MpegRecorder::audRateL1[] = { 32, 64, 96, 128, 160, 192, 224, 
@@ -41,11 +47,12 @@ const char* MpegRecorder::streamType[] = { "MPEG-2 PS", "MPEG-2 TS",
 const char* MpegRecorder::aspectRatio[] = { "Square", "4:3", "16:9", 
                                             "2.21:1", 0 };
 
-MpegRecorder::MpegRecorder()
-            : RecorderBase()
+#define BUILDBUFFERMAX (1024 * 1024)
+
+MpegRecorder::MpegRecorder(TVRec *rec) :
+    RecorderBase(rec, "MpegRecorder")
 {
-    paused = false;
-    mainpaused = false;
+    errored = false;
     recording = false;
 
     framesWritten = 0;
@@ -68,14 +75,38 @@ MpegRecorder::MpegRecorder()
     audbitratel1 = 14;
     audbitratel2 = 14;
     audvolume = 80;
+
+    deviceIsMpegFile = false;
+    bufferSize = 4096;
+
+    buildbuffer = new unsigned char[BUILDBUFFERMAX + 1];
+    buildbuffersize = 0;
 }
 
 MpegRecorder::~MpegRecorder()
 {
-    if (chanfd > 0)
+    TeardownAll();
+    delete [] buildbuffer;
+}
+
+void MpegRecorder::deleteLater(void)
+{
+    TeardownAll();
+    RecorderBase::deleteLater();
+}
+
+void MpegRecorder::TeardownAll(void)
+{
+    if (chanfd >= 0)
+    {
         close(chanfd);
-    if (readfd > 0)
+        chanfd = -1;
+    }
+    if (readfd >= 0)
+    {
         close(readfd);
+        readfd = -1;
+    }
 }
 
 void MpegRecorder::SetOption(const QString &opt, int value)
@@ -94,26 +125,30 @@ void MpegRecorder::SetOption(const QString &opt, int value)
     else if (opt == "mpeg2audbitratel1")
     {
         for (int i = 0; audRateL1[i] != 0; i++)
+        {
             if (audRateL1[i] == value)
             {
                 audbitratel1 = i + 1;
                 found = true;
                 break;
             }
-        if (! found)
+        }
+        if (!found)
             cerr << "Audiorate(L1): " << value << " is invalid\n";
 
     }
     else if (opt == "mpeg2audbitratel2")
     {
         for (int i = 0; audRateL2[i] != 0; i++)
+        {
             if (audRateL2[i] == value)
             {
                 audbitratel2 = i + 1;
                 found = true;
                 break;
             }
-        if (! found)
+        }
+        if (!found)
             cerr << "Audiorate(L2): " << value << " is invalid\n";
     }
     else if (opt == "mpeg2audvolume")
@@ -173,12 +208,24 @@ void MpegRecorder::SetOption(const QString &opt, const QString &value)
 void MpegRecorder::SetOptionsFromProfile(RecordingProfile *profile, 
                                          const QString &videodev, 
                                          const QString &audiodev,
-                                         const QString &vbidev, int ispip)
+                                         const QString &vbidev)
 {
     (void)audiodev;
     (void)vbidev;
 
-    SetOption("videodevice", videodev);
+    if (videodev.lower().left(5) == "file:")
+    {
+        deviceIsMpegFile = true;
+        bufferSize = 64000;
+        QString newVideoDev = videodev;
+        newVideoDev.replace(QRegExp("^file:", false), QString(""));
+        SetOption("videodevice", newVideoDev);
+    }
+    else
+    {
+        SetOption("videodevice", videodev);
+    }
+
     SetOption("tvformat", gContext->GetSetting("TVFormat"));
     SetOption("vbiformat", gContext->GetSetting("VbiFormat"));
 
@@ -197,34 +244,41 @@ void MpegRecorder::SetOptionsFromProfile(RecordingProfile *profile,
 
     SetIntOption(profile, "width");
     SetIntOption(profile, "height");
-
-    if (ispip)
-    {
-        SetOption("width", 320);
-        SetOption("height", 240);
-        SetOption("mpeg2bitrate", 1000);
-        SetOption("mpeg2maxbitrate", 2000);
-    }
 }
 
-void MpegRecorder::StartRecording(void)
+bool MpegRecorder::OpenMpegFileAsInput(void)
+{
+    chanfd = readfd = open(videodevice.ascii(), O_RDONLY);
+
+    if (readfd < 0)
+    {
+        cerr << "Can't open MPEG File: " << videodevice << endl;
+        perror("open mpeg file:");
+        return false;
+    }
+    return true;
+}
+
+bool MpegRecorder::OpenV4L2DeviceAsInput(void)
 {
     chanfd = open(videodevice.ascii(), O_RDWR);
-    if (chanfd <= 0)
+    if (chanfd < 0)
     {
         cerr << "Can't open video device: " << videodevice << endl;
         perror("open video:");
-        return;
+        return false;
     }
 
     struct v4l2_format vfmt;
     memset(&vfmt, 0, sizeof(vfmt));
 
+    vfmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+
     if (ioctl(chanfd, VIDIOC_G_FMT, &vfmt) < 0)
     {
         cerr << "Error getting format\n";
         perror("VIDIOC_G_FMT:");
-        return;
+        return false;
     }
 
     vfmt.fmt.pix.width = width;
@@ -234,7 +288,7 @@ void MpegRecorder::StartRecording(void)
     {
         cerr << "Error setting format\n";
         perror("VIDIOC_S_FMT:");
-        return;
+        return false;
     }
 
     struct ivtv_ioctl_codec ivtvcodec;
@@ -244,7 +298,7 @@ void MpegRecorder::StartRecording(void)
     {
         cerr << "Error getting codec params\n";
         perror("IVTV_IOC_G_CODEC:");
-        return;
+        return false;
     }
 
     // only 48kHz works properly.
@@ -290,16 +344,19 @@ void MpegRecorder::StartRecording(void)
         ivtvcodec.bitrate = ivtvcodec.bitrate_peak;
 
     // framerate (1 = 25fps, 0 = 30fps)
-    ivtvcodec.framerate = (!ntsc);
+    ivtvcodec.framerate = (!ntsc_framerate);
     ivtvcodec.stream_type = streamtype;
 
     if (ioctl(chanfd, IVTV_IOC_S_CODEC, &ivtvcodec) < 0)
     {
         cerr << "Error setting codec params\n";
         perror("IVTV_IOC_S_CODEC:");
-        return;
+        return false;
     }
 
+    if (ivtvcodec.framerate)
+        keyframedist = 12;
+
     struct v4l2_control ctrl;
     ctrl.id = V4L2_CID_AUDIO_VOLUME;
     ctrl.value = 65536 / 100 *audvolume;
@@ -308,51 +365,143 @@ void MpegRecorder::StartRecording(void)
     {
         cerr << "Error setting codec params\n";
         perror("VIDIOC_S_CTRL:");
-        return;
+        return false;
     }
 
-    readfd = open(videodevice.ascii(), O_RDWR);
-    if (readfd <= 0)
+    if (vbimode) {
+        struct ivtv_sliced_vbi_format vbifmt;
+        memset(&vbifmt, 0, sizeof(struct ivtv_sliced_vbi_format));
+        vbifmt.service_set = (1==vbimode) ? VBI_TYPE_TELETEXT : VBI_TYPE_CC;
+        if (ioctl(chanfd, IVTV_IOC_S_VBI_MODE, &vbifmt) < 0) 
+        {
+            struct v4l2_format vbi_fmt;
+            memset(&vbi_fmt, 0, sizeof(struct v4l2_format));
+            vbi_fmt.type = V4L2_BUF_TYPE_SLICED_VBI_CAPTURE;
+            vbi_fmt.fmt.sliced.service_set = 0;
+            if (1 == vbimode)
+                vbi_fmt.fmt.sliced.service_set |= V4L2_SLICED_VBI_625;
+            else
+                vbi_fmt.fmt.sliced.service_set |= V4L2_SLICED_VBI_525;
+
+            if (ioctl(chanfd, VIDIOC_S_FMT, &vbi_fmt) < 0)
+            {
+                VERBOSE(VB_IMPORTANT, QString("Can't enable VBI recording"));
+                perror("vbi");
+            }
+        }
+
+        int embedon = 1;
+        if (ioctl(chanfd, IVTV_IOC_S_VBI_EMBED, &embedon) < 0)
+        {
+            VERBOSE(VB_IMPORTANT, QString("Can't enable VBI recording (2)"));
+            perror("vbi");
+        }
+
+        ioctl(chanfd, IVTV_IOC_G_VBI_MODE, &vbifmt);
+        VERBOSE(VB_RECORD, QString("VBI service:%1, packet size:%2, io size:%3").
+                arg(vbifmt.service_set).arg(vbifmt.packet_size).arg(vbifmt.io_size));
+    }
+
+    readfd = open(videodevice.ascii(), O_RDWR | O_NONBLOCK);
+    if (readfd < 0)
     {
         cerr << "Can't open video device: " << videodevice << endl;
         perror("open video:");
-        return;
+        return false;
     }
+    return true;
+}
 
-    unsigned char buffer[256001];
-    int ret;
+bool MpegRecorder::Open(void)
+{
+    if (deviceIsMpegFile)
+        return OpenMpegFileAsInput();
+    else
+        return OpenV4L2DeviceAsInput();
+}
+
+void MpegRecorder::StartRecording(void)
+{
+    if (!Open())
+    {
+	errored = true;
+	return;
+    }
 
     if (!SetupRecording())
     {
-        cerr << "Error initializing recording\n";
-        return;
+        VERBOSE(VB_IMPORTANT, "Error initializing recording");
+	errored = true;
+	return;
     }
 
     encoding = true;
     recording = true;
+    unsigned char *buffer = new unsigned char[bufferSize + 1];
+    int ret;
+
+    MythTimer elapsedTimer;
+    float elapsed;
+
+    struct timeval tv;
+    fd_set rdset;
+
+    if (deviceIsMpegFile)
+        elapsedTimer.start();
 
     while (encoding)
     {
-        if (paused)
-        {
-            mainpaused = true;
-            pauseWait.wakeAll();
-
-            if (readfd > 0)
-            {
-                close(readfd);
-                readfd = -1;
-            }
-            usleep(50);
+        if (PauseAndWait(100))
             continue;
+
+        if ((deviceIsMpegFile) && (framesWritten))
+        {
+            elapsed = (elapsedTimer.elapsed() / 1000.0) + 1;
+            while ((framesWritten / elapsed) > 30)
+            {
+                usleep(50000);
+                elapsed = (elapsedTimer.elapsed() / 1000.0) + 1;
+            }
         }
 
         if (readfd < 0)
             readfd = open(videodevice.ascii(), O_RDWR);
 
-        ret = read(readfd, buffer, 256000);
+        tv.tv_sec = 5;
+        tv.tv_usec = 0;
+        FD_ZERO(&rdset);
+        FD_SET(readfd, &rdset);
 
-        if (ret < 0)
+        switch (select(readfd + 1, &rdset, NULL, NULL, &tv))
+        {
+            case -1:
+                if (errno == EINTR)
+                    continue;
+                perror("select");
+                continue;
+            case 0:
+                printf("select timeout - ivtv driver has stopped responding\n");
+                continue;
+           default: break;
+        }
+
+        ret = read(readfd, buffer, bufferSize);
+
+        if ((ret == 0) &&
+            (deviceIsMpegFile))
+        {
+            close(readfd);
+            readfd = open(videodevice.ascii(), O_RDONLY);
+
+            if (readfd >= 0)
+                ret = read(readfd, buffer, bufferSize);
+            if (ret <= 0)
+            {
+                encoding = false;
+                continue;
+            }
+        }
+        else if (ret < 0 && errno != EAGAIN)
         {
             cerr << "error reading from: " << videodevice << endl;
             perror("read");
@@ -364,6 +513,7 @@ void MpegRecorder::StartRecording(void)
 
     FinishRecording();
 
+    delete[] buffer;
     recording = false;
 }
 
@@ -372,85 +522,27 @@ int MpegRecorder::GetVideoFd(void)
     return chanfd;
 }
 
-// start common code to the dvbrecorder class.
-
-static void mpg_write_packet(void *opaque, uint8_t *buf, int buf_size)
-{
-    (void)opaque;
-    (void)buf;
-    (void)buf_size;
-}
-
-static int mpg_read_packet(void *opaque, uint8_t *buf, int buf_size)
-{
-    (void)opaque;
-    (void)buf;
-    (void)buf_size;
-    return 0;
-}
-
-static int mpg_seek_packet(void *opaque, int64_t offset, int whence)
-{
-    (void)opaque;
-    (void)offset;
-    (void)whence;
-    return 0;
-}
-
 bool MpegRecorder::SetupRecording(void)
 {
-    AVInputFormat *fmt = &mpegps_demux;
-    fmt->flags |= AVFMT_NOFILE;
-
-    ic = av_alloc_format_context();
-    if (!ic)
-    {
-        cerr << "Couldn't allocate context\n";
-        return false;
-    }
-
-    QString filename = "blah.mpg";
-    char *cfilename = (char *)filename.ascii();
-    AVFormatParameters params;
-
-    ic->pb.buffer_size = 256001;
-    ic->pb.buffer = NULL;
-    ic->pb.buf_ptr = NULL;
-    ic->pb.write_flag = 0;
-    ic->pb.buf_end = NULL;
-    ic->pb.opaque = this;
-    ic->pb.read_packet = mpg_read_packet;
-    ic->pb.write_packet = mpg_write_packet;
-    ic->pb.seek = mpg_seek_packet;
-    ic->pb.pos = 0;
-    ic->pb.must_flush = 0;
-    ic->pb.eof_reached = 0;
-    ic->pb.is_streamed = 0;
-    ic->pb.max_packet_size = 0;
-
-    int err = av_open_input_file(&ic, cfilename, fmt, 0, &params);
-    if (err < 0)
-    {
-        cerr << "Couldn't initialize decocder\n";
-        return false;
-    }    
-
-    ic->build_index = 0;
-
-    prev_gop_save_pos = -1;
-
+    leftovers = 0xFFFFFFFF;
+    numgops = 0;
+    lastseqstart = 0;
     return true;
 }
 
 void MpegRecorder::FinishRecording(void)
 {
-    if (curRecording && db_lock && db_conn)
+    ringBuffer->WriterFlush();
+
+    if (curRecording)
     {
-        pthread_mutex_lock(db_lock);
-        MythContext::KickDatabase(db_conn);
-        curRecording->SetPositionMap(positionMap, MARK_GOP_START, db_conn);
-        pthread_mutex_unlock(db_lock);
+        curRecording->SetFilesize(ringBuffer->GetRealFileSize());
+        SavePositionMap(true);
     }
+    positionMapLock.lock();
+    positionMap.clear();
+    positionMapDelta.clear();
+    positionMapLock.unlock();
 }
 
 void MpegRecorder::SetVideoFilters(QString &filters)
@@ -462,98 +554,87 @@ void MpegRecorder::Initialize(void)
 {
 }
 
+#define PACK_HEADER   0x000001BA
 #define GOP_START     0x000001B8
-#define PICTURE_START 0x00000100
+#define SEQ_START     0x000001B3
 #define SLICE_MIN     0x00000101
 #define SLICE_MAX     0x000001af
 
-bool MpegRecorder::PacketHasHeader(unsigned char *buf, int len,
-                                   unsigned int startcode)
+void MpegRecorder::ProcessData(unsigned char *buffer, int len)
 {
-    unsigned char *bufptr = buf;
-    unsigned int state = 0xFFFFFFFF, v = 0;
+    unsigned char *bufptr = buffer, *bufstart = buffer;
+    unsigned int state = leftovers, v = 0;
+    int leftlen = len;
 
-    while (bufptr < buf + len)
+    while (bufptr < buffer + len)
     {
         v = *bufptr++;
         if (state == 0x000001)
         {
             state = ((state << 8) | v) & 0xFFFFFF;
-            if (state >= SLICE_MIN && state <= SLICE_MAX)
-                return false;
-            if (state == startcode)
-                return true;
+            
+            if (state == PACK_HEADER)
+            {
+                long long startpos = ringBuffer->GetWritePosition();
+                startpos += buildbuffersize + bufptr - bufstart - 4;
+                lastpackheaderpos = startpos;
+
+                int curpos = bufptr - bufstart - 4;
+                if (curpos < 0)
+                {
+                    // header was split
+                    buildbuffersize += curpos;
+                    if (buildbuffersize > 0)
+                        ringBuffer->Write(buildbuffer, buildbuffersize);
+
+                    buildbuffersize = 4;
+                    memcpy(buildbuffer, &state, 4);
+
+                    leftlen = leftlen - curpos + 4;
+                    bufstart = bufptr;
+                }
+                else
+                {
+                    // header was entirely in this packet
+                    memcpy(buildbuffer + buildbuffersize, bufstart, curpos);
+                    buildbuffersize += curpos;
+                    bufstart += curpos;
+                    leftlen -= curpos;
+
+                    if (buildbuffersize > 0)
+                        ringBuffer->Write(buildbuffer, buildbuffersize);
+
+                    buildbuffersize = 0;
+                }
+            }
+
+            if (state == SEQ_START)
+            {
+                lastseqstart = lastpackheaderpos;
+            }
+
+            if (state == GOP_START && lastseqstart == lastpackheaderpos)
+            {
+                framesWritten = numgops * keyframedist;
+                numgops++;
+                HandleKeyframe();
+            }
         }
-        state = ((state << 8) | v) & 0xFFFFFF;
+        else
+            state = ((state << 8) | v) & 0xFFFFFF;
     }
 
-    return false;
-}
+    leftovers = state;
 
-void MpegRecorder::ProcessData(unsigned char *buffer, int len)
-{
-    AVPacket pkt;
-
-    ic->pb.buffer = buffer;
-    ic->pb.buf_ptr = ic->pb.buffer;
-    ic->pb.buf_end = ic->pb.buffer + len;
-    ic->pb.eof_reached = 0;
-    ic->pb.pos = len;
-
-    while (ic->pb.eof_reached == 0)
+    if (buildbuffersize + leftlen > BUILDBUFFERMAX)
     {
-        if (av_read_packet(ic, &pkt) < 0)
-            break;
-        
-        if (pkt.stream_index > ic->nb_streams)
-        {
-            cerr << "bad stream\n";
-            av_free_packet(&pkt);
-            continue;
-        }
-
-        AVStream *curstream = ic->streams[pkt.stream_index];
-        if (pkt.size > 0 && curstream->codec.codec_type == CODEC_TYPE_VIDEO)
-        {
-            if (PacketHasHeader(pkt.data, pkt.size, PICTURE_START))
-            {
-                framesWritten++;
-            }
-
-            if (PacketHasHeader(pkt.data, pkt.size, GOP_START))
-            {
-                int frameNum = framesWritten - 1;
-
-                if (!gopset && frameNum > 0)
-                {
-                    keyframedist = frameNum;
-                    gopset = true;
-                }
-
-                long long startpos = ringBuffer->GetFileWritePosition();
-                startpos += pkt.startpos;
-
-                long long keyCount = frameNum / keyframedist;
-
-                positionMap[keyCount] = startpos;
-
-                if (curRecording && db_lock && db_conn &&
-                    ((positionMap.size() % 30) == 0))
-                {
-                    pthread_mutex_lock(db_lock);
-                    MythContext::KickDatabase(db_conn);
-                    curRecording->SetPositionMap(positionMap, MARK_GOP_START,
-                            db_conn, prev_gop_save_pos, keyCount);
-                    pthread_mutex_unlock(db_lock);
-                    prev_gop_save_pos = keyCount + 1;
-                }
-            }
-        }
-
-        av_free_packet(&pkt);
+        ringBuffer->Write(buildbuffer, buildbuffersize);
+        buildbuffersize = 0;
     }
 
-    ringBuffer->Write(buffer, len);
+    // copy remaining..
+    memcpy(buildbuffer + buildbuffersize, bufstart, leftlen);
+    buildbuffersize += leftlen;
 }
 
 void MpegRecorder::StopRecording(void)
@@ -561,47 +642,33 @@ void MpegRecorder::StopRecording(void)
     encoding = false;
 }
 
-void MpegRecorder::Reset(void)
+void MpegRecorder::ResetForNewFile(void)
 {
-    AVPacketList *pktl = NULL;
-    while ((pktl = ic->packet_buffer))
-    {
-        ic->packet_buffer = pktl->next;
-        av_free_packet(&pktl->pkt);
-        av_free(pktl);
-    }
-
-    ic->pb.pos = 0;
-    ic->pb.buf_ptr = ic->pb.buffer;
-    ic->pb.buf_end = ic->pb.buffer;
-
+    errored = false;
     framesWritten = 0;
+    numgops = 0;
+    lastseqstart = lastpackheaderpos = 0;
 
     positionMap.clear();
+    positionMapDelta.clear();
+}
+
+void MpegRecorder::Reset(void)
+{
+    ResetForNewFile();
+
+    leftovers = 0xFFFFFFFF;
+    buildbuffersize = 0;
+
+    if (curRecording)
+        curRecording->ClearPositionMap(MARK_GOP_START);
 }
 
 void MpegRecorder::Pause(bool clear)
 {
     cleartimeonpause = clear;
-    mainpaused = false;
-    paused = true;
-}
-
-void MpegRecorder::Unpause(void)
-{
     paused = false;
-}
-
-bool MpegRecorder::GetPause(void)
-{
-    return mainpaused;
-}
-
-void MpegRecorder::WaitForPause(void)
-{
-    if (!mainpaused)
-        if (!pauseWait.wait(1000))
-            cerr << "Waited too long for recorder to pause\n";
+    request_pause = true;
 }
 
 bool MpegRecorder::IsRecording(void)
@@ -616,6 +683,7 @@ long long MpegRecorder::GetFramesWritten(void)
 
 long long MpegRecorder::GetKeyframePosition(long long desired)
 {
+    QMutexLocker locker(&positionMapLock);
     long long ret = -1;
 
     if (positionMap.find(desired) != positionMap.end())
@@ -624,7 +692,68 @@ long long MpegRecorder::GetKeyframePosition(long long desired)
     return ret;
 }
 
-void MpegRecorder::GetBlankFrameMap(QMap<long long, int> &blank_frame_map)
+// documented in recorderbase.h
+void MpegRecorder::SetNextRecording(const ProgramInfo *progInf, RingBuffer *rb)
 {
-    (void)blank_frame_map;
+    // First we do some of the time consuming stuff we can do now
+    SavePositionMap(true);
+    ringBuffer->WriterFlush();
+
+    // Then we set the next info
+    {
+        QMutexLocker locker(&nextRingBufferLock);
+        nextRecording = NULL;
+        if (progInf)
+            nextRecording = new ProgramInfo(*progInf);
+        nextRingBuffer = rb;
+    }
+}
+
+/** \fn MpegRecorder::HandleKeyframe()
+ *  \brief This save the current frame to the position maps
+ *         and handles ringbuffer switching.
+ */
+void MpegRecorder::HandleKeyframe(void)
+{
+    // Add key frame to position map
+    bool save_map = false;
+    positionMapLock.lock();
+    if (!positionMap.contains(numgops))
+    {
+        positionMapDelta[numgops] = lastpackheaderpos;
+        positionMap[numgops]      = lastpackheaderpos;
+        save_map = true;
+    }
+    positionMapLock.unlock();
+    // Save the position map delta, but don't force a save.
+    if (save_map)
+        SavePositionMap(false);
+
+    // Perform ringbuffer switch if needed.
+    CheckForRingBufferSwitch();
+}
+
+/** \fn MpegRecorder::SavePositionMap(bool)
+ *  \brief This saves the postition map delta to the database if force
+ *         is true or there are 30 frames in the map or there are five
+ *         frames in the map with less than 30 frames in the non-delta
+ *         position map.
+ *  \param force If true this forces a DB sync.
+ */
+void MpegRecorder::SavePositionMap(bool force)
+{
+    QMutexLocker locker(&positionMapLock);
+
+    // save on every 5th key frame if in the first few frames of a recording
+    force |= (positionMap.size() < 30) && (positionMap.size()%5 == 1);
+    // save every 30th key frame later on
+    force |= positionMapDelta.size() >= 30;
+
+    if (curRecording && force)
+    {
+        curRecording->SetPositionMapDelta(positionMapDelta,
+                                          MARK_GOP_START);
+        curRecording->SetFilesize(lastpackheaderpos);
+        positionMapDelta.clear();
+    }
 }
