@@ -1,12 +1,18 @@
+// C headers
+#include <cerrno>
+#include <cassert>
+#include <cstring>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <assert.h>
-
-#include <iostream>
 #include <pthread.h>
+
+// C++ headers
+#include <iostream>
 using namespace std;
 
+// MythTV headers
+#include "mythconfig.h"
 #include "nuppeldecoder.h"
 #include "NuppelVideoPlayer.h"
 #include "remoteencoder.h"
@@ -15,60 +21,58 @@ using namespace std;
 
 #include "minilzo.h"
 
-pthread_mutex_t avcodeclock = PTHREAD_MUTEX_INITIALIZER;
+#ifdef WORDS_BIGENDIAN
+#include "bswap.h"
+#endif
 
-NuppelDecoder::NuppelDecoder(NuppelVideoPlayer *parent, MythSqlDatabase *db,
-                             ProgramInfo *pginfo)
-             : DecoderBase(parent, db, pginfo)
+#define LOC QString("NVD: ")
+#define LOC_ERR QString("NVD Error: ")
+
+NuppelDecoder::NuppelDecoder(NuppelVideoPlayer *parent, ProgramInfo *pginfo)
+    : DecoderBase(parent, pginfo),
+      gf(0), rtjd(0), video_width(0), video_height(0), video_size(0),
+      video_frame_rate(0.0f), audio_samplerate(44100), 
+#ifdef WORDS_BIGENDIAN
+      audio_bits_per_sample(0),
+#endif
+      ffmpeg_extradatasize(0), ffmpeg_extradata(0), usingextradata(false),
+      disablevideo(false), totalLength(0), totalFrames(0), effdsp(0), 
+      directframe(NULL),            decoded_video_frame(NULL),
+      mpa_codec(0), mpa_ctx(0), directrendering(false),
+      lastct('1'), strm(0), buf(0), buf2(0), 
+      videosizetotal(0), videoframesread(0), setreadahead(false)
 {
-    ffmpeg_extradata = NULL;
-    ffmpeg_extradatasize = 0;
-
-    usingextradata = false;
+    // initialize structures
+    memset(&fileheader, 0, sizeof(rtfileheader));
+    memset(&frameheader, 0, sizeof(rtframeheader));
     memset(&extradata, 0, sizeof(extendeddata));
+    memset(&tmppicture, 0, sizeof(AVPicture));
+    planes[0] = planes[1] = planes[2] = 0;
 
+    // set parent class variables
     positionMapType = MARK_KEYFRAME;
-
-    totalLength = 0;
-    totalFrames = 0;
+    lastKey = 0;
+    framesPlayed = 0;
+    getrawframes = false;
+    getrawvideo = false;
 
     gf = lame_init();
     lame_set_decode_only(gf, 1);
     lame_decode_init();
     lame_init_params(gf);
 
-    int i;
     rtjd = new RTjpeg();
-    i = RTJ_YUV420;
-    rtjd->SetFormat(&i);
-
-    if (lzo_init() != LZO_E_OK)
-    {
-        cerr << "lzo_init() failed, exiting\n";
-    }
+    int format = RTJ_YUV420;
+    rtjd->SetFormat(&format);
 
     avcodec_init();
     avcodec_register_all();
-
-    mpa_codec = 0;
-    mpa_ctx = NULL;
-    mpa_pic = NULL;
-    directrendering = false;
-    directbuf = NULL;
-
-    buf = NULL;
-    buf2 = NULL;
-    strm = NULL;
-
-    lastct = '1';
-
-    audio_samplerate = 44100;
-
-    lastKey = 0;
-    framesPlayed = 0;
-
-    getrawframes = false;
-    getrawvideo = false;
+    if (lzo_init() != LZO_E_OK)
+    {
+        VERBOSE(VB_IMPORTANT, "NuppelDecoder: lzo_init() failed, aborting");
+        errored = true;
+        return;
+    }
 }
 
 NuppelDecoder::~NuppelDecoder()
@@ -100,7 +104,7 @@ bool NuppelDecoder::CanHandle(char testbuf[2048])
     return false;
 }
 
-QString NuppelDecoder::GetEncodingType(void)
+QString NuppelDecoder::GetEncodingType(void) const
 {
     QString value = "Unknown";
     if (mpa_codec)
@@ -116,6 +120,40 @@ QString NuppelDecoder::GetEncodingType(void)
     return (value);
 }
 
+bool NuppelDecoder::ReadFileheader(struct rtfileheader *fh)
+{
+    if (ringBuffer->Read(fh, FILEHEADERSIZE) != FILEHEADERSIZE)
+        return false;
+
+#ifdef WORDS_BIGENDIAN
+    fh->width         = bswap_32(fh->width);
+    fh->height        = bswap_32(fh->height);
+    fh->desiredwidth  = bswap_32(fh->desiredwidth);
+    fh->desiredheight = bswap_32(fh->desiredheight);
+    fh->aspect        = bswap_dbl(fh->aspect);
+    fh->fps           = bswap_dbl(fh->fps);
+    fh->videoblocks   = bswap_32(fh->videoblocks);
+    fh->audioblocks   = bswap_32(fh->audioblocks);
+    fh->textsblocks   = bswap_32(fh->textsblocks);
+    fh->keyframedist  = bswap_32(fh->keyframedist);
+#endif
+
+    return true;
+}
+
+bool NuppelDecoder::ReadFrameheader(struct rtframeheader *fh)
+{
+    if (ringBuffer->Read(fh, FRAMEHEADERSIZE) != FRAMEHEADERSIZE)
+        return false;
+
+#ifdef WORDS_BIGENDIAN
+    fh->timecode     = bswap_32(fh->timecode);
+    fh->packetlength = bswap_32(fh->packetlength);
+#endif
+
+    return true;
+}
+
 int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo, 
                             char testbuf[2048])
 {
@@ -130,9 +168,9 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
     char ftype;
     char *space;
 
-    if (ringBuffer->Read(&fileheader, FILEHEADERSIZE) != FILEHEADERSIZE)
+    if (!ReadFileheader(&fileheader))
     {
-        cerr << "Error reading file: " << ringBuffer->GetFilename() << endl;
+        VERBOSE(VB_IMPORTANT, QString( "Error reading file: %1").arg( ringBuffer->GetFilename()));
         return -1;
     }
 
@@ -143,24 +181,28 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
         char dummychar;
         ringBuffer->Read(&dummychar, 1);
 
-        startpos = ringBuffer->GetTotalReadPosition();
+        startpos = ringBuffer->GetReadPosition();
 
-        if (ringBuffer->Read(&fileheader, FILEHEADERSIZE) != FILEHEADERSIZE)
+        if (!ReadFileheader(&fileheader))
         {
-            cerr << "Error reading file: " << ringBuffer->GetFilename() << endl;
+            VERBOSE(VB_IMPORTANT, QString( "Error reading file: %1").arg( ringBuffer->GetFilename()));
             return -1;
         }
 
         if (startpos > 20000)
         {
-            cerr << "Bad file: " << ringBuffer->GetFilename().ascii() << endl;
+            VERBOSE(VB_IMPORTANT, QString( "Bad file: %1").arg(ringBuffer->GetFilename().ascii()));
             return -1;
         }
     }
 
-    m_parent->SetVideoParams(fileheader.width, fileheader.height,
+    if (fileheader.aspect > .999 && fileheader.aspect < 1.001)
+        fileheader.aspect = 4.0 / 3;
+    current_aspect = fileheader.aspect;
+
+    GetNVP()->SetVideoParams(fileheader.width, fileheader.height,
                              fileheader.fps, fileheader.keyframedist,
-                             4.0 / 3, kScan_Detect);
+                             fileheader.aspect, kScan_Detect);
 
     video_width = fileheader.width;
     video_height = fileheader.height;
@@ -168,14 +210,14 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
     keyframedist = fileheader.keyframedist;
     video_frame_rate = fileheader.fps;
 
-    if (FRAMEHEADERSIZE != ringBuffer->Read(&frameheader, FRAMEHEADERSIZE))
+    if (!ReadFrameheader(&frameheader))
     {
-        cerr << "File not big enough for a header\n";
+        VERBOSE(VB_IMPORTANT, "File not big enough for a header");
         return -1;
     }
     if (frameheader.frametype != 'D')
     {
-        cerr << "Illegal file format\n";
+        VERBOSE(VB_IMPORTANT,"Illegal file format");
         return -1;
     }
 
@@ -190,7 +232,7 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
             if (frameheader.packetlength != ringBuffer->Read(ffmpeg_extradata,
                                                      frameheader.packetlength))
             {
-                cerr << "File not big enough for first frame data\n";
+                VERBOSE(VB_IMPORTANT,"File not big enough for first frame data");
                 delete [] ffmpeg_extradata;
                 ffmpeg_extradata = NULL;
                 delete [] space;
@@ -203,7 +245,7 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
         if (frameheader.packetlength != ringBuffer->Read(space,
                                                      frameheader.packetlength))
         {
-            cerr << "File not big enough for first frame data\n";
+            VERBOSE(VB_IMPORTANT,"File not big enough for first frame data");
             delete [] space;
             return -1;
         }
@@ -212,44 +254,67 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
     if ((video_height & 1) == 1)
     {
         video_height--;
-        cerr << "Incompatible video height, reducing to " << video_height
-             << endl;
+        VERBOSE(VB_IMPORTANT,QString("Incompatible video height, reducing to %1")
+                .arg( video_height));
     }
 
-    startpos = ringBuffer->GetTotalReadPosition();
+    startpos = ringBuffer->GetReadPosition();
 
-    ringBuffer->Read(&frameheader, FRAMEHEADERSIZE);
+    ReadFrameheader(&frameheader);
 
     if (frameheader.frametype == 'X')
     {
         if (frameheader.packetlength != EXTENDEDSIZE)
         {
-            cerr << "Corrupt file.  Bad extended frame.\n";
+            VERBOSE(VB_IMPORTANT,"Corrupt file.  Bad extended frame.");
         }
         else
         {
             ringBuffer->Read(&extradata, frameheader.packetlength);
+#ifdef WORDS_BIGENDIAN
+            struct extendeddata *ed = &extradata;
+            ed->version                 = bswap_32(ed->version);
+            ed->video_fourcc            = bswap_32(ed->video_fourcc);
+            ed->audio_fourcc            = bswap_32(ed->audio_fourcc);
+            ed->audio_sample_rate       = bswap_32(ed->audio_sample_rate);
+            ed->audio_bits_per_sample   = bswap_32(ed->audio_bits_per_sample);
+            ed->audio_channels          = bswap_32(ed->audio_channels);
+            ed->audio_compression_ratio = bswap_32(ed->audio_compression_ratio);
+            ed->audio_quality           = bswap_32(ed->audio_quality);
+            ed->rtjpeg_quality          = bswap_32(ed->rtjpeg_quality);
+            ed->rtjpeg_luma_filter      = bswap_32(ed->rtjpeg_luma_filter);
+            ed->rtjpeg_chroma_filter    = bswap_32(ed->rtjpeg_chroma_filter);
+            ed->lavc_bitrate            = bswap_32(ed->lavc_bitrate);
+            ed->lavc_qmin               = bswap_32(ed->lavc_qmin);
+            ed->lavc_qmax               = bswap_32(ed->lavc_qmax);
+            ed->lavc_maxqdiff           = bswap_32(ed->lavc_maxqdiff);
+            ed->seektable_offset        = bswap_64(ed->seektable_offset);
+            ed->keyframeadjust_offset   = bswap_64(ed->keyframeadjust_offset);
+#endif
             usingextradata = true;
-            ringBuffer->Read(&frameheader, FRAMEHEADERSIZE);
+            ReadFrameheader(&frameheader);
         }
     }
 
     if (usingextradata && extradata.seektable_offset > 0)
     {
-        long long currentpos = ringBuffer->GetTotalReadPosition();
+        long long currentpos = ringBuffer->GetReadPosition();
         struct rtframeheader seek_frameheader;
 
         int seekret = ringBuffer->Seek(extradata.seektable_offset, SEEK_SET);
-        if (seekret == -1) {
-            perror("seek");
+        if (seekret == -1)
+        {
+            VERBOSE(VB_IMPORTANT,
+                    QString("NuppelDecoder::OpenFile(): seek error (%1)")
+                    .arg(strerror(errno)));
         }
 
-        ringBuffer->Read(&seek_frameheader, FRAMEHEADERSIZE);
+        ReadFrameheader(&seek_frameheader);
 
         if (seek_frameheader.frametype != 'Q')
         {
-            cerr << "Invalid seektable (frametype "
-                 << (int)seek_frameheader.frametype << ")\n";
+            VERBOSE(VB_IMPORTANT, QString( "Invalid seektable (frametype %1)")
+                    .arg((int)seek_frameheader.frametype));
         }
         else
         {
@@ -270,6 +335,10 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                 {
                     memcpy(&ste, seekbuf + offset,
                            sizeof(struct seektable_entry));
+#ifdef WORDS_BIGENDIAN
+                    ste.file_offset     = bswap_64(ste.file_offset);
+                    ste.keyframe_number = bswap_32(ste.keyframe_number);
+#endif
                     offset += sizeof(struct seektable_entry);
 
                     PosMapEntry e = {ste.keyframe_number,
@@ -281,12 +350,12 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                 totalLength = (int)((ste.keyframe_number * keyframedist * 1.0) /
                                      video_frame_rate);
                 totalFrames = (long long)ste.keyframe_number * keyframedist;
-                m_parent->SetFileLength(totalLength, totalFrames);
+                GetNVP()->SetFileLength(totalLength, totalFrames);
 
                 delete [] seekbuf;
             }
             else
-                cerr << "0 length seek table\n";
+                VERBOSE(VB_IMPORTANT, "0 length seek table");
         }
 
         ringBuffer->Seek(currentpos, SEEK_SET);
@@ -295,21 +364,24 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
     if (usingextradata && extradata.keyframeadjust_offset > 0 &&
         hasFullPositionMap)
     {
-        long long currentpos = ringBuffer->GetTotalReadPosition();
+        long long currentpos = ringBuffer->GetReadPosition();
         struct rtframeheader kfa_frameheader;
 
         int kfa_ret = ringBuffer->Seek(extradata.keyframeadjust_offset, 
                                        SEEK_SET);
-        if (kfa_ret == -1) {
-            perror("keyframeadjust");
+        if (kfa_ret == -1)
+        {
+            VERBOSE(VB_IMPORTANT,
+                    QString("NuppelDecoder::OpenFile(): keyframeadjust (%1)")
+                    .arg(strerror(errno)));
         }
 
         ringBuffer->Read(&kfa_frameheader, FRAMEHEADERSIZE);
 
         if (kfa_frameheader.frametype != 'K')
         {
-            cerr << "Invalid key frame adjust table (frametype "
-                 << (int)kfa_frameheader.frametype << ")\n";
+            VERBOSE(VB_IMPORTANT, QString( "Invalid key frame adjust table (frametype %1)")
+                    .arg((int)kfa_frameheader.frametype));
         }
         else
         {
@@ -329,6 +401,10 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                 {
                     memcpy(&kfate, kfa_buf + offset,
                            sizeof(struct kfatable_entry));
+#ifdef WORDS_BIGENDIAN
+                    kfate.adjust          = bswap_32(kfate.adjust);
+                    kfate.keyframe_number = bswap_32(kfate.keyframe_number);
+#endif
                     offset += sizeof(struct kfatable_entry);
 
                     keyFrameAdjustMap[kfate.keyframe_number] = kfate.adjust;
@@ -338,7 +414,7 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
 
                 totalLength -= (int)(adjust / video_frame_rate);
                 totalFrames -= adjust;
-                m_parent->SetFileLength(totalLength, totalFrames);
+                GetNVP()->SetFileLength(totalLength, totalFrames);
 
                 adjust = 0;
                 for (unsigned int i=0; i < m_positionMap.size(); i++) 
@@ -352,7 +428,7 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                 delete [] kfa_buf;
             }
             else
-                cerr << "0 length key frame adjust table\n";
+                VERBOSE(VB_IMPORTANT,"0 length key frame adjust table");
         }
 
         ringBuffer->Seek(currentpos, SEEK_SET);
@@ -367,9 +443,9 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
         char dummychar;
         ringBuffer->Read(&dummychar, 1);
 
-        startpos = ringBuffer->GetTotalReadPosition();
+        startpos = ringBuffer->GetReadPosition();
 
-        if (ringBuffer->Read(&frameheader, FRAMEHEADERSIZE) != FRAMEHEADERSIZE)
+        if (!ReadFrameheader(&frameheader))
         {
             delete [] space;
             return -1;
@@ -385,16 +461,21 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
     foundit = 0;
 
     effdsp = audio_samplerate * 100;
-    m_parent->SetEffDsp(effdsp);
+    GetNVP()->SetEffDsp(effdsp);
 
     if (usingextradata)
     {
         effdsp = extradata.audio_sample_rate * 100;
-        m_parent->SetEffDsp(effdsp);
+        GetNVP()->SetEffDsp(effdsp);
         audio_samplerate = extradata.audio_sample_rate;
-        m_parent->SetAudioParams(extradata.audio_bits_per_sample,
+#ifdef WORDS_BIGENDIAN
+        // Why only if using extradata?
+        audio_bits_per_sample = extradata.audio_bits_per_sample;
+#endif
+        GetNVP()->SetAudioParams(extradata.audio_bits_per_sample,
                                  extradata.audio_channels, 
                                  extradata.audio_sample_rate);
+        GetNVP()->ReinitAudio();
         foundit = 1;
     }
 
@@ -408,7 +489,7 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                 effdsp = frameheader.timecode;
                 if (effdsp > 0)
                 {
-                    m_parent->SetEffDsp(effdsp);
+                    GetNVP()->SetEffDsp(effdsp);
                     foundit = 1;
                     continue;
                 }
@@ -424,10 +505,9 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
             }
         }
 
-        long long startpos2 = ringBuffer->GetTotalReadPosition();
+        long long startpos2 = ringBuffer->GetReadPosition();
 
-        foundit = (FRAMEHEADERSIZE != ringBuffer->Read(&frameheader,
-                                                       FRAMEHEADERSIZE));
+        foundit = !ReadFrameheader(&frameheader);
 
         bool framesearch = false;
 
@@ -445,10 +525,9 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
             char dummychar;
             ringBuffer->Read(&dummychar, 1);
 
-            startpos2 = ringBuffer->GetTotalReadPosition();
+            startpos2 = ringBuffer->GetReadPosition();
 
-            foundit = (FRAMEHEADERSIZE != ringBuffer->Read(&frameheader,
-                                                           FRAMEHEADERSIZE));
+            foundit = !ReadFrameheader(&frameheader);
             if (foundit)
                 break;
         }
@@ -462,7 +541,7 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
     if (usingextradata && extradata.video_fourcc == MKTAG('D', 'I', 'V', 'X'))
         setreadahead = true;
 
-    ringBuffer->CalcReadAheadThresh(0);
+    ringBuffer->UpdateRawBitrate(0);
 
     videosizetotal = 0;
     videoframesread = 0;
@@ -488,7 +567,7 @@ int get_nuppel_buffer(struct AVCodecContext *c, AVFrame *pic)
     int width = c->width;
     int height = c->height;
 
-    pic->data[0] = nd->directbuf;
+    pic->data[0] = nd->directframe->buf;
     pic->data[1] = pic->data[0] + width * height;
     pic->data[2] = pic->data[1] + width * height / 4;
 
@@ -496,7 +575,7 @@ int get_nuppel_buffer(struct AVCodecContext *c, AVFrame *pic)
     pic->linesize[1] = width / 2;
     pic->linesize[2] = width / 2;
 
-    pic->opaque = nd;
+    pic->opaque = nd->directframe;
     pic->type = FF_BUFFER_TYPE_USER;
 
     pic->age = 256 * 256 * 256 * 64;
@@ -509,6 +588,10 @@ void release_nuppel_buffer(struct AVCodecContext *c, AVFrame *pic)
     (void)c;
     assert(pic->type == FF_BUFFER_TYPE_USER);
 
+    NuppelDecoder *nd = (NuppelDecoder *)(c->opaque);
+    if (nd && nd->GetNVP() && nd->GetNVP()->getVideoOutput())
+        nd->GetNVP()->getVideoOutput()->DeLimboFrame((VideoFrame*)pic->opaque);
+
     int i;
     for (i = 0; i < 4; i++)
         pic->data[i] = NULL;
@@ -540,10 +623,10 @@ bool NuppelDecoder::InitAVCodec(int codec)
 
     if (!mpa_codec)
     {
-        cerr << "couldn't find codec " << codec;
         if (usingextradata)
-            cerr << " (" << extradata.video_fourcc << ")";
-        cerr << endl;
+            VERBOSE(VB_IMPORTANT, QString("couldn't find codec (%1)").arg(extradata.video_fourcc));
+        else
+            VERBOSE(VB_IMPORTANT, "couldn't find codec");
         return false;
     }
 
@@ -551,12 +634,7 @@ bool NuppelDecoder::InitAVCodec(int codec)
         directrendering = true;
 
     if (mpa_ctx)
-        free(mpa_ctx);
-
-    if (mpa_pic)
-        free(mpa_pic);
-
-    mpa_pic = avcodec_alloc_frame();
+        av_free(mpa_ctx);
 
     mpa_ctx = avcodec_alloc_context();
 
@@ -581,9 +659,10 @@ bool NuppelDecoder::InitAVCodec(int codec)
         mpa_ctx->extradata_size = ffmpeg_extradatasize;
     }
 
+    QMutexLocker locker(&avcodeclock);
     if (avcodec_open(mpa_ctx, mpa_codec) < 0)
     {
-        cerr << "Couldn't find lavc codec\n";
+        VERBOSE(VB_IMPORTANT, LOC + "Couldn't find lavc codec");
         return false;
     }
 
@@ -592,17 +671,18 @@ bool NuppelDecoder::InitAVCodec(int codec)
 
 void NuppelDecoder::CloseAVCodec(void)
 {
-    if (!mpa_codec)
-        return;
+    QMutexLocker locker(&avcodeclock);
 
-    avcodec_close(mpa_ctx);
+    if (mpa_codec)
+    {
+        avcodec_close(mpa_ctx);
 
-    if (mpa_ctx)
-        free(mpa_ctx);
-    if (mpa_pic)
-        free(mpa_pic);
-    mpa_ctx = NULL;
-    mpa_pic = NULL;
+        if (mpa_ctx)
+        {
+            av_free(mpa_ctx);
+            mpa_ctx = NULL;
+        }
+    }
 }
 
 bool NuppelDecoder::DecodeFrame(struct rtframeheader *frameheader,
@@ -613,6 +693,7 @@ bool NuppelDecoder::DecodeFrame(struct rtframeheader *frameheader,
     int compoff = 0;
 
     unsigned char *outbuf = frame->buf;
+    directframe = frame;
 
     if (!buf2)
     {
@@ -654,7 +735,7 @@ bool NuppelDecoder::DecodeFrame(struct rtframeheader *frameheader,
                               NULL);
         if (r != LZO_E_OK)
         {
-            cerr << "minilzo: can't decompress illegal data\n";
+            VERBOSE(VB_IMPORTANT, "minilzo: can't decompress illegal data");
         }
     }
 
@@ -684,23 +765,25 @@ bool NuppelDecoder::DecodeFrame(struct rtframeheader *frameheader,
         if (!mpa_codec)
             InitAVCodec(frameheader->comptype - '3');
 
-        int gotpicture = 0;
-        pthread_mutex_lock(&avcodeclock);
-        // if directrendering, writes into buf
-        directbuf = outbuf;
-        int ret = avcodec_decode_video(mpa_ctx, mpa_pic, &gotpicture,
-                                       lstrm, frameheader->packetlength);
-        pthread_mutex_unlock(&avcodeclock);
-        if (ret < 0)
-        {
-            VERBOSE(VB_PLAYBACK, QString("decoding error: %1 back from avcodec")
-                                 .arg(ret));
-            return false;
-        }
+        AVFrame mpa_pic;
 
-        if (!gotpicture)
         {
-            return false;
+            QMutexLocker locker(&avcodeclock);
+            // if directrendering, writes into buf
+            int gotpicture = 0;
+            int ret = avcodec_decode_video(mpa_ctx, &mpa_pic, &gotpicture,
+                                           lstrm, frameheader->packetlength);
+            directframe = NULL;
+            if (ret < 0)
+            {
+                VERBOSE(VB_PLAYBACK, LOC_ERR +
+                        "avcodec_decode_video returned: "<<ret);
+                return false;
+            }
+            else if (!gotpicture)
+            {
+                return false;
+            }
         }
 
 /* XXX: Broken
@@ -728,13 +811,8 @@ bool NuppelDecoder::DecodeFrame(struct rtframeheader *frameheader,
 
         avpicture_fill(&tmppicture, outbuf, PIX_FMT_YUV420P, video_width,
                        video_height);
-        for (int i = 0; i < 4; i++)
-        {
-            mpa_pic_tmp.data[i] = mpa_pic->data[i];
-            mpa_pic_tmp.linesize[i] = mpa_pic->linesize[i];
-        }
 
-        img_convert(&tmppicture, PIX_FMT_YUV420P, &mpa_pic_tmp,
+        img_convert(&tmppicture, PIX_FMT_YUV420P, (AVPicture *)&mpa_pic,
                     mpa_ctx->pix_fmt, video_width, video_height);
     }
 
@@ -746,7 +824,7 @@ bool NuppelDecoder::isValidFrametype(char type)
     switch (type)
     {
         case 'A': case 'V': case 'S': case 'T': case 'R': case 'X':
-        case 'M': case 'D':
+        case 'M': case 'D': case 'Q': case 'K':
             return true;
         default:
             return false;
@@ -788,11 +866,16 @@ long NuppelDecoder::UpdateStoredFrameNum(long framenum)
     return 0;
 }
 
-void NuppelDecoder::WriteStoredData(RingBuffer *rb, bool storevid)
+void NuppelDecoder::WriteStoredData(RingBuffer *rb, bool storevid,
+                                    long timecodeOffset)
 {
     RawDataList *data;
     while(! StoredData.isEmpty()) {
         data = StoredData.first();
+
+        if (data->frameheader.frametype != 'S')
+            data->frameheader.timecode -= timecodeOffset;
+
         if (storevid || data->frameheader.frametype != 'V')
         {
             rb->Write(&(data->frameheader), FRAMEHEADERSIZE);
@@ -804,26 +887,51 @@ void NuppelDecoder::WriteStoredData(RingBuffer *rb, bool storevid)
     }
 }
 
+void NuppelDecoder::ClearStoredData()
+{
+    RawDataList *data;
+    while(!StoredData.isEmpty()) {
+        data = StoredData.first();
+        StoredData.removeFirst();
+        delete data;
+    }
+}
+
 // avignore = 0  : get audio and video
 //          = 1  : video only
 //          = -1 : neither, just parse
 
-void NuppelDecoder::GetFrame(int avignore)
+bool NuppelDecoder::GetFrame(int avignore)
 {
     bool gotvideo = false;
     bool ret = false;
     int seeklen = 0;
 
+    decoded_video_frame = NULL;
+
     while (!gotvideo)
     {
-        long long currentposition = ringBuffer->GetTotalReadPosition();
+        long long currentposition = ringBuffer->GetReadPosition();
+        if (waitingForChange && currentposition + 4 >= readAdjust)
+        {
+            FileChanged();
+            currentposition = ringBuffer->GetReadPosition();
+        }
 
-        if ((ringBuffer->Read(&frameheader, FRAMEHEADERSIZE) != FRAMEHEADERSIZE)
-            || (frameheader.frametype == 'Q') || (frameheader.frametype == 'K'))
+        if (!ReadFrameheader(&frameheader))
         {
             ateof = true;
-            m_parent->SetEof();
-            return;
+            GetNVP()->SetEof();
+            return false;
+        }
+
+
+        if (!ringBuffer->LiveMode() && 
+            ((frameheader.frametype == 'Q') || (frameheader.frametype == 'K')))
+        {
+            ateof = true;
+            GetNVP()->SetEof();
+            return false;
         }
 
         bool framesearch = false;
@@ -837,12 +945,11 @@ void NuppelDecoder::GetFrame(int avignore)
 
             ringBuffer->Seek((long long)seeklen-FRAMEHEADERSIZE, SEEK_CUR);
 
-            if (ringBuffer->Read(&frameheader, FRAMEHEADERSIZE)
-                != FRAMEHEADERSIZE)
+            if (!ReadFrameheader(&frameheader))
             {
                 ateof = true;
-                m_parent->SetEof();
-                return;
+                GetNVP()->SetEof();
+                return false;
             }
             seeklen = 1;
         }
@@ -856,8 +963,8 @@ void NuppelDecoder::GetFrame(int avignore)
             {
                 delete [] dummy;
                 ateof = true;
-                m_parent->SetEof();
-                return;
+                GetNVP()->SetEof();
+                return false;
             }
 
             delete [] dummy;
@@ -879,7 +986,7 @@ void NuppelDecoder::GetFrame(int avignore)
                     frameheader.timecode < 5500000)
                 {
                     effdsp = frameheader.timecode;
-                    m_parent->SetEffDsp(effdsp);
+                    GetNVP()->SetEffDsp(effdsp);
                 }
             }
             else if (frameheader.comptype == 'V')
@@ -914,18 +1021,19 @@ void NuppelDecoder::GetFrame(int avignore)
         {
             if (frameheader.packetlength > 10485760) // arbitrary 10MB limit
             {
-                cerr << "Broken packet: " << frameheader.frametype
-                     << " " << frameheader.packetlength << endl;
+                VERBOSE(VB_IMPORTANT, QString("Broken packet: %1 %2")
+                        .arg(frameheader.frametype)
+                        .arg(frameheader.packetlength));
                 ateof = true;
-                m_parent->SetEof();
-                return;
+                GetNVP()->SetEof();
+                return false;
             }
             if (ringBuffer->Read(strm, frameheader.packetlength) !=
                 frameheader.packetlength)
             {
                 ateof = true;
-                m_parent->SetEof();
-                return;
+                GetNVP()->SetEof();
+                return false;
             }
         }
         else
@@ -940,16 +1048,24 @@ void NuppelDecoder::GetFrame(int avignore)
                 continue;
             }
 
-            VideoFrame *buf = m_parent->GetNextVideoFrame();
+            VideoFrame *buf = GetNVP()->GetNextVideoFrame();
 
             ret = DecodeFrame(&frameheader, strm, buf);
             if (!ret)
             {
-                m_parent->DiscardVideoFrame(buf);
+                GetNVP()->DiscardVideoFrame(buf);
                 continue;
             }
 
-            m_parent->ReleaseNextVideoFrame(buf, frameheader.timecode);
+            buf->frameNumber = framesPlayed;
+            GetNVP()->ReleaseNextVideoFrame(buf, frameheader.timecode);
+            
+            // We need to make the frame available ourselves
+            // if we are not using ffmpeg/avlib.
+            if (directframe)
+                GetNVP()->getVideoOutput()->DeLimboFrame(buf);
+
+            decoded_video_frame = buf;
             gotvideo = 1;
             if (getrawframes && getrawvideo)
                 StoreRawData(strm);
@@ -967,7 +1083,7 @@ void NuppelDecoder::GetFrame(int avignore)
                     float bps = videosizetotal * 8.0 / 1024 * video_frame_rate;
                     bps = bps * 3 / 2;
 
-                    ringBuffer->CalcReadAheadThresh((int)bps);
+                    ringBuffer->UpdateRawBitrate((uint) bps);
                     setreadahead = true;
                 }
             }
@@ -993,13 +1109,14 @@ void NuppelDecoder::GetFrame(int avignore)
 
                     if (lameret > 0)
                     {
-                        m_parent->AddAudioData(pcmlbuffer, pcmrbuffer,
+                        GetNVP()->AddAudioData(pcmlbuffer, pcmrbuffer,
                                                lameret, frameheader.timecode);
                     }
                     else if (lameret < 0)
                     {
                         VERBOSE(VB_IMPORTANT, QString("lame decode error: %1, exiting").arg(lameret));
-                        exit(-15);
+                        errored = true;
+                        return false;
                     }
                     packetlen = 0;
                 } while (lameret > 0);
@@ -1007,7 +1124,21 @@ void NuppelDecoder::GetFrame(int avignore)
             else
             {
                 getrawframes = 0;
-                m_parent->AddAudioData((char *)strm, frameheader.packetlength, 
+#ifdef WORDS_BIGENDIAN
+                // Why endian correct the audio buffer here?
+                // Don't big-endian clients have to do it in audiooutBlah.cpp?
+                if (audio_bits_per_sample == 16) {
+                    // swap bytes
+                    for (int i = 0; i < (frameheader.packetlength & ~1); i+=2) {
+                        char tmp;
+                        tmp = strm[i+1];
+                        strm[i+1] = strm[i];
+                        strm[i] = tmp;
+                    }
+                }
+#endif
+                VERBOSE(VB_PLAYBACK, QString("A audio timecode %1").arg(frameheader.timecode));
+                GetNVP()->AddAudioData((char *)strm, frameheader.packetlength, 
                                        frameheader.timecode);
             }
         }
@@ -1017,26 +1148,38 @@ void NuppelDecoder::GetFrame(int avignore)
             if (getrawframes)
                 StoreRawData(strm);
 
-            m_parent->AddTextData((char *)strm, frameheader.packetlength,
+            GetNVP()->AddTextData((char *)strm, frameheader.packetlength,
                                   frameheader.timecode, frameheader.comptype);
         }
     }
 
-    UpdateFramesPlayed();
+    framesRead = framesPlayed;
+
+    return true;
 }
 
-void NuppelDecoder::SeekReset(long long newKey, int skipFrames,
-                              bool needFlush)
+void NuppelDecoder::SeekReset(long long newKey, uint skipFrames,
+                              bool doFlush, bool discardFrames)
 {
-    if (mpa_codec && needFlush)
+    VERBOSE(VB_PLAYBACK, LOC +
+            QString("SeekReset(%1, %2, %3 flush, %4 discard)")
+            .arg(newKey).arg(skipFrames)
+            .arg((doFlush) ? "do" : "don't")
+            .arg((discardFrames) ? "do" : "don't"));
+    
+    DecoderBase::SeekReset(newKey, skipFrames, doFlush, discardFrames);
+
+    if (mpa_codec && doFlush)
         avcodec_flush_buffers(mpa_ctx);
 
-    while (skipFrames > 0)
+    if (discardFrames)
+        GetNVP()->DiscardVideoFrames(doFlush);
+
+    for (;(skipFrames > 0) && !ateof; skipFrames--)
     {
         GetFrame(0);
-        if (ateof)
-            break;
-        skipFrames--;
+        if (decoded_video_frame)
+            GetNVP()->DiscardVideoFrame(decoded_video_frame);
     }
 }
 
