@@ -38,23 +38,27 @@ class RawDataList
 class NuppelDecoder : public DecoderBase
 {
   public:
-    NuppelDecoder(NuppelVideoPlayer *parent, MythSqlDatabase *db,
-                  ProgramInfo *pginfo);
+    NuppelDecoder(NuppelVideoPlayer *parent, ProgramInfo *pginfo);
    ~NuppelDecoder();
 
     static bool CanHandle(char testbuf[2048]);
 
     int OpenFile(RingBuffer *rbuffer, bool novideo, char testbuf[2048]);
-    void GetFrame(int onlyvideo);
+    bool GetFrame(int onlyvideo);
 
+    // lastFrame is really (framesPlayed - 1) since we increment after getting
     bool isLastFrameKey(void) { return (lastKey == framesPlayed); }
-    void WriteStoredData(RingBuffer *rb, bool writevid);
+    void WriteStoredData(RingBuffer *rb, bool writevid, long timecodeOffset);
+    void ClearStoredData(void);
 
     long UpdateStoredFrameNum(long framenumber);
 
-    QString GetEncodingType(void);
+    QString GetEncodingType(void) const;
 
   private:
+    inline bool ReadFileheader(struct rtfileheader *fileheader);
+    inline bool ReadFrameheader(struct rtframeheader *frameheader);
+
     bool DecodeFrame(struct rtframeheader *frameheader,
                      unsigned char *lstrm, VideoFrame *frame);
     bool isValidFrametype(char type);
@@ -63,8 +67,8 @@ class NuppelDecoder : public DecoderBase
     void CloseAVCodec(void);
     void StoreRawData(unsigned char *strm);
 
-    void SeekReset(long long newKey = 0, int skipFrames = 0,
-                   bool needFlush = false);
+    void SeekReset(long long newKey = 0, uint skipFrames = 0,
+                   bool needFlush = false, bool discardFrames = false);
 
     friend int get_nuppel_buffer(struct AVCodecContext *c, AVFrame *pic);
     friend void release_nuppel_buffer(struct AVCodecContext *c, AVFrame *pic);
@@ -78,6 +82,9 @@ class NuppelDecoder : public DecoderBase
     int video_width, video_height, video_size;
     double video_frame_rate;
     int audio_samplerate;
+#ifdef WORDS_BIGENDIAN
+    int audio_bits_per_sample;
+#endif
 
     int ffmpeg_extradatasize;
     char *ffmpeg_extradata;
@@ -92,13 +99,12 @@ class NuppelDecoder : public DecoderBase
 
     int effdsp;
 
-    unsigned char *directbuf;
+    VideoFrame *directframe;
+    VideoFrame *decoded_video_frame;
 
     AVCodec *mpa_codec;
     AVCodecContext *mpa_ctx;
-    AVFrame *mpa_pic;
     AVPicture tmppicture;
-    AVPicture mpa_pic_tmp;
 
     bool directrendering;
 
