@@ -20,9 +20,20 @@ enum OSDFunctionalType
     kOSDFunctionalType_Default = 0,
     kOSDFunctionalType_PictureAdjust,
     kOSDFunctionalType_RecPictureAdjust,
-    kOSDFunctionalType_SmartForward
+    kOSDFunctionalType_SmartForward,
+    kOSDFunctionalType_TimeStretchAdjust,
+    kOSDFunctionalType_AudioSyncAdjust
 };
 
+struct StatusPosInfo
+{
+    QString desc;
+    int position;
+    bool progBefore;
+    bool progAfter;
+};
+
+
 class QImage;
 class TTFFont;
 class OSDSet;
@@ -40,13 +51,13 @@ class OSD : public QObject
 {
     Q_OBJECT
  public:
-    OSD(int width, int height, int framerate, const QString &font, 
-        const QString &ccfont, const QString &prefix, const QString &osdtheme,
-        int dispx, int dispy, int dispw, int disph);
+    OSD(const QRect &totalBounds,   int   frameRate,
+        const QRect &visibleBounds, float visibleAspect, float fontScaling);
    ~OSD(void);
 
     OSDSurface *Display(void);
 
+    void ClearAll(const QString &name);
     void ClearAllText(const QString &name);
     void SetText(const QString &name, QMap<QString, QString> &infoMap,
                          int length);
@@ -66,7 +77,7 @@ class OSD : public QObject
     void SetSettingsText(const QString &text, int length);
 
     void NewDialogBox(const QString &name, const QString &message, 
-                      QStringList &options, int length);
+                      QStringList &options, int length, int sel = 0);
     void DialogUp(const QString &name);
     void DialogDown(const QString &name);
     bool DialogShowing(const QString &name);
@@ -77,15 +88,19 @@ class OSD : public QObject
     void SetUpOSDClosedHandler(TV *tv);
 
     // position is 0 - 1000 
-    void StartPause(int position, bool fill, QString msgtext,
-                    QString slidertext, int displaytime,
+    void ShowStatus(int pos, bool fill, QString msgtext, QString desc,
+                    int displaytime,
                     int osdFunctionalType = kOSDFunctionalType_Default);
-    void UpdatePause(int position, QString slidertext);
-    void EndPause(void);
+    void ShowStatus(struct StatusPosInfo posInfo,
+                      bool fill, QString msgtext, int displaytime,
+                      int osdFunctionalType = kOSDFunctionalType_Default);
+    void UpdateStatus(struct StatusPosInfo posInfo);
+    void EndStatus(void);
 
     bool Visible(void);
 
-    void HideAll(void);
+    bool HideAll(void) { return HideAllExcept(QString::null); };
+    bool HideAllExcept(const QString &name);
     bool HideSet(const QString &name);
     bool HideSets(QStringList &name);
 
@@ -105,8 +120,9 @@ class OSD : public QObject
 
     int getTimeType(void) { return timeType; }
 
-    void Reinit(int width, int height, int frint, int dispx, int dispy, 
-                int dispw, int disph);
+    void Reinit(const QRect &totalBounds,   int   frameRate,
+                const QRect &visibleBounds,
+                float visibleAspect, float fontScaling);
 
     void SetFrameInterval(int frint);
 
@@ -118,6 +134,11 @@ class OSD : public QObject
     OSDListTreeType *ShowTreeMenu(const QString &name, 
                                   OSDGenericTree *treeToShow);
 
+    void DisableFade(void);
+    bool IsSetDisplaying(const QString &name);
+    bool HasSet(const QString &name);
+    QRect GetSubtitleBounds();
+
  private:
     void SetDefaults();
     TTFFont *LoadFont(QString name, int size); 
@@ -128,6 +149,7 @@ class OSD : public QObject
     bool LoadTheme();
     void normalizeRect(QRect &rect);
     QPoint parsePoint(QString text);
+    QColor parseColor(QString text);
     QRect parseRect(QString text);
 
     void RemoveSet(OSDSet *set);
@@ -144,17 +166,13 @@ class OSD : public QObject
     void parsePositionImage(OSDSet *container, QDomElement &element);
     void parseListTree(OSDSet *container, QDomElement &element);
 
-    QString fontname;
-    QString ccfontname;
-    QString fontSizeType;
+    QRect osdBounds;
+    int   frameint;
+    bool  needPillarBox;
 
-    int vid_width;
-    int vid_height;
-    int frameint;
-
-    QString fontprefix;
     QString themepath;
 
+    float wscale, fscale;
     float hmult, wmult;
     int xoffset, yoffset, displaywidth, displayheight;
 
