@@ -21,6 +21,7 @@
 
 #include <iostream>
 
+#include <qapplication.h>
 #include <qpixmap.h>
 #include <qpainter.h>
 #include <qimage.h>
@@ -34,12 +35,18 @@
 
 OSDGenericTree::OSDGenericTree(OSDGenericTree *parent, const QString &name, 
                                const QString &action, int check, 
-                               OSDTypeImage *image)
+                               OSDTypeImage *image, QString group)
               : GenericTree(name)
 {
     m_checkable = check;
     m_action = action;
     m_image = image;
+    m_parentButton = NULL;
+
+    if (group != "")
+        m_group = group;
+    else
+        m_group = action;
 
     if (!action.isEmpty() && !action.isNull())
         setSelectable(true);
@@ -87,6 +94,50 @@ OSDListTreeType::OSDListTreeType(const QString &name, const QRect &area,
     m_margin = 0;
 }
 
+void OSDListTreeType::Reinit(float wchange, float hchange, float wmult,
+                             float hmult)
+{
+    m_wmult = wmult;
+    m_hmult = hmult;
+
+    m_spacing = (int)(m_spacing * wchange);
+    m_margin = (int)(m_margin * wchange);
+
+    int width = (int)(m_totalarea.width() * wchange);
+    int height = (int)(m_totalarea.height() * hchange);
+    int x = (int)(m_totalarea.x() * wchange);
+    int y = (int)(m_totalarea.y() * hchange);
+
+    m_totalarea = QRect(x, y, width, height);
+
+    width = (int)(m_levelsize.width() * wchange);
+    height = (int)(m_levelsize.height() * hchange);
+    x = (int)(m_levelsize.x() * wchange);
+    y = (int)(m_levelsize.y() * hchange);
+
+    m_levelsize = QRect(x, y, width, height);
+
+    QPtrListIterator<OSDListBtnType> it(listLevels);
+    OSDListBtnType *child;
+
+    while ((child = it.current()) != 0)
+    {
+        child->Reinit(wchange, hchange, wmult, hmult);
+        ++it;
+    }
+}
+
+void OSDListTreeType::SetGroupCheckState(QString group, int newState)
+{
+    QPtrListIterator<OSDListBtnType> it(listLevels);
+    OSDListBtnType *child;
+    while ((child = it.current()) != 0)
+    {
+        child->SetGroupCheckState(group, newState);
+        ++it;
+    }
+}
+
 void OSDListTreeType::SetItemRegColor(const QColor& beg, const QColor& end,
                                       uint alpha)
 {
@@ -210,7 +261,8 @@ bool OSDListTreeType::HandleKeypress(QKeyEvent *e)
 
     bool handled = false;
     QStringList actions;
-    if (gContext->GetMainWindow()->TranslateKeyPress("qt", e, actions))
+    if (gContext->GetMainWindow()->TranslateKeyPress("TV Playback", e, 
+                                                     actions))
     {
         for (unsigned int i = 0; i < actions.size() && !handled; i++)
         {
@@ -264,13 +316,24 @@ bool OSDListTreeType::HandleKeypress(QKeyEvent *e)
                 }
                 else if (m_arrowAccel)
                 {
+                    SetGroupCheckState(currentpos->getGroup(),
+                                       OSDListBtnTypeItem::NotChecked);
+                    currentpos->getParentButton()->setChecked(
+                                       OSDListBtnTypeItem::FullChecked);
                     emit itemSelected(this, currentpos);
                 }
             }
-            else if (action == "ESCAPE" || action == "MENU")
+            else if (action == "ESCAPE" || action == "MENU" ||
+                     action == "CLEAROSD")
                 m_visible = false;
             else if (action == "SELECT")
+            {
+                SetGroupCheckState(currentpos->getGroup(),
+                                   OSDListBtnTypeItem::NotChecked);
+                currentpos->getParentButton()->setChecked(
+                                   OSDListBtnTypeItem::FullChecked);
                 emit itemSelected(this, currentpos);
+            }
             else
                 handled = false;
         }
@@ -295,6 +358,12 @@ void OSDListTreeType::Draw(OSDSurface *surface, int fade, int maxfade,
 void OSDListTreeType::FillLevelFromTree(OSDGenericTree *item, 
                                         OSDListBtnType *list)
 {
+    if (!list)
+    {
+        VERBOSE(VB_IMPORTANT, "OSDListTreeType::FillLevelFromTree() "
+                "called with no list. Ignoring call.");
+        return;
+    }
     list->Reset();
 
     QPtrList<GenericTree> *itemlist = item->getAllChildren();
@@ -313,7 +382,9 @@ void OSDListTreeType::FillLevelFromTree(OSDGenericTree *item,
                                          (child->childCount() > 0));
         if (osdchild->getCheckable() == 1)
             newitem->setChecked(OSDListBtnTypeItem::FullChecked);
+        newitem->setGroup(osdchild->getGroup());
         newitem->setData(osdchild);
+        osdchild->setParentButton(newitem);
 
         ++it;
     }
@@ -386,6 +457,41 @@ OSDListBtnType::~OSDListBtnType()
     Reset();
 }
 
+void OSDListBtnType::Reinit(float wchange, float hchange, float wmult,
+                            float hmult)
+{
+    m_wmult = wmult;
+    m_hmult = hmult;
+
+    m_itemHeight = (int)(m_itemHeight * hchange);
+    m_itemSpacing = (int)(m_itemSpacing * wchange);
+    m_itemMargin = (int)(m_itemMargin * wchange);
+
+    int width = (int)(m_rect.width() * wchange);
+    int height = (int)(m_rect.height() * hchange);
+    int x = (int)(m_rect.x() * wchange);
+    int y = (int)(m_rect.y() * hchange);
+
+    m_rect = QRect(x, y, width, height);
+
+    Init();
+
+    OSDListBtnTypeItem* item = 0;
+    for (item = m_itemList.first(); item; item = m_itemList.next()) {
+        item->Reinit(wchange, hchange, wmult, hmult);
+    }
+
+}
+
+void OSDListBtnType::SetGroupCheckState(QString group, int newState)
+{
+    OSDListBtnTypeItem* item = 0;
+    for (item = m_itemList.first(); item; item = m_itemList.next()) {
+        if (item->getGroup() == group)
+            item->setChecked((OSDListBtnTypeItem::CheckState)newState);
+    }
+}
+
 void OSDListBtnType::SetItemRegColor(const QColor& beg, const QColor& end, 
                                      uint alpha)
 {
@@ -656,7 +762,7 @@ void OSDListBtnType::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
     OSDListBtnTypeItem *it = m_itemList.current();
     while (it && (y - m_rect.y()) <= (m_contentsRect.height() - m_itemHeight)) 
     {
-        it->paint(surface, font, fade, maxfade, m_rect.x(), y);
+        it->paint(surface, font, fade, maxfade, m_rect.x()+ xoff, y + yoff);
 
         y += m_itemHeight + m_itemSpacing;
 
@@ -667,29 +773,27 @@ void OSDListBtnType::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
     {
         if (m_showUpArrow)
             m_upArrowActPix.Draw(surface, fade, maxfade, 
-                                 m_rect.x() + m_arrowsRect.x(),
-                                 m_rect.y() + m_arrowsRect.y());
+                                 m_rect.x() + m_arrowsRect.x() + xoff,
+                                 m_rect.y() + m_arrowsRect.y() + yoff);
         else
             m_upArrowRegPix.Draw(surface, fade, maxfade,
-                                 m_rect.x() + m_arrowsRect.x(),
-                                 m_rect.y() + m_arrowsRect.y());
+                                 m_rect.x() + m_arrowsRect.x() + xoff,
+                                 m_rect.y() + m_arrowsRect.y() + yoff);
         if (m_showDnArrow)
             m_dnArrowActPix.Draw(surface, fade, maxfade,
                                  m_rect.x() + m_arrowsRect.x() +
-                                 m_upArrowRegPix.width() + m_itemMargin,
-                                 m_rect.y() + m_arrowsRect.y());
+                                 m_upArrowRegPix.width() + m_itemMargin + xoff,
+                                 m_rect.y() + m_arrowsRect.y() + yoff);
         else
             m_dnArrowRegPix.Draw(surface, fade, maxfade,
                                  m_rect.x() + m_arrowsRect.x() +
-                                 m_upArrowRegPix.width() + m_itemMargin,
-                                 m_rect.y() + m_arrowsRect.y());
+                                 m_upArrowRegPix.width() + m_itemMargin + xoff,
+                                 m_rect.y() + m_arrowsRect.y() + yoff);
     }
 }
 
 void OSDListBtnType::Init()
 {
-    qApp->lock();
-
     int sz1 = m_fontActive->Size() * 3 / 2;
     int sz2 = m_fontInactive->Size() * 3 / 2;
     m_itemHeight = QMAX(sz1, sz2) + (int)(2 * m_itemMargin);
@@ -725,114 +829,14 @@ void OSDListBtnType::Init()
     LoadPixmap(m_checkFullPix, "check-full");
     LoadPixmap(m_arrowPix, "arrow");
 
-    int itemWidth = (m_rect.width() / 2) * 2;
+    uint itemWidth = (m_rect.width() + 1) & (~1);
 
-    QImage img(itemWidth, m_itemHeight, 32);
-    img.setAlphaBuffer(true);
-
-    for (int y = 0; y < img.height(); y++) 
-    {
-        for (int x = 0; x < img.width(); x++) 
-        {
-            uint *p = (uint *)img.scanLine(y) + x;
-            *p = qRgba(0, 0, 0, m_itemRegAlpha);
-        }
-    }
-
-    {
-        float rstep = float(m_itemRegEnd.red() - m_itemRegBeg.red()) / 
-                      float(m_itemHeight);
-        float gstep = float(m_itemRegEnd.green() - m_itemRegBeg.green()) / 
-                      float(m_itemHeight);
-        float bstep = float(m_itemRegEnd.blue() - m_itemRegBeg.blue()) / 
-                      float(m_itemHeight);
-
-        QPixmap tmpRegPix = QPixmap(img);
-        QPainter p(&tmpRegPix);
-
-        float r = m_itemRegBeg.red();
-        float g = m_itemRegBeg.green();
-        float b = m_itemRegBeg.blue();
-        for (int y = 0; y < img.height(); y++) 
-        {
-            QColor c((int)r, (int)g, (int)b);
-            p.setPen(c);
-            p.drawLine(0, y, img.width(), y);
-            r += rstep;
-            g += gstep;
-            b += bstep;
-        }
-        p.setPen(Qt::black);
-        p.drawLine(0, 0, 0, img.height() - 1);
-        p.drawLine(0, 0, img.width() - 1, 0);
-        p.drawLine(0, img.height() - 1, img.width() - 1, img.height() - 1);
-        p.drawLine(img.width() - 1, 0, img.width() - 1, img.height() - 1);
-        p.end();
-
-        QImage tmpImg = tmpRegPix.convertToImage();
-        m_itemRegPix.LoadFromQImage(tmpImg);
-    }   
-
-    {
-        float rstep = float(m_itemSelEnd.red() - m_itemSelBeg.red()) /
-                      float(m_itemHeight);
-        float gstep = float(m_itemSelEnd.green() - m_itemSelBeg.green()) / 
-                      float(m_itemHeight);
-        float bstep = float(m_itemSelEnd.blue() - m_itemSelBeg.blue()) /
-                      float(m_itemHeight);
-
-        QPixmap tmpSelInactPix = QPixmap(img);
-        QPainter p(&tmpSelInactPix);
-
-        float r = m_itemSelBeg.red();
-        float g = m_itemSelBeg.green();
-        float b = m_itemSelBeg.blue();
-        for (int y = 0; y < img.height(); y++) 
-        {
-            QColor c((int)r, (int)g, (int)b);
-            p.setPen(c);
-            p.drawLine(0, y, img.width(), y);
-            r += rstep;
-            g += gstep;
-            b += bstep;
-        }
-        p.setPen(Qt::black);
-        p.drawLine(0, 0, 0, img.height() - 1);
-        p.drawLine(0, 0, img.width() - 1, 0);
-        p.drawLine(0, img.height() - 1, img.width() - 1, img.height() - 1);
-        p.drawLine(img.width() - 1, 0, img.width() - 1, img.height() - 1);
-        p.end();
-
-        QImage tmpImg = tmpSelInactPix.convertToImage();
-        m_itemSelInactPix.LoadFromQImage(tmpImg);
-
-        img.setAlphaBuffer(false);
-        
-        QPixmap tmpSelActPix = QPixmap(img);
-        p.begin(&tmpSelActPix);
-
-        r = m_itemSelBeg.red();
-        g = m_itemSelBeg.green();
-        b = m_itemSelBeg.blue();
-        for (int y = 0; y < img.height(); y++) 
-        {
-            QColor c((int)r, (int)g, (int)b);
-            p.setPen(c);
-            p.drawLine(0, y, img.width(), y);
-            r += rstep;
-            g += gstep;
-            b += bstep;
-        }
-        p.setPen(Qt::black);
-        p.drawLine(0, 0, 0, img.height() - 1);
-        p.drawLine(0, 0, img.width() - 1, 0);
-        p.drawLine(0, img.height() - 1, img.width() - 1, img.height() - 1);
-        p.drawLine(img.width() - 1, 0, img.width() - 1, img.height() - 1);
-        p.end();
-
-        tmpImg = tmpSelActPix.convertToImage();
-        m_itemSelActPix.LoadFromQImage(tmpImg);
-    }
+    InitItem(m_itemRegPix,      itemWidth,    m_itemHeight,
+             m_itemRegBeg,      m_itemRegEnd, m_itemRegAlpha);
+    InitItem(m_itemSelInactPix, itemWidth,    m_itemHeight,
+             m_itemSelBeg,      m_itemSelEnd, m_itemRegAlpha);
+    InitItem(m_itemSelActPix,   itemWidth,    m_itemHeight,
+             m_itemSelBeg,      m_itemSelEnd, 255);
 
     if (m_itemList.count() > m_itemsVisible && m_showScrollArrows)
         m_showDnArrow = true;
@@ -840,15 +844,48 @@ void OSDListBtnType::Init()
         m_showDnArrow = false;
 
     m_initialized = true;
+}
 
-    qApp->unlock();
+void OSDListBtnType::InitItem(
+    OSDTypeImage &osdImg, uint width, uint height,
+    QColor beg, QColor end, int alpha)
+{
+    float rstep = ((float) (end.red()   - beg.red()))   / m_itemHeight;
+    float gstep = ((float) (end.green() - beg.green())) / m_itemHeight;
+    float bstep = ((float) (end.blue()  - beg.blue()))  / m_itemHeight;
+
+    unsigned char *data = new unsigned char[4 * width * height];
+    uint32_t *ptr = (uint32_t*) data;
+
+    uint black = qRgba(0,0,0,alpha);
+    for (uint x = 0; x < width; x++, ptr++)
+        *ptr = black;
+    for (uint y = 1; y < height - 1; y++) 
+    {
+        int r = (int) (beg.red()   + (y * rstep));
+        int g = (int) (beg.green() + (y * gstep));
+        int b = (int) (beg.blue()  + (y * bstep));
+        uint32_t color = qRgba(r,g,b,alpha);
+        *ptr = black; ptr++;
+        for (uint x = 1; x < width - 1; x++, ptr++)
+            *ptr = color;
+        *ptr = black; ptr++;
+    }
+    for (uint x = 0; x < width; x++, ptr++)
+        *ptr = black;
+
+    {
+        QImage img(data, width, height, 32, NULL, 65536 * 65536,
+                   QImage::LittleEndian);
+        img.setAlphaBuffer(alpha<255);
+        osdImg.LoadFromQImage(img);
+    }
+    delete[] data;
 }
 
 void OSDListBtnType::LoadPixmap(OSDTypeImage& pix, const QString& fileName)
 {
-    QString file = "/usr/local/share/mythtv/themes/default/lb-" + fileName + 
-                   ".png";
-    
+    QString file = gContext->GetThemesParentDir() + "default/lb-" + fileName + ".png";
     pix.LoadImage(file, m_wmult, m_hmult);
 }
 
@@ -858,11 +895,6 @@ OSDListBtnTypeItem::OSDListBtnTypeItem(OSDListBtnType* lbtype,
                                        OSDTypeImage *pixmap, bool checkable,
                                        bool showArrow, CheckState state)
 {
-    if (!lbtype) {
-        VERBOSE(VB_IMPORTANT, "OSDListBtnTypeItem: trying to creating item without parent");
-        exit(-16);
-    }
-    
     m_parent    = lbtype;
     m_text      = text;
     m_pixmap    = pixmap;
@@ -893,7 +925,7 @@ OSDListBtnTypeItem::OSDListBtnTypeItem(OSDListBtnType* lbtype,
     else
         m_checkRect = QRect(0,0,0,0);
 
-    if (showArrow) 
+    if (m_showArrow) 
         m_arrowRect = QRect(width - aw - margin, (height - ah)/2,
                             aw, ah);
     else
@@ -912,7 +944,7 @@ OSDListBtnTypeItem::OSDListBtnTypeItem(OSDListBtnType* lbtype,
                        0,
                        width - 2*margin -
                        (m_checkable ? m_checkRect.width() + margin : 0) -
-                       (showArrow ? m_arrowRect.width() + margin : 0) -
+                       (m_showArrow ? m_arrowRect.width() + margin : 0) -
                        (m_pixmap ? m_pixmapRect.width() + margin : 0),
                        height);
 
@@ -1018,3 +1050,38 @@ void OSDListBtnTypeItem::paint(OSDSurface *surface, TTFFont *font,
     font->DrawString(surface, tr.x(), tr.y(), m_text, tr.right(), tr.bottom());
 }
 
+void OSDListBtnTypeItem::Reinit(float wchange, float hchange, 
+                                float wmult, float hmult)
+{
+    (void)wmult;
+    (void)hmult;
+
+    int width = (int)(m_checkRect.width() * wchange);
+    int height = (int)(m_checkRect.height() * hchange);
+    int x = (int)(m_checkRect.x() * wchange);
+    int y = (int)(m_checkRect.y() * hchange);
+
+    m_checkRect = QRect(x, y, width, height);
+
+    width = (int)(m_pixmapRect.width() * wchange);
+    height = (int)(m_pixmapRect.height() * hchange);
+    x = (int)(m_pixmapRect.x() * wchange);
+    y = (int)(m_pixmapRect.y() * hchange);
+
+    m_pixmapRect = QRect(x, y, width, height);
+
+    width = (int)(m_textRect.width() * wchange);
+    height = (int)(m_textRect.height() * hchange);
+    x = (int)(m_textRect.x() * wchange);
+    y = (int)(m_textRect.y() * hchange);
+
+    m_textRect = QRect(x, y, width, height);
+
+    width = (int)(m_arrowRect.width() * wchange);
+    height = (int)(m_arrowRect.height() * hchange);
+    x = (int)(m_arrowRect.x() * wchange);
+    y = (int)(m_arrowRect.y() * hchange);
+
+    m_arrowRect = QRect(x, y, width, height);
+}
+
