@@ -132,6 +132,12 @@ OSDSet::OSDSet(const OSDSet &other)
 }
 
 OSDSet::~OSDSet()
+{
+    Clear();
+    delete allTypes;
+}
+
+void OSDSet::Clear()
 {
     vector<OSDType *>::iterator i = allTypes->begin();
     for (; i != allTypes->end(); i++)
@@ -140,7 +146,7 @@ OSDSet::~OSDSet()
         if (type)
             delete type;
     }
-    delete allTypes;
+    allTypes->clear();
 }
  
 void OSDSet::AddType(OSDType *type)
@@ -195,7 +201,7 @@ void OSDSet::Reinit(int screenwidth, int screenheight, int xoff, int yoff,
         }
         else if (OSDTypeImage *item = dynamic_cast<OSDTypeImage*>(type))
         {
-            item->Reinit(wmult, hmult);
+            item->Reinit(wchange, hchange, wmult, hmult);
         }
         else if (OSDTypeBox *item = dynamic_cast<OSDTypeBox*>(type))
         {
@@ -426,6 +432,7 @@ void OSDSet::Draw(OSDSurface *surface, bool actuallydraw)
 
 OSDType::OSDType(const QString &name)
 {
+    m_hidden = false;
     m_name = name;
 }
 
@@ -709,8 +716,6 @@ OSDTypeImage::OSDTypeImage(const QString &name, const QString &filename,
 
     m_scalew = scalew;
     m_scaleh = scaleh;
-    m_wmult = wmult;
-    m_hmult = hmult;
 
     LoadImage(filename, wmult, hmult, scalew, scaleh);
 }
@@ -796,18 +801,15 @@ void OSDTypeImage::SetName(const QString &name)
     m_name = name;
 }
 
-void OSDTypeImage::Reinit(float wmult, float hmult)
+void OSDTypeImage::Reinit(float wchange, float hchange, float wmult, float hmult)
 {
-    int x = (int)(m_displaypos.x() * wmult / m_wmult);
-    int y = (int)(m_displaypos.y() * hmult / m_hmult);
+    int x = (int)(m_displaypos.x() * wchange);
+    int y = (int)(m_displaypos.y() * hchange);
 
     m_displaypos.setX(x);
     m_displaypos.setY(y);
 
-    m_wmult = wmult;
-    m_hmult = hmult;
-
-    LoadImage(m_filename, m_wmult, m_hmult, m_scalew, m_scaleh);
+    LoadImage(m_filename, wmult, hmult, m_scalew, m_scaleh);
 }
 
 void OSDTypeImage::LoadImage(const QString &filename, float wmult, float hmult,
@@ -883,8 +885,8 @@ void OSDTypeImage::LoadFromQImage(const QImage &img)
 
     m_isvalid = true;
 
-    int imwidth = (img.width() / 2) * 2;
-    int imheight = (img.height() / 2) * 2;
+    int imwidth = (img.width() + 1) & ~1;
+    int imheight = (img.height() + 1) & ~1;
 
     m_yuv = new unsigned char[imwidth * imheight * 3 / 2];
     m_ybuffer = m_yuv;
@@ -894,35 +896,22 @@ void OSDTypeImage::LoadFromQImage(const QImage &img)
     m_alpha = new unsigned char[imwidth * imheight];
 
     rgb32_to_yuv420p(m_ybuffer, m_ubuffer, m_vbuffer, m_alpha, img.bits(),
-                     imwidth, imheight, img.width());
+                     img.width(), img.height(), img.bytesPerLine() / 4);
 
     m_imagesize = QRect(0, 0, imwidth, imheight);
 }
 
-void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff, 
-                        int yoff)
+void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade,
+                        int xoff, int yoff)
 {
+    if (m_hidden)
+        return;
+
     if (!m_isvalid)
         return;
 
-    unsigned char *dest, *destalpha, *src, *srcalpha;
-    unsigned char *udest, *vdest, *usrc, *vsrc;
-    int alpha, iwidth, drawwidth;
-
-    iwidth = drawwidth = m_imagesize.width();
-    int drawheight = m_imagesize.height();
-
-    if (m_drawwidth >= 0)
-        drawwidth = m_drawwidth;
-
-    int ystart = m_displaypos.y();
-    int xstart = m_displaypos.x();
-
-    xstart += xoff;
-    ystart += yoff;
-
-    ystart = (ystart / 2) * 2;
-    xstart = ((xstart + 1) / 2) * 2;
+    int xstart = ((m_displaypos.x() + xoff + 1) / 2) * 2;
+    int ystart = ((m_displaypos.y() + yoff)     / 2) * 2;
 
     int startline = 0;
     int startcol = 0;
@@ -939,27 +928,24 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
         xstart = 0;
     }
 
-    if (drawheight + ystart > surface->height)
-        drawheight = surface->height - ystart - 1;
-    if (drawwidth + xstart > surface->width)
-        drawwidth = surface->width - xstart - 1;
+    int iwidth     = m_imagesize.width();
+    int drawwidth  = (m_drawwidth >= 0) ? m_drawwidth : m_imagesize.width();
+    int drawheight = m_imagesize.height();
 
-    if (drawwidth == 0 || drawheight == 0)
+    drawwidth = (drawwidth + xstart > surface->width) ?
+        surface->width - xstart - 1 : drawwidth;
+
+    drawheight = (drawheight + ystart > surface->height) ?
+        surface->height - ystart - 1 : drawheight;
+ 
+    if ((drawwidth <= 0) || (drawheight <= 0))
         return;
 
     QRect destRect = QRect(xstart, ystart, drawwidth, drawheight);
-    bool needblend = false;
+    bool needblend = m_onlyusefirst || surface->IntersectsDrawn(destRect);
 
-    if (m_onlyusefirst || surface->IntersectsDrawn(destRect))
-        needblend = true;
     surface->AddRect(destRect);
     
-    int ysrcwidth;
-    int ydestwidth;
-
-    int uvsrcwidth;
-    int uvdestwidth;
-
     int alphamod = 255;
 
     if (maxfade > 0 && fade >= 0)
@@ -969,17 +955,17 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
     {
         for (int y = startline; y < drawheight; y++)
         {
-            ysrcwidth = y * iwidth;
-            ydestwidth = (y + ystart - startline) * surface->width;
+            int ysrcwidth = y * iwidth;
+            int ydestwidth = (y + ystart - startline) * surface->width;
 
             memcpy(surface->y + xstart + ydestwidth,
                    m_ybuffer + startcol + ysrcwidth, drawwidth);
 
-            destalpha = surface->alpha + xstart + ydestwidth;
+            unsigned char *destalpha = surface->alpha + xstart + ydestwidth;
 
             for (int x = startcol; x < drawwidth; x++)
             {
-                alpha = *(m_alpha + x + ysrcwidth);
+                int alpha = *(m_alpha + x + ysrcwidth);
   
                 if (alpha == 0)
                     *destalpha = 0;
@@ -990,20 +976,18 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
             }
         }
 
-        iwidth /= 2;
-        drawwidth /= 2;
+        iwidth     /= 2;
+        drawwidth  /= 2;
         drawheight /= 2;
-
-        ystart /= 2;
-        xstart /= 2;
-
-        startline /= 2;
-        startcol /= 2;
+        ystart     /= 2;
+        xstart     /= 2;
+        startline  /= 2;
+        startcol   /= 2;
 
         for (int y = startline; y < drawheight; y++)
         {
-            uvsrcwidth = y * iwidth;
-            uvdestwidth = (y + ystart - startline) * (surface->width / 2);
+            int uvsrcwidth = y * iwidth;
+            int uvdestwidth = (y + ystart - startline) * (surface->width / 2);
 
             memcpy(surface->u + xstart + uvdestwidth,
                    m_ubuffer + startcol + uvsrcwidth, drawwidth);
@@ -1014,27 +998,22 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
         return;
     }
 
-    int startingx = startcol;
-    if (m_onlyusefirst)
-        startingx = 0;
+    int ysrcwidth   = startline * iwidth;
+    int uvsrcwidth  = ysrcwidth  / 4;
+    int startingx   = (m_onlyusefirst) ? 0 : startcol;
 
-    ysrcwidth = startline * iwidth;
-    ydestwidth = ystart * surface->width;
+    unsigned char *src       = m_ybuffer      + ysrcwidth  + startingx;
+    unsigned char *usrc      = m_ubuffer      + uvsrcwidth + startingx / 2;
+    unsigned char *vsrc      = m_vbuffer      + uvsrcwidth + startingx / 2;
+    unsigned char *srcalpha  = m_alpha        + ysrcwidth  + startingx;
 
-    dest = surface->y + xstart + ydestwidth;
-    destalpha = surface->alpha + xstart + ydestwidth;
-    src = m_ybuffer + ysrcwidth + startingx;
+    int ydestwidth  = ystart * surface->width;
+    int uvdestwidth = ydestwidth / 4;
 
-    srcalpha = m_alpha + ysrcwidth + startingx;
-
-    uvsrcwidth = ysrcwidth / 4;
-    uvdestwidth = ydestwidth / 4;
-
-    udest = surface->u + xstart / 2 + uvdestwidth;
-    usrc = m_ubuffer + uvsrcwidth + startingx / 2;
-
-    vdest = surface->v + xstart / 2 + uvdestwidth;
-    vsrc = m_vbuffer + uvsrcwidth + startingx / 2;
+    unsigned char *dest      = surface->y     + xstart     + ydestwidth;
+    unsigned char *udest     = surface->u     + xstart / 2 + uvdestwidth;
+    unsigned char *vdest     = surface->v     + xstart / 2 + uvdestwidth;
+    unsigned char *destalpha = surface->alpha + xstart     + ydestwidth;
 
     if (m_onlyusefirst)
         (surface->blendcolumnfunc) (src, usrc, vsrc, srcalpha, iwidth, dest,
@@ -1052,7 +1031,7 @@ void OSDTypeImage::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
                                     surface->pow_lut);
 }
 
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 OSDTypePosSlider::OSDTypePosSlider(const QString &name,
                                    const QString &filename,
@@ -1080,7 +1059,7 @@ void OSDTypePosSlider::Reinit(float wchange, float hchange, float wmult,
 
     m_displayrect = QRect(x, y, width, height);
 
-    OSDTypeImage::Reinit(wmult, hmult);
+    OSDTypeImage::Reinit(wchange, hchange, wmult, hmult);
 }
 
 void OSDTypePosSlider::SetPosition(int pos)
@@ -1129,7 +1108,7 @@ void OSDTypeFillSlider::Reinit(float wchange, float hchange, float wmult,
 
     m_displayrect = QRect(x, y, width, height);
 
-    OSDTypeImage::Reinit(wmult, hmult);
+    OSDTypeImage::Reinit(wchange, hchange, wmult, hmult);
 }
 
 void OSDTypeFillSlider::SetPosition(int pos)
@@ -1236,9 +1215,6 @@ void OSDTypeEditSlider::Reinit(float wchange, float hchange, float wmult,
     if (m_ralpha)
         delete [] m_ralpha;
 
-    m_wmult = wmult;
-    m_hmult = hmult;
-
     LoadImage(m_redname, wmult, hmult, m_scalew, m_scaleh);
     if (m_isvalid)
     {
@@ -1312,14 +1288,6 @@ void OSDTypeEditSlider::Draw(OSDSurface *surface, int fade, int maxfade,
     ystart = (ystart / 2) * 2;
     xstart = (xstart / 2) * 2;
 
-    if (height + ystart > surface->height)
-        height = surface->height - ystart - 1;
-    if (width + xstart > surface->width)
-        width = surface->width - xstart - 1;
-
-    if (width == 0 || height == 0)
-        return;
-
     int startline = 0;
     int startcol = 0;
 
@@ -1335,6 +1303,14 @@ void OSDTypeEditSlider::Draw(OSDSurface *surface, int fade, int maxfade,
         xstart = 0;
     }
 
+    if (height + ystart > surface->height)
+        height = surface->height - ystart - 1;
+    if (width + xstart > surface->width)
+        width = surface->width - xstart - 1;
+
+    if (width == 0 || height == 0)
+        return;
+
     QRect destRect = QRect(xstart, ystart, width, height);
     surface->AddRect(destRect);
  
@@ -1577,6 +1553,15 @@ void OSDTypePositionRectangle::Draw(OSDSurface *surface, int fade, int maxfade,
     int xstart = rect.left() + xoff;
     int xend = rect.right() + xoff;
 
+    if (xstart < 0)
+        xstart = 0;
+    if (xend > surface->width)
+        xend = surface->width;
+    if (ystart < 0)
+        ystart = 0;
+    if (yend > surface->height)
+        yend = surface->height;
+
     int height = yend - ystart + 1, width = xend - xstart + 1;
 
     QRect destRect = QRect(xstart, ystart, width, height);
@@ -1654,7 +1639,7 @@ OSDTypePositionImage::~OSDTypePositionImage()
 void OSDTypePositionImage::Reinit(float wchange, float hchange, float wmult, 
                                   float hmult)
 {
-    OSDTypeImage::Reinit(wmult, hmult);
+    OSDTypeImage::Reinit(wchange, hchange, wmult, hmult);
 
     for (int i = 0; i < m_numpositions; i++)
     {
