@@ -21,7 +21,7 @@
 #include <cassert>
 
 #include "progfind.h"
-#include "infodialog.h"
+#include "proglist.h"
 #include "infostructs.h"
 #include "programinfo.h"
 #include "oldsettings.h"
@@ -31,12 +31,15 @@
 using namespace std;
 
 #include "libmyth/mythcontext.h"
+#include "libmyth/mythdbcon.h"
 
 void RunProgramFind(bool thread, bool ggActive)
 {
     if (thread)
         qApp->lock();
 
+    gContext->addCurrentLocation("ProgFinder");
+
     // Language specific progfinder, if needed
 
     ProgFinder *programFind = NULL;
@@ -62,6 +65,8 @@ void RunProgramFind(bool thread, bool ggActive)
 
     delete programFind;
 
+    gContext->removeCurrentLocation();
+
     return;
 }
 
@@ -78,10 +83,10 @@ ProgFinder::ProgFinder(MythMainWindow *parent, const char *name, bool gg)
 void ProgFinder::Initialize(void)
 {
     running = true;
-    m_db = QSqlDatabase::database();
-    
     
     allowkeypress = true;
+    inFill = false;
+    needFill = false;
 
     baseDir = gContext->GetInstallPrefix();
 
@@ -117,7 +122,6 @@ void ProgFinder::Initialize(void)
 
     inSearch = 0;
     pastInitial = false;
-    recordingCount = 0;
 
     initData = new QString[(int)(searchCount*showsPerListing)];
     gotInitData = new int[searchCount];
@@ -134,16 +138,17 @@ void ProgFinder::Initialize(void)
     update_Timer->start((int)(100));
 
     setNoErase();
+    gContext->addListener(this);
 
     showInfo = false;
 }
 
 ProgFinder::~ProgFinder()
 {
+    gContext->removeListener(this);
+
     if (inSearch > 0)
         delete [] progData;
-    if (inSearch == 2)
-            delete [] showData;
     delete [] searchData;
     delete [] initData;
 
@@ -158,6 +163,7 @@ void ProgFinder::keyPressEvent(QKeyEvent *e)
 {
     if (!allowkeypress)
         return;
+    allowkeypress = false;
 
     bool handled = false;
 
@@ -183,6 +189,10 @@ void ProgFinder::keyPressEvent(QKeyEvent *e)
             pageDown();
         else if (action == "SELECT" || action == "INFO")
             select();
+        else if (action == "UPCOMING")
+            upcoming();
+        else if (action == "DETAILS")
+            details();
         else if (action == "MENU" || action == "ESCAPE")
             escape();
         else if (action == "TOGGLERECORD")
@@ -195,6 +205,19 @@ void ProgFinder::keyPressEvent(QKeyEvent *e)
 
     if (!handled)
         MythDialog::keyPressEvent(e);
+
+    if (needFill && inSearch == 2)
+    {
+        do
+        {
+            needFill = false;
+            ProgramInfo *curPick = showData[curShow];
+            if (curPick)
+                selectShowData(curPick->title, curShow);
+        } while (needFill);
+    }
+
+    allowkeypress = true;
 }
 
 void ProgFinder::updateBackground(void)
@@ -220,17 +243,16 @@ void ProgFinder::updateBackground(void)
 
 void ProgFinder::paintEvent(QPaintEvent *e)
 {
+    if (inFill)
+        return;
+
     QRect r = e->rect();
     QPainter p(this);
 
     if (r.intersects(listRect))
-    {
         updateList(&p);
-    }
     if (r.intersects(infoRect))
-    {
         updateInfo(&p);
-    }
 }
 
 void ProgFinder::updateList(QPainter *p)
@@ -328,79 +350,27 @@ void ProgFinder::updateInfo(QPainter *p)
     }
     else if (inSearch == 2)
     {
-        QString title = "";
-        QString subtitle = "";
-        QString timedate = "";
-        QString description = "";
-        QString channum = "";
-        QString channame = "";
-        QString recording = "";
-        QString seriesid = "";
-        QString programid = "";
-
-        if (channelFormat.contains("<num>"))
-            channum = showData[curShow].channelNum;
-        if (channelFormat.contains("<sign>"))
-            channame = showData[curShow].channelCallsign;
-        else if (channelFormat.contains("<name>"))
-            channame = showData[curShow].channelName;
-        title = progData[curProgram];
-        timedate = showData[curShow].startDisplay + " - " +
-                   showData[curShow].endDisplay;
-        if (showData[curShow].subtitle.stripWhiteSpace().length() > 0)
-            subtitle = "\"" + showData[curShow].subtitle + "\"";
-        else
-            subtitle = "";
-        description = showData[curShow].description;
-        recording = showData[curShow].recText;
-        seriesid = showData[curShow].seriesid;
-        programid = showData[curShow].programid;
-
-        if (gotInitData[curSearch] == 1)
-        {
-            title = tr("No Programs");
-            description = tr("There are no available programs under this "
-                             "search. Please select another search.");
-        }
-
         container = theme->GetSet("program_info");
         if (container)
         {
-            UITextType *type = (UITextType *)container->GetType("title");
-            if (type)
-                type->SetText(title);
-
-            type = (UITextType *)container->GetType("subtitle");
-            if (type)
-                type->SetText(subtitle);
-
-            type = (UITextType *)container->GetType("timedate");
-            if (type)
-                type->SetText(timedate);
-
-            type = (UITextType *)container->GetType("description");
-            if (type)
-                type->SetText(description);
-
-            type = (UITextType *)container->GetType("channelnum");
-            if (type)
-                type->SetText(channum);
-
-            type = (UITextType *)container->GetType("channelname");
-            if (type)
-                type->SetText(channame);
-
-            type = (UITextType *)container->GetType("recordingstatus");
-            if (type)
-                type->SetText(recording);
-
-            type = (UITextType *)container->GetType("seriesid");
-            if (type)
-                type->SetText(seriesid);
-
-            type = (UITextType *)container->GetType("programid");
-            if (type)
-                type->SetText(programid);
+            container->ClearAllText();
+            if (gotInitData[curSearch] == 1)
+            {
+                UITextType *type = (UITextType *)container->GetType("title");
+                if (type)
+                    type->SetText(tr("No Programs"));
+                type = (UITextType *)container->GetType("description");
+                if (type)
+                    type->SetText(tr("There are no available programs under "
+                                     "this search. Please select another "
+                                     "search."));
+            }
+            else
+            {
+                QMap<QString, QString> infoMap;
+                showData[curShow]->ToMap(infoMap);
+                container->SetText(infoMap);
+            }
         }
     }
 
@@ -422,15 +392,14 @@ void ProgFinder::escape()
 
 void ProgFinder::showGuide()
 {
-    if(!ggActive)
+    if (!ggActive)
     {
         QString startchannel = gContext->GetSetting("DefaultTVChannel");
         if (startchannel == "")
             startchannel = "3";
-
-        RunProgramGuide(startchannel);
+        uint startchanid = 0;
+        RunProgramGuide(startchanid, startchannel);
     }
-
                 
     running = false;
     hide();
@@ -479,90 +448,25 @@ void ProgFinder::parseContainer(QDomElement &element)
 
 void ProgFinder::getInfo(bool toggle)
 {
-    int rectype = 0;
-    QString data = "";
     if (inSearch == 2)
     {
         showInfo = 1;
-        ProgramInfo *curPick = ProgramInfo::GetProgramAtDateTime(m_db,
-                                                                 curChannel,
-                                                            curDateTime + "50");
+        ProgramInfo *curPick = showData[curShow];
 
         if (curPick)
         {
             if (toggle)
-            {
-                curPick->ToggleRecord(m_db);
-            }
+                curPick->ToggleRecord();
             else
-            {
-                if ((gContext->GetNumSetting("AdvancedRecord", 0)) ||
-                    (curPick->GetProgramRecordingStatus(m_db) > kAllRecord))
-                {
-                    ScheduledRecording record;
-                    record.loadByProgram(m_db, curPick);
-                    record.exec(m_db);
-                }
-                else
-                {
-                    InfoDialog diag(curPick, gContext->GetMainWindow(),
-                                    "Program Info");
-                    diag.exec();
-                }
-            }
+                curPick->EditRecording();
         }
         else
             return;
 
         showInfo = 0;
 
-        curPick->GetProgramRecordingStatus(m_db);
+        selectShowData(curPick->title, curShow);
 
-        getRecordingInfo();
-
-        for (int i = 0; i < showCount; i++)
-        {
-            rectype = checkRecordingStatus(i);
-            showData[i].recording = rectype;
-
-            switch (rectype)
-            {
-                case kSingleRecord:
-                    data = tr("Recording just this showing");
-                    break;
-                case kOverrideRecord:
-                    data = tr("Recording this showing with override options");
-                    break;
-                case kFindOneRecord:
-                    data = tr("Recording one showing of this program");
-                    break;
-                case kTimeslotRecord:
-                    data = tr("Recording every day when shown in this timeslot");
-                    break;
-                case kWeekslotRecord:
-                    data = tr("Recording every week when shown in this timeslot");
-                    break;
-                case kChannelRecord:
-                    data = tr("Recording when shown on this channel");
-                    break;
-                case kAllRecord:
-                    data = tr("Recording all showings");
-                    break;
-                case kDontRecord:
-                    data = tr("Manually not recording this showing");
-                    break;
-                case kNotRecording:
-                    data = tr("Not recording this showing");
-                    break;
-                default:
-                    data = tr("Error!");
-                    break;
-            }
-
-            showData[i].recText = data;
-        }
-
-        showShowingList();
         setActiveWindow();
         setFocus();
     }
@@ -650,7 +554,7 @@ void ProgFinder::cursorRight()
         if (inSearch == 2)
         {
             if (gotInitData[curSearch] > 10)
-                selectShowData(progData[curProgram]);
+                selectShowData(progData[curProgram], 0);
             else
                 inSearch = 1;
         }
@@ -667,6 +571,39 @@ void ProgFinder::select()
         cursorRight();
 }
 
+void ProgFinder::upcoming()
+{
+    if (inSearch == 2)
+    {
+        ProgramInfo *curPick = showData[curShow];
+
+        if (!curPick)
+            return;
+
+        ProgLister *pl = new ProgLister(plTitle, curPick->title, "",
+                                        gContext->GetMainWindow(), "proglist");
+        pl->exec();
+        delete pl;
+    }
+    else
+        cursorRight();
+}
+
+void ProgFinder::details()
+{
+    if (inSearch == 2)
+    {
+        ProgramInfo *curPick = showData[curShow];
+
+        if (!curPick)
+            return;
+
+        curPick->showDetails();
+    }
+    else
+        cursorRight();
+}
+
 void ProgFinder::quickRecord()
 {
     if (inSearch == 2)
@@ -769,7 +706,7 @@ void ProgFinder::cursorUp()
         if (curShow == -1)
         {
             curShow = showCount - 1;
-            while (showData[curShow].title == "**!0")
+            while (!showData[curShow])
                 curShow--;
         }
 
@@ -814,7 +751,7 @@ void ProgFinder::cursorDown()
         if ((curShow + 1) >= showCount)
             curShow = -1;
 
-        if (showData[curShow + 1].title != "**!0")
+        if (showData[curShow + 1])
         {
             curShow++;
             if (curShow == showCount)
@@ -981,30 +918,24 @@ void ProgFinder::showShowingList()
                     if (i >= showCount)
                         t = i - showCount;
 
-                    if ((&showData[t]) != NULL)
+                    if (showData[t])
                     {
-                        if (showData[t].title != "**!0")
-                        {
-                            ltype->SetItemText(curLabel, " " +
-                                               showData[t].startDisplay);
+                        ltype->SetItemText(curLabel, " "
+                            + showData[t]->startts.toString(dateFormat)
+                            + " " + showData[t]->startts.toString(timeFormat));
 
-                            if (showData[t].recording > 0)
-                                ltype->EnableForcedFont(curLabel, "recording");
+                        if (showData[t]->recstatus == rsRecording)
+                            ltype->EnableForcedFont(curLabel, "recording");
+                        else if (showData[t]->recstatus == rsWillRecord)
+                            ltype->EnableForcedFont(curLabel, "record");
 
-                        }
-                        else
-                            ltype->SetItemText(curLabel, "");
                     }
                     else
-                    {
                         ltype->SetItemText(curLabel, "");
-                    }
                     curLabel++;
                 }
             }
         }
-        curChannel = showData[curShow].channelID;
-        curDateTime = showData[curShow].starttime;
     }
     update(infoRect);
     update(listRect);
@@ -1015,15 +946,19 @@ void ProgFinder::selectSearchData()
     if (running == false)
         return;
 
-    allowkeypress = false;
+    inFill = true;
     QString thequery;
     QString data;
+    MSqlBindings bindings;
 
-    thequery = whereClauseGetSearchData(curSearch);
+    MSqlQuery query(MSqlQuery::InitCon());
+    whereClauseGetSearchData(curSearch, thequery, bindings);
 
-    QSqlQuery query = m_db->exec(thequery);
-
-    int rows = query.numRowsAffected();
+    query.prepare(thequery);
+    query.bindValues(bindings);
+    query.exec();
+    
+    int rows = query.size();
 
     if (rows == -1)
     {
@@ -1081,7 +1016,7 @@ void ProgFinder::selectSearchData()
         listCount = showsPerListing;
 
     curProgram = 0;
-    allowkeypress = true;
+    inFill = false;
     showProgramList();
 }
 
@@ -1095,10 +1030,10 @@ void ProgFinder::clearProgramList()
         {
             UIListType *ltype = (UIListType *)container->GetType("shows");
             if (ltype)
+            {
                 for (int i = 0; i < showsPerListing; i++)
-                {
                     ltype->SetItemText(i, "");
-                }
+            }
         }
 
         for (int j = 0; j < searchCount; j++)
@@ -1128,7 +1063,7 @@ void ProgFinder::clearProgramList()
 
 void ProgFinder::clearShowData()
 {
-    delete [] showData;
+    showData.clear();
 
     LayerSet *container = theme->GetSet("selector");
     if (container)
@@ -1137,274 +1072,41 @@ void ProgFinder::clearShowData()
         if (ltype)
         {
             for (int i = 0; i < showsPerListing; i++)
-            {
                 ltype->SetItemText(i, "");
-            }
         }
     }
     update(infoRect);
 }
 
-int ProgFinder::checkRecordingStatus(int showNum)
-{
-    for (int j = 0; j < recordingCount; j++)
-    {
-        if (showData[showNum].title == curRecordings[j].title)
-        {
-            if (showData[showNum].subtitle == curRecordings[j].subtitle &&
-                showData[showNum].description == curRecordings[j].description)
-            {
-                if (curRecordings[j].type == kSingleRecord ||
-                    curRecordings[j].type == kOverrideRecord ||
-                    curRecordings[j].type == kDontRecord)
-                {
-                    if (showData[showNum].startdatetime ==
-                        curRecordings[j].startdatetime)
-                    {
-                        return curRecordings[j].type;
-                    }
-                }
-            }
-            if (curRecordings[j].type == kTimeslotRecord)
-            {
-                if ((showData[showNum].startdatetime).time() ==
-                     (curRecordings[j].startdatetime).time()
-                    && (showData[showNum].channelID == curRecordings[j].chanid
-                      || (showData[showNum].channelCallsign != ""
-                         && showData[showNum].channelCallsign ==
-                            curRecordings[j].chansign)))
-                {
-                    return curRecordings[j].type;
-                }
-            }
-            if (curRecordings[j].type == kWeekslotRecord)
-            {
-                if ((showData[showNum].startdatetime).time() ==
-                     (curRecordings[j].startdatetime).time()
-                    && (showData[showNum].startdatetime).toString("dddd") ==
-                     (curRecordings[j].startdatetime).toString("dddd")
-                    && (showData[showNum].channelID == curRecordings[j].chanid
-                      || (showData[showNum].channelCallsign != ""
-                         && showData[showNum].channelCallsign ==
-                            curRecordings[j].chansign)))
-                {
-                    return curRecordings[j].type;
-                }
-            }
-            if (curRecordings[j].type == kChannelRecord)
-            {
-                if (showData[showNum].channelID == curRecordings[j].chanid
-                      || (showData[showNum].channelCallsign != ""
-                         && showData[showNum].channelCallsign ==
-                            curRecordings[j].chansign))
-                {
-                    return curRecordings[j].type;
-                }
-            }
-            if (curRecordings[j].type == kAllRecord)
-            {
-                return curRecordings[j].type;
-            }
-            if (curRecordings[j].type == kFindOneRecord)
-            {
-                return curRecordings[j].type;
-            }
-        }
-    }
-    return 0;
-}
-
-void ProgFinder::getRecordingInfo()
-{
-    if (running == false)
-        return;
-    QDateTime recDateTime;
-    QString thequery;
-    QString data;
-
-    thequery = QString("SELECT record.chanid,starttime,startdate,"
-                       "title,subtitle,description,type,callsign,"
-                       "seriesid,programid "
-                       "FROM record,channel "
-                       "WHERE record.chanid = channel.chanid;");
-
-    QSqlQuery query = m_db->exec(thequery);
-
-    int rows = query.numRowsAffected();
-
-    if (rows == -1)
-    {
-        cerr << "MythProgFind: Error executing query! (getRecordingInfo)\n";
-        cerr << "MythProgFind: QUERY = " << thequery.local8Bit() << endl;
-        return;
-    }
-
-    recordingCount = 0;
-    if (rows > 0)
-    {
-        curRecordings = new recordingRecord[(int)rows];
-
-        if (query.isActive() && rows > 0)
-        {
-            while (query.next())
-            {
-                recDateTime = QDateTime::fromString(query.value(2).toString() +
-                                                    "T" +
-                                                    query.value(1).toString(),
-                                                    Qt::ISODate);
-
-                curRecordings[recordingCount].chanid = query.value(0).toString();
-                curRecordings[recordingCount].startdatetime = recDateTime;
-                curRecordings[recordingCount].title = QString::fromUtf8(query.value(3).toString());
-                curRecordings[recordingCount].subtitle = QString::fromUtf8(query.value(4).toString());
-                curRecordings[recordingCount].description = QString::fromUtf8(query.value(5).toString());
-                curRecordings[recordingCount].type = query.value(6).toInt();
-                curRecordings[recordingCount].chansign = QString::fromUtf8(query.value(7).toString());
-                curRecordings[recordingCount].seriesid = QString::fromUtf8(query.value(8).toString());
-                curRecordings[recordingCount].programid = QString::fromUtf8(query.value(9).toString());
-
-                recordingCount++;
-            }
-        }
-    }
-}
-
-void ProgFinder::selectShowData(QString progTitle)
+void ProgFinder::selectShowData(QString progTitle, int newCurShow)
 {
     if (running == false)
         return;
 
-    allowkeypress = false;
+    inFill = true;
     QDateTime progStart = QDateTime::currentDateTime();
-    QDateTime progEnd;
-    QString thequery;
-    QString data;
-    int rectype = 0;
 
-    QSqlQuery query(QString::null, m_db);
-    query.prepare("SELECT subtitle,starttime,channel.channum,"
-                  "channel.callsign,description,endtime,channel.chanid,"
-                  "channel.sourceid,channel.name,seriesid,programid "
-                  "FROM program,channel "
-                  "WHERE program.title = :TITLE AND "
-                  "program.chanid = channel.chanid "
-                  "AND program.starttime > :STARTTIME "
-                  "GROUP BY starttime,endtime,channum,callsign "
-                  "ORDER BY starttime," +
-                  gContext->GetSetting("ChannelOrdering", "channum + 0") +";");
-    query.bindValue(":TITLE", progTitle.utf8());
-    query.bindValue(":STARTTIME", progStart.toString("yyyyMMddhhmm50"));
+    schedList.FromScheduler();
 
-    if (!query.exec())
-    {
-        cerr << "MythProgFind: Error executing query! (selectShowData)\n";
-        cerr << "MythProgFind: QUERY = " << thequery.local8Bit() << endl;
-        return;
-    }
+    MSqlBindings bindings;
+    QString querystr = "WHERE program.title = :TITLE "
+                       "  AND program.endtime > :ENDTIME ";
+    bindings[":TITLE"] = progTitle.utf8();
+    bindings[":ENDTIME"] = progStart.toString("yyyy-MM-ddThh:mm:50");
 
-    int rows = query.numRowsAffected();
+    showData.FromProgram(querystr, bindings, schedList);
 
-    if (rows == -1)
-    {
-        cerr << "MythProgFind: Error executing query! (selectShowData)\n";
-        cerr << "MythProgFind: QUERY = " << thequery.local8Bit() << endl;
-        return;
-    }
-
-    showCount = 0;
-
-    if (rows < showsPerListing)
-    {
-        showData = new showRecord[showsPerListing];
-        for (int i = 0; i < showsPerListing; i++)
-            showData[i].title = "**!0";
-    }
-    else
-    {
-        showData = new showRecord[rows];
-    }
-
-    if (query.isActive() && rows > 0)
-    {
-        while (query.next())
-        {
-            progStart = QDateTime::fromString(query.value(1).toString(),
-                                              Qt::ISODate);
-            progEnd = QDateTime::fromString(query.value(5).toString(),
-                                            Qt::ISODate);
-
-            // DATE, CHANNEL NUM, CHANNEL CALLSIGN, SUBTITLE, DESCRIPTION, SQL START, SQL END
-
-            showData[showCount].startdatetime = progStart;
-            QString DateTimeFormat = dateFormat + " " + timeFormat;
-
-            showData[showCount].startDisplay = progStart.toString(DateTimeFormat);
-            showData[showCount].endDisplay = progEnd.toString(timeFormat);
-            showData[showCount].channelNum = query.value(2).toString();
-            showData[showCount].channelCallsign = query.value(3).toString();
-            showData[showCount].title = progTitle;
-            showData[showCount].subtitle = QString::fromUtf8(query.value(0).toString());
-            showData[showCount].description = QString::fromUtf8(query.value(4).toString());
-            showData[showCount].channelID = query.value(6).toString();
-            showData[showCount].channelName = query.value(8).toString();
-            showData[showCount].starttime = progStart.toString("yyyyMMddhhmm");
-            showData[showCount].endtime = progEnd.toString("yyyyMMddhhmm");
-            rectype = checkRecordingStatus(showCount);
-            showData[showCount].recording = rectype;
-            showData[showCount].seriesid = query.value(9).toString();
-            showData[showCount].programid = query.value(10).toString();
-
-            switch (rectype)
-            {
-                case kSingleRecord:
-                    data = tr("Recording just this showing");
-                    break;
-                case kOverrideRecord:
-                    data = tr("Recording this showing with override options");
-                    break;
-                case kFindOneRecord:
-                    data = tr("Recording one showing of this program");
-                    break;
-                case kTimeslotRecord:
-                    data = tr("Recording every day when shown in this timeslot");
-                    break;
-                case kWeekslotRecord:
-                    data = tr("Recording every week when shown in this timeslot");
-                    break;
-                case kChannelRecord:
-                    data = tr("Recording when shown on this channel");
-                    break;
-                case kAllRecord:
-                    data = tr("Recording all showings");
-                    break;
-                case kDontRecord:
-                    data = tr("Manually not recording this showing");
-                    break;
-                case kNotRecording:
-                    data = tr("Not recording this showing");
-                    break;
-                default:
-                    data = tr("Error!");
-                    break;
-            }
-
-            showData[showCount].recText = data;
-
-            showCount++;
-        }
-    }
-
-    if (rows < showsPerListing)
+    showCount = showData.count();
+    if (showCount < showsPerListing)
         showCount = showsPerListing;
 
-    curShow = 0;
-    allowkeypress = true;
+    curShow = newCurShow;
+    inFill = false;
     showShowingList();
 }
 
 void ProgFinder::getInitialProgramData()
 {
-    getRecordingInfo();
     getAllProgramData();
 }
 
@@ -1418,12 +1120,16 @@ void ProgFinder::getSearchData(int charNum)
     int startPlace = 0;
     QString thequery;
     QString data;
+    MSqlBindings bindings;
 
-    thequery = whereClauseGetSearchData(charNum);
+    whereClauseGetSearchData(charNum, thequery, bindings);
 
-    QSqlQuery query = m_db->exec(thequery);
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare(thequery);
+    query.bindValues(bindings);
+    query.exec();
 
-    int rows = query.numRowsAffected();
+    int rows = query.size();
 
     if (rows == -1)
     {
@@ -1507,35 +1213,36 @@ void ProgFinder::fillSearchData(void)
     searchData[curLabel] = '@';
 }
 
-QString ProgFinder::whereClauseGetSearchData(int charNum)
+void ProgFinder::whereClauseGetSearchData(int charNum, QString &where,
+                                          MSqlBindings &bindings)
 {
     QDateTime progStart = QDateTime::currentDateTime();
-    QString thequery;
 
     if (searchData[charNum].contains('@'))
     {
-        thequery = QString("SELECT DISTINCT title FROM program WHERE ( "
-                           "title NOT REGEXP '^[A-Z0-9]' AND "
-                           "title NOT REGEXP '^The [A-Z0-9]' AND "
-                           "title NOT REGEXP '^A [A-Z0-9]' AND "
-                           "starttime > %1 ) ORDER BY title;")
-                           .arg(progStart.toString("yyyyMMddhhmm50"));
+        where = "SELECT DISTINCT title FROM program WHERE ( "
+                   "title NOT REGEXP '^[A-Z0-9]' AND "
+                   "title NOT REGEXP '^The [A-Z0-9]' AND "
+                   "title NOT REGEXP '^A [A-Z0-9]' AND "
+                   "starttime > :STARTTIME ) ORDER BY title;";
+        bindings[":STARTTIME"] = progStart.toString("yyyy-MM-ddThh:mm:50");
     }
     else
     {
-        thequery = QString("SELECT DISTINCT title "
-                           "FROM program "
-                           "WHERE ( title LIKE '%1%' OR title LIKE 'The %2%' "
-                           "OR title LIKE 'A %3%' ) "
-                           "AND starttime > %4 "
-                           "ORDER BY title;")
-                           .arg(searchData[charNum])
-                           .arg(searchData[charNum])
-                           .arg(searchData[charNum])
-                           .arg(progStart.toString("yyyyMMddhhmm50"));
+        QString one = searchData[charNum] + "%";
+        QString two = QString("The ") + one;
+        QString three = QString("A ") + one;
+        where = "SELECT DISTINCT title "
+                "FROM program "
+                "WHERE ( title LIKE :ONE OR title LIKE :TWO "
+                "        OR title LIKE :THREE ) "
+                "AND starttime > :STARTTIME "
+                "ORDER BY title;";
+        bindings[":ONE"] = one.utf8();
+        bindings[":TWO"] = two.utf8();
+        bindings[":THREE"] = three.utf8();
+        bindings[":STARTTIME"] = progStart.toString("yyyy-MM-ddThh:mm:50");
     }
-
-    return thequery;
 }
 
 bool ProgFinder::formatSelectedData(QString& data)
@@ -1632,6 +1339,39 @@ void ProgFinder::getAllProgramData()
     }
 }
 
+void ProgFinder::customEvent(QCustomEvent *e)
+{
+    if ((MythEvent::Type)(e->type()) == MythEvent::MythEventMessage)
+    {
+        MythEvent *me = (MythEvent *)e;
+        QString message = me->Message();
+
+        if (message == "SCHEDULE_CHANGE")
+        {
+            if (inSearch == 2)
+            {
+                if (!allowkeypress)
+                {
+                    needFill = true;
+                    return;
+                }
+
+                allowkeypress = false;
+
+                do
+                {
+                    needFill = false;
+                    ProgramInfo *curPick = showData[curShow];
+                    if (curPick)
+                        selectShowData(curPick->title, curShow);
+                } while (needFill);
+
+                allowkeypress = true;
+            }
+        }
+    }
+}
+
 
 // Japanese specific program finder
 
@@ -1668,55 +1408,54 @@ void JaProgFinder::fillSearchData()
 // we hope japanese HIRAGANA and alphabets, numerics is inserted
 // these character must required ZENKAKU
 // because query work not fine, if mysql's default charset latin1
-QString JaProgFinder::whereClauseGetSearchData(int charNum)
+void JaProgFinder::whereClauseGetSearchData(int charNum, QString &where,
+                                            MSqlBindings &bindings)
 {
     QDateTime progStart = QDateTime::currentDateTime();
-    QString thequery;
 
-    thequery = QString("SELECT DISTINCT title FROM program ");
+    where = "SELECT DISTINCT title FROM program ";
 
     switch (charNum) {
     case 0:
-        thequery += QString("WHERE ( title_pronounce >= 'あ' AND title_pronounce <= 'お') ");
+        where += "WHERE ( title_pronounce >= 'あ' AND title_pronounce <= 'お') ";
         break;
     case 1:
-        thequery += QString("WHERE ( title_pronounce >= 'か' AND title_pronounce <= 'ご') ");
+        where += "WHERE ( title_pronounce >= 'か' AND title_pronounce <= 'ご') ";
         break;
     case 2:
-        thequery += QString("WHERE ( title_pronounce >= 'さ' AND title_pronounce <= 'そ') ");
+        where += "WHERE ( title_pronounce >= 'さ' AND title_pronounce <= 'そ') ";
         break;
     case 3:
-        thequery += QString("WHERE ( title_pronounce >= 'た' AND title_pronounce <= 'ど') ");
+        where += "WHERE ( title_pronounce >= 'た' AND title_pronounce <= 'ど') ";
         break;
     case 4:
-        thequery += QString("WHERE ( title_pronounce >= 'な' AND title_pronounce <= 'の') ");
+        where += "WHERE ( title_pronounce >= 'な' AND title_pronounce <= 'の') ";
         break;
     case 5:
-        thequery += QString("WHERE ( title_pronounce >= 'は' AND title_pronounce <= 'ぽ') ");
+        where += "WHERE ( title_pronounce >= 'は' AND title_pronounce <= 'ぽ') ";
         break;
     case 6:
-        thequery += QString("WHERE ( title_pronounce >= 'ま' AND title_pronounce <= 'も') ");
+        where += "WHERE ( title_pronounce >= 'ま' AND title_pronounce <= 'も') ";
         break;
     case 7:
-        thequery += QString("WHERE ( title_pronounce >= 'や' AND title_pronounce <= 'よ') ");
+        where += "WHERE ( title_pronounce >= 'や' AND title_pronounce <= 'よ') ";
         break;
     case 8:
-        thequery += QString("WHERE ( title_pronounce >= 'ら' AND title_pronounce <= 'ろ') ");
+        where += "WHERE ( title_pronounce >= 'ら' AND title_pronounce <= 'ろ') ";
         break;
     case 9:
-        thequery += QString("WHERE ( title_pronounce >= 'わ' AND title_pronounce <= 'ん') ");
+        where += "WHERE ( title_pronounce >= 'わ' AND title_pronounce <= 'ん') ";
         break;
     case 10:
-        thequery += QString("WHERE ( title_pronounce >= 'Ａ' AND title_pronounce <= 'ｚ') ");
+        where += "WHERE ( title_pronounce >= 'Ａ' AND title_pronounce <= 'ｚ') ";
         break;
     case 11:
-        thequery += QString("WHERE ( title_pronounce >= '０' AND title_pronounce <= '９') ");
+        where += "WHERE ( title_pronounce >= '０' AND title_pronounce <= '９') ";
         break;
     }
 
-    thequery += QString("AND starttime > %1 ORDER BY title_pronounce;")
-                       .arg(progStart.toString("yyyyMMddhhmm50"));
-    return thequery;
+    where += "AND starttime > :STARTTIME ORDER BY title_pronounce;";
+    bindings[":STARTTIME"] = progStart.toString("yyyy-MM-ddThh:mm:50");
 }
 
 
