@@ -22,7 +22,6 @@
 
 class QListView;
 class ProgramInfo;
-class QSqlDatabase;
 class QWidget;
 class TV;
 
@@ -63,7 +62,8 @@ class ProgFinder : public MythDialog
     virtual bool formatSelectedData(QString &data);
     virtual bool formatSelectedData(QString &data, int charNum);
     virtual void restoreSelectedData(QString &data);
-    virtual QString whereClauseGetSearchData(int canNum);
+    virtual void whereClauseGetSearchData(int canNum, QString &where, 
+                                          MSqlBindings &bindings);
 
     void LoadWindow(QDomElement &);
     void parseContainer(QDomElement &);
@@ -100,8 +100,6 @@ class ProgFinder : public MythDialog
     QString *initData;
     QString *progData;
 
-    QSqlDatabase *m_db;
-
     void showSearchList();
     void showProgramList();
     void showShowingList();
@@ -135,7 +133,8 @@ class JaProgFinder : public ProgFinder
     virtual bool formatSelectedData(QString &data);
     virtual bool formatSelectedData(QString &data, int charNum);
     virtual void restoreSelectedData(QString &data);
-    virtual QString whereClauseGetSearchData(int canNum);
+    virtual void whereClauseGetSearchData(int canNum, QString &where,
+                                          MSqlBindings &bindings);
 
   private:
     static const char* searchChars[];
