@@ -23,12 +23,14 @@ using namespace std;
 #include "remoteutil.h"
 #include "mythdbcon.h"
 
-ProgLister::ProgLister(ProgListType pltype, const QString &view,
+ProgLister::ProgLister(ProgListType pltype,
+                       const QString &view, const QString &from,
                        MythMainWindow *parent,
                        const char *name)
             : MythDialog(parent, name)
 {
     type = pltype;
+    addTables = from;
     startTime = QDateTime::currentDateTime();
     searchTime = startTime;
 
@@ -117,12 +119,14 @@ ProgLister::ProgLister(ProgListType pltype, const QString &view,
     setNoErase();
 
     gContext->addListener(this);
+    gContext->addCurrentLocation("ProgLister");
 }
 
 ProgLister::~ProgLister()
 {
     itemList.clear();
     gContext->removeListener(this);
+    gContext->removeCurrentLocation();
     delete theme;
 }
 
@@ -242,8 +246,9 @@ void ProgLister::LoadWindow(QDomElement &element)
             }
             else
             {
-                VERBOSE(VB_IMPORTANT, QString("Unknown element: %1, exiting").arg(e.tagName()));
-                exit(-17);
+                VERBOSE(VB_IMPORTANT,
+                        QString("ProgLister::LoadWindow(): Error, unknown "
+                                "element '%1'. Ignoring.").arg(e.tagName()));
             }
         }
     }
@@ -299,13 +304,14 @@ void ProgLister::paintEvent(QPaintEvent *e)
     QRect r = e->rect();
     QPainter p(this);
  
-    if (updateAll || r.intersects(viewRect))
-        updateView(&p);
+    
     if (updateAll || r.intersects(listRect))
         updateList(&p);
     if (updateAll || r.intersects(infoRect))
         updateInfo(&p);
-
+    if (updateAll || r.intersects(viewRect))
+        updateView(&p);
+    
     updateAll = false;
 }
 
@@ -450,6 +456,40 @@ void ProgLister::chooseListBoxChanged(void)
     }
 }
 
+void ProgLister::updateKeywordInDB(const QString &text)
+{
+    int oldview = chooseListBox->currentItem() - 1;
+    int newview = viewList.findIndex(text);
+
+    QString qphrase = NULL;
+
+    if (newview < 0 || newview != oldview)
+    {
+        if (oldview >= 0)
+        {
+            qphrase = viewList[oldview].utf8();
+
+            MSqlQuery query(MSqlQuery::InitCon());
+            query.prepare("DELETE FROM keyword "
+                          "WHERE phrase = :PHRASE AND searchtype = :TYPE;");
+            query.bindValue(":PHRASE", qphrase);
+            query.bindValue(":TYPE", searchtype);
+            query.exec();
+        }
+        if (newview < 0)
+        {
+            qphrase = text.utf8();
+
+            MSqlQuery query(MSqlQuery::InitCon());
+            query.prepare("REPLACE INTO keyword (phrase, searchtype)"
+                          "VALUES(:PHRASE, :TYPE );");
+            query.bindValue(":PHRASE", qphrase);
+            query.bindValue(":TYPE", searchtype);
+            query.exec();
+        }
+    }
+}
+
 void ProgLister::setViewFromEdit(void)
 {
     if (!choosePopup || !chooseListBox || !chooseLineEdit)
@@ -460,40 +500,8 @@ void ProgLister::setViewFromEdit(void)
     if (text.stripWhiteSpace().length() == 0)
         return;
 
-    int oldview = chooseListBox->currentItem() - 1;
-    int newview = viewList.findIndex(text);
-
-    QString querystr = NULL;
-    QString qphrase = NULL;
-
-    if (newview < 0 || newview != oldview)
-    {
-        if (oldview >= 0)
-        {
-            qphrase = viewList[oldview].utf8();
-            qphrase.replace("\'", "\\\'");
-
-            MSqlQuery query(MSqlQuery::InitCon());
-            querystr = QString("DELETE FROM keyword "
-                               "WHERE phrase = '%1' AND searchtype = '%2';")
-                               .arg(qphrase).arg(searchtype);
-            query.prepare(querystr);
-            query.exec();
-        }
-        if (newview < 0)
-        {
-            qphrase = text.utf8();
-            qphrase.replace("\'", "\\\'");
-
-            MSqlQuery query(MSqlQuery::InitCon());
-            querystr = QString("REPLACE INTO keyword (phrase, searchtype)"
-                               "VALUES('%1','%2');")
-                               .arg(qphrase).arg(searchtype);
-            query.prepare(querystr);
-            query.exec();
-        }
-    }
-
+    updateKeywordInDB(text);
+  
     choosePopup->done(0);
 
     fillViewList(text);
@@ -524,39 +532,8 @@ void ProgLister::setViewFromPowerEdit()
     if (text == ":::::")
         return;
 
-    int oldview = chooseListBox->currentItem() - 1;
-    int newview = viewList.findIndex(text);
+    updateKeywordInDB(text);
 
-    QString querystr = NULL;
-    QString qphrase = NULL;
-
-    if (newview < 0 || newview != oldview)
-    {
-        if (oldview >= 0)
-        {
-            qphrase = viewList[oldview].utf8();
-            qphrase.replace("\'", "\\\'");
-
-            MSqlQuery query(MSqlQuery::InitCon());
-            querystr = QString("DELETE FROM keyword "
-                               "WHERE phrase = '%1' AND searchtype = '%2';")
-                               .arg(qphrase).arg(searchtype);
-            query.prepare(querystr);
-            query.exec();
-        }
-        if (newview < 0)
-        {
-            qphrase = text.utf8();
-            qphrase.replace("\'", "\\\'");
-
-            MSqlQuery query(MSqlQuery::InitCon());
-            querystr = QString("REPLACE INTO keyword (phrase, searchtype)"
-                               "VALUES('%1','%2');")
-                               .arg(qphrase).arg(searchtype);
-            query.prepare(querystr);
-            query.exec();
-        }
-    }
     powerPopup->done(0);
 
     fillViewList(text);
@@ -591,15 +568,19 @@ void ProgLister::addSearchRecord(void)
         VERBOSE(VB_IMPORTANT, "Unknown search in ProgLister");
         return;
     }
+
     if (searchtype == kPowerSearch)
     {
         if (text == "" || text == ":::::")
             return;
 
-        what = powerStringToSQL(text);
+        MSqlBindings bindings;
+        powerStringToSQL(text.utf8(), what, bindings);
 
         if (what == "")
             return;
+
+        MSqlEscapeAsAQuery(what, bindings);
     }
 
 
@@ -623,13 +604,12 @@ void ProgLister::deleteKeyword(void)
 
     QString text = viewList[view];
     QString qphrase = text.utf8();
-    qphrase.replace("\'", "\\\'");
 
     MSqlQuery query(MSqlQuery::InitCon());
-    QString querystr = QString("DELETE FROM keyword "
-                               "WHERE phrase = '%1' AND searchtype = '%2';")
-                               .arg(qphrase).arg(searchtype);
-    query.prepare(querystr);
+    query.prepare("DELETE FROM keyword "
+                  "WHERE phrase = :PHRASE AND searchtype = :TYPE;");
+    query.bindValue(":PHRASE", qphrase);
+    query.bindValue(":TYPE", searchtype);
     query.exec();
 
     chooseListBox->removeItem(view + 1);
@@ -927,7 +907,7 @@ void ProgLister::powerEdit()
 
     QStringList field = QStringList::split( ":", text, true);
 
-    if ( field.count() != 6)
+    if (field.count() != 6)
     {
         VERBOSE(VB_IMPORTANT, QString("Error. PowerSearch %1 has %2 fields")
                 .arg(text).arg(field.count()));
@@ -970,8 +950,7 @@ void ProgLister::powerEdit()
 
     MSqlQuery query(MSqlQuery::InitCon());
 
-    QString querystr = "SELECT category FROM program GROUP BY category;";
-    query.prepare(querystr);
+    query.prepare("SELECT category FROM program GROUP BY category;");
     query.exec();
 
     if (query.isActive() && query.size())
@@ -995,15 +974,14 @@ void ProgLister::powerEdit()
     stationList.clear();
     stationList << "";
 
-    querystr = "SELECT channel.chanid, channel.channum, "
-               "channel.callsign, channel.name FROM channel "
-               "WHERE channel.visible = 1 "
-               "GROUP BY callsign "
-               "ORDER BY " + channelOrdering + ";";
-    query.prepare(querystr);
+    query.prepare(QString("SELECT channel.chanid, channel.channum, "
+                  "channel.callsign, channel.name FROM channel "
+                  "WHERE channel.visible = 1 "
+                  "GROUP BY callsign "
+                  "ORDER BY ") + channelOrdering + ";");
     query.exec();
 
-    if (query.isActive() && query.numRowsAffected())
+    if (query.isActive() && query.size())
     {
         while (query.next())
         {
@@ -1063,58 +1041,74 @@ void ProgLister::powerEdit()
     powerPopup = NULL;
 }
 
-QString ProgLister::powerStringToSQL(QString &qphrase)
+void ProgLister::powerStringToSQL(const QString &qphrase, QString &output,
+                                  MSqlBindings &bindings)
 {
-    QString str = "";
+    output = "";
+    QString curfield;
 
-    qphrase.replace("\"","\\\"");
-    QStringList field = QStringList::split( ":", qphrase, true);
+    QStringList field = QStringList::split(":", qphrase, true);
 
-    if ( field.count() != 6)
+    if (field.count() != 6)
     {
         VERBOSE(VB_IMPORTANT, QString("Error. PowerSearch %1 has %2 fields")
                 .arg(qphrase).arg(field.count()));
-        return str;
+        return;
     }
+
     if (field[0])
-        str += QString("program.title LIKE \"\%%1\%\" ")
-                       .arg(field[0]);
+    {
+        curfield = "%" + field[0] + "%";
+        output += "program.title LIKE :POWERTITLE ";
+        bindings[":POWERTITLE"] = curfield;
+    }
+
     if (field[1])
     {
-        if (str > "")
-            str += "AND ";
-        str += QString("program.subtitle LIKE \"\%%1\%\" ")
-                       .arg(field[1]);
+        if (output > "")
+            output += "\nAND ";
+
+        curfield = "%" + field[1] + "%";
+        output += "program.subtitle LIKE :POWERSUB ";
+        bindings[":POWERSUB"] = curfield;
     }
+
     if (field[2])
     {
-        if (str > "")
-            str += "AND ";
-        str += QString("program.description LIKE \"\%%1\%\" ")
-                       .arg(field[2]);
+        if (output > "")
+            output += "\nAND ";
+
+        curfield = "%" + field[2] + "%";
+        output += "program.description LIKE :POWERDESC ";
+        bindings[":POWERDESC"] = curfield;
     }
+
     if (field[3])
     {
-        if (str > "")
-            str += "AND ";
-        str += QString("program.category_type=\"%1\" ")
-                       .arg(field[3]);
+        if (output > "")
+            output += "\nAND ";
+
+        output += "program.category_type = :POWERCATTYPE ";
+        bindings[":POWERCATTYPE"] = field[3];
     }
+
     if (field[4])
     {
-        if (str > "")
-            str += "AND ";
-        str += QString("program.category=\"%1\" ")
-                       .arg(field[4]);
+        if (output > "")
+            output += "\nAND ";
+
+        output += "program.category = :POWERCAT ";
+        bindings[":POWERCAT"] = field[4];
     }
+
     if (field[5])
     {
-        if (str > "")
-            str += "AND ";
-        str += QString("channel.callsign=\"%1\" ")
-                       .arg(field[5]);
+        if (output > "")
+            output += "\nAND ";
+
+        output += "channel.callsign = :POWERCALLSIGN ";
+        bindings[":POWERCALLSIGN"] = field[5];
     }
-    return str;
 }
 
 void ProgLister::quickRecord()
@@ -1154,7 +1148,7 @@ void ProgLister::upcoming()
     if (!pi || type == plTitle)
         return;
 
-    ProgLister *pl = new ProgLister(plTitle, pi->title,
+    ProgLister *pl = new ProgLister(plTitle, pi->title, "",
                                    gContext->GetMainWindow(), "proglist");
     pl->exec();
     delete pl;
@@ -1176,12 +1170,11 @@ void ProgLister::fillViewList(const QString &view)
     if (type == plChannel) // list by channel
     {
         MSqlQuery query(MSqlQuery::InitCon()); 
-        QString querystr = "SELECT channel.chanid, channel.channum, "
-            "channel.callsign, channel.name FROM channel "
-            "WHERE channel.visible = 1 "
-            "GROUP BY channum, callsign "
-            "ORDER BY " + channelOrdering + ";";
-        query.prepare(querystr);
+        query.prepare(QString("SELECT channel.chanid, channel.channum, "
+                      "channel.callsign, channel.name FROM channel "
+                      "WHERE channel.visible = 1 "
+                      "GROUP BY channum, callsign "
+                      "ORDER BY ") + channelOrdering + ";");
         query.exec();
 
         if (query.isActive() && query.size())
@@ -1208,8 +1201,7 @@ void ProgLister::fillViewList(const QString &view)
     else if (type == plCategory) // list by category
     {
         MSqlQuery query(MSqlQuery::InitCon()); 
-        QString querystr = "SELECT category FROM program GROUP BY category;";
-        query.prepare(querystr);
+        query.prepare("SELECT category FROM program GROUP BY category;");
         query.exec();
 
         if (query.isActive() && query.size())
@@ -1231,10 +1223,9 @@ void ProgLister::fillViewList(const QString &view)
              type == plPeopleSearch || type == plPowerSearch)
     {
         MSqlQuery query(MSqlQuery::InitCon()); 
-        QString querystr = QString("SELECT phrase FROM keyword "
-                                   "WHERE searchtype = '%1';")
-                                   .arg(searchtype);
-        query.prepare(querystr);
+        query.prepare("SELECT phrase FROM keyword "
+                      "WHERE searchtype = :SEARCHTYPE;");
+        query.bindValue(":SEARCHTYPE", searchtype);
         query.exec();
 
         if (query.isActive() && query.size())
@@ -1256,13 +1247,12 @@ void ProgLister::fillViewList(const QString &view)
             if (curView < 0)
             {
                 QString qphrase = view.utf8();
-                qphrase.replace("\'", "\\\'");
 
                 MSqlQuery query(MSqlQuery::InitCon()); 
-                querystr = QString("REPLACE INTO keyword (phrase, searchtype)"
-                                   "VALUES('%1','%2');")
-                                    .arg(qphrase).arg(searchtype);
-                query.prepare(querystr);
+                query.prepare("REPLACE INTO keyword (phrase, searchtype)"
+                              "VALUES(:VIEW, :SEARCHTYPE );");
+                query.bindValue(":VIEW", qphrase);
+                query.bindValue(":SEARCHTYPE", searchtype);
                 query.exec();
 
                 viewList << qphrase;
@@ -1290,6 +1280,9 @@ void ProgLister::fillViewList(const QString &view)
         viewList << "all";
         viewTextList << tr("All");
 
+        viewList << "premieres";
+        viewTextList << tr("Premieres");
+
         viewList << "movies";
         viewTextList << tr("Movies");
 
@@ -1333,192 +1326,227 @@ void ProgLister::fillViewList(const QString &view)
 class plTitleSort
 {
     public:
-        plTitleSort(bool reverseSort = false) {m_reverse = reverseSort;}
+        plTitleSort(void) {;}
 
         bool operator()(const ProgramInfo *a, const ProgramInfo *b) 
         {
-            if (m_reverse)
-                return (a->sortTitle > b->sortTitle);
-            else
-                return (a->sortTitle < b->sortTitle);
-        }
+            if (a->sortTitle != b->sortTitle)
+                    return (a->sortTitle < b->sortTitle);
 
-    private:
-        bool m_reverse;
+            if (a->recstatus == b->recstatus)
+                return a->startts < b->startts;
+
+            if (a->recstatus == rsRecording) return true;
+            if (b->recstatus == rsRecording) return false;
+
+            if (a->recstatus == rsWillRecord) return true;
+            if (b->recstatus == rsWillRecord) return false;
+
+            return a->startts < b->startts;
+        }
 };
 
 class plTimeSort
 {
     public:
-        plTimeSort(bool reverseSort = false) {m_reverse = reverseSort;}
+        plTimeSort(void) {;}
 
         bool operator()(const ProgramInfo *a, const ProgramInfo *b) 
         {
             if (a->startts == b->startts)
                 return (a->chanid < b->chanid);
 
-            if (m_reverse)
-                return (a->startts > b->startts);
-            else
-                return (a->startts < b->startts);
+            return (a->startts < b->startts);
         }
-
-    private:
-        bool m_reverse;
 };
 
 void ProgLister::fillItemList(void)
 {
     if (curView < 0)
-        return;
+         return;
 
     QString where;
-    QString startstr = startTime.toString("yyyyMMddhhmm50");
+    QString startstr = startTime.toString("yyyy-MM-ddThh:mm:50");
     QString qphrase = viewList[curView].utf8();
-    qphrase.replace("\'", "\\\'");
+
+    MSqlBindings bindings;
+    bindings[":PGILSTART"] = startstr;
+    bindings[":PGILPHRASE"] = qphrase;
+    bindings[":PGILLIKEPHRASE"] = QString("%") + qphrase + "%";
 
     if (type == plTitle) // per title listings
     {
-        where = QString("WHERE channel.visible = 1 "
-                        "  AND program.endtime > %1 "
-                        "  AND program.title = '%2' ")
-                        .arg(startstr).arg(qphrase);
+        where = "WHERE channel.visible = 1 "
+                "  AND program.endtime > :PGILSTART "
+                "  AND program.title = :PGILPHRASE ";
     }
     else if (type == plNewListings) // what's new list
     {
-        where = QString("LEFT JOIN oldprogram ON "
-                        "  oldprogram.oldtitle = program.title "
-                        "WHERE channel.visible = 1 "
-                        "  AND program.endtime > %1 "
-                        "  AND oldprogram.oldtitle IS NULL "
-                        "  AND program.manualid = 0 ")
-                        .arg(startstr);
+        where = "LEFT JOIN oldprogram ON "
+                "  oldprogram.oldtitle = program.title "
+                "WHERE channel.visible = 1 "
+                "  AND program.endtime > :PGILSTART "
+                "  AND oldprogram.oldtitle IS NULL "
+                "  AND program.manualid = 0 ";
 
-        if (qphrase == "movies")
+        if (qphrase == "premieres")
         {
-            where += "AND program.category_type = 'movie' ";
+            where += "  AND ( ";
+            where += "    ( program.previouslyshown = 0 ";
+            where += "      AND (program.category = 'Special' ";
+            where += "        OR program.programid LIKE 'EP%0001') ";
+            where += "      AND DAYOFYEAR(program.originalairdate) = "; 
+            where += "          DAYOFYEAR(program.starttime)) ";
+            where += "    OR (program.category_type='movie' ";
+            where += "      AND program.stars > 0.5 ";
+            where += "      AND program.airdate >= YEAR(NOW()) - 2) ";
+            where += "  ) ";
+        }
+        else if (qphrase == "movies")
+        {
+            where += "  AND program.category_type = 'movie' ";
         }
         else if (qphrase == "series")
         {
-            where += "AND program.category_type = 'series' ";
+            where += "  AND program.category_type = 'series' ";
         }
         else if (qphrase == "specials")
         {
-            where += "AND program.category_type = 'tvshow' ";
+            where += "  AND program.category_type = 'tvshow' ";
         }
         else
         {
-            where += "AND (program.category_type <> 'movie' ";
-            where += "OR program.airdate >= YEAR(NOW() - INTERVAL 2 YEAR)) ";
+            where += "  AND (program.category_type <> 'movie' ";
+            where += "  OR program.airdate >= YEAR(NOW()) - 3) ";
         }
     }
     else if (type == plTitleSearch) // keyword search
     {
-        where = QString("WHERE channel.visible = 1 "
-                        "  AND program.endtime > %1 "
-                        "  AND program.title LIKE '\%%2\%' ")
-                        .arg(startstr).arg(qphrase);
+        where = "WHERE channel.visible = 1 "
+                "  AND program.endtime > :PGILSTART "
+                "  AND program.title LIKE :PGILLIKEPHRASE ";
     }
     else if (type == plKeywordSearch) // keyword search
     {
-        where = QString("WHERE channel.visible = 1 "
-                        "  AND program.endtime > %1 "
-                        "  AND (program.title LIKE '\%%2\%' "
-                        "    OR program.subtitle LIKE '\%%3\%' "
-                        "    OR program.description LIKE '\%%4\%') ")
-                        .arg(startstr).arg(qphrase).arg(qphrase).arg(qphrase);
-
+        where = "WHERE channel.visible = 1 "
+                "  AND program.endtime > :PGILSTART "
+                "  AND (program.title LIKE :PGILLIKEPHRASE "
+                "    OR program.subtitle LIKE :PGILLIKEPHRASE "
+                "    OR program.description LIKE :PGILLIKEPHRASE ) ";
     }
     else if (type == plPeopleSearch) // people search
     {
-        where = QString(", people, credits WHERE channel.visible = 1 "
-                        "  AND program.endtime > %1 "
-                        "  AND people.name LIKE '%2' "
-                        "  AND credits.person = people.person "
-                        "  AND program.chanid = credits.chanid "
-                        "  AND program.starttime = credits.starttime")
-                        .arg(startstr).arg(qphrase);
+        where = ", people, credits WHERE channel.visible = 1 "
+                "  AND program.endtime > :PGILSTART "
+                "  AND people.name LIKE :PGILPHRASE "
+                "  AND credits.person = people.person "
+                "  AND program.chanid = credits.chanid "
+                "  AND program.starttime = credits.starttime";
     }
     else if (type == plPowerSearch) // complex search
     {
-        where = QString("WHERE channel.visible = 1 "
-                        "  AND program.endtime > %1 "
-                        "  AND ( %2 ) ")
-                        .arg(startstr).arg(powerStringToSQL(qphrase));
+        QString powerWhere;
+        MSqlBindings powerBindings;
+
+        powerStringToSQL(qphrase, powerWhere, powerBindings);
+
+        if (powerWhere != "")
+        {
+            where = QString("WHERE channel.visible = 1 "
+                    "  AND program.endtime > :PGILSTART "
+                    "  AND ( ") + powerWhere + " ) ";
+            MSqlAddMoreBindings(bindings, powerBindings);
+        }
     }
     else if (type == plSQLSearch) // complex search
     {
         qphrase.remove(QRegExp("^\\s*AND\\s+", false));
         where = QString("WHERE channel.visible = 1 "
-                        "  AND program.endtime > %1 "
-                        "  AND ( %2 ) ")
-                        .arg(startstr).arg(qphrase);
+                        "  AND program.endtime > :PGILSTART "
+                        "  AND ( %1 ) ").arg(qphrase);
+        if (addTables > "")
+            where = addTables + " " + where;
     }
     else if (type == plChannel) // list by channel
     {
-        where = QString("WHERE channel.visible = 1 "
-                        "  AND program.endtime > %1 "
-                        "  AND channel.chanid = '%2' ")
-                        .arg(startstr).arg(qphrase);
+        where = "WHERE channel.visible = 1 "
+                "  AND program.endtime > :PGILSTART "
+                "  AND channel.chanid = :PGILPHRASE ";
     }
     else if (type == plCategory) // list by category
     {
-        where = QString("WHERE channel.visible = 1 "
-                        "  AND program.endtime > %1 "
-                        "  AND program.category = '\%2' ")
-                        .arg(startstr).arg(qphrase);
+        where = "WHERE channel.visible = 1 "
+                "  AND program.endtime > :PGILSTART "
+                "  AND program.category = :PGILPHRASE ";
     }
     else if (type == plMovies) // list movies
     {
-        where = QString("WHERE channel.visible = 1 "
-                        "  AND program.endtime > %1 "
-                        "  AND program.category_type = 'movie' "
-                        "  AND program.stars >= '\%2' ")
-                        .arg(startstr).arg(qphrase);
+        where = "WHERE channel.visible = 1 "
+                "  AND program.endtime > :PGILSTART "
+                "  AND program.category_type = 'movie' "
+                "  AND program.stars >= :PGILPHRASE ";
     }
     else if (type == plTime) // list by time
     {
-        where = QString("WHERE channel.visible = 1 "
-                        "  AND program.starttime >= %1 ")
-                        .arg(searchTime.toString("yyyyMMddhh0000"));
+        bindings[":PGILSEARCHTIME"] = searchTime.toString("yyyy-MM-dd hh:00:00");
+        where = "WHERE channel.visible = 1 "
+                "  AND program.starttime >= :PGILSEARCHTIME ";
         if (titleSort)
-            where += QString("  AND program.starttime < DATE_ADD(%1, "
-                             "INTERVAL 1 HOUR) ")
-                             .arg(searchTime.toString("yyyyMMddhh0000"));
+            where += "  AND program.starttime < DATE_ADD(:PGILSEARCHTIME, "
+                     "INTERVAL '1' HOUR) ";
     }
 
-    if (titleSort && type == plTitle)
-        where += " GROUP BY subtitle ";
-    else if (type == plNewListings || titleSort)
-        where += " GROUP BY title ";
-
     schedList.FromScheduler();
-    itemList.FromProgram(where, schedList);
+    itemList.FromProgram(where, bindings, schedList);
 
-    if (titleSort || reverseSort)
+    ProgramInfo *s;
+    vector<ProgramInfo *> sortedList;
+
+    while (itemList.count())
     {
-        ProgramInfo *s;
-
-        vector<ProgramInfo *> sortedList;
-        while (itemList.count())
-        {
-            s = itemList.take();
-            if (type == plTitle)
-                s->sortTitle = s->subtitle;
-            else
-                s->sortTitle = s->title;
-
-            s->sortTitle.remove(QRegExp("^(The |A |An )"));
-            sortedList.push_back(s);
-        }
-
-        if (titleSort)
-            sort(sortedList.begin(), sortedList.end(),
-                 plTitleSort(reverseSort));
+        s = itemList.take();
+        if (type == plTitle)
+            s->sortTitle = s->subtitle;
         else
-            sort(sortedList.begin(), sortedList.end(),
-                 plTimeSort(reverseSort));
+            s->sortTitle = s->title;
 
+        s->sortTitle.remove(QRegExp("^(The |A |An )"));
+        sortedList.push_back(s);
+    }
+
+    if (type == plNewListings || titleSort)
+    {
+        // Prune to one per title
+        sort(sortedList.begin(), sortedList.end(), plTitleSort());
+
+        QString curtitle = "";
+        vector<ProgramInfo *>::iterator i = sortedList.begin();
+        while (i != sortedList.end())
+        {
+            ProgramInfo *p = *i;
+            if (p->sortTitle != curtitle)
+            {
+                curtitle = p->sortTitle;
+                i++;
+            }
+            else
+            {
+                delete p;
+                i = sortedList.erase(i);
+            }
+        }
+    }
+    if (!titleSort)
+        sort(sortedList.begin(), sortedList.end(), plTimeSort());
+
+    if (reverseSort)
+    {
+        vector<ProgramInfo *>::reverse_iterator r = sortedList.rbegin();
+        for (; r != sortedList.rend(); r++)
+            itemList.append(*r);
+    }
+    else
+    {
         vector<ProgramInfo *>::iterator i = sortedList.begin();
         for (; i != sortedList.end(); i++)
             itemList.append(*i);
@@ -1731,3 +1759,4 @@ void ProgLister::customEvent(QCustomEvent *e)
 
     allowEvents = true;
 }
+
