@@ -1,5 +1,7 @@
 #include "recordingprofile.h"
 #include "libmyth/mythcontext.h"
+#include "libmyth/mythdbcon.h"
+#include "libmyth/mythwizard.h"
 #include <qsqldatabase.h>
 #include <qheader.h>
 #include <qcursor.h>
@@ -8,8 +10,6 @@
 
 #include "managedlist.h"
 
-#define TRANSCODER_GROUP 6
-
 QString RecordingProfileParam::whereClause(void) {
   return QString("id = %1").arg(parentProfile.getProfileNum());
 }
@@ -411,6 +411,22 @@ public:
     };
 };
 
+class EncodingThreadCount: public CodecParam, public SliderSetting {
+public:
+    EncodingThreadCount(const RecordingProfile& parent):
+        CodecParam(parent, "encodingthreadcount"),
+        SliderSetting(1,8,1) {
+
+        setLabel(QObject::tr("Number of threads"));
+        setValue(1);
+        setHelpText(
+            QObject::tr("Threads to use for software encoding.") + " " +
+            QObject::tr("Set to a value less than or equal to the "
+                        "number of processors on the backend that "
+                        "will be doing the encoding."));
+    };
+};
+
 class MPEG2bitrate: public CodecParam, public SliderSetting {
 public:
     MPEG2bitrate(const RecordingProfile& parent):
@@ -554,18 +570,24 @@ public:
         inter->addChild(new MPEG4OptionIDCT(parent));
         inter->addChild(new MPEG4OptionIME(parent));
         params->addChild(inter);
+#ifdef HAVE_PTHREADS
+        params->addChild(new EncodingThreadCount(parent));
+#endif
         addTarget("MPEG-4", params);
 
         // We'll eventually want to add MPEG2 software encoding params here
         params = new VerticalConfigurationGroup(false);
-        params->setLabel(QObject::tr("MPEG-4 Parameters"));
-        params->addChild(new MPEG4bitrate(parent));
-        params->addChild(new MPEG4MaxQuality(parent));
-        params->addChild(new MPEG4MinQuality(parent));
-        params->addChild(new MPEG4QualDiff(parent));
-        params->addChild(new MPEG4ScaleBitrate(parent));
-        params->addChild(new MPEG4OptionVHQ(parent));
-        params->addChild(new MPEG4Option4MV(parent));
+        params->setLabel(QObject::tr("MPEG-2 Parameters"));
+        //params->addChild(new MPEG4bitrate(parent));
+        //params->addChild(new MPEG4MaxQuality(parent));
+        //params->addChild(new MPEG4MinQuality(parent));
+        //params->addChild(new MPEG4QualDiff(parent));
+        //params->addChild(new MPEG4ScaleBitrate(parent));
+        //params->addChild(new MPEG4OptionVHQ(parent));
+        //params->addChild(new MPEG4Option4MV(parent));
+#ifdef HAVE_PTHREADS
+        params->addChild(new EncodingThreadCount(parent));
+#endif
         addTarget("MPEG-2", params);
 
         params = new VerticalConfigurationGroup();
@@ -594,6 +616,8 @@ public:
                codecName->addSelection("MPEG-2 Hardware Encoder");
             else if (groupType == "MJPEG")
                 codecName->addSelection("Hardware MJPEG");
+            else if (groupType == "GO7007")
+                codecName->addSelection("MPEG-4");
             else
             {
                 // V4L, TRANSCODE (and any undefined types)
@@ -618,12 +642,54 @@ class AutoTranscode: public CodecParam, public CheckBoxSetting {
 public:
     AutoTranscode(const RecordingProfile& parent):
         CodecParam(parent, "autotranscode") {
-        setLabel(QObject::tr("Automatically transcode after recording"));
+        setLabel(QObject::tr("Enable auto-transcode after recording"));
         setValue(false);
+	    setHelpText(QObject::tr("Automatically transcode when a recording is "
+                                "made using this profile and the recording's "
+                                "schedule is configurd to allow transcoding."));
     };
 };
 
-class ImageSize: public HorizontalConfigurationGroup {
+class TranscodeResize: public CodecParam, public CheckBoxSetting {
+public:
+    TranscodeResize(const RecordingProfile& parent):
+        CodecParam(parent, "transcoderesize") {
+        setLabel(QObject::tr("Resize Video while transcoding"));
+        setValue(false);
+        setHelpText(QObject::tr("Allows the transcoder to "
+                                "resize the video during transcoding."));
+    };
+};
+
+class TranscodeLossless: public CodecParam, public CheckBoxSetting {
+public:
+    TranscodeLossless(const RecordingProfile& parent):
+        CodecParam(parent, "transcodelossless") {
+        setLabel(QObject::tr("Lossless transcoding"));
+        setValue(false);
+        setHelpText(QObject::tr("Only reencode where absolutely needed "
+                                "(normally only around cutpoints).  Otherwise "
+                                "keep audio and video formats identical to "
+                                "the source.  This should result in the "
+                                "highest quality, but won't save as much "
+                                "space."));
+    };
+};
+
+class TranscodePreserveAspect: public CodecParam, public CheckBoxSetting {
+public:
+    TranscodePreserveAspect(const RecordingProfile& parent):
+        CodecParam(parent, "transcodepreserveaspect") {
+        setLabel(QObject::tr("Preserve Aspect Ratio of Video when resizing"));
+        setValue(false);
+        setHelpText(QObject::tr("Preserve the aspect ratio of the original "
+                                "recording file by ignoring the height "
+                                "setting above, and calculating the new height "
+                                "based on the aspect ratio and width."));
+    };
+};
+
+class ImageSize: public VerticalConfigurationGroup {
 public:
     class Width: public SpinBoxSetting, public CodecParam {
     public:
@@ -647,8 +713,9 @@ public:
 
     ImageSize(const RecordingProfile& parent, QString tvFormat,
               QString profName) 
-         : HorizontalConfigurationGroup(false) 
+         : VerticalConfigurationGroup(false) 
     {
+        ConfigurationGroup* imgSize = new HorizontalConfigurationGroup(false);
         QString labelName;
         if (profName.isNull())
             labelName = QObject::tr("Image size");
@@ -660,7 +727,11 @@ public:
 
         QString fullsize, halfsize;
         int maxwidth, maxheight;
-        if (tvFormat.lower() == "ntsc" || tvFormat.lower() == "ntsc-jp") {
+        if (profName.left(11) == "Transcoders") {
+            maxwidth = 1920;
+            maxheight = 1088;
+        } else if ((tvFormat.lower() == "ntsc") ||
+                   (tvFormat.lower() == "ntsc-jp")) {
             maxwidth = 720;
             maxheight = 480;
         } else if (tvFormat.lower() == "atsc") {
@@ -671,15 +742,30 @@ public:
             maxheight = 576;
         }
 
-        addChild(new Width(parent,maxwidth));
-        addChild(new Height(parent,maxheight));
+        imgSize->addChild(new Width(parent,maxwidth));
+        imgSize->addChild(new Height(parent,maxheight));
+        addChild(imgSize);
+
+        if (profName != NULL)
+        {
+            if (profName.left(11) == "Transcoders")
+                addChild(new TranscodePreserveAspect(parent));
+        }
+        else
+        {
+            addChild(new TranscodePreserveAspect(parent));
+        }
     };
 };
 
 RecordingProfile::RecordingProfile(QString profName)
+    : id(new ID()),        name(new Name(*this)),
+      imageSize(NULL),     videoSettings(NULL),
+      audioSettings(NULL), profileName(profName),
+      isEncoder(true)
 {
     // This must be first because it is needed to load/save the other settings
-    addChild(id = new ID());
+    addChild(id);
 
     ConfigurationGroup* profile = new VerticalConfigurationGroup(false);
     QString labelName;
@@ -688,36 +774,118 @@ RecordingProfile::RecordingProfile(QString profName)
     else
         labelName = profName + "->" + QObject::tr("Profile");
     profile->setLabel(labelName);
-    profile->addChild(name = new Name(*this));
-    profile->addChild(new AutoTranscode(*this));
+    profile->addChild(name);
+
+    tr_lossless = NULL;
+    tr_resize = NULL;
+
+    if (profName != NULL)
+    {
+        if (profName.left(11) == "Transcoders")
+        {
+            tr_lossless = new TranscodeLossless(*this);
+            tr_resize = new TranscodeResize(*this);
+            profile->addChild(tr_lossless);
+            profile->addChild(tr_resize);
+        }
+        else
+            profile->addChild(new AutoTranscode(*this));
+    }
+    else
+    {
+        tr_lossless = new TranscodeLossless(*this);
+        tr_resize = new TranscodeResize(*this);
+        profile->addChild(tr_lossless);
+        profile->addChild(tr_resize);
+        profile->addChild(new AutoTranscode(*this));
+    }
+
     addChild(profile);
-
-    QString tvFormat = gContext->GetSetting("TVFormat");
-    addChild(new ImageSize(*this, tvFormat, profName));
-    vc = new VideoCompressionSettings(*this, profName);
-    addChild(vc);
-
-    ac = new AudioCompressionSettings(*this, profName);
-    addChild(ac);
 };
 
-void RecordingProfile::loadByID(QSqlDatabase* db, int profileId) {
-    id->setValue(profileId);
-    load(db);
+void RecordingProfile::ResizeTranscode(bool resize)
+{
+    MythWizard *wizard = (MythWizard *)dialog;
+    //page '1' is the Image Size page
+    QWidget *size_page = wizard->page(1);
+    wizard->setAppropriate(size_page, resize);
 }
 
-bool RecordingProfile::loadByCard(QSqlDatabase* db, QString name, int cardid) {
+void RecordingProfile::SetLosslessTranscode(bool lossless)
+{
+    MythWizard *wizard = (MythWizard *)dialog;
+
+    bool show_size = (lossless) ? false : tr_resize->boolValue();
+    wizard->setAppropriate(wizard->page(1), show_size);
+    wizard->setAppropriate(wizard->page(2), ! lossless);
+    wizard->setAppropriate(wizard->page(3), ! lossless);
+    tr_resize->setEnabled(! lossless);
+    wizard->setNextEnabled(wizard->page(0), ! lossless);
+    wizard->setFinishEnabled(wizard->page(0), lossless);
+    
+}
+
+void RecordingProfile::loadByID(int profileId) 
+{
+    MSqlQuery result(MSqlQuery::InitCon());
+    result.prepare(
+        "SELECT cardtype "
+        "FROM profilegroups, recordingprofiles "
+        "WHERE profilegroups.id     = recordingprofiles.profilegroup AND "
+        "      recordingprofiles.id = :PROFILEID");
+    result.bindValue(":PROFILEID", profileId);
+
+    if (!result.exec() || !result.isActive())
+        MythContext::DBError("RecordingProfile::loadByID -- cardtype", result);
+    else if (result.next())
+    {
+        QString type = result.value(0).toString();
+        isEncoder = (type != "DVB" && type != "HDTV");
+    }
+
+    if (isEncoder)
+    {
+        QString tvFormat = gContext->GetSetting("TVFormat");
+        addChild(new ImageSize(*this, tvFormat, profileName));
+
+        videoSettings = new VideoCompressionSettings(*this, profileName);
+        addChild(videoSettings);
+
+        audioSettings = new AudioCompressionSettings(*this, profileName);
+        addChild(audioSettings);
+
+        if (profileName && profileName.left(11) == "Transcoders")
+        {
+            connect(tr_resize, SIGNAL(valueChanged   (bool)),
+                    this,      SLOT(  ResizeTranscode(bool)));
+            connect(tr_lossless, SIGNAL(valueChanged        (bool)),
+                    this,        SLOT(  SetLosslessTranscode(bool)));
+        }
+    }
+
+    id->setValue(profileId);
+    load();
+}
+
+bool RecordingProfile::loadByCard(QString name, int cardid) 
+{
     QString hostname = gContext->GetHostName();
     int recid = 0;
-    QString query = QString("SELECT recordingprofiles.id, "
-                   "profilegroups.hostname, profilegroups.is_default FROM "
-                   "recordingprofiles,profilegroups,capturecard WHERE "
-                   "profilegroups.id = recordingprofiles.profilegroup "
-                   "AND capturecard.cardid = %1 "
-                   "AND capturecard.cardtype = profilegroups.cardtype "
-                   "AND recordingprofiles.name = '%2';").arg(cardid).arg(name);
-    QSqlQuery result = db->exec(query);
-    if (result.isActive() && result.numRowsAffected() > 0) {
+
+    MSqlQuery result(MSqlQuery::InitCon());
+    result.prepare(
+        "SELECT recordingprofiles.id, profilegroups.hostname, "
+        "       profilegroups.is_default "
+        "FROM recordingprofiles, profilegroups, capturecard "
+        "WHERE profilegroups.id       = recordingprofiles.profilegroup AND "
+        "      capturecard.cardtype   = profilegroups.cardtype         AND "
+        "      capturecard.cardid     = :CARDID                        AND "
+        "      recordingprofiles.name = :NAME");
+    result.bindValue(":CARDID", cardid);
+    result.bindValue(":NAME", name);
+
+    if (result.exec() && result.isActive() && result.size() > 0)
+    {
         while (result.next())
         {
             if (result.value(1).toString() == hostname)
@@ -731,75 +899,100 @@ bool RecordingProfile::loadByCard(QSqlDatabase* db, QString name, int cardid) {
     }
     if (recid)
     {
-        loadByID(db, recid);
+        loadByID(recid);
         return true;
     }
-    else
-        return false;
+    return false;
 }
 
-bool RecordingProfile::loadByGroup(QSqlDatabase* db, QString name, QString group) {
-    QString query = QString("SELECT recordingprofiles.id FROM "
-                 "recordingprofiles, profilegroups WHERE "
-                 "recordingprofiles.profilegroup = profilegroups.id AND "
-                 "profilegroups.name = '%1' AND recordingprofiles.name = '%2';")
-                 .arg(group).arg(name);
-    QSqlQuery result = db->exec(query);
-    if (result.isActive() && result.numRowsAffected() > 0) {
-        result.next();
-        loadByID(db, result.value(0).toInt());
-        return true;
-    } else {
-        return false;
-    }
-}
-
-void RecordingProfile::setCodecTypes(QSqlDatabase *db)
+bool RecordingProfile::loadByGroup(QString name, QString group) 
 {
-    vc->selectCodecs(groupType(db));
-    ac->selectCodecs(groupType(db));
+    MSqlQuery result(MSqlQuery::InitCon());
+    result.prepare(
+        "SELECT recordingprofiles.id "
+        "FROM recordingprofiles, profilegroups "
+        "WHERE recordingprofiles.profilegroup = profilegroups.id AND "
+        "      profilegroups.name             = :GROUPNAME       AND "
+        "      recordingprofiles.name         = :NAME");
+    result.bindValue(":GROUPNAME", group);
+    result.bindValue(":NAME", name);
+
+    if (result.exec() && result.isActive() && result.next())
+    {
+        loadByID(result.value(0).toInt());
+        return true;
+    }
+    return false;
 }
 
-void RecordingProfileEditor::open(int id) {
-    QString profName = RecordingProfile::getName(db, id);
+void RecordingProfile::setCodecTypes()
+{
+    if (videoSettings)
+        videoSettings->selectCodecs(groupType());
+    if (audioSettings)
+        audioSettings->selectCodecs(groupType());
+}
+
+void RecordingProfile::setName(const QString& newName)
+{
+    name->setValue(newName);
+    name->setRW(isEncoder);
+}
+
+int RecordingProfile::exec()
+{
+    MythDialog* dialog = dialogWidget(gContext->GetMainWindow());
+    dialog->Show();
+    if (tr_lossless)
+        SetLosslessTranscode(tr_lossless->boolValue());
+
+    int ret = dialog->exec();
+
+    delete dialog;
+
+    return ret;
+}
+
+void RecordingProfileEditor::open(int id) 
+{
+    QString profName = RecordingProfile::getName(id);
     if (profName.isNull())
         profName = labelName;
     else
         profName = labelName + "->" + profName;
     RecordingProfile* profile = new RecordingProfile(profName);
 
-    profile->loadByID(db,id);
-    profile->setCodecTypes(db);
+    profile->loadByID(id);
+    profile->setCodecTypes();
 
-    if (profile->exec(db) == QDialog::Accepted)
-        profile->save(db);
+    if (profile->exec() == QDialog::Accepted)
+        profile->save();
     delete profile;
 };
 
-RecordingProfileEditor::RecordingProfileEditor(QSqlDatabase* _db,
-                          int id, QString profName)
+RecordingProfileEditor::RecordingProfileEditor(int id, QString profName)
 {
     labelName = profName;
-    db = _db;
     group = id;
     if (! labelName.isNull())
         this->setLabel(labelName);
 }
 
-void RecordingProfileEditor::load(QSqlDatabase* db) {
+void RecordingProfileEditor::load() {
     clearSelections();
     //addSelection("(Create new profile)", "0");
-    RecordingProfile::fillSelections(db, this, group);
+    RecordingProfile::fillSelections(this, group);
 }
 
-int RecordingProfileEditor::exec(QSqlDatabase* db) {
-    while (ConfigurationDialog::exec(db) == QDialog::Accepted)
+int RecordingProfileEditor::exec() {
+    while (ConfigurationDialog::exec() == QDialog::Accepted)
         open(getValue().toInt());
 
     return QDialog::Rejected;
 }
 
-void RecordingProfile::fillSelections(QSqlDatabase* db, SelectSetting* setting, int group)
+void RecordingProfile::fillSelections(SelectSetting* setting, int group,
+                                      bool foldautodetect)
 {
     if (group == 0)
     {
@@ -808,19 +1001,43 @@ void RecordingProfile::fillSelections(QSqlDatabase* db, SelectSetting* setting,
     }
     else
     {
-        QString query = QString("SELECT name, id FROM recordingprofiles "
+        MSqlQuery result(MSqlQuery::InitCon());
+        QString querystr = QString("SELECT name, id FROM recordingprofiles "
                                 "WHERE profilegroup = %1 ORDER BY id;")
                                 .arg(group);
+        result.prepare(querystr);
 
-        QSqlQuery result = db->exec(query);
-        if (result.isActive() && result.numRowsAffected() > 0)
+        if (result.exec() && result.isActive() && result.size() > 0)
         {
+            if (group == RecordingProfile::TranscoderGroup && foldautodetect)
+            {
+                setting->addSelection(QObject::tr("Autodetect"),
+                                      QString::number(
+                                      RecordingProfile::TranscoderAutodetect));
+            }
             while (result.next())
             {
-                if (group == TRANSCODER_GROUP)
+                if (group == RecordingProfile::TranscoderGroup)
                 {
-                    setting->addSelection(QString( "From %1").arg(result.value(0).toString()),
-                                          result.value(1).toString());
+                    if (result.value(0).toString() == "RTjpeg/MPEG4" ||
+                        result.value(0).toString() == "MPEG2")
+                    {
+                        if (foldautodetect)
+                        {
+                            /* Hide; used by "Autodetect". */
+                        }
+                        else
+                        {
+                            setting->addSelection(QObject::tr("Autodetect from %1")
+                                              .arg(result.value(0).toString()),
+                                              result.value(1).toString());
+                        }
+                    }
+                    else
+                    {
+                        setting->addSelection(result.value(0).toString(),
+                                              result.value(1).toString());
+                    }
                 }
                 else
                 {
@@ -832,7 +1049,7 @@ void RecordingProfile::fillSelections(QSqlDatabase* db, SelectSetting* setting,
     }
 }
 
-void RecordingProfile::fillSelections(QSqlDatabase* db, SelectManagedListItem* setting, int group)
+void RecordingProfile::fillSelections(SelectManagedListItem* setting, int group)
 {
     if (group == 0)
     {
@@ -844,62 +1061,81 @@ void RecordingProfile::fillSelections(QSqlDatabase* db, SelectManagedListItem* s
     }
     else
     {
-        QString query = QString("SELECT name, id FROM recordingprofiles "
+        MSqlQuery result(MSqlQuery::InitCon());
+        QString querystr = QString("SELECT name, id FROM recordingprofiles "
                                 "WHERE profilegroup = %1 ORDER BY id;")
                                 .arg(group);
+        result.prepare(querystr);
 
-        QSqlQuery result = db->exec(query);
-        if (result.isActive() && result.numRowsAffected() > 0)
+        if (result.exec() && result.isActive() && result.size() > 0)
         {
+            if (group == RecordingProfile::TranscoderGroup)
+            {
+                setting->addSelection(QObject::tr("Transcode using Autodetect"),
+                                      QString::number(
+                                      RecordingProfile::TranscoderAutodetect));
+            }
             while (result.next())
             {
-                if (group == TRANSCODER_GROUP)
+                if (group == RecordingProfile::TranscoderGroup)
                 {
-                    setting->addSelection(QString( "From %1").arg(result.value(0).toString()),
-                                          result.value(1).toString());
+                    /* RTjpeg/MPEG4 and MPEG2 are used by "Autodetect". */
+                    if (result.value(0).toString() != "RTjpeg/MPEG4" &&
+                        result.value(0).toString() != "MPEG2")
+                    {
+                        QString tempLabel(QObject::tr(
+                                          "Transcode using \"%1\""));
+                        setting->addSelection(QString(tempLabel)
+                                              .arg(result.value(0).toString()),
+                                              result.value(1).toString(),
+                                              false);
+                    }
                 }
                 else
                 {
-                    setting->addSelection(result.value(0).toString() ,result.value(1).toString(), false);
-
+                    QString tempLabel(QObject::tr(
+                                      "Record using the \"%1\" profile"));
+                    setting->addSelection(QString(tempLabel)
+                                          .arg(result.value(0).toString()),
+                                          result.value(1).toString(), false);
                 }
             }
         }
     }
 }
 
-QString RecordingProfile::groupType(QSqlDatabase *db)
+QString RecordingProfile::groupType(void) const
 {
-    if (db != NULL)
+    MSqlQuery result(MSqlQuery::InitCon());
+    QString querystr = QString("SELECT profilegroups.cardtype FROM "
+                        "profilegroups, recordingprofiles WHERE "
+                        "profilegroups.id = recordingprofiles.profilegroup "
+                        "AND recordingprofiles.id = %1;")
+                        .arg(getProfileNum());
+    result.prepare(querystr);
+
+    if (result.exec() && result.isActive() && result.size() > 0)
     {
-        QString query = QString("SELECT profilegroups.cardtype FROM "
-                            "profilegroups, recordingprofiles WHERE "
-                            "profilegroups.id = recordingprofiles.profilegroup "
-                            "AND recordingprofiles.id = %1;")
-                            .arg(getProfileNum());
-        QSqlQuery result = db->exec(query);
-        if (result.isActive() && result.numRowsAffected() > 0)
-        {
-            result.next();
-            return (result.value(0).toString());
-        }
+        result.next();
+        return (result.value(0).toString());
     }
+
     return NULL;
 }
 
-QString RecordingProfile::getName(QSqlDatabase *db, int id)
+QString RecordingProfile::getName(int id)
 {
-    if (db != NULL)
-    {
-        QString query = QString("SELECT name FROM recordingprofiles WHERE "
+    MSqlQuery result(MSqlQuery::InitCon());
+    QString querystr = QString("SELECT name FROM recordingprofiles WHERE "
                                 "id = %1;").arg(id);
-        QSqlQuery result = db->exec(query);
-        if (result.isActive() && result.numRowsAffected() > 0)
-        {
-            result.next();
-            return (result.value(0).toString());
-        }
+    result.prepare(querystr);
+
+    if (result.exec() && result.isActive() && result.size() > 0)
+    {
+        result.next();
+        return (result.value(0).toString());
     }
+
     return NULL;
 }
 
