@@ -11,6 +11,9 @@ class RecordingProfile;
 class VideoCompressionSettings;
 class AudioCompressionSettings;
 
+class SelectManagedListItem;
+
+
 // Any setting associated with a recording profile
 class RecordingProfileSetting: virtual public Setting {
 protected:
@@ -33,12 +36,16 @@ protected:
 };
 
 class ImageSize;
+class TranscodeResize;
+class TranscodeLossless;
 
-class RecordingProfile: public ConfigurationWizard {
-protected:
+class RecordingProfile: public ConfigurationWizard
+{
+  Q_OBJECT
+  protected:
     class ID: virtual public IntegerSetting,
               public AutoIncrementStorage {
-    public:
+      public:
         ID():
             AutoIncrementStorage("recordingprofiles", "id") {
             setVisible(false);
@@ -54,7 +61,7 @@ protected:
     };
 
     class Name: public LineEditSetting, public RecordingProfileParam {
-    public:
+      public:
         Name(const RecordingProfile& parent):
             LineEditSetting(false),
             RecordingProfileParam(parent, "name") {
@@ -62,48 +69,65 @@ protected:
             setLabel(QObject::tr("Profile name"));
         };
     };
-public:
+
+  public:
+    // initializers
     RecordingProfile(QString profName = NULL);
+    virtual void loadByID(int id);
+    virtual bool loadByCard(QString name, int cardid);
+    virtual bool loadByGroup(QString name, QString group);
+    virtual int exec();
 
-    virtual void loadByID(QSqlDatabase* db, int id);
-    virtual bool loadByCard(QSqlDatabase* db, QString name, int cardid);
-    virtual bool loadByGroup(QSqlDatabase* db, QString name, QString group);
+    // sets
+    void setCodecTypes();
+    void setName(const QString& newName);
 
-    static void fillSelections(QSqlDatabase* db, SelectSetting* setting,
-                               int group);
-    QString groupType(QSqlDatabase *db);
-    void setCodecTypes(QSqlDatabase *db);
-    int getProfileNum(void) const {
-        return id->intValue();
-    };
+    // gets
+    const ImageSize& getImageSize(void) const { return *imageSize;       }
+    int     getProfileNum(void)         const { return id->intValue();   }
+    QString getName(void)               const { return name->getValue(); }
+    QString groupType(void)             const;
 
-    QString getName(void) const { return name->getValue(); };
-    static QString getName(QSqlDatabase* db, int id);
-    void setName(QString newName) { name->setValue(newName); name->setRW(); };
-    const ImageSize& getImageSize(void) const { return *imageSize; };
-    
-private:
-    ID* id;
-    Name* name;
-    ImageSize* imageSize;
-    VideoCompressionSettings *vc;
-    AudioCompressionSettings *ac;
+    // static functions
+    static QString getName(int id);    
+    static void fillSelections(SelectSetting* setting,
+                               int group, bool foldautodetect = false);
+    static void fillSelections(SelectManagedListItem* setting,
+                               int group);                           
+
+    // constants
+    static const int TranscoderAutodetect = 0;  ///< sentinel value
+    static const int TranscoderGroup = 6;       ///< hard-coded DB value
+
+  private slots:
+    void ResizeTranscode(bool resize); 
+    void SetLosslessTranscode(bool lossless);
+
+  private:
+    ID                       *id;
+    Name                     *name;
+    ImageSize                *imageSize;
+    TranscodeResize          *tr_resize;
+    TranscodeLossless        *tr_lossless;
+    VideoCompressionSettings *videoSettings;
+    AudioCompressionSettings *audioSettings;
+    QString                   profileName;
+    bool                      isEncoder;
 };
 
 class RecordingProfileEditor: public ListBoxSetting, public ConfigurationDialog {
     Q_OBJECT
 public:
-    RecordingProfileEditor(QSqlDatabase* _db, int id, QString profName);
+    RecordingProfileEditor(int id, QString profName);
 
-    virtual int exec(QSqlDatabase* db);
-    virtual void load(QSqlDatabase* db);
-    virtual void save(QSqlDatabase* db) { (void)db; };
+    virtual int exec();
+    virtual void load();
+    virtual void save() { };
 
 protected slots:
     void open(int id);
 
 protected:
-    QSqlDatabase* db;
     int group;
     QString labelName;
 };
