@@ -1,6 +1,5 @@
 #include <qapplication.h>
 
-#include <iostream>
 #include <unistd.h>
 
 using namespace std;
@@ -11,15 +10,10 @@ using namespace std;
 #include "mythcontext.h"
 
 RemoteEncoder::RemoteEncoder(int num, const QString &host, short port)
+    : recordernum(num),       controlSock(NULL),      remotehost(host),
+      remoteport(port),       lastchannel(""),        backendError(false),
+      cachedFramesWritten(0)
 {
-    recordernum = num;
-    remotehost = host;
-    remoteport = port;
-    backendError = false;
-
-    lastchannel = "";
-
-    controlSock = NULL;
     pthread_mutex_init(&lock, NULL); 
 }
 
@@ -59,7 +53,8 @@ void RemoteEncoder::SendReceiveStringList(QStringList &strlist)
     WriteStringList(controlSock, strlist);
     if (!ReadStringList(controlSock, strlist, true))
     {
-        cerr << "Remote encoder not responding.\n";
+        VERBOSE(VB_IMPORTANT,
+                "RemoteEncoder::SendReceiveStringList(): No response.");
         backendError = true;
     }
 
@@ -71,7 +66,8 @@ QSocketDevice *RemoteEncoder::openControlSocket(const QString &host, short port)
     QSocketDevice *sock = new QSocketDevice(QSocketDevice::Stream);
     if (!connectSocket(sock, host, port))
     {
-        cerr << "Connection timed out.\n";
+        VERBOSE(VB_IMPORTANT,
+                "RemoteEncoder::openControlSocket(): Connection timed out.");
         delete sock;
         sock = NULL;
     }
@@ -117,6 +113,12 @@ ProgramInfo *RemoteEncoder::GetRecording(void)
     return proginfo;
 }
 
+/** \fn RemoteEncoder::GetFrameRate(void)
+ *  \brief Returns recordering frame rate set by nvr.
+ *  \sa TVRec::GetFramerate(void), EncoderLink::GetFramerate(void),
+ *      RecorderBase::GetFrameRate(void)
+ *  \return Frames per second if query succeeds -1 otherwise.
+ */
 float RemoteEncoder::GetFrameRate(void)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
@@ -128,6 +130,13 @@ float RemoteEncoder::GetFrameRate(void)
     return retval;
 }
 
+/** \fn RemoteEncoder::GetFramesWritten(void)
+ *  \brief Returns number of frames written to disk by TVRec's RecorderBase
+ *         instance.
+ *
+ *  \sa TVRec::GetFramesWritten(void), EncoderLink::GetFramesWritten(void)
+ *  \return Number of frames if query succeeds, -1 otherwise.
+ */
 long long RemoteEncoder::GetFramesWritten(void)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
@@ -135,11 +144,17 @@ long long RemoteEncoder::GetFramesWritten(void)
 
     SendReceiveStringList(strlist);
 
-    long long retval = decodeLongLong(strlist, 0);
+    cachedFramesWritten = decodeLongLong(strlist, 0);
 
-    return retval;
+    return cachedFramesWritten;
 }
 
+/** \fn RemoteEncoder::GetFilePosition(void)
+ *  \brief Returns total number of bytes written by TVRec's RingBuffer.
+ *
+ *  \sa TVRec::GetFilePosition(void), EncoderLink::GetFilePosition(void)
+ *  \return Bytes written if query succeeds, -1 otherwise.
+ */
 long long RemoteEncoder::GetFilePosition(void)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
@@ -152,11 +167,14 @@ long long RemoteEncoder::GetFilePosition(void)
     return retval;
 }
 
-long long RemoteEncoder::GetFreeSpace(long long totalreadpos)
+/** \fn TVRec::GetMaxBitrate(void)
+ *   Returns the maximum bits per second this recorder can produce.
+ *  \sa TVRec::GetMaxBitrate(void), EncoderLink::GetMaxBitrate(void)
+ */
+long long RemoteEncoder::GetMaxBitrate(void)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
-    strlist << "GET_FREE_SPACE";
-    encodeLongLong(strlist, totalreadpos);
+    strlist << "GET_MAX_BITRATE";
 
     SendReceiveStringList(strlist);
 
@@ -165,6 +183,13 @@ long long RemoteEncoder::GetFreeSpace(long long totalreadpos)
     return retval;
 }
 
+/** \fn RemoteEncoder::GetKeyframePosition(long long)
+ *  \brief Returns byte position in RingBuffer of a keyframe.
+ *
+ *  \sa TVRec::GetKeyframePosition(long long),
+ *      EncoderLink::GetKeyframePosition(long long)
+ *  \return Byte position of keyframe if query succeeds, -1 otherwise.
+ */
 long long RemoteEncoder::GetKeyframePosition(long long desired)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
@@ -202,12 +227,11 @@ void RemoteEncoder::FillPositionMap(int start, int end,
     }
 }
 
-void RemoteEncoder::CancelNextRecording(void)
+void RemoteEncoder::CancelNextRecording(bool cancel)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
     strlist << "CANCEL_NEXT_RECORDING";
-    VERBOSE(VB_IMPORTANT, QString("Sending QUERY_RECORDER %1 - CANCEL_NEXT_RECORDING")
-                          .arg(recordernum));
+    strlist << QString::number((cancel) ? 1 : 0);
                           
     SendReceiveStringList(strlist);
 }
@@ -220,6 +244,10 @@ void RemoteEncoder::FrontendReady(void)
     SendReceiveStringList(strlist);
 }
 
+/** \fn RemoteEncoder::StopPlaying(void)
+ *  \brief Tells TVRec to stop streaming a recording to the frontend.
+ *  \sa TVRec::StopPlaying(void), EncoderLink::StopPlaying(void)
+ */
 void RemoteEncoder::StopPlaying(void)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
@@ -228,29 +256,26 @@ void RemoteEncoder::StopPlaying(void)
     SendReceiveStringList(strlist);
 }
 
-void RemoteEncoder::SetupRingBuffer(QString &path, long long &filesize,
-                                    long long &fillamount, bool pip)
-{
-    QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
-    strlist << "SETUP_RING_BUFFER";
-    strlist << QString::number((int)pip);
-
-    SendReceiveStringList(strlist);
-
-    path = strlist[0];
-
-    filesize = decodeLongLong(strlist, 1);
-    fillamount = decodeLongLong(strlist, 3);
-}
-
-void RemoteEncoder::SpawnLiveTV(void)
+/** \fn RemoteEncoder::SpawnLiveTV(QString,bool)
+ *  \brief Tells TVRec to Spawn a "Live TV" recorder.
+ *  \sa TVRec::SpawnLiveTV(LiveTVChain*,bool),
+ *      EncoderLink::SpawnLiveTV(LiveTVChain*,bool)
+ */
+void RemoteEncoder::SpawnLiveTV(QString chainId, bool pip)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
     strlist << "SPAWN_LIVETV";
+    strlist << chainId;
+    strlist << QString::number((int)pip);
 
     SendReceiveStringList(strlist);
 }
 
+/** \fn EncoderLink::StopLiveTV(void)
+ *  \brief Tells TVRec to stop a "Live TV" recorder.
+ *         <b>This only works on local recorders.</b>
+ *  \sa TVRec::StopLiveTV(void), EncoderLink::StopLiveTV(void)
+ */
 void RemoteEncoder::StopLiveTV(void)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
@@ -259,7 +284,12 @@ void RemoteEncoder::StopLiveTV(void)
     SendReceiveStringList(strlist);
 }
 
-void RemoteEncoder::Pause(void)
+/** \fn RemoteEncoder::PauseRecorder(void)
+ *  \brief Tells TVRec to pause a recorder, used for channel and input changes.
+ *  \sa TVRec::PauseRecorder(void), EncoderLink::PauseRecorder(void),
+ *      RecorderBase::Pause(void)
+ */
+void RemoteEncoder::PauseRecorder(void)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
     strlist << "PAUSE";
@@ -275,6 +305,15 @@ void RemoteEncoder::FinishRecording(void)
     SendReceiveStringList(strlist);
 }
 
+void RemoteEncoder::SetLiveRecording(bool recording)
+{
+    QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
+    strlist << "SET_LIVE_RECORDING";
+    strlist << QString::number(recording);
+
+    SendReceiveStringList(strlist);
+}
+
 void RemoteEncoder::ToggleInputs(void)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
@@ -315,6 +354,42 @@ void RemoteEncoder::SetChannel(QString channel)
     lastchannel = "";
 }
 
+/** \fn RemoteEncoder::SetSignalMonitoringRate(int,bool)
+ *  \brief Sets the signal monitoring rate.
+ *
+ *  This will actually call SetupSignalMonitor() and 
+ *  TeardownSignalMonitor(bool) as needed, so it can
+ *  be used directly, without worrying about the 
+ *  SignalMonitor instance.
+ *
+ *  \sa TVRec::SetSignalMonitoringRate(int,int),
+ *      EncoderLink::SetSignalMonitoringRate(int,bool)
+ *  \param rate           The update rate to use in milliseconds,
+ *                        use 0 to disable.
+ *  \param notifyFrontend If true, SIGNAL messages will be sent to
+ *                        the frontend using this recorder.
+ *  \return Previous update rate
+ */
+int RemoteEncoder::SetSignalMonitoringRate(int rate, bool notifyFrontend)
+{
+    QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
+    strlist << "SET_SIGNAL_MONITORING_RATE";
+    strlist << QString::number(rate);
+    strlist << QString::number((int)notifyFrontend);
+
+    SendReceiveStringList(strlist);
+  
+    int retval = strlist[0].toInt();
+    return retval;
+}
+
+/** \fn RemoteEncoder::ChangeContrast(bool)
+ *  \brief Changes contrast of a recording.
+ *
+ *  Note: In practice this only works with frame grabbing recorders.
+ *
+ *  \return contrast if it succeeds, -1 otherwise.
+ */
 int RemoteEncoder::ChangeContrast(bool direction)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
@@ -327,6 +402,13 @@ int RemoteEncoder::ChangeContrast(bool direction)
     return retval;
 }
 
+/** \fn RemoteEncoder::ChangeBrightness(bool)
+ *  \brief Changes the brightness of a recording.
+ *
+ *   Note: In practice this only works with frame grabbing recorders.
+ *
+ *  \return brightness if it succeeds, -1 otherwise.
+ */
 int RemoteEncoder::ChangeBrightness(bool direction)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
@@ -339,6 +421,13 @@ int RemoteEncoder::ChangeBrightness(bool direction)
     return retval;
 }
 
+/** \fn RemoteEncoder::ChangeColour(bool)
+ *  \brief Changes the colour phase of a recording.
+ *
+ *   Note: In practice this only works with frame grabbing recorders.
+ *
+ *  \return colour if it succeeds, -1 otherwise.
+ */
 int RemoteEncoder::ChangeColour(bool direction)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
@@ -351,6 +440,13 @@ int RemoteEncoder::ChangeColour(bool direction)
     return retval;
 }
 
+/** \fn RemoteEncoder::ChangeHue(bool)
+ *  \brief Changes the hue of a recording.
+ *
+ *   Note: In practice this only works with frame grabbing recorders.
+ *
+ *  \return hue if it succeeds, -1 otherwise.
+ */
 int RemoteEncoder::ChangeHue(bool direction)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
@@ -372,6 +468,15 @@ void RemoteEncoder::ChangeDeinterlacer(int deint_mode)
     SendReceiveStringList(strlist);
 }
 
+/** \fn RemoteEncoder::CheckChannel(QString)
+ *  \brief Checks if named channel exists on current tuner.
+ *
+ *  \param channel Channel to verify against current tuner.
+ *  \return true if it succeeds, false otherwise.
+ *  \sa TVRec::CheckChannel(QString),
+ *      EncoderLink::CheckChannel(const QString&),
+ *      ShouldSwitchToAnotherCard(QString)
+ */
 bool RemoteEncoder::CheckChannel(QString channel)
 {
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
@@ -384,20 +489,61 @@ bool RemoteEncoder::CheckChannel(QString channel)
     return retval;
 }
 
-bool RemoteEncoder::CheckChannelPrefix(QString channel, bool &unique) 
+/** \fn RemoteEncoder::ShouldSwitchToAnotherCard(QString)
+ *  \brief Checks if named channel exists on current tuner, or
+ *         another tuner.
+ *         <b>This only works on local recorders.</b>
+ *  \param channelid channel to verify against tuners.
+ *  \return true if the channel on another tuner and not current tuner,
+ *          false otherwise.
+ *  \sa CheckChannel(const QString&)
+ */
+bool RemoteEncoder::ShouldSwitchToAnotherCard(QString channelid)
 {
+    // this function returns true if the channelid is not a valid
+    // channel on the current recorder. It queries to server in order
+    // to determine this.
     QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
-    strlist << "CHECK_CHANNEL_PREFIX";
-    strlist << channel;
+    strlist << "SHOULD_SWITCH_CARD";
+    strlist << channelid;
 
     SendReceiveStringList(strlist);
 
     bool retval = strlist[0].toInt();
-    unique = strlist[1].toInt();
-
     return retval;
 }
 
+/** \fn RemoteEncoder::CheckChannelPrefix(const QString&,uint&,bool&,QString&)
+ *  \brief Checks a prefix against the channels in the DB.
+ *
+ *  \sa TVRec::CheckChannelPrefix(const QString&,uint&,bool&,QString&)
+ *      for details.
+ */
+bool RemoteEncoder::CheckChannelPrefix(
+    const QString &prefix,
+    uint          &is_complete_valid_channel_on_rec,
+    bool          &is_extra_char_useful,
+    QString       &needed_spacer)
+{
+    QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
+    strlist << "CHECK_CHANNEL_PREFIX";
+    strlist << prefix;
+
+    SendReceiveStringList(strlist);
+
+    is_complete_valid_channel_on_rec = strlist[1].toInt();
+    is_extra_char_useful = strlist[2].toInt();
+    needed_spacer = (strlist[3] == "X") ? "" : strlist[3];
+
+    return strlist[0].toInt();
+}
+
+/** \fn RemoteEncoder::GetNextProgram(int,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&)
+ *  \brief Returns information about the program that would be seen if we changed
+ *         the channel using ChangeChannel(int) with "direction".
+ *  \sa TVRec::GetNextProgram(int,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&),
+ *      EncoderLink::GetNextProgram(int,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&,QString&)
+ */
 void RemoteEncoder::GetNextProgram(int direction,
                                    QString &title, QString &subtitle,
                                    QString &desc, QString &category,
@@ -454,101 +600,3 @@ void RemoteEncoder::GetNextProgram(int direction,
         programid = "";
 }
 
-void RemoteEncoder::GetChannelInfo(QString &title, QString &subtitle,
-                                   QString &desc, QString &category,
-                                   QString &starttime, QString &endtime,
-                                   QString &callsign, QString &iconpath,
-                                   QString &channelname, QString &chanid,
-                                   QString &seriesid, QString &programid,
-                                   QString &outputFilters, QString &repeat, 
-                                   QString &airdate, QString &stars)
-{
-    QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
-    strlist << "GET_PROGRAM_INFO";
-
-    SendReceiveStringList(strlist);
-
-    title = strlist[0];
-    subtitle = strlist[1];
-    desc = strlist[2];
-    category = strlist[3];
-    starttime = strlist[4];
-    endtime = strlist[5];
-    callsign = strlist[6];
-    iconpath = strlist[7];
-    channelname = strlist[8];
-    chanid = strlist[9];
-    seriesid = strlist[10];
-    programid = strlist[11];
-    outputFilters = strlist[12];
-    
-    repeat = strlist[13];
-    airdate = strlist[14];
-    stars = strlist[15];
-
-    if (title == " ")
-        title = "";
-    if (subtitle == " ")
-        subtitle = "";
-    if (desc == " ")
-        desc = "";
-    if (category == " ")
-        category = "";
-    if (starttime == " ")
-        starttime = "";
-    if (endtime == " ")
-        endtime = "";
-    if (callsign == " ")
-        callsign = "";
-    if (iconpath == " ")
-        iconpath = "";
-    if (channelname == " ")
-        channelname = "";
-    if (chanid == " ")
-        chanid = "";
-    if (seriesid == " ")
-        seriesid = "";
-    if (programid == " ")
-        programid = "";
-    if (outputFilters == " ")
-        outputFilters = "";        
-    if (repeat == " ")
-        repeat = "0";
-    if (airdate == " ")
-        airdate = "";
-    if (stars == " ")
-        stars = "";
-         
-    lastchannel = channelname;
-}
-
-void RemoteEncoder::GetInputName(QString &inputname)
-{
-    QStringList strlist = QString("QUERY_RECORDER %1").arg(recordernum);
-    strlist << "GET_INPUT_NAME";
-
-    SendReceiveStringList(strlist);
-
-    inputname = strlist[0];
-}
-
-QString RemoteEncoder::GetCurrentChannel(void)
-{
-    if (lastchannel == "" || lastchannel == "0")
-    {
-        QString dummy;
-        GetChannelInfo(dummy, dummy, dummy, dummy, dummy, dummy,
-                       dummy, dummy, lastchannel, dummy, dummy, 
-                       dummy, dummy, dummy, dummy, dummy);
-    }
-    return lastchannel;
-}
-
-void RemoteEncoder::GetOutputFilters(QString& filters)
-{
-    QString dummy;
-    GetChannelInfo(dummy, dummy, dummy, dummy, dummy, dummy,
-                   dummy, dummy, dummy, dummy, dummy, dummy, filters,
-                   dummy, dummy, dummy);
-}
-
