@@ -22,21 +22,23 @@ class RemoteEncoder
     bool IsRecording(void);
     float GetFrameRate(void);
     long long GetFramesWritten(void);
+    /// \brief Return value last returned by GetFramesWritten().
+    long long GetCachedFramesWritten(void) const { return cachedFramesWritten; }
     long long GetFilePosition(void);
-    long long GetFreeSpace(long long totalreadpos);
+    long long GetFreeDiskSpace();
+    long long GetMaxBitrate();
     long long GetKeyframePosition(long long desired);
     void FillPositionMap(int start, int end,
                          QMap<long long, long long> &positionMap);
     void StopPlaying(void);
-    void SetupRingBuffer(QString &path, long long &filesize, 
-                         long long &fillamount, bool pip = false);
-    void SpawnLiveTV(void);
+    void SpawnLiveTV(QString chainid, bool pip);
     void StopLiveTV(void);
-    void Pause(void);
+    void PauseRecorder(void);
     void FinishRecording(void);
     void FrontendReady(void);
-    void CancelNextRecording(void);
+    void CancelNextRecording(bool cancel);
 
+    void SetLiveRecording(bool recording);
     void ToggleInputs(void);
     int ChangeContrast(bool direction);
     int ChangeBrightness(bool direction);
@@ -46,22 +48,19 @@ class RemoteEncoder
     void ChangeDeinterlacer(int deint_mode);
     void ToggleChannelFavorite(void);
     void SetChannel(QString channel);
+    int SetSignalMonitoringRate(int msec, bool notifyFrontend = true);
     bool CheckChannel(QString channel);
-    bool CheckChannelPrefix(QString channel, bool &unique);
+    bool ShouldSwitchToAnotherCard(QString channelid);
+    bool CheckChannelPrefix(const QString&,uint&,bool&,QString&);
     void GetNextProgram(int direction,
                         QString &title, QString &subtitle, QString &desc, 
                         QString &category, QString &starttime, QString &endtime,
                         QString &callsign, QString &iconpath,
                         QString &channelname, QString &chanid,
                         QString &seriesid, QString &programid);
-    void GetChannelInfo(QString &title, QString &subtitle, QString &desc, 
-                        QString &category, QString &starttime, QString &endtime,
-                        QString &callsign, QString &iconpath,
-                        QString &channelname, QString &chanid,
-                        QString &seriesid, QString &programid);
-    void GetInputName(QString &inputname);
- 
-    QString GetCurrentChannel(void);
+                        
+    bool GetErrorStatus(void) { bool v = backendError; backendError = false; 
+                                return v; }
  
   private:
     QSocketDevice *openControlSocket(const QString &host, short port);
@@ -76,6 +75,9 @@ class RemoteEncoder
     short remoteport;
 
     QString lastchannel;
+
+    bool backendError;
+    long long cachedFramesWritten;
 };
 
 #endif
