@@ -3,16 +3,19 @@
 #include "recordingprofile.h"
 #include "mythcontext.h"
 #include "proglist.h"
+#include "previouslist.h"
 #include "sr_items.h"
 #include "sr_root.h"
 #include "sr_dialog.h"
 #include "jobqueue.h"
 #include "mythdbcon.h"
+#include "viewschdiff.h"
 
 #include <qlayout.h>
 #include <qlabel.h>
 #include <qapplication.h>
 #include <qregexp.h>
+#include <qsqlquery.h>
 
 // NOTE: if this changes, you _MUST_ update the RecTypePriority function 
 // in recordingtypes.cpp.
@@ -27,6 +30,7 @@ ScheduledRecording::ScheduledRecording()
     dupmethod = NULL;
     autoexpire = NULL;
     autotranscode = NULL;
+    transcoder = NULL;
     autocommflag = NULL;
     autouserjob1 = NULL;
     autouserjob2 = NULL;
@@ -39,6 +43,7 @@ ScheduledRecording::ScheduledRecording()
     m_dialog = NULL;
     recpriority = NULL;
     recgroup = NULL;
+    playgroup = NULL;
     inactive = NULL;
     searchType = "";
     searchForWhat = "";
@@ -51,27 +56,33 @@ ScheduledRecording::ScheduledRecording()
     
     addChild(id = new ID());
     
-    channel = new SRChannel(*this);
-    station = new SRStation(*this);
-    title = new SRTitle(*this);
-    subtitle = new SRSubtitle(*this);
-    description = new SRDescription(*this);
-    startTime = new SRStartTime(*this);
-    endTime = new SREndTime(*this);
-    startDate = new SRStartDate(*this);
-    endDate = new SREndDate(*this);
-    category = new SRCategory(*this);
-    seriesid = new SRSeriesid(*this);
-    programid = new SRProgramid(*this);
-    findday = new SRFindDay(*this);
-    findtime = new SRFindTime(*this);
-    findid = new SRFindId(*this);
-    parentid = new SRParentId(*this);
-    search = new SRRecSearchType(*this);
-    
-    rootGroup = new RootSRGroup(*this);
+    channel = new SRChannel(this);
+    station = new SRStation(this);
+    title = new SRTitle(this);
+    subtitle = new SRSubtitle(this);
+    description = new SRDescription(this);
+    startTime = new SRStartTime(this);
+    endTime = new SREndTime(this);
+    startDate = new SRStartDate(this);
+    endDate = new SREndDate(this);
+    category = new SRCategory(this);
+    seriesid = new SRSeriesid(this);
+    programid = new SRProgramid(this);
+    findday = new SRFindDay(this);
+    findtime = new SRFindTime(this);
+    findid = new SRFindId(this);
+    parentid = new SRParentId(this);
+    search = new SRRecSearchType(this);
+    rootGroup = new RootSRGroup(this);
 }
 
+ScheduledRecording::~ScheduledRecording() 
+{
+    // rootGroup is unique among this class' member variables in
+    // that it's not self-managed
+    if (!rootGroup.isNull())
+        delete rootGroup;
+}
 
 void ScheduledRecording::load()
 {
@@ -87,16 +98,17 @@ void ScheduledRecording::load()
         else
             type->addNormalSelections(!station->getValue().isEmpty(),
                                       search->intValue() == kManualSearch);
-        
+       
         type->setValue(tmpType);
         type->setUnchanged();
+
         fetchChannelInfo();
     }
 }
 
 
 
-void ScheduledRecording::loadByProgram(ProgramInfo* proginfo) 
+void ScheduledRecording::loadByProgram(const ProgramInfo* proginfo) 
 {
     m_pginfo = proginfo;
     
@@ -107,12 +119,24 @@ void ScheduledRecording::loadByProgram(ProgramInfo* proginfo)
 
     if (search->intValue() == kNoSearch ||
         search->intValue() == kManualSearch)
+    {
         setProgram(proginfo);
+        if (!proginfo->recordid)
+            playgroup->setValue(PlayGroup::GetInitialName(proginfo));
+    }
 }
 
 void ScheduledRecording::loadBySearch(RecSearchType lsearch,
                                       QString textname,
                                       QString forwhat)
+{
+    QString from = "";
+    loadBySearch(lsearch, textname, from, forwhat);
+}
+void ScheduledRecording::loadBySearch(RecSearchType lsearch,
+                                      QString textname,
+                                      QString from,
+                                      QString forwhat)
 {
     MSqlQuery query(MSqlQuery::InitCon());
 
@@ -158,11 +182,40 @@ void ScheduledRecording::loadBySearch(RecSearchType lsearch,
         }
         QString ltitle = QString("%1 %2").arg(textname).arg(searchType);
         title->setValue(ltitle);
+        subtitle->setValue(from);
         description->setValue(forwhat);
         findday->setValue((startDate->dateValue().dayOfWeek() + 1) % 7);
+        QDate epoch = QDate::QDate (1970, 1, 1);
+        findid->setValue(epoch.daysTo(startDate->dateValue()) + 719528);
     } 
 }
 
+void ScheduledRecording::modifyPowerSearchByID(int rid,
+                                               QString textname,
+                                               QString forwhat)
+{
+    QString from = "";
+    modifyPowerSearchByID(rid, textname, from, forwhat);
+}
+
+void ScheduledRecording::modifyPowerSearchByID(int rid,
+                                               QString textname,
+                                               QString from,
+                                               QString forwhat)
+{
+    if (rid <= 0)
+        return;
+
+    loadByID(rid);
+    if (search->intValue() != kPowerSearch)
+        return;
+
+    QString ltitle = textname + " (" + QObject::tr("Power Search") + ")";
+    title->setValue(ltitle);
+    subtitle->setValue(from);
+    description->setValue(forwhat);
+}
+
 void ScheduledRecording::fetchChannelInfo()
 {
     
@@ -274,8 +327,9 @@ void ScheduledRecording::ToMap(QMap<QString, QString>& progMap)
         QString findfrom = findtime->timeValue().toString(timeFormat);
         if (recType == kFindWeeklyRecord)
         {
+            int daynum = (findday->intValue() + 5) % 7 + 1;
             findfrom = QString("%1, %2")
-                               .arg(QDate::shortDayName(findday->intValue()-1))
+                               .arg(QDate::shortDayName(daynum))
                                .arg(findfrom);
         }
         progMap["subtitle"] = QString("(%1 %2) %3")
@@ -336,14 +390,18 @@ void ScheduledRecording::setSearchType(RecSearchType newType) {
                               search->intValue() == kManualSearch);
 }
 
-bool ScheduledRecording::GetAutoExpire(void) const {
+int ScheduledRecording::GetAutoExpire(void) const {
     return(autoexpire->getValue().toInt());
 }
 
-void ScheduledRecording::SetAutoExpire(bool expire) {
+void ScheduledRecording::SetAutoExpire(int expire) {
     autoexpire->setValue(expire);
 }
 
+int ScheduledRecording::GetTranscoder(void) const {
+    return transcoder->getValue().toInt();
+}
+
 int ScheduledRecording::GetAutoRunJobs(void) const {
     int result = 0;
 
@@ -371,7 +429,15 @@ bool ScheduledRecording::GetMaxNewest(void) const {
     return(maxnewest->getValue().toInt());
 }
 
-void ScheduledRecording::save() 
+void ScheduledRecording::save(void)
+{
+    // NOTE: we can not use a default value for send(bool) because
+    // anyone calling save on a parent type pointer will then not 
+    // use our version of the virtual funcation.
+    save(true);
+}
+
+void ScheduledRecording::save(bool sendSig)
 {
     if (type->isChanged() && getRecordingType() == kNotRecording)
     {
@@ -380,8 +446,36 @@ void ScheduledRecording::save()
     else
     {
         ConfigurationGroup::save();
+
+        MSqlQuery query(MSqlQuery::InitCon());
+        query.prepare(
+            "UPDATE recorded "
+                "SET recpriority = :RECPRIORITY, recgroup = :RECGROUP, "
+                    "transcoder = :TRANSCODER, playgroup = :PLAYGROUP "
+                "WHERE recordid = :RECORDID ;");
+        query.bindValue(":RECPRIORITY", getRecPriority());
+        query.bindValue(":RECGROUP", recgroup->getValue());
+        query.bindValue(":TRANSCODER", transcoder->getValue().toInt());
+        query.bindValue(":PLAYGROUP", playgroup->getValue());
+        query.bindValue(":RECORDID", getRecordID());
+
+        if (!query.exec())
+            MythContext::DBError("UPDATE recorded", query);
+    }
+    if (sendSig)
+        signalChange(getRecordID());
+}
+
+void ScheduledRecording::save(QString destination) 
+{
+    if (type->isChanged() && getRecordingType() == kNotRecording)
+    {
+        remove();
+    }
+    else
+    {
+        ConfigurationGroup::save(destination);
     }
-    signalChange(getRecordID());
 }
 
 void ScheduledRecording::remove() 
@@ -414,120 +508,33 @@ void ScheduledRecording::signalChange(int recordid)
         QStringList slist;
         slist << QString("RESCHEDULE_RECORDINGS %1").arg(recordid);
         if (!gContext->SendReceiveStringList(slist))
-            cerr << "error resceduling id " << recordid << 
-                " in ScheduledRecording::signalChange" << endl;
+            VERBOSE(VB_IMPORTANT, QString("Error rescheduling id %1 "
+                                    "in ScheduledRecording::signalChange")
+                                    .arg(recordid));
     }
 }
 
-void ScheduledRecording::doneRecording(const ProgramInfo& proginfo) 
+void ScheduledRecording::doneRecording(ProgramInfo& proginfo) 
 {
-    if (getRecordingType() == kFindOneRecord)
-        remove();
+    proginfo.recstatus = rsRecorded;
 
-    QString msg;
-  
-    msg = QString("Finished recording %1 on channel: %2")
-                  .arg(proginfo.title)
-                  .arg(proginfo.chanid);
+    QString msg = "Finished recording";
+    QString subtitle = proginfo.subtitle.isEmpty() ? "" :
+                                    QString(" \"%1\"").arg(proginfo.subtitle);
+    QString details = QString("%1%2: channel %3")
+                                    .arg(proginfo.title)
+                                    .arg(subtitle)
+                                    .arg(proginfo.chanid);
 
-    VERBOSE(VB_GENERAL, msg.local8Bit());
-    gContext->LogEntry("scheduler", LP_NOTICE, "Finished recording", 
-                       msg);
-
-    addHistory(proginfo);
+    VERBOSE(VB_GENERAL, QString("%1 %2").arg(msg).arg(details).local8Bit());
+    gContext->LogEntry("scheduler", LP_NOTICE, msg, details);
 }
 
-void ScheduledRecording::addHistory(const ProgramInfo& proginfo) 
-{
-    MSqlQuery result(MSqlQuery::InitCon());
-
-    result.prepare("REPLACE INTO oldrecorded (chanid,starttime,"
-                   "endtime,title,subtitle,description,category,"
-                   "seriesid,programid,findid,recordid) "
-                   "VALUES(:CHANID,:START,:END,:TITLE,:SUBTITLE,:DESC,"
-                   ":CATEGORY,:SERIESID,:PROGRAMID,:FINDID,:RECORDID);");
-    result.bindValue(":CHANID", proginfo.chanid);
-    result.bindValue(":START", proginfo.recstartts.toString(Qt::ISODate));
-    result.bindValue(":END", proginfo.recendts.toString(Qt::ISODate));
-    result.bindValue(":TITLE", proginfo.title.utf8());
-    result.bindValue(":SUBTITLE", proginfo.subtitle.utf8());
-    result.bindValue(":DESC", proginfo.description.utf8());
-    result.bindValue(":CATEGORY", proginfo.category.utf8());
-    result.bindValue(":SERIESID", proginfo.seriesid.utf8());
-    result.bindValue(":PROGRAMID", proginfo.programid.utf8());
-    result.bindValue(":FINDID", proginfo.findid);
-    result.bindValue(":RECORDID", proginfo.recordid);
-
-    result.exec();
-
-    if (!result.isActive())
-    {
-        MythContext::DBError("addHistory", result);
-    }
-
-    if (proginfo.findid)
-    {
-        result.prepare("REPLACE INTO oldfind (recordid, findid) "
-                       "VALUES(:RECORDID,:FINDID);");
-        result.bindValue(":RECORDID", proginfo.recordid);
-        result.bindValue(":FINDID", proginfo.findid);
-    
-        result.exec();
-        if (!result.isActive())
-        {
-            MythContext::DBError("addFindHistory", result);
-        }
-    }
-    // The removal of an entry from oldrecorded may affect near-future
-    // scheduling decisions, so recalculate
-    signalChange(0);
-}
-
-void ScheduledRecording::forgetHistory(const ProgramInfo& proginfo)
-{
-    MSqlQuery result(MSqlQuery::InitCon());
-
-    result.prepare("DELETE FROM oldrecorded WHERE title = :TITLE AND "
-                   "((subtitle = :SUBTITLE AND description = :DESC) OR "
-                   "(programid <> '' AND programid = :PROGRAMID) OR "
-                   "(findid <> 0 AND findid = :FINDID))");
-    result.bindValue(":TITLE", proginfo.title.utf8());
-    result.bindValue(":SUBTITLE", proginfo.subtitle.utf8());
-    result.bindValue(":DESC", proginfo.description.utf8());
-    result.bindValue(":PROGRAMID", proginfo.programid);
-    result.bindValue(":FINDID", proginfo.findid);
-    
-    result.exec();
-    if (!result.isActive())
-    {
-        MythContext::DBError("forgetHistory", result);
-    }
-
-    if (proginfo.findid)
-    {
-        result.prepare("DELETE FROM oldfind WHERE "
-                       "recordid = :RECORDID AND findid = :FINDID");
-        result.bindValue(":RECORDID", proginfo.recordid);
-        result.bindValue(":FINDID", proginfo.findid);
-    
-        result.exec();
-        if (!result.isActive())
-        {
-            MythContext::DBError("forgetFindHistory", result);
-        }
-    }
-    // The removal of an entry from oldrecorded may affect near-future
-    // scheduling decisions, so recalculate
-    signalChange(0);
-}
-
-
-
 void ScheduledRecording::runProgList(void)
 {
     ProgLister *pl = NULL;
 
-    if(search->intValue() && getRecordID())
+    if (search->intValue() && getRecordID())
     {
         ScheduledRecording rule;
         rule.loadByID(getRecordID());
@@ -535,34 +542,48 @@ void ScheduledRecording::runProgList(void)
         switch (search->intValue())
         {
         case kTitleSearch:
-            pl = new ProgLister(plTitleSearch, rule.description->getValue(),
+            pl = new ProgLister(plTitleSearch,
+                                rule.description->getValue(), "",
                                 gContext->GetMainWindow(), "proglist");
             break;
         case kKeywordSearch:
-            pl = new ProgLister(plKeywordSearch, rule.description->getValue(),
+            pl = new ProgLister(plKeywordSearch,
+                                rule.description->getValue(), "",
                                 gContext->GetMainWindow(), "proglist");
             break;
         case kPeopleSearch:
-            pl = new ProgLister(plPeopleSearch, rule.description->getValue(),
+            pl = new ProgLister(plPeopleSearch,
+                                rule.description->getValue(), "",
                                 gContext->GetMainWindow(), "proglist");
             break;
         case kPowerSearch:
             pl = new ProgLister(plSQLSearch, rule.description->getValue(),
+                                rule.subtitle->getValue(),
                                 gContext->GetMainWindow(), "proglist");
             break;
         default:
-            pl = new ProgLister(plTitle, title->getValue(),
+            pl = new ProgLister(plTitle, title->getValue(), "",
                                 gContext->GetMainWindow(), "proglist");
             break;
         }
     }
     else
-        pl = new ProgLister(plTitle, title->getValue(),
+        pl = new ProgLister(plTitle, title->getValue(), "",
                             gContext->GetMainWindow(), "proglist");
     pl->exec();
     delete pl;
 }
 
+void ScheduledRecording::runPrevList(void)
+{
+    PreviousList *pb = NULL;
+
+    pb = new PreviousList(gContext->GetMainWindow(), "proglist",
+                          getRecordID(), title->getValue());
+    pb->exec();
+    delete pb;
+}
+
 void ScheduledRecording::runShowDetails(void)
 {
     if (m_pginfo)
@@ -670,6 +691,10 @@ void ScheduledRecording::setEnd(const QDateTime& end) {
     endDate->setValue(end.date());
 }
 
+int ScheduledRecording::getRecPriority(void) const {
+    return recpriority->getValue().toInt();
+}
+
 void ScheduledRecording::setRecPriority(int newrecpriority) {
     recpriority->setValue(newrecpriority);
 }
@@ -678,18 +703,40 @@ void ScheduledRecording::setRecGroup(const QString& newrecgroup) {
     recgroup->setValue(newrecgroup);
 }
 
+void ScheduledRecording::setPlayGroup(const QString& newplaygroup) {
+    playgroup->setValue(newplaygroup);
+}
+
 QString ScheduledRecording::getProfileName(void) const {
     return profile->getValue();
 }
 
+QString ScheduledRecording::getRecordTitle(void) const {
+    return title->getValue();
+}
 
+QString ScheduledRecording::getRecordSubTitle(void) const {
+    return subtitle->getValue();
+}
 
-MythDialog* ScheduledRecording::dialogWidget(MythMainWindow *parent, const char *name)
+QString ScheduledRecording::getRecordDescription(void) const {
+    return description->getValue();
+}
+
+MythDialog* ScheduledRecording::dialogWidget(MythMainWindow *parent,
+                                             const char *name)
 {
-
+    (void) parent;
+    (void) name;
+#ifdef USING_FRONTEND
     MythDialog* dlg = new RecOptDialog(this, parent, name);
     rootGroup->setDialog(dlg);
     return dlg;
+#else
+    VERBOSE(VB_IMPORTANT, "You must compile the frontend "
+            "to use a dialogWidget.");
+    return NULL;
+#endif
 }
 
 
@@ -742,7 +789,10 @@ void ScheduledRecording::setDefault(bool haschannel)
     
     autoexpire->setValue(gContext->GetNumSetting("AutoExpireDefault", 0));
 
-    autotranscode->setValue(0);
+    autotranscode->setValue(gContext->GetNumSetting("AutoTranscode", 0));
+    transcoder->fillSelections();
+    transcoder->setValue(gContext->GetNumSetting("DefaultTranscoder",
+                         RecordingProfile::TranscoderAutodetect));
     autocommflag->setValue(gContext->GetNumSetting("AutoCommercialFlag", 1));
     autouserjob1->setValue(gContext->GetNumSetting("AutoRunUserJob1", 0));
     autouserjob2->setValue(gContext->GetNumSetting("AutoRunUserJob2", 0));
@@ -751,11 +801,13 @@ void ScheduledRecording::setDefault(bool haschannel)
 
     recgroup->fillSelections();    
     recgroup->setValue("Default");
+    playgroup->fillSelections();    
+    playgroup->setValue("Default");
 
     inactive->setValue(0);
 }
 
-void ScheduledRecording::setProgram(ProgramInfo *proginfo)
+void ScheduledRecording::setProgram(const ProgramInfo *proginfo)
 {
     m_pginfo = proginfo;
     if (proginfo)
@@ -775,8 +827,21 @@ void ScheduledRecording::setProgram(ProgramInfo *proginfo)
         {
             findday->setValue((proginfo->startts.date().dayOfWeek() + 1) % 7);
             findtime->setValue(proginfo->startts.time());
+
+            QDate epoch = QDate::QDate (1970, 1, 1);
+            findid->setValue(epoch.daysTo(proginfo->startts.date()) + 719528);
+        }
+        else
+        {
+            if (findid->intValue() > 0)
+                findid->setValue(proginfo->findid);
+            else
+            {
+                QDate epoch = QDate::QDate (1970, 1, 1);
+                findid->setValue(epoch.daysTo(proginfo->startts.date()) +
+                                              719528);
+            }
         }
-        findid->setValue(proginfo->findid);
         category->setValue(proginfo->category);
         
         fetchChannelInfo();
@@ -820,8 +885,98 @@ void ScheduledRecording::makeOverride(void)
     endoffset->setChanged();
     recpriority->setChanged();
     recgroup->setChanged();
+    playgroup->setChanged();
 }
 
+void
+ScheduledRecording::testRecording()
+{
+
+    QString ttable = "record_tmp";
+
+    MSqlQueryInfo dbcon = MSqlQuery::SchedCon();
+    MSqlQuery query(dbcon);
+    QString thequery;
+
+    thequery ="SELECT GET_LOCK(:LOCK, 2);";
+    query.prepare(thequery);
+    query.bindValue(":LOCK", "DiffSchedule");
+    query.exec();
+    if (query.lastError().type() != QSqlError::None)
+    {
+        QString msg =
+            QString("DB Error (Obtaining lock in testRecording): \n"
+                    "Query was: %1 \nError was: %2 \n")
+            .arg(thequery)
+            .arg(MythContext::DBErrorMessage(query.lastError()));
+        VERBOSE(VB_IMPORTANT, msg);
+        return;
+    }
 
 
+    thequery = QString("DROP TABLE IF EXISTS %1;").arg(ttable);
+    query.prepare(thequery);
+    query.exec();
+    if (query.lastError().type() != QSqlError::None)
+    {
+        QString msg =
+            QString("DB Error (deleting old table in testRecording): \n"
+                    "Query was: %1 \nError was: %2 \n")
+            .arg(thequery)
+            .arg(MythContext::DBErrorMessage(query.lastError()));
+        VERBOSE(VB_IMPORTANT, msg);
+        return;
+    }
 
+    thequery = QString("CREATE TABLE %1 SELECT * FROM record;").arg(ttable);
+    query.prepare(thequery);
+    query.exec();
+    if (query.lastError().type() != QSqlError::None)
+    {
+        QString msg =
+            QString("DB Error (create new table): \n"
+                    "Query was: %1 \nError was: %2 \n")
+            .arg(thequery)
+            .arg(MythContext::DBErrorMessage(query.lastError()));
+        VERBOSE(VB_IMPORTANT, msg);
+        return;
+    }
+
+    bool resetID = false;
+    if (getRecordID() == 0) {
+        thequery = QString("SELECT MAX(recordid) FROM %1 ORDER BY recordid;")
+                           .arg(ttable);
+        query.prepare(thequery);
+        query.exec();
+        if (query.isActive() && query.next())
+            id->setValue(query.value(0).toInt() + 1);
+        else
+            id->setValue(100000);
+    }
+    save(ttable);
+
+    ViewScheduleDiff vsd(gContext->GetMainWindow(), "Preview Schedule Changes",
+                         ttable, getRecordID(), title->getValue());
+
+    thequery = "SELECT RELEASE_LOCK(:LOCK);";
+    query.prepare(thequery);
+    query.bindValue(":LOCK", "DiffSchedule");
+    query.exec();
+    if (query.lastError().type() != QSqlError::None)
+    {
+        QString msg =
+            QString("DB Error (free lock): \n"
+                    "Query was: %1 \nError was: %2 \n")
+            .arg(thequery)
+            .arg(MythContext::DBErrorMessage(query.lastError()));
+        VERBOSE(VB_IMPORTANT, msg);
+        return;
+    }
+
+    if (resetID)
+        id->setValue(0);
+
+    vsd.exec();
+}
+
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
