@@ -10,17 +10,23 @@
 using namespace std;
 
 class ProgramInfo;
-class QSqlDatabase;
 
 class RootSRGroup;
 class RecOptDialog;
 
-
+class SRInactive;
 class SRRecordingType;
 class SRRecSearchType;
 class SRProfileSelector;
 class SRDupIn;
 class SRDupMethod;
+class SRAutoTranscode;
+class SRTranscoderSelector;
+class SRAutoCommFlag;
+class SRAutoUserJob1;
+class SRAutoUserJob2;
+class SRAutoUserJob3;
+class SRAutoUserJob4;
 class SRAutoExpire;
 class SRStartOffset;
 class SREndOffset;
@@ -38,78 +44,104 @@ class SREndDate;
 class SRCategory;
 class SRRecPriority;
 class SRRecGroup;
+class SRPlayGroup;
 class SRSeriesid;
 class SRProgramid;
+class SRFindDay;
+class SRFindTime;
+class SRFindId;
+class SRParentId;
 
 class ScheduledRecording: public ConfigurationGroup, public ConfigurationDialog {
     Q_OBJECT
 public:
     ScheduledRecording();
+    ~ScheduledRecording();
     ScheduledRecording(const ScheduledRecording& other);
 
-    virtual void load(QSqlDatabase* db);
+    virtual void load();
     
     virtual MythDialog* dialogWidget(MythMainWindow* parent,
                                      const char* widgetName=0);    
     
     void makeOverride(void);
-    ProgramInfo* getProgramInfo() { return m_pginfo; }
+    const ProgramInfo* getProgramInfo() const { return m_pginfo; }
     RootSRGroup* getRootGroup() { return rootGroup; }
 
     RecordingType getRecordingType(void) const;
     void setRecordingType(RecordingType);
     RecSearchType getSearchType(void) const;
+    void setSearchType(RecSearchType);
 
-    bool GetAutoExpire(void) const;
-    void SetAutoExpire(bool expire);
+    int GetAutoExpire(void) const;
+    void SetAutoExpire(int expire);
 
     int GetMaxEpisodes(void) const;
     bool GetMaxNewest(void) const;
 
+    int GetTranscoder(void) const;
+
+    int GetAutoRunJobs(void) const;
+
     void setStart(const QDateTime& start);
     void setEnd(const QDateTime& end);
+    int getRecPriority(void) const;
     void setRecPriority(int recpriority);
     void setRecGroup(const QString& recgroup);
+    void setPlayGroup(const QString& recgroup);
 
-    
-    virtual void save(QSqlDatabase* db);
+    virtual void save(void);
+    virtual void save(bool send_reschedule_signal);
+    virtual void save(QString);
 
-    virtual void loadByID(QSqlDatabase* db, int id);
-    virtual void loadByProgram(QSqlDatabase* db, ProgramInfo* proginfo);
-    virtual void loadBySearch(QSqlDatabase *db, RecSearchType lsearch, QString what);
+    virtual void loadByID(int id);
+    virtual void loadByProgram(const ProgramInfo* proginfo);
+    virtual void loadBySearch(RecSearchType lsearch,
+                              QString textname, QString forwhat);
+    virtual void loadBySearch(RecSearchType lsearch, QString textname,
+                              QString from, QString forwhat);
+    virtual void modifyPowerSearchByID(int rid,
+                                       QString textname, QString forwhat);
+    virtual void modifyPowerSearchByID(int rid, QString textname,
+                                       QString from, QString forwhat);
 
-    virtual int exec(QSqlDatabase* db, bool saveOnExec = true, bool doLoad = false)
-        { return ConfigurationDialog::exec(db, saveOnExec, doLoad); }    
+    virtual int exec(bool saveOnExec = true, bool doLoad = false)
+        { return ConfigurationDialog::exec(saveOnExec, doLoad); }    
         
-    void remove(QSqlDatabase* db);
+    void remove();
     int getRecordID(void) const { return id->intValue(); };
+    QString getRecordTitle(void) const;
+    QString getRecordSubTitle(void) const;
+    QString getRecordDescription(void) const;
     QString getProfileName(void) const;
 
-    void findMatchingPrograms(QSqlDatabase* db, list<ProgramInfo*>& proglist);
+    void findMatchingPrograms(list<ProgramInfo*>& proglist);
 
     // Do any necessary bookkeeping after a matching program has been
     // recorded
-    void doneRecording(QSqlDatabase* db, const ProgramInfo& proginfo);
+    void doneRecording(ProgramInfo& proginfo);
 
-    // Remember a program has been reccorded before (means it won't
-    // be recorded again)
-    void addHistory(QSqlDatabase* db, const ProgramInfo& proginfo);
+    static void fillSelections(SelectSetting* setting);
 
-    // Forget whether a program has been recorded before (allows it to
-    // be recorded again)
-    void forgetHistory(QSqlDatabase* db, const ProgramInfo& proginfo);
-
-    static bool hasChanged(QSqlDatabase* db);
-
-    static void fillSelections(QSqlDatabase* db, SelectSetting* setting);
-
-    static void signalChange(QSqlDatabase* db);
+    static void signalChange(int recordid);
+    // Use -1 for recordid when all recordids are potentially
+    // affected, such as when the program table is updated.  
+    // Use 0 for recordid when a reschdule isn't specific to a single
+    // recordid, such as when a recording type priority is changed.
     
+    void setInactiveObj(SRInactive* val) {inactive = val;}
     void setRecTypeObj(SRRecordingType* val) {type = val;}
     void setSearchTypeObj(SRRecSearchType* val) {search = val;}
     void setProfileObj( SRProfileSelector* val) {profile = val;}
     void setDupInObj(SRDupIn* val) {dupin = val;}
     void setDupMethodObj(SRDupMethod* val) {dupmethod = val;}
+    void setAutoTranscodeObj(SRAutoTranscode* val) {autotranscode = val;}
+    void setTranscoderObj(SRTranscoderSelector* val) {transcoder = val;}
+    void setAutoCommFlagObj(SRAutoCommFlag* val) {autocommflag = val;}
+    void setAutoUserJob1Obj(SRAutoUserJob1* val) {autouserjob1 = val;}
+    void setAutoUserJob2Obj(SRAutoUserJob2* val) {autouserjob2 = val;}
+    void setAutoUserJob3Obj(SRAutoUserJob3* val) {autouserjob3 = val;}
+    void setAutoUserJob4Obj(SRAutoUserJob4* val) {autouserjob4 = val;}
     void setAutoExpireObj(SRAutoExpire* val) {autoexpire = val;}
     void setStartOffsetObj(SRStartOffset* val) {startoffset = val;}
     void setEndOffsetObj(SREndOffset* val) {endoffset = val;}
@@ -127,21 +159,30 @@ public:
     void setCategoryObj(SRCategory* val) {category = val;}
     void setRecPriorityObj(SRRecPriority* val) {recpriority = val;}
     void setRecGroupObj(SRRecGroup* val) {recgroup = val;}
+    void setPlayGroupObj(SRPlayGroup* val) {playgroup = val;}
     void setSeriesIDObj(SRSeriesid* val) {seriesid = val;}
     void setProgramIDObj(SRProgramid* val) {programid = val;}
+    void setFindDayObj(SRFindDay* val) {findday = val;}
+    void setFindTimeObj(SRFindTime* val) {findtime = val;}
+    void setFindIdObj(SRFindId* val) {findid = val;}
+    void setParentIdObj(SRParentId* val) {parentid = val;}
     
     void ToMap(QMap<QString, QString>& infoMap);
     
     QString ChannelText(QString format);
 
-protected slots:
+public slots:
     void runProgList();
+    void runPrevList();
+    void testRecording();
+
+protected slots:
     void runShowDetails();
 
 protected:
-    virtual void setDefault(QSqlDatabase *db, bool haschannel);
-    virtual void setProgram(ProgramInfo *proginfo);
-    void fetchChannelInfo(QSqlDatabase *db);
+    virtual void setDefault(bool haschannel);
+    virtual void setProgram(const ProgramInfo *proginfo);
+    void fetchChannelInfo();
     
     class ID: virtual public IntegerSetting,
               public AutoIncrementStorage 
@@ -156,11 +197,19 @@ protected:
     };
 
     ID* id;
+    class SRInactive* inactive;
     class SRRecordingType* type;
     class SRRecSearchType* search;
     class SRProfileSelector* profile;
     class SRDupIn* dupin;
     class SRDupMethod* dupmethod;
+    class SRAutoTranscode* autotranscode;
+    class SRTranscoderSelector* transcoder;
+    class SRAutoCommFlag* autocommflag;
+    class SRAutoUserJob1* autouserjob1;
+    class SRAutoUserJob2* autouserjob2;
+    class SRAutoUserJob3* autouserjob3;
+    class SRAutoUserJob4* autouserjob4;
     class SRAutoExpire* autoexpire;
     class SRStartOffset* startoffset;
     class SREndOffset* endoffset;
@@ -178,10 +227,15 @@ protected:
     class SRCategory* category;
     class SRRecPriority* recpriority;
     class SRRecGroup* recgroup;
+    class SRPlayGroup* playgroup;
     class SRSeriesid* seriesid;
     class SRProgramid* programid;
+    class SRFindDay* findday;
+    class SRFindTime* findtime;
+    class SRFindId* findid;
+    class SRParentId* parentid;
     
-    ProgramInfo* m_pginfo;
+    const ProgramInfo* m_pginfo;
     QGuardedPtr<RootSRGroup> rootGroup;
     QGuardedPtr<RecOptDialog> m_dialog;
     QString chanstr;
@@ -200,19 +254,17 @@ protected:
 class ScheduledRecordingEditor: public ListBoxSetting, public ConfigurationDialog {
     Q_OBJECT
 public:
-    ScheduledRecordingEditor(QSqlDatabase* _db):
-        db(_db) {};
+    ScheduledRecordingEditor() {};
     virtual ~ScheduledRecordingEditor() {};
 
-    virtual int exec(QSqlDatabase* db);
-    virtual void load(QSqlDatabase* db);
-    virtual void save(QSqlDatabase* db) { (void)db; };
+    virtual int exec();
+    virtual void load();
+    virtual void save() { };
 
 protected slots:
     void open(int id);
 
 protected:
-    QSqlDatabase* db;
 };
 
 
