@@ -1,5 +1,5 @@
 /*
- * $Id: sitypes.cpp,v 1.8 2005/03/27 18:18:56 taylor Exp $
+ * $Id: sitypes.cpp 7799 2005-11-08 16:09:01Z danielk $
  * vim: set expandtab tabstop=4 shiftwidth=4:
  *
  * Original Project
@@ -29,8 +29,15 @@
  *
  */
 
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
+
+#include <qdeepcopy.h>
+
 #include "sitypes.h"
+#include "mythconfig.h"
+#include "mythcontext.h"
 
 // Set EIT_DEBUG_SID to a valid serviceid to enable EIT debugging
 // #define EIT_DEBUG_SID 1602
@@ -83,79 +90,91 @@ void TableSourcePIDObject::Reset()
     ChannelETT = 0;
 }
 
-void SectionTracker::Reset()
+void SectionTracker::Reset(void)
 {
-    MaxSections = -1;
-    Version = -1;
-    memset(&Filled, 0, sizeof(Filled));
+    maxSections = -1;
+    version     = -1;
+    empty       = true;
+    fillStatus.clear();
 }
 
-void SectionTracker::MarkUnused(int Section)
+void SectionTracker::MarkUnused(int section)
 {
-    if (Section > (MaxSections+1))
+    if (section > maxSections)
         return;
-    else
-        Filled[Section]=2;
+    
+    fillStatus[section] = kStatusUnused;
 }
 
-int SectionTracker::Complete()
+bool SectionTracker::IsComplete(void) const
 {
-    if (MaxSections == -1)
-        return 0;
-    for (int x = 0; x <= MaxSections; x++)
+    if (maxSections < 0)
+        return false;
+
+    for (int x = 0; x <= maxSections; x++)
     {
-        if (Filled[x] == 0)
-            return 0;
+        if (kStatusEmpty == fillStatus[x])
+            return false;
     }
-    return 1;
+
+    return true;
 }
 
-QString SectionTracker::loadStatus()
+QString SectionTracker::Status(void) const
 {
-    QString retval = "";
-    if (MaxSections == -1)
+    if (maxSections < 0)
         return QString("[---] ");
-    retval += QString("[");
-    for (int x=0;x<MaxSections+1;x++)
+
+    QString retval = "[";
+    for (int x = 0; x <= maxSections; x++)
     {
-        if (Filled[x] == 1)
-            retval += QString("P");
-        else if (Filled[x] == 2)
-            retval += QString("u");
-        else
-            retval += QString("m");
+        switch (fillStatus[x])
+        {
+            case kStatusEmpty:  retval += "m"; break;
+            case kStatusFilled: retval += "P"; break;
+            case kStatusUnused: retval += "u"; break;
+            default:            retval += "i"; break;
+        }
     }
-    retval += QString("] ");
-    return retval;
+
+    return retval + "] ";
 }
 
-int SectionTracker::AddSection(tablehead *head)
+int SectionTracker::AddSection(const tablehead *head)
 {
-    if (MaxSections == -1)
+    empty = false;
+    if (maxSections < 0)
     {
-         MaxSections = head->section_last;
-         Version = head->version;
-         Filled[head->section_number]=1;
+         maxSections = head->section_last;
+         version     = head->version;
+
+         fillStatus.clear();
+         fillStatus.resize(maxSections+10, kStatusEmpty);
+         fillStatus[head->section_number] = kStatusFilled;
          return 0;
     }
-    else if (Version != head->version)
+    else if (version != head->version)
     {
          Reset();
-         MaxSections = head->section_last;
-         Version = head->version;
-         Filled[head->section_number]=1;
+         maxSections = head->section_last;
+         version     = head->version;
+
+         fillStatus.clear();
+         fillStatus.resize(maxSections+10, kStatusEmpty);
+         fillStatus[head->section_number] = kStatusFilled;
          return -1;
     }
-    else
+    else if (head->section_number <= maxSections)
     {
-        if (Filled[head->section_number] == 1)
+        if (kStatusFilled == fillStatus[head->section_number])
              return 1;
         else
         {
-             Filled[head->section_number] = 1;
+             fillStatus[head->section_number] = kStatusFilled;
              return 0;
         }
     }
+    return 1; // just ignore it if it is outside bounds
 }
 
 void SDTObject::Reset()
@@ -210,6 +229,21 @@ Descriptor::~Descriptor()
         delete[] Data;
 }
 
+void ElementaryPIDObject::deepCopy(const ElementaryPIDObject &e)
+{
+    Type        = e.Type;
+    Orig_Type   = e.Orig_Type;
+    PID         = e.PID;
+    Description = QDeepCopy<QString>(e.Description);
+    Language    = QDeepCopy<QString>(e.Language);
+    CA          = QDeepCopy<CAList>(e.CA);
+    Record      = e.Record;
+
+    Descriptors.clear();
+    DescriptorList::const_iterator dit;
+    for (dit = e.Descriptors.begin(); dit != e.Descriptors.end(); ++dit)
+        Descriptors.append(Descriptor(*dit));
+}
 
 void ElementaryPIDObject::Reset()
 {
@@ -232,23 +266,61 @@ void Event::Reset()
     clearEventValues();
 }
 
+void Event::deepCopy(const Event &e)
+{
+    SourcePID       = e.SourcePID;
+    TransportID     = e.TransportID;
+    NetworkID       = e.NetworkID;
+    ServiceID       = e.ServiceID;
+    EventID         = e.EventID;
+    StartTime       = e.StartTime;
+    EndTime         = e.EndTime;
+    OriginalAirDate = e.OriginalAirDate;
+    Stereo          = e.Stereo;
+    HDTV            = e.HDTV;
+    SubTitled       = e.SubTitled;
+    ETM_Location    = e.ETM_Location;
+    ATSC            = e.ATSC;
+    PartNumber      = e.PartNumber;
+    PartTotal       = e.PartTotal;
+
+    // QString is not thread safe, so we must make deep copies
+    LanguageCode       = QDeepCopy<QString>(e.LanguageCode);
+    Event_Name         = QDeepCopy<QString>(e.Event_Name);
+    Event_Subtitle     = QDeepCopy<QString>(e.Event_Subtitle);
+    Description        = QDeepCopy<QString>(e.Description);
+    ContentDescription = QDeepCopy<QString>(e.ContentDescription);
+    Year               = QDeepCopy<QString>(e.Year);
+    CategoryType       = QDeepCopy<QString>(e.CategoryType);
+    Actors             = QDeepCopy<QStringList>(e.Actors);
+
+    Credits.clear();
+    QValueList<Person>::const_iterator pit = e.Credits.begin();
+    for (; pit != e.Credits.end(); ++pit)
+        Credits.push_back(Person(QDeepCopy<QString>((*pit).role),
+                                 QDeepCopy<QString>((*pit).name)));
+}
+
 void Event::clearEventValues()
 {
-    SourcePID = 0;
+    SourcePID    = 0;
     LanguageCode = "";
-    Event_Name = "";
-    Description = "";
-    EventID = 0;
+    Event_Name   = "";
+    Description  = "";
+    EventID      = 0;
     ETM_Location = 0;
-    Event_Subtitle = "";
+    Event_Subtitle     = "";
     ContentDescription = "";
-    Year = "";
-    SubTitled = false;
-    Stereo = false;
-    HDTV = false;
-    ATSC = false;
-    //PreviouslyShown = false;
+    Year         = "";
+    SubTitled    = false;
+    Stereo       = false;
+    HDTV         = false;
+    ATSC         = false;
+    PartNumber   = 0;
+    PartTotal    = 0;
+    CategoryType = "";
     OriginalAirDate = QDate();
+    Actors.clear();
     Credits.clear();
 }
 
@@ -285,6 +357,73 @@ void NetworkObject::Reset()
     LinkagePresent = 0;
 }
 
+/** \fn PMTObject::PMTObject(const PMTObject&)
+ *  \brief Performs shallow copy.
+ */
+PMTObject::PMTObject(const PMTObject &other)
+{
+    shallowCopy(other);
+}
+
+/** \fn PMTObject::operator=(const PMTObject&)
+ *  \brief Performs shallow copy.
+ */
+PMTObject &PMTObject::operator=(const PMTObject &other)
+{
+    shallowCopy(other);
+    return *this;
+}
+
+/** \fn PMTObject::shallowCopy(const PMTObject&)
+ *  \brief Copies each field in PMT, but shares QPtrVector data.
+ */
+void PMTObject::shallowCopy(const PMTObject &other)
+{
+    PCRPID      = other.PCRPID;
+    ServiceID   = other.ServiceID;
+    PMTPID      = other.PMTPID;
+    CA          = other.CA;
+    Descriptors = other.Descriptors;
+    Components  = other.Components;
+    hasCA       = other.hasCA;
+    hasAudio    = other.hasAudio;
+    hasVideo    = other.hasVideo;
+}
+
+/** \fn PMTObject::deepCopy(const PMTObject&)
+ *  \brief Copies each simple field in PMT, and copies each element in the CA,
+ *         Descriptors and Components QPtrVector fields into new QPtrVector's.
+ */
+void PMTObject::deepCopy(const PMTObject &other)
+{
+    PCRPID      = other.PCRPID;
+    ServiceID   = other.ServiceID;
+    PMTPID      = other.PMTPID;
+
+    CA          = QDeepCopy<CAList>(other.CA);
+
+    Descriptors.clear();
+    DescriptorList::const_iterator dit;
+    for (dit = other.Descriptors.begin(); dit!=other.Descriptors.end(); ++dit)
+        Descriptors.append(Descriptor(*dit));
+
+    Components.clear();
+    QValueList<ElementaryPIDObject>::const_iterator eit;
+    for (eit = other.Components.begin(); eit!=other.Components.end(); ++eit)
+    {
+        ElementaryPIDObject obj;
+        obj.deepCopy(*eit);
+        Components.append(obj);
+    }
+
+    hasCA       = other.hasCA;
+    hasAudio    = other.hasAudio;
+    hasVideo    = other.hasVideo;
+}
+
+/** \fn PMTObject::Reset()
+ *  \brief Resets all values in the PMTObject to their initial values.
+ */
 void PMTObject::Reset()
 {
     ServiceID = 0;
@@ -331,7 +470,7 @@ void PATHandler::Request(uint16_t key)
 
 bool PATHandler::Complete()
 {
-    if (Tracker.Complete() && (!status.emitted))
+    if (Tracker.IsComplete() && (!status.emitted))
     {
         if (status.requestedEmit == false)
             status.emitted = true;
@@ -391,8 +530,11 @@ bool PMTHandler::EmitRequired()
     QMap_pullStatus::Iterator i;
     for (i = status.begin() ; i != status.end() ; ++i)
     {
-        if (i.data().requestedEmit && (i.data().emitted == false) && Tracker[i.key()].Complete())
+        if (i.data().requestedEmit && !i.data().emitted &&
+            Tracker[i.key()].IsComplete())
+        {
             return true;
+        }
     }
     return false;
 }
@@ -402,7 +544,8 @@ bool PMTHandler::GetEmitID(uint16_t& key0, uint16_t& key1)
     QMap_pullStatus::Iterator i;
     for (i = status.begin() ; i != status.end() ; ++i)
     {
-        if ((i.data().requestedEmit) && (i.data().emitted == false) && Tracker[i.key()].Complete())
+        if (i.data().requestedEmit && !i.data().emitted &&
+            Tracker[i.key()].IsComplete())
         {
             i.data().emitted = true;
             key0 = i.key();
@@ -468,7 +611,7 @@ void MGTHandler::Request(uint16_t key)
 
 bool MGTHandler::Complete()
 {
-    if (Tracker.Complete() && (!status.emitted))
+    if (Tracker.IsComplete() && !status.emitted)
     {
         if (status.requestedEmit == false)
             status.emitted = true;
@@ -515,7 +658,7 @@ void STTHandler::Request(uint16_t key)
 
 bool STTHandler::Complete()
 {
-    if (Tracker.Complete() && (!status.emitted))
+    if (Tracker.IsComplete() && !status.emitted)
     {
         if (status.requestedEmit == false)
             status.emitted = true;
@@ -619,7 +762,7 @@ bool EventHandler::RequirePIDs()
     QMap_pidHandler::Iterator i;
     for (i = EITpid.begin() ; i != EITpid.end() ; ++i)
     {
-        if(i.data().pulling == false)
+        if (i.data().pulling == false)
         {
             SetupTrackers();
             return true;
@@ -627,7 +770,7 @@ bool EventHandler::RequirePIDs()
     }
     for (i = ETTpid.begin() ; i != ETTpid.end() ; ++i)
     {
-        if(i.data().pulling == false)
+        if (i.data().pulling == false)
         {
             SetupTrackers();
             return true;
@@ -641,7 +784,7 @@ bool EventHandler::GetPIDs(uint16_t& pid, uint8_t& filter, uint8_t& mask)
     QMap_pidHandler::Iterator i;
     for (i = EITpid.begin() ; i != EITpid.end() ; ++i)
     {
-        if(i.data().pulling == false)
+        if (i.data().pulling == false)
         {
             pid = i.data().pid;
             filter = i.data().filter;
@@ -652,7 +795,7 @@ bool EventHandler::GetPIDs(uint16_t& pid, uint8_t& filter, uint8_t& mask)
     }
     for (i = ETTpid.begin() ; i != ETTpid.end() ; ++i)
     {
-        if(i.data().pulling == false)
+        if (i.data().pulling == false)
         {
             pid = i.data().pid;
             filter = i.data().filter;
@@ -667,11 +810,16 @@ bool EventHandler::GetPIDs(uint16_t& pid, uint8_t& filter, uint8_t& mask)
 void EventHandler::RequestEmit(uint16_t key)
 {
 #ifdef EIT_DEBUG_SID
-    fprintf(stdout,"EventHandler::RequestEmit for serviceid=%i\n", key);
+    VERBOSE(VB_EIT, QString("EventHandler::RequestEmit for serviceid = %1")
+            .arg(key));
 #endif
+
     status[key].requested = true;
     status[key].requestedEmit = true;
     status[key].emitted = false;
+// HACK begin -- this is probably not the right place to set this
+    status[key].pulling = true;
+// HACK end
 }
 
 bool EventHandler::EmitRequired()
@@ -689,29 +837,52 @@ bool EventHandler::EmitRequired()
         if (!(sttloaded))
             return false;
     }
+
     if (!(servicesloaded))
+    {
+#ifdef EIT_DEBUG_SID
+        static int n =0;
+        if (!(n++%100))
+            VERBOSE(VB_EIT, "EventHandler::EmitRequired no services");
+#endif
         return false;
+    }
 
     for (s = status.begin() ; s != status.end() ; ++s)
     {
-        if (s.data().emitted || !s.data().pulling)
+#ifdef EIT_DEBUG_SID
+        if (s.key() == EIT_DEBUG_SID &&
+            (s.data().emitted || !s.data().pulling))
         {
-            continue;
+            VERBOSE(VB_EIT, QString("EventHandler::EmitRequired %1: "
+                                    "not pulling").arg(EIT_DEBUG_SID));
         }
+#endif
+        if (s.data().emitted || !s.data().pulling)
+            continue;
+
+        if (!TrackerSetup[s.key()])
+            continue;
 
         AllComplete = true;
         /* Make sure all sections are being pulled otherwise your not done */
-        if (TrackerSetup[s.key()] == false)
-            AllComplete = false;
         for (i = Tracker[s.key()].begin() ; i != Tracker[s.key()].end() ; ++i)
         {
-            if (!i.data().Complete())
+            if (!i.data().IsComplete())
             {
+#ifdef EIT_DEBUG_SID
+                if (s.key() == EIT_DEBUG_SID)
+                {
+                    VERBOSE(VB_EIT, QString("EventHandler::EmitRequired %1:"
+                                            " 0x%2 is not complete")
+                            .arg(EIT_DEBUG_SID).arg(i.key(),0,16));
+                }
+#endif
                 AllComplete = false;
                 break;
             }
         }
-        if (SIStandard == SI_STANDARD_ATSC)
+        if (AllComplete && (SIStandard == SI_STANDARD_ATSC))
         {
             for (e = Events[s.key()].begin() ; e != Events[s.key()].end() ; ++e)
             {
@@ -732,6 +903,13 @@ bool EventHandler::EmitRequired()
                 }
             }
         }
+#ifdef EIT_DEBUG_SID
+        if (s.key() == EIT_DEBUG_SID)
+        {
+            VERBOSE(VB_EIT, "EventHandler::EmitRequired (End) "
+                    <<EIT_DEBUG_SID<<": AllComplete = "<<AllComplete);
+        }
+#endif
         if (AllComplete)
             return true;
     }
@@ -741,7 +919,6 @@ bool EventHandler::EmitRequired()
 bool EventHandler::GetEmitID(uint16_t& key0, uint16_t& key1)
 {
     QMap_pidHandler::Iterator p;
-    QMap_SectionTracker::Iterator i;
     QMap_pullStatus::Iterator s;
     QMap_Events::Iterator e;
     bool AllComplete;
@@ -752,14 +929,22 @@ bool EventHandler::GetEmitID(uint16_t& key0, uint16_t& key1)
         {
             AllComplete = true;
             if (TrackerSetup[s.key()] == false)
-                AllComplete = false;
-            /* Make sure all sections are being pulled otherwise your not done */
-            for (i = Tracker[s.key()].begin() ; i != Tracker[s.key()].end() ; ++i)
+                continue;
+
+            // Make sure all sections are being pulled,
+            // otherwise your not done.
+            QMap_SectionTracker::Iterator it = Tracker[s.key()].begin();
+            while (it != Tracker[s.key()].end())
             {
-                if (!(i.data().Complete()))
+                if (!(it.data().IsComplete()))
+                {
                     AllComplete = false;
+                    break;
+                }
+                ++it;
             }
-            if (SIStandard == SI_STANDARD_ATSC)
+
+            if (AllComplete && (SIStandard == SI_STANDARD_ATSC))
             {
                 for (e = Events[s.key()].begin() ; e != Events[s.key()].end() ; ++e)
                 {
@@ -837,11 +1022,26 @@ bool EventHandler::AddSection(tablehead_t *head, uint16_t key0, uint16_t key1)
     int retval = false;
 
 #ifdef EIT_DEBUG_SID
-    if (key0 == EIT_DEBUG_SID) {
-        printf("EventHandler::AddSection sid=%i eid=%i version=%i section %02x of %02x\n", key0, key1, head->version, head->section_number, head->section_last);
+    if (key0 == EIT_DEBUG_SID)
+    {
+        VERBOSE(VB_EIT, QString("EventHandler::AddSection sid(%1) eid(%2) "
+                                "version(%3) section 0x%4 of %0x%5")
+                .arg(key0).arg(key1).arg(head->version)
+                .arg(head->section_number,0,16).arg(head->section_last,0,16));
     }
 #endif
 
+// HACK begin -- this is probably not the right place to set this
+    // If we are interested in this but the tracker does not have any
+    // data in it then set the status back to pulling if appropriate.
+    if (status[key0].requestedEmit && status[key0].emitted &&
+        Tracker[key0][key1].IsEmpty())
+    {
+        status[key0].pulling = true;
+        status[key0].emitted = false;
+    }
+// HACK end
+
     if (SIStandard == SI_STANDARD_ATSC)
     {
         QMap_pidHandler::Iterator p;
@@ -857,9 +1057,10 @@ bool EventHandler::AddSection(tablehead_t *head, uint16_t key0, uint16_t key1)
     if (SIStandard == SI_STANDARD_DVB)
     {
 #ifdef EIT_DEBUG_SID
-        if (key0 == EIT_DEBUG_SID) {
-            QString foo = Tracker[key0][key1].loadStatus();
-            printf("%s\n", foo.ascii());
+        if (key0 == EIT_DEBUG_SID)
+        {
+            QString foo = Tracker[key0][key1].Status();
+            VERBOSE(VB_EIT, "EventHandler::AddSection Status: "<<foo);
         }
 #endif
         return Tracker[key0][key1].AddSection(head);
@@ -1001,7 +1202,7 @@ bool NetworkHandler::Complete()
     if (CompleteSent)
         return false;
 
-    if ((status.pulling == true) && (Tracker.Complete()) )
+    if (status.pulling && Tracker.IsComplete())
     {
         CompleteSent = true;
         return true;
@@ -1049,9 +1250,7 @@ void NetworkHandler::RequestEmit(uint16_t key)
 
 bool NetworkHandler::EmitRequired()
 {
-    if ((status.emitted == false) && (Tracker.Complete()) && (status.requestedEmit))
-        return true;
-    return false;
+    return !status.emitted && status.requestedEmit && Tracker.IsComplete();
 }
 
 bool NetworkHandler::GetEmitID(uint16_t& key0, uint16_t& key1)
