@@ -6,65 +6,67 @@
 #include "managedlist.h"
 #include "scheduledrecording.h"
 #include "recordingprofile.h"
-
+#include "playgroup.h"
 
 class SimpleSRSetting: public SimpleDBStorage
 {
     protected:
-        SimpleSRSetting(ScheduledRecording& _parent, QString name)
+        SimpleSRSetting(ScheduledRecording *_parent, QString name)
                       : SimpleDBStorage("record", name),
                         parent(_parent)
        {
-            parent.addChild(this);
+            parent->addChild(this);
             setName(name);
         }
 
         virtual QString setClause(void)
         {
             QString value = getValue();
-            value.replace(QRegExp("\""), QString("\\\""));
+            value.replace(QChar('\''), "''");
+            value.replace("\\", "\\\\");
 
-            return QString("recordid = %1, %2 = \"%3\"")
-                .arg(parent.getRecordID())
+            return QString("recordid = %1, %2 = '%3'")
+                .arg(parent->getRecordID())
                 .arg(getColumn())
                 .arg(value.utf8());
         }
 
         virtual QString whereClause(void)
         {
-            return QString("recordid = %1").arg(parent.getRecordID());
+            return QString("recordid = %1").arg(parent->getRecordID());
         }
 
-        ScheduledRecording& parent;
+        ScheduledRecording *parent;
 };
 
 
 class SRSetting: public ManagedListSetting
 {
     protected:
-        SRSetting(ScheduledRecording& _parent, QString name, ManagedList* _parentList=NULL)
+        SRSetting(ScheduledRecording *_parent, QString name, ManagedList* _parentList=NULL)
             : ManagedListSetting("record", name, _parentList),
               parent(_parent)
         {
-            _parent.addChild(this);
+            parent->addChild(this);
             setName(name);
         }
 
         virtual QString setClause(void) {
             QString value = getValue();
-            value.replace(QRegExp("\""), QString("\\\""));
+            value.replace(QChar('\''), "''");
+            value.replace("\\", "\\\\");
 
-            return QString("recordid = %1, %2 = \"%3\"")
-                .arg(parent.getRecordID())
+            return QString("recordid = %1, %2 = '%3'")
+                .arg(parent->getRecordID())
                 .arg(getColumn())
                 .arg(value.utf8());
         };
 
         virtual QString whereClause(void) {
-            return QString("recordid = %1").arg(parent.getRecordID());
+            return QString("recordid = %1").arg(parent->getRecordID());
             }
 
-        ScheduledRecording& parent;
+        ScheduledRecording *parent;
 };
 
 
@@ -72,65 +74,67 @@ class SRSetting: public ManagedListSetting
 class SRSelectSetting : public SelectManagedListSetting
 {
     protected:
-        SRSelectSetting(ScheduledRecording& _parent, const QString& listName,  const QString& listText,
+        SRSelectSetting(ScheduledRecording *_parent, const QString& listName,  const QString& listText,
                         ManagedListGroup* _group, QString _column, ManagedList* _parentList=NULL)
             : SelectManagedListSetting(listName, listText, _group, "record", _column, _parentList),
               parent(_parent)
         {
-            _parent.addChild(this);
+            parent->addChild(this);
             setName(_column);
         }
         virtual QString setClause(void) {
             QString value = getValue();
-            value.replace(QRegExp("\""), QString("\\\""));
+            value.replace(QChar('\''), "''");
+            value.replace("\\", "\\\\");
 
-            return QString("recordid = %1, %2 = \"%3\"")
-                .arg(parent.getRecordID())
+            return QString("recordid = %1, %2 = '%3'")
+                .arg(parent->getRecordID())
                 .arg(getColumn())
                 .arg(value.utf8());
         };
 
         virtual QString whereClause(void) {
-            return QString("recordid = %1").arg(parent.getRecordID());
+            return QString("recordid = %1").arg(parent->getRecordID());
             }
-        ScheduledRecording& parent;
+        ScheduledRecording *parent;
 };
 
 class SRBoolSetting : public BoolManagedListSetting
 {
     public:
-        SRBoolSetting(ScheduledRecording& _parent, const QString& trueText, const QString& falseText,
+        SRBoolSetting(ScheduledRecording *_parent, const QString& trueText, const QString& falseText,
                       const QString& ItemName, QString _column, ManagedListGroup* _group,
                       ManagedList* _parentList=NULL)
             : BoolManagedListSetting(trueText, falseText, ItemName, "record", _column, _group, _parentList),
               parent(_parent)
 
         {
-            _parent.addChild(this);
+            parent->addChild(this);
             setName(_column);
         }
 
         virtual QString setClause(void) {
             QString value = getValue();
-            value.replace(QRegExp("\""), QString("\\\""));
+            value.replace(QChar('\''), "''");
+            value.replace("\\", "\\\\");
 
-            return QString("recordid = %1, %2 = \"%3\"")
-                .arg(parent.getRecordID())
+            return QString("recordid = %1, %2 = '%3'")
+                .arg(parent->getRecordID())
                 .arg(getColumn())
                 .arg(value.utf8());
         };
 
         virtual QString whereClause(void) {
-            return QString("recordid = %1").arg(parent.getRecordID());
+            return QString("recordid = %1").arg(parent->getRecordID());
             }
-        ScheduledRecording& parent;
+        ScheduledRecording *parent;
 };
 
 
 class SRBoundedIntegerSetting : public BoundedIntegerManagedListSetting
 {
     public:
-        SRBoundedIntegerSetting(int _min, int _max, int _bigStep, int _step, ScheduledRecording& _parent,
+        SRBoundedIntegerSetting(int _min, int _max, int _bigStep, int _step, ScheduledRecording *_parent,
                                 const QString& ItemName, QString _column, ManagedListGroup* _group,
                                 ManagedList* _parentList=NULL,  bool _invert = false)
             : BoundedIntegerManagedListSetting(_min, _max, _bigStep, _step, ItemName, "record",
@@ -138,37 +142,38 @@ class SRBoundedIntegerSetting : public BoundedIntegerManagedListSetting
               parent(_parent)
 
         {
-            _parent.addChild(this);
+            parent->addChild(this);
             setName(_column);
         }
         virtual QString setClause(void)
         {
             QString value = getValue();
-            value.replace(QRegExp("\""), QString("\\\""));
+            value.replace(QChar('\''), "''");
+            value.replace("\\", "\\\\");
 
-            return QString("recordid = %1, %2 = \"%3\"")
-                .arg(parent.getRecordID())
+            return QString("recordid = %1, %2 = '%3'")
+                .arg(parent->getRecordID())
                 .arg(getColumn())
                 .arg(value.utf8());
         };
 
         virtual QString whereClause(void)
         {
-            return QString("recordid = %1").arg(parent.getRecordID());
+            return QString("recordid = %1").arg(parent->getRecordID());
         }
 
-        ScheduledRecording& parent;
+        ScheduledRecording *parent;
 };
 
 
 class SRRecordingType : public SRSelectSetting
 {
     public:
-        SRRecordingType(ScheduledRecording& _parent, ManagedList* _list, ManagedListGroup* _group)
+        SRRecordingType(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "typeList", QString("[ %1 ]").arg(QObject::tr("Select Recording Schedule")),
                               _group, "type", _list)
         {
-            _parent.setRecTypeObj(this);
+            _parent->setRecTypeObj(this);
         }
 
 
@@ -211,7 +216,7 @@ class SRRecordingType : public SRSelectSetting
 class SRStartOffset : public SRBoundedIntegerSetting
 {
     public:
-        SRStartOffset(ScheduledRecording& _parent, ManagedListGroup* _group, ManagedList* _list)
+        SRStartOffset(ScheduledRecording *_parent, ManagedListGroup* _group, ManagedList* _list)
                      : SRBoundedIntegerSetting( -480, 480, 10, 1, _parent, "startoffsetList", "startoffset",
                                                 _group, _list, true)
         {
@@ -221,7 +226,7 @@ class SRStartOffset : public SRBoundedIntegerSetting
                          QObject::tr("Start recording %1 minute early"),
                          QObject::tr("Start recording %1 minutes early"));
 
-            _parent.setStartOffsetObj(this);
+            _parent->setStartOffsetObj(this);
         }
 };
 
@@ -230,7 +235,7 @@ class SRStartOffset : public SRBoundedIntegerSetting
 class SREndOffset : public SRBoundedIntegerSetting
 {
     public:
-        SREndOffset(ScheduledRecording& _parent, ManagedListGroup* _group, ManagedList* _list)
+        SREndOffset(ScheduledRecording *_parent, ManagedListGroup* _group, ManagedList* _list)
                      : SRBoundedIntegerSetting( -480, 480, 10, 1, _parent, "endoffsetList", "endoffset",
                                                 _group, _list)
         {
@@ -239,7 +244,7 @@ class SREndOffset : public SRBoundedIntegerSetting
                          QObject::tr("End recording on time"), QObject::tr("End recording %1 minute late"),
                          QObject::tr("End recording %1 minutes late"));
 
-            _parent.setEndOffsetObj(this);
+            _parent->setEndOffsetObj(this);
         }
 };
 
@@ -247,7 +252,7 @@ class SREndOffset : public SRBoundedIntegerSetting
 class SRDupIn : public SRSelectSetting
 {
     public:
-        SRDupIn(ScheduledRecording& _parent, ManagedList* _list, ManagedListGroup* _group)
+        SRDupIn(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "dupInList", "[ Check for duplicates in ]", _group, "dupin", _list )
         {
             if (gContext->GetNumSetting("HaveRepeats", 0))
@@ -256,7 +261,7 @@ class SRDupIn : public SRSelectSetting
             addSelection(QObject::tr("Look for duplicates in current recordings only"), kDupsInRecorded);
             addSelection(QObject::tr("Look for duplicates in previous recordings only"), kDupsInOldRecorded);
             setValue(kDupsInAll);
-            _parent.setDupInObj(this);
+            _parent->setDupInObj(this);
         }
 };
 
@@ -264,7 +269,7 @@ class SRDupIn : public SRSelectSetting
 class SRDupMethod: public SRSelectSetting
 {
     public:
-        SRDupMethod(ScheduledRecording& _parent, ManagedList* _list, ManagedListGroup* _group)
+        SRDupMethod(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "dupMethodList", QObject::tr("[ Match duplicates with ]"), _group,
                               "dupmethod", _list)
         {
@@ -273,7 +278,7 @@ class SRDupMethod: public SRSelectSetting
             addSelection(QObject::tr("Match duplicates using description"), kDupCheckDesc);
             addSelection(QObject::tr("Don't match duplicates"), kDupCheckNone);
             setValue(kDupCheckSubDesc);
-            _parent.setDupMethodObj(this);
+            _parent->setDupMethodObj(this);
         }
 };
 
@@ -281,7 +286,7 @@ class SRDupMethod: public SRSelectSetting
 class SRRecSearchType: public IntegerSetting, public SimpleSRSetting
 {
     public:
-        SRRecSearchType(ScheduledRecording& parent):
+        SRRecSearchType(ScheduledRecording *parent):
             SimpleSRSetting(parent, "search") {
             setVisible(false);
         }
@@ -290,11 +295,11 @@ class SRRecSearchType: public IntegerSetting, public SimpleSRSetting
 
 class SRProfileSelector: public SRSelectSetting {
     public:
-        SRProfileSelector(ScheduledRecording& _parent, ManagedList* _list, ManagedListGroup* _group)
+        SRProfileSelector(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "profileList", QObject::tr("[ Select recording Profile ]"), _group,
                               "profile", _list )
         {
-            _parent.setProfileObj(this);
+            _parent->setProfileObj(this);
         }
 
 
@@ -314,7 +319,7 @@ class SRSchedOptionsGroup : public ManagedListGroup
     Q_OBJECT
 
     public:
-        SRSchedOptionsGroup(ScheduledRecording& _rec, ManagedList* _list, ManagedListGroup* _group, 
+        SRSchedOptionsGroup(ScheduledRecording *_rec, ManagedList* _list, ManagedListGroup* _group, 
                             QObject* _parent);
 
         void setEnabled(bool isScheduled, bool multiEpisode);
@@ -332,7 +337,7 @@ class SRSchedOptionsGroup : public ManagedListGroup
         class SRDupMethod* dupMethItem;
         class SRDupIn* dupLocItem;
 
-        ScheduledRecording& schedRec;
+        ScheduledRecording *schedRec;
 };
 
 class SRJobQueueGroup : public ManagedListGroup
@@ -340,19 +345,37 @@ class SRJobQueueGroup : public ManagedListGroup
     Q_OBJECT
     
     public:
-        SRJobQueueGroup(ScheduledRecording& _rec, ManagedList* _list, ManagedListGroup* _group, QObject* _parent);
+        SRJobQueueGroup(ScheduledRecording *_rec, ManagedList* _list, ManagedListGroup* _group, QObject* _parent);
         
     protected:
 
         friend class SRRootGroup;
         class SRAutoTranscode* autoTranscode;
+        class SRTranscoderSelector* transcoder;
         class SRAutoCommFlag* autoCommFlag;
         class SRAutoUserJob1* autoUserJob1;
         class SRAutoUserJob2* autoUserJob2;
         class SRAutoUserJob3* autoUserJob3;
         class SRAutoUserJob4* autoUserJob4;
 
-        ScheduledRecording& schedRec;
+        ScheduledRecording *schedRec;
+};
+
+class SRSchedInfoGroup : public ManagedListGroup
+{
+    Q_OBJECT
+    
+    public:
+        SRSchedInfoGroup(ScheduledRecording *_rec, ManagedList* _list, ManagedListGroup* _group, QObject* _parent);
+        
+    protected:
+
+        friend class SRRootGroup;
+        ManagedListItem* detailsButton;
+        ManagedListItem* previousButton;
+        ManagedListItem* upcomingButton;
+
+        ScheduledRecording *schedRec;
 };
 
 
@@ -361,7 +384,7 @@ class SRStorageOptionsGroup : public ManagedListGroup
     Q_OBJECT
 
     public:
-        SRStorageOptionsGroup(ScheduledRecording& _rec, ManagedList* _list, ManagedListGroup* _group, 
+        SRStorageOptionsGroup(ScheduledRecording *_rec, ManagedList* _list, ManagedListGroup* _group, 
                               QObject* _parent);
 
         void setEnabled(bool isScheduled, bool multiEpisode);
@@ -374,18 +397,19 @@ class SRStorageOptionsGroup : public ManagedListGroup
         friend class RootSRGroup;
         class SRProfileSelector* profile;
         class SRRecGroup* recGroup;
+        class SRPlayGroup* playGroup;
         class SRAutoExpire* autoExpire;
         class SRMaxEpisodes* maxEpisodes;
         class SRMaxNewest* maxNewest;
 
-        ScheduledRecording& schedRec;
+        ScheduledRecording *schedRec;
 };
 
 
 class SRChannel: public ChannelSetting, public SimpleSRSetting
 {
     public:
-        SRChannel(ScheduledRecording& parent): SimpleSRSetting(parent, "chanid") {
+        SRChannel(ScheduledRecording *parent): SimpleSRSetting(parent, "chanid") {
             setVisible(false);
         }
 };
@@ -393,7 +417,7 @@ class SRChannel: public ChannelSetting, public SimpleSRSetting
 class SRStation: public LineEditSetting, public SimpleSRSetting
 {
     public:
-        SRStation(ScheduledRecording& parent): SimpleSRSetting(parent, "station") {
+        SRStation(ScheduledRecording *parent): SimpleSRSetting(parent, "station") {
             setVisible(false);
         }
 };
@@ -401,7 +425,7 @@ class SRStation: public LineEditSetting, public SimpleSRSetting
 class SRTitle: public LineEditSetting, public SimpleSRSetting
 {
     public:
-        SRTitle(ScheduledRecording& parent)
+        SRTitle(ScheduledRecording *parent)
               : SimpleSRSetting(parent, "title")
         {
             setVisible(false);
@@ -411,7 +435,7 @@ class SRTitle: public LineEditSetting, public SimpleSRSetting
 class SRSubtitle: public LineEditSetting, public SimpleSRSetting
 {
     public:
-        SRSubtitle(ScheduledRecording& parent)
+        SRSubtitle(ScheduledRecording *parent)
                  : SimpleSRSetting(parent, "subtitle")
         {
             setVisible(false);
@@ -421,7 +445,7 @@ class SRSubtitle: public LineEditSetting, public SimpleSRSetting
 class SRDescription: public LineEditSetting, public SimpleSRSetting
 {
     public:
-        SRDescription(ScheduledRecording& parent)
+        SRDescription(ScheduledRecording *parent)
                     : SimpleSRSetting(parent, "description")
         {
             setVisible(false);
@@ -431,7 +455,7 @@ class SRDescription: public LineEditSetting, public SimpleSRSetting
 class SRStartTime: public TimeSetting, public SimpleSRSetting
 {
     public:
-        SRStartTime(ScheduledRecording& parent)
+        SRStartTime(ScheduledRecording *parent)
                   : SimpleSRSetting(parent, "starttime")
         {
             setVisible(false);
@@ -441,7 +465,7 @@ class SRStartTime: public TimeSetting, public SimpleSRSetting
 class SRStartDate: public DateSetting, public SimpleSRSetting
 {
     public:
-        SRStartDate(ScheduledRecording& parent)
+        SRStartDate(ScheduledRecording *parent)
                   : SimpleSRSetting(parent, "startdate")
         {
             setVisible(false);
@@ -451,7 +475,7 @@ class SRStartDate: public DateSetting, public SimpleSRSetting
 class SREndTime: public TimeSetting, public SimpleSRSetting
 {
     public:
-        SREndTime(ScheduledRecording& parent)
+        SREndTime(ScheduledRecording *parent)
                 : SimpleSRSetting(parent, "endtime")
         {
             setVisible(false);
@@ -461,7 +485,7 @@ class SREndTime: public TimeSetting, public SimpleSRSetting
 class SREndDate: public DateSetting, public SimpleSRSetting
 {
     public:
-        SREndDate(ScheduledRecording& parent)
+        SREndDate(ScheduledRecording *parent)
                 : SimpleSRSetting(parent, "enddate")
         {
             setVisible(false);
@@ -472,7 +496,7 @@ class SREndDate: public DateSetting, public SimpleSRSetting
 class SRCategory: public LineEditSetting, public SimpleSRSetting
 {
     public:
-        SRCategory(ScheduledRecording& parent)
+        SRCategory(ScheduledRecording *parent)
                  : SimpleSRSetting(parent, "category")
         {
             setVisible(false);
@@ -482,7 +506,7 @@ class SRCategory: public LineEditSetting, public SimpleSRSetting
 class SRSeriesid: public LineEditSetting, public SimpleSRSetting
 {
     public:
-        SRSeriesid(ScheduledRecording& parent)
+        SRSeriesid(ScheduledRecording *parent)
                  : SimpleSRSetting(parent, "seriesid")
         {
             setVisible(false);
@@ -492,7 +516,7 @@ class SRSeriesid: public LineEditSetting, public SimpleSRSetting
 class SRProgramid: public LineEditSetting, public SimpleSRSetting
 {
     public:
-        SRProgramid(ScheduledRecording& parent)
+        SRProgramid(ScheduledRecording *parent)
                   : SimpleSRSetting(parent, "programid")
         {
             setVisible(false);
@@ -502,7 +526,7 @@ class SRProgramid: public LineEditSetting, public SimpleSRSetting
 class SRFindDay: public IntegerSetting, public SimpleSRSetting
 {
     public:
-        SRFindDay(ScheduledRecording& parent)
+        SRFindDay(ScheduledRecording *parent)
             : SimpleSRSetting(parent, "findday")
         {
             setVisible(false);
@@ -512,7 +536,7 @@ class SRFindDay: public IntegerSetting, public SimpleSRSetting
 class SRFindTime: public TimeSetting, public SimpleSRSetting
 {
     public:
-        SRFindTime(ScheduledRecording& parent)
+        SRFindTime(ScheduledRecording *parent)
             : SimpleSRSetting(parent, "findtime")
         {
             setVisible(false);
@@ -522,7 +546,7 @@ class SRFindTime: public TimeSetting, public SimpleSRSetting
 class SRFindId: public IntegerSetting, public SimpleSRSetting
 {
     public:
-        SRFindId(ScheduledRecording& parent)
+        SRFindId(ScheduledRecording *parent)
             : SimpleSRSetting(parent, "findid")
         {
             setVisible(false);
@@ -532,7 +556,7 @@ class SRFindId: public IntegerSetting, public SimpleSRSetting
 class SRParentId: public IntegerSetting, public SimpleSRSetting
 {
     public:
-        SRParentId(ScheduledRecording& parent)
+        SRParentId(ScheduledRecording *parent)
             : SimpleSRSetting(parent, "parentid")
         {
             setVisible(false);
@@ -542,35 +566,59 @@ class SRParentId: public IntegerSetting, public SimpleSRSetting
 class SRAutoTranscode: public SRSelectSetting 
 {
     public:
-        SRAutoTranscode(ScheduledRecording& _parent, ManagedList* _list, ManagedListGroup* _group)
+        SRAutoTranscode(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "autoTranscodeList", "[ Automatically Transcode ]", _group, 
                               "autotranscode", _list) 
         {
-            addSelection("Transcode new recordings", 1);
-            addSelection("Do not Transcode new recordings", 0);
+            addSelection(QObject::tr("Transcode new recordings"), 1);
+            addSelection(QObject::tr("Do not Transcode new recordings"), 0);
             setValue(0);
-            _parent.setAutoTranscodeObj(this);
+            _parent->setAutoTranscodeObj(this);
+        }
+};
+
+class SRTranscoderSelector: public SRSelectSetting {
+    public:
+        SRTranscoderSelector(ScheduledRecording *_parent, ManagedList* _list,
+                             ManagedListGroup* _group)
+            : SRSelectSetting(_parent, "transcoderList",
+                              QObject::tr("[ Select transcoder ]"), _group,
+                              "transcoder", _list )
+        {
+            _parent->setTranscoderObj(this);
+        }
+
+
+        virtual void load() {
+            fillSelections();
+            SRSelectSetting::load();
+        }
+
+        virtual void fillSelections() {
+            clearSelections();
+            RecordingProfile::fillSelections(selectItem,
+                RecordingProfile::TranscoderGroup);
         }
 };
 
 class SRAutoCommFlag: public SRSelectSetting 
 {
     public:
-        SRAutoCommFlag(ScheduledRecording& _parent, ManagedList* _list, ManagedListGroup* _group)
+        SRAutoCommFlag(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "autoCommFlagList", "[ Automatically Commercial Flag ]", _group, 
                               "autocommflag", _list) 
         {
             addSelection(QObject::tr("Commercial Flag new recordings"), 1);
             addSelection(QObject::tr("Do not Commercial Flag new recordings"), 0);
             setValue(1);
-            _parent.setAutoCommFlagObj(this);
+            _parent->setAutoCommFlagObj(this);
         }
 };
 
 class SRAutoUserJob1: public SRSelectSetting 
 {
     public:
-        SRAutoUserJob1(ScheduledRecording& _parent, ManagedList* _list, ManagedListGroup* _group)
+        SRAutoUserJob1(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "autoUserJob1List", "[ Automatically Run User Job #1 ]", _group, 
                               "autouserjob1", _list) 
         {
@@ -578,14 +626,14 @@ class SRAutoUserJob1: public SRSelectSetting
             addSelection(QObject::tr("Run '%1'").arg(desc), 1);
             addSelection(QObject::tr("Do not run '%1' for new recordings").arg(desc), 0);
             setValue(0);
-            _parent.setAutoUserJob1Obj(this);
+            _parent->setAutoUserJob1Obj(this);
         }
 };
 
 class SRAutoUserJob2: public SRSelectSetting 
 {
     public:
-        SRAutoUserJob2(ScheduledRecording& _parent, ManagedList* _list, ManagedListGroup* _group)
+        SRAutoUserJob2(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "autoUserJob2List", "[ Automatically Run User Job #2 ]", _group, 
                               "autouserjob2", _list) 
         {
@@ -593,14 +641,14 @@ class SRAutoUserJob2: public SRSelectSetting
             addSelection(QObject::tr("Run '%1'").arg(desc), 1);
             addSelection(QObject::tr("Do not run '%1' for new recordings").arg(desc), 0);
             setValue(0);
-            _parent.setAutoUserJob2Obj(this);
+            _parent->setAutoUserJob2Obj(this);
         }
 };
 
 class SRAutoUserJob3: public SRSelectSetting 
 {
     public:
-        SRAutoUserJob3(ScheduledRecording& _parent, ManagedList* _list, ManagedListGroup* _group)
+        SRAutoUserJob3(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "autoUserJob3List", "[ Automatically Run User Job #3 ]", _group, 
                               "autouserjob3", _list) 
         {
@@ -608,14 +656,14 @@ class SRAutoUserJob3: public SRSelectSetting
             addSelection(QObject::tr("Run '%1'").arg(desc), 1);
             addSelection(QObject::tr("Do not run '%1' for new recordings").arg(desc), 0);
             setValue(0);
-            _parent.setAutoUserJob3Obj(this);
+            _parent->setAutoUserJob3Obj(this);
         }
 };
 
 class SRAutoUserJob4: public SRSelectSetting 
 {
     public:
-        SRAutoUserJob4(ScheduledRecording& _parent, ManagedList* _list, ManagedListGroup* _group)
+        SRAutoUserJob4(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "autoUserJob4List", "[ Automatically Run User Job #4 ]", _group, 
                               "autouserjob4", _list) 
         {
@@ -623,44 +671,44 @@ class SRAutoUserJob4: public SRSelectSetting
             addSelection(QObject::tr("Run '%1'").arg(desc), 1);
             addSelection(QObject::tr("Do not run '%1' for new recordings").arg(desc), 0);
             setValue(0);
-            _parent.setAutoUserJob4Obj(this);
+            _parent->setAutoUserJob4Obj(this);
         }
 };
 
 class SRAutoExpire: public SRBoolSetting
 {
     public:
-        SRAutoExpire(ScheduledRecording& _parent, ManagedListGroup* _group, ManagedList* _list)
+        SRAutoExpire(ScheduledRecording *_parent, ManagedListGroup* _group, ManagedList* _list)
                     : SRBoolSetting(_parent, QObject::tr("Allow auto expire"), 
                                      QObject::tr("Don't allow auto expire"),
                                     "autoExpireItem", "autoexpire", _group, _list )
         {
-            _parent.setAutoExpireObj(this);
+            _parent->setAutoExpireObj(this);
         }
 };
 
 class SRInactive: public SRBoolSetting
 {
     public:
-        SRInactive(ScheduledRecording& _parent, ManagedListGroup* _group, ManagedList* _list)
+        SRInactive(ScheduledRecording *_parent, ManagedListGroup* _group, ManagedList* _list)
                     : SRBoolSetting(_parent, QObject::tr("This recording schedule is inactive"), 
                                      QObject::tr("This recording schedule is active"),
                                     "inactiveItem", "inactive", _group, _list )
         {
-            _parent.setInactiveObj(this);
+            _parent->setInactiveObj(this);
         }
 };
 
 class SRMaxNewest: public SRBoolSetting
 {
     public:
-        SRMaxNewest(ScheduledRecording& _parent, ManagedListGroup* _group, ManagedList* _list)
+        SRMaxNewest(ScheduledRecording *_parent, ManagedListGroup* _group, ManagedList* _list)
                    : SRBoolSetting(_parent, QObject::tr("Delete oldest if this would exceed the max episodes"),
                                    QObject::tr("Don't record if this would exceed the max episodes"),
                                    "maxnewestItem", "maxnewest", _group, _list )
         {
             setValue(false);
-            _parent.setMaxNewestObj(this);
+            _parent->setMaxNewestObj(this);
         }
 };
 
@@ -670,13 +718,13 @@ class SRMaxNewest: public SRBoolSetting
 class SRMaxEpisodes : public SRBoundedIntegerSetting
 {
     public:
-    SRMaxEpisodes(ScheduledRecording& _parent, ManagedListGroup* _group, ManagedList* _list)
+    SRMaxEpisodes(ScheduledRecording *_parent, ManagedListGroup* _group, ManagedList* _list)
                  : SRBoundedIntegerSetting( 0, 100, 5, 1, _parent, "maxepisodesList", "maxepisodes",
                                             _group, _list)
     {
         setTemplates("", "", QObject::tr("No episode limit"), 
                      QObject::tr("Keep only one episode."), QObject::tr("Keep at most %1 episodes"));
-        _parent.setMaxEpisodesObj(this);
+        _parent->setMaxEpisodesObj(this);
     }
 };
 
@@ -684,7 +732,7 @@ class SRMaxEpisodes : public SRBoundedIntegerSetting
 class SRRecPriority: public SRBoundedIntegerSetting
 {
     public:
-        SRRecPriority(ScheduledRecording& _parent, ManagedListGroup* _group, ManagedList* _list)
+        SRRecPriority(ScheduledRecording *_parent, ManagedListGroup* _group, ManagedList* _list)
                     : SRBoundedIntegerSetting( -99, 99, 5, 1, _parent, "recpriorityList", "recpriority",
                                                _group, _list)
         {
@@ -692,7 +740,7 @@ class SRRecPriority: public SRBoundedIntegerSetting
                           QObject::tr("Normal recording priority"),
                           QObject::tr("Raise priority by %1"), QObject::tr("Raise priority by %1") );
             setValue(0);
-            _parent.setRecPriorityObj(this);
+            _parent->setRecPriorityObj(this);
         }
 };
 
@@ -702,13 +750,13 @@ class SRRecGroup: public SRSelectSetting
     Q_OBJECT
 
     public:
-        SRRecGroup(ScheduledRecording& _parent, ManagedList* _list, ManagedListGroup* _group)
+        SRRecGroup(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "recgroupList", QString("[ %1 ]")
                                                        .arg(QObject::tr("Select Recording Group")),
                               _group, "recgroup", _list )
         {
             setValue("Default");
-            _parent.setRecGroupObj(this);
+            _parent->setRecGroupObj(this);
             connect(selectItem, SIGNAL(goingBack()), this, SLOT(onGoingBack()));
         }
 
@@ -727,40 +775,61 @@ class SRRecGroup: public SRSelectSetting
 
         virtual void fillSelections()
         {
+            QStringList groups;
+            QStringList::Iterator it;
+            QString value, dispValue;
+            bool foundDefault = false;
+
             addButton(QString("[ %1 ]").arg(QObject::tr("Create a new recording group")), "__NEW_GROUP__");
             connect(selectItem, SIGNAL(buttonPressed(ManagedListItem*, ManagedListItem*)), this,
                                        SLOT(showNewRecGroup()));
 
-            addSelection(QString(QObject::tr("Store in the \"%1\" recording group"))
-                         .arg(QObject::tr("Default")), "Default");
             MSqlQuery query(MSqlQuery::InitCon());
-            QString thequery = QString("SELECT DISTINCT recgroup from recorded "
-                                       "WHERE recgroup <> 'Default'");
-            query.prepare(thequery);
+            query.prepare("SELECT DISTINCT recgroup FROM recorded");
 
             if (query.exec() && query.isActive() && query.size() > 0)
+                while (query.next())
+                {
+                    value = QString::fromUtf8(query.value(0).toString());
+                    groups += value;
+
+                    if (value == "Default")
+                        foundDefault = true;
+                }
+
+
+            query.prepare("SELECT DISTINCT recgroup FROM record");
+
+            if (query.exec() && query.isActive() && query.size() > 0)
+                while (query.next())
+                {
+                    value = QString::fromUtf8(query.value(0).toString());
+                    groups += value;
+
+                    if (value == "Default")
+                        foundDefault = true;
+                }
+
+            groups.sort();
+            for (it = groups.begin(); it != groups.end(); ++it)
             {
-                while (query.next())
+                if (!foundDefault && *it > QObject::tr("Default"))
                 {
-                    QString recgroup = QString::fromUtf8(query.value(0).toString());
-
-                    addSelection(QString(QObject::tr("Store in the \"%1\" recording group")).arg(recgroup),
-                                 recgroup);
+                    addSelection(QObject::tr(
+                                 "Store in the \"%1\" recording group")
+                                 .arg(QObject::tr("Default")), "Default");
+                    foundDefault = true;
                 }
+
+                if (*it == "Default")
+                    dispValue = QObject::tr("Default");
+                else
+                    dispValue = *it;
+
+                addSelection(QObject::tr("Store in the \"%1\" recording group")
+                                         .arg(dispValue), *it);
             }
 
-            thequery = QString("SELECT DISTINCT recgroup from record "
-                               "WHERE recgroup <> 'Default'");
-            query.prepare(thequery);
-
-            if (query.exec() && query.isActive() && query.size() > 0)
-                while (query.next())
-                {
-                    QString recgroup = QString::fromUtf8(query.value(0).toString());
-                    addSelection(QString(QObject::tr("Store in the \"%1\" recording group")).arg(recgroup),
-                                 recgroup);
-                }
-
         }
 
 
@@ -768,5 +837,32 @@ class SRRecGroup: public SRSelectSetting
         void showNewRecGroup();
         void onGoingBack();
 };
+
+class SRPlayGroup: public SRSelectSetting
+{
+    Q_OBJECT
+
+    public:
+        SRPlayGroup(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
+            : SRSelectSetting(_parent, "playgroupList", QString("[ %1 ]")
+                              .arg(QObject::tr("Select Playback Group")),
+                              _group, "playgroup", _list )
+        {
+            setValue("Default");
+            _parent->setPlayGroupObj(this);
+        }
+
+        virtual void load() {
+            fillSelections();
+            SRSelectSetting::load();
+        }
+
+        virtual QString getValue(void) const {
+            return settingValue;
+        }
+
+        virtual void fillSelections();
+};
+
 #endif
 
