@@ -1,9 +1,7 @@
 #ifndef CC_H
 #define CC_H
 
-#include <qstring.h>
-
-#define CC_VBIBUFSIZE 65536
+#define CC_VBIBUFSIZE 65536*2
 
 //cc is 32 columns per row, this allows for extra characters
 #define CC_BUFSIZE 256
@@ -12,16 +10,20 @@ struct cc
 {
     int fd;
     char buffer[CC_VBIBUFSIZE];
-    int lastcode;
-    int ccmode;	// 0=cc1 or 1=cc2
-    QString ccbuf[2];
+    int code1;
+    int code2;
+
+    int samples_per_line;
+    int start_line;
+    int line_count;
 };
 
 int cc_decode(unsigned char *vbiline);
+void cc_decode(struct cc *cc);
 
 struct cc *cc_open(const char *vbi_name);
 void cc_close(struct cc *cc);
-int cc_handler(struct cc *cc);
+void cc_handler(struct cc *cc);
 
 #endif
 
