@@ -17,6 +17,7 @@ using namespace std;
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <directfb_version.h>
 
 #define DFBCHECKFAIL(dfbcommand, returnstmt...)\
 {\
@@ -182,7 +183,8 @@ static const unsigned int QT_KEYS[DIKI_NUMBER_OF_KEYS][2] =
 //8 1 3 2 works, but not for everyone :-(
 const int kNumBuffers = 31;
 const int kNeedFreeFrames = 1;
-const int kPrebufferFrames = 12;
+const int kPrebufferFramesNormal = 12;
+const int kPrebufferFramesSmall = 4;
 const int kKeepPrebuffer = 2;
 typedef map<unsigned char *, IDirectFBSurface *> BufferMap;
 
@@ -191,7 +193,11 @@ struct DirectfbData
     //DirectFB hook
     IDirectFB *dfb;
     //video output
+#if (DIRECTFB_MINOR_VERSION <= 9) && (DIRECTFB_MICRO_VERSION <= 22)
     DFBCardCapabilities cardCapabilities;
+#else
+    DFBGraphicsDeviceDescription cardDescription;
+#endif
     IDirectFBDisplayLayer *primaryLayer;
     IDirectFBSurface *primarySurface;
     IDirectFBDisplayLayer *videoLayer;
@@ -306,12 +312,13 @@ int VideoOutputDirectfb::GetRefreshRate(void)
 }
 
 bool VideoOutputDirectfb::Init(int width, int height, float aspect, WId winid,
-                               int winx, int winy, int winw, int winh, WId embedid)
+                               int winx, int winy, int /*winw*/, int /*winh*/,
+                               WId embedid)
 {
     DFBResult ret;
     DFBSurfaceDescription desc;
     DFBDisplayLayerConfig conf;
-    DFBDisplayLayerDescription ldesc;
+    //DFBDisplayLayerDescription ldesc;
 
     widget = QWidget::find(winid);
 
@@ -331,17 +338,30 @@ bool VideoOutputDirectfb::Init(int width, int height, float aspect, WId winid,
     DFBCHECKFAIL(data->primaryLayer->GetConfiguration(data->primaryLayer, &conf), false);
     data->screen_width = conf.width;
     data->screen_height = conf.height;
+    display_aspect = ((float)(conf.width)) / ((float)(conf.height));
+
 
     //determine output card capacities
+#if (DIRECTFB_MINOR_VERSION <= 9) && (DIRECTFB_MICRO_VERSION <= 22)
     DFBCHECKFAIL(data->dfb->GetCardCapabilities(data->dfb, &(data->cardCapabilities)), false);
     VERBOSE(VB_GENERAL, QString("DirectFB output : card : %1")
             .arg((data->cardCapabilities.acceleration_mask & DFXL_BLIT) > 0 ?
                  "hardware blit support" : "NO hardware blit support"));
+#else
+    DFBCHECKFAIL(data->dfb->GetDeviceDescription(data->dfb, &(data->cardDescription)), false);
+    VERBOSE(VB_GENERAL, QString("DirectFB output : card : %1")
+            .arg((data->cardDescription.acceleration_mask & DFXL_BLIT) > 0 ?
+                 "hardware blit support" : "NO hardware blit support"));
+#endif
 
     //clear primary layer
     desc.flags = DSDESC_CAPS;
     desc.caps = DSCAPS_PRIMARY;
+#if (DIRECTFB_MINOR_VERSION <= 9) && (DIRECTFB_MICRO_VERSION <= 22)
     if (data->cardCapabilities.acceleration_mask & DFXL_BLIT)
+#else
+    if (data->cardDescription.acceleration_mask & DFXL_BLIT)
+#endif
         desc.caps = (DFBSurfaceCapabilities)(desc.caps | DSCAPS_DOUBLE);
     DFBCHECKFAIL(data->dfb->CreateSurface(data->dfb, &desc, &(data->primarySurface)), false);
     DFBCHECKFAIL(data->primarySurface->Clear(data->primarySurface, 0, 0, 0, 0xff), false);
@@ -414,8 +434,9 @@ bool VideoOutputDirectfb::Init(int width, int height, float aspect, WId winid,
            );
 
     //setup video input buffers
-    VideoOutput::InitBuffers(kNumBuffers, true, kNeedFreeFrames,
-                             kPrebufferFrames, kKeepPrebuffer);
+    vbuffers.Init(kNumBuffers, true, kNeedFreeFrames,
+                  kPrebufferFramesNormal, kPrebufferFramesSmall, 
+                  kKeepPrebuffer);
     desc.flags = (DFBSurfaceDescriptionFlags)(DSDESC_HEIGHT | DSDESC_WIDTH | DSDESC_PIXELFORMAT);
     desc.width = width;
     desc.height = height;
@@ -449,13 +470,13 @@ bool VideoOutputDirectfb::Init(int width, int height, float aspect, WId winid,
 
     //this stuff is right from Xv - look at this sometime
     //first frame of the buffers
-    scratchFrame = &(vbuffers[kNumBuffers]);
 
-    pauseFrame.height = scratchFrame->height;
-    pauseFrame.width = scratchFrame->width;
-    pauseFrame.bpp = scratchFrame->bpp;
-    pauseFrame.size = scratchFrame->size;
-    pauseFrame.buf = new unsigned char[pauseFrame.size];
+    pauseFrame.height = vbuffers.GetScratchFrame()->height;
+    pauseFrame.width  = vbuffers.GetScratchFrame()->width;
+    pauseFrame.bpp    = vbuffers.GetScratchFrame()->bpp;
+    pauseFrame.size   = vbuffers.GetScratchFrame()->size;
+    pauseFrame.buf    = new unsigned char[pauseFrame.size];
+    pauseFrame.frameNumber = vbuffers.GetScratchFrame()->frameNumber;
 
     VideoOutput::Init(width, height, aspect, winid, winx, winy, data->screen_width, data->screen_height,
                       embedid);
@@ -474,29 +495,20 @@ bool VideoOutputDirectfb::Init(int width, int height, float aspect, WId winid,
     //display video output
     DFBCHECK(data->videoLayer->SetOpacity(data->videoLayer, 0xff));
 
-    if (myth_dsw != 0)
-        w_mm = myth_dsw;
-    else
-        w_mm = data->screen_width;
-
-    if (myth_dsh != 0)
-        h_mm = myth_dsh;
-    else
-        h_mm = data->screen_height;
+    w_mm = (myth_dsw != 0) ? myth_dsw : data->screen_width;
+    h_mm = (myth_dsh != 0) ? myth_dsh : data->screen_height;
+    display_aspect = ((float)w_mm) / ((float)h_mm);
 
     XJ_started = true;
     return true;
 }
 
-float VideoOutputDirectfb::GetDisplayAspect(void)
-{
-    return (float)data->screen_width / (float)data->screen_height;
-}
-
-void VideoOutputDirectfb::PrepareFrame(VideoFrame *buffer, FrameScanType t)
+void VideoOutputDirectfb::PrepareFrame(VideoFrame *buffer, FrameScanType)
 {
     if (!buffer)
-        buffer=scratchFrame;
+        buffer = vbuffers.GetScratchFrame();
+
+    framesPlayed = buffer->frameNumber + 1;
 
     IDirectFBSurface *bufferSurface = data->buffers[buffer->buf];
 
@@ -526,7 +538,11 @@ void VideoOutputDirectfb::PrepareFrame(VideoFrame *buffer, FrameScanType t)
     }
     if (!bufferSurface)
         return;
+#if (DIRECTFB_MINOR_VERSION <= 9) && (DIRECTFB_MICRO_VERSION <= 22)
     if ((data->cardCapabilities.acceleration_mask & DFXL_BLIT) > 0)
+#else
+    if ((data->cardDescription.acceleration_mask & DFXL_BLIT) > 0)
+#endif
     {
         DFBCHECK(data->videoSurface->Blit(data->videoSurface, bufferSurface, NULL, 0, 0));
     }
@@ -561,7 +577,7 @@ void VideoOutputDirectfb::PrepareFrame(VideoFrame *buffer, FrameScanType t)
     }
 }
 
-void VideoOutputDirectfb::Show(FrameScanType t)
+void VideoOutputDirectfb::Show(FrameScanType)
 {
 
     DFBCHECK(data->videoSurface->Flip(data->videoSurface, NULL, DSFLIP_ONSYNC));
@@ -580,7 +596,7 @@ void VideoOutputDirectfb::Show(FrameScanType t)
 */
 }
 
-void VideoOutputDirectfb::DrawUnusedRects(void)
+void VideoOutputDirectfb::DrawUnusedRects(bool)
 {
     /* DirectFB only draws what is needed :-) */
 }
@@ -588,11 +604,12 @@ void VideoOutputDirectfb::DrawUnusedRects(void)
 void VideoOutputDirectfb::UpdatePauseFrame(void)
 {
     //**FIXME - is this all we need?
-    VideoFrame *pauseb = scratchFrame;
-    if (usedVideoBuffers.count() > 0)
-        pauseb = usedVideoBuffers.head();
-
-    memcpy(pauseFrame.buf, pauseb->buf, pauseb->size);
+    VideoFrame *pauseb = vbuffers.GetScratchFrame();
+    VideoFrame *pauseu = vbuffers.head(kVideoBuffer_used);
+    if (pauseu)
+        memcpy(pauseFrame.buf, pauseu->buf, pauseu->size);
+    else
+        memcpy(pauseFrame.buf, pauseb->buf, pauseb->size);
 }
 
 void VideoOutputDirectfb::ProcessFrame(VideoFrame *frame, OSD *osd,
@@ -601,8 +618,8 @@ void VideoOutputDirectfb::ProcessFrame(VideoFrame *frame, OSD *osd,
 {
     if (!frame)
     {
-        frame = scratchFrame;
-        CopyFrame(scratchFrame, &pauseFrame);
+        frame = vbuffers.GetScratchFrame();
+        CopyFrame(vbuffers.GetScratchFrame(), &pauseFrame);
     }
 
     if (m_deinterlacing && m_deintFilter != NULL)
@@ -630,22 +647,15 @@ void VideoOutputDirectfb::InputChanged(int width, int height, float aspect)
     CreateDirectfbBuffers(desc);
     MoveResize();
 
-    scratchFrame = &(vbuffers[kNumBuffers]);
-
     if (pauseFrame.buf)
         delete [] pauseFrame.buf;
 
-    pauseFrame.height = scratchFrame->height;
-    pauseFrame.width = scratchFrame->width;
-    pauseFrame.bpp = scratchFrame->bpp;
-    pauseFrame.size = scratchFrame->size;
-    pauseFrame.buf = new unsigned char[pauseFrame.size];
-}
-
-void VideoOutputDirectfb::AspectChanged(float aspect)
-{
-    VideoOutput::AspectChanged(aspect);
-    MoveResize();
+    pauseFrame.height = vbuffers.GetScratchFrame()->height;
+    pauseFrame.width  = vbuffers.GetScratchFrame()->width;
+    pauseFrame.bpp    = vbuffers.GetScratchFrame()->bpp;
+    pauseFrame.size   = vbuffers.GetScratchFrame()->size;
+    pauseFrame.buf    = new unsigned char[pauseFrame.size];
+    pauseFrame.frameNumber = vbuffers.GetScratchFrame()->frameNumber;
 }
 
 void VideoOutputDirectfb::Zoom(int direction)
@@ -684,21 +694,22 @@ bool VideoOutputDirectfb::CreateDirectfbBuffers(DFBSurfaceDescription desc)
     desc.flags = (DFBSurfaceDescriptionFlags)(desc.flags | DSDESC_CAPS);
     desc.caps=DSCAPS_SYSTEMONLY;
 
-    for (int i = 0; i < numbuffers + 1; i++)
+    for (uint i = 0; i < vbuffers.allocSize(); i++)
     {
         IDirectFBSurface *bufferSurface;
         unsigned char *bufferSurfaceData;
         DFBCHECKFAIL(data->dfb->CreateSurface(data->dfb, &desc, &bufferSurface), false);
         DFBCHECKFAIL(bufferSurface->Lock(bufferSurface, DSLF_WRITE, (void **)&bufferSurfaceData, &pitch), false);
         data->buffers[bufferSurfaceData] = bufferSurface;
-        vbuffers[i].height = desc.height;
-        vbuffers[i].width = desc.width;
+
+        vbuffers.at(i)->height = desc.height;
+        vbuffers.at(i)->width  = desc.width;
         //**FIXME set the three following parameters correctly
-        vbuffers[i].bpp = 12;
-        vbuffers[i].size = desc.height * desc.width * 3 / 2;
+        vbuffers.at(i)->bpp    = 12;
+        vbuffers.at(i)->size   = desc.height * desc.width * 3 / 2;
         //The format FMT_YV12 is a I420 ???? - Can the buffer format change ????
-        vbuffers[i].codec = FMT_YV12;
-        vbuffers[i].buf = bufferSurfaceData;
+        vbuffers.at(i)->codec  = FMT_YV12;
+        vbuffers.at(i)->buf    = bufferSurfaceData;
     }
 
     return true;
@@ -706,14 +717,14 @@ bool VideoOutputDirectfb::CreateDirectfbBuffers(DFBSurfaceDescription desc)
 
 void VideoOutputDirectfb::DeleteDirectfbBuffers(void)
 {
-    for (int i = 0; i < numbuffers + 1; i++)
+    for (uint i = 0; i < vbuffers.allocSize(); i++)
     {
-        vbuffers[i].buf = NULL;
+        vbuffers.at(i)->buf = NULL;
 
-        if (vbuffers[i].qscale_table)
+        if (vbuffers.at(i)->qscale_table)
         {
-            delete [] vbuffers[i].qscale_table;
-            vbuffers[i].qscale_table = NULL;
+            delete [] vbuffers.at(i)->qscale_table;
+            vbuffers.at(i)->qscale_table = NULL;
         }
     }
     BufferMap::iterator iter;
@@ -727,32 +738,32 @@ void VideoOutputDirectfb::DeleteDirectfbBuffers(void)
     data->buffers.clear();
 }
 
-int VideoOutputDirectfb::ChangePictureAttribute(int attributeType, int newValue)
+int VideoOutputDirectfb::ChangePictureAttribute(int attribute, int newValue)
 {
     data->videoLayer->GetColorAdjustment(data->videoLayer, (DFBColorAdjustment*) &adj);
     adj.flags = 0;
 
-    switch (attributeType) {
+    switch (attribute) {
     case kPictureAttribute_Brightness:
-        if ( data->videoLayerDesc.caps & DLCAPS_BRIGHTNESS ) {
+        if (data->videoLayerDesc.caps & DLCAPS_BRIGHTNESS ) {
             adj.flags = DCAF_BRIGHTNESS;
             adj.brightness = (unsigned short)(0xffff*newValue/100);
         }
         break;
     case kPictureAttribute_Contrast:
-        if ( data->videoLayerDesc.caps & DLCAPS_CONTRAST ) {
+        if (data->videoLayerDesc.caps & DLCAPS_CONTRAST ) {
             adj.flags = DCAF_CONTRAST;
             adj.contrast = (unsigned short)(0xffff*newValue/100);
         }
         break;
     case kPictureAttribute_Colour:
-        if ( data->videoLayerDesc.caps & DLCAPS_SATURATION ) {
+        if (data->videoLayerDesc.caps & DLCAPS_SATURATION ) {
             adj.flags = DCAF_SATURATION;
             adj.saturation = (unsigned short)(0xffff*newValue/100);
         }
         break;
     case kPictureAttribute_Hue:
-        if ( data->videoLayerDesc.caps & DLCAPS_HUE ) {
+        if (data->videoLayerDesc.caps & DLCAPS_HUE ) {
             adj.flags = DCAF_HUE;
             adj.hue = (unsigned short)(0xffff*newValue/100);
         }
@@ -773,7 +784,7 @@ DFBEnumerationResult LayerCallback(unsigned int id,
 {
     struct DirectfbData *vodata = (DirectfbData*)data;
     DFBResult ret;
-    IDirectFBSurface *surface;
+    //IDirectFBSurface *surface;
 
     if (id == DLID_PRIMARY)
         return DFENUM_OK;
