@@ -3,6 +3,7 @@
 
 #include <qstring.h>
 #include <qmutex.h>
+#include <cassert>
 
 #include "videooutbase.h"
 
@@ -21,9 +22,6 @@ class VideoOutputIvtv: public VideoOutput
 
     void InputChanged(int width, int height, float aspect);
 
-    void EmbedInWidget(WId wid, int x, int y, int w, int h);
-    void StopEmbedding(void);
-
     int GetRefreshRate(void);
 
     void DrawUnusedRects(bool sync = true);
@@ -33,7 +31,7 @@ class VideoOutputIvtv: public VideoOutput
                       FilterChain *filterList,
                       NuppelVideoPlayer *pipPlayer);
 
-    int WriteBuffer(unsigned char *buf, int count);
+    uint WriteBuffer(unsigned char *buf, int count);
     int Poll(int delay);
     void Pause(void);
     void Start(int skip, int mute);
@@ -52,11 +50,31 @@ class VideoOutputIvtv: public VideoOutput
         { last_speed = speed; last_normal = normal; last_mask = mask; };
     void Flush(void);
     void Step(void);
-    int GetFramesPlayed(void);
+    long long GetFramesPlayed(void);
+
+    VideoFrame *GetNextFreeFrame(bool with_lock = false,
+                                 bool allow_unsafe = false)
+    {
+        (void) with_lock;
+        (void) allow_unsafe;
+        return NULL;
+    }
+
+    int ValidVideoFrames(void) const;
 
   private:
+    typedef enum
+    {
+        kAlpha_Solid,
+        kAlpha_Local,
+        kAlpha_Clear,
+        kAlpha_Embedded
+    } eAlphaState;
+
     void ShowPip(VideoFrame *frame, NuppelVideoPlayer *pipplayer);
- 
+    void SetAlpha(eAlphaState newAlpha);
+    long long GetFirmwareFramesPlayed(void);
+
     int videofd;
     int fbfd;
 
@@ -74,15 +92,21 @@ class VideoOutputIvtv: public VideoOutput
     int stride;
 
     bool lastcleared;
+    bool pipon;
+    bool osdon;
 
     char *osdbuffer;
     char *osdbuf_aligned;
-
-    int osdbufsize;
+    int   osdbufsize;
+    int   osdbuf_revision;
 
     float last_speed;
+    int   internal_offset;
+    int   frame_at_speed_change;
+
     bool last_normal;
     int last_mask;
+    eAlphaState alphaState;
 };
 
 #endif
