@@ -15,7 +15,6 @@ class VideoOutputNull : public VideoOutput
     void Show(FrameScanType );
 
     void InputChanged(int width, int height, float aspect);
-    void AspectChanged(float aspect);
     void Zoom(int direction);
 
     void EmbedInWidget(WId wid, int x, int y, int w, int h);
@@ -23,7 +22,7 @@ class VideoOutputNull : public VideoOutput
 
     int GetRefreshRate(void);
 
-    void DrawUnusedRects(void);
+    void DrawUnusedRects(bool sync = true);
 
     void UpdatePauseFrame(void);
     void ProcessFrame(VideoFrame *frame, OSD *osd,
@@ -32,12 +31,9 @@ class VideoOutputNull : public VideoOutput
 
   private:
     void Exit(void);
-    bool CreateNullBuffers(void);
-    void DeleteNullBuffers(void);
 
     bool XJ_started;
 
-    VideoFrame *scratchFrame;
     VideoFrame pauseFrame;
 };
 
