@@ -17,7 +17,7 @@ class VideoOutputQuartz : public VideoOutput
     void Show(FrameScanType);
 
     void InputChanged(int width, int height, float aspect);
-    void AspectChanged(float aspect);
+    void VideoAspectRatioChanged(float aspect);
     void Zoom(int direction);
 
     void EmbedInWidget(WId wid, int x, int y, int w, int h);
@@ -25,7 +25,7 @@ class VideoOutputQuartz : public VideoOutput
 
     int GetRefreshRate(void);
 
-    void DrawUnusedRects(void);
+    void DrawUnusedRects(bool sync = true);
 
     void UpdatePauseFrame(void);
     void ProcessFrame(VideoFrame *frame, OSD *osd,
@@ -41,7 +41,6 @@ class VideoOutputQuartz : public VideoOutput
 
     QuartzData * data;
 
-    VideoFrame * scratchFrame;
     VideoFrame   pauseFrame;
 };
 
