@@ -14,22 +14,81 @@
 #include "dvbchannel.h"
 #endif
 
+class SignalTimeout;
+class ChannelTimeout;
+class UseEIT;
+
+typedef QMap<int,QString> InputNames;
+
+/** \class CardUtil
+ *  \brief Collection of helper utilities for capture card DB use
+ */
 class CardUtil
 {
   public:
-    enum DVB_TYPES {ERROR_OPEN, ERROR_PROBE, QPSK, QAM, OFDM, ATSC};
-    enum DISEQC_TYPES { SINGLE=0, MINI_2, SWITCH_2_1_0, SWITCH_2_1_1,
-                        SWITCH_4_1_0, SWITCH_4_1_1, POSITIONER_1_2,
-                        POSITIONER_X, POSITIONER_1_2_SWITCH_2,
-                        POSITIONER_X_SWITCH_2 };
+    /// \brief all the different capture cards
+    enum CARD_TYPES
+    {
+        ERROR_OPEN,
+        ERROR_PROBE,
+        QPSK,
+        QAM,
+        OFDM,
+        ATSC,
+        V4L,
+        MPEG,
+        HDTV,
+        FIREWIRE,
+    };
+    /// \brief all the different dvb DiSEqC devices
+    enum DISEQC_TYPES
+    {
+        SINGLE=0,
+        MINI_2,
+        SWITCH_2_1_0,
+        SWITCH_2_1_1,
+        SWITCH_4_1_0,
+        SWITCH_4_1_1,
+        POSITIONER_1_2,
+        POSITIONER_X,
+        POSITIONER_1_2_SWITCH_2,
+        POSITIONER_X_SWITCH_2,
+    };
+    /// \brief dvb card type
     static const QString DVB;
 
-    static bool isCardPresent(const QString &strType);
-    static DVB_TYPES cardDVBType(unsigned nVideoDev);
-    static DVB_TYPES cardDVBType(unsigned nVideoDev, QString &name);
-    static int CardUtil::videoDeviceFromCardID(unsigned nCardID);
-    static bool CardUtil::isDVB(unsigned nCardID);
-    static DISEQC_TYPES CardUtil::diseqcType(unsigned nCardID);
+    static int          GetCardID(const QString &videodevice,
+                                  QString hostname = QString::null);
+
+    static bool         IsCardTypePresent(const QString &strType);
+
+    static CARD_TYPES   GetCardType(uint cardid, QString &name, QString &card_type);
+    static CARD_TYPES   GetCardType(uint cardid, QString &name);
+    static CARD_TYPES   GetCardType(uint cardid);
+    static bool         IsDVBCardType(const QString card_type);
+
+    static bool         GetVideoDevice(uint cardid, QString& device, QString& vbi);
+    static bool         GetVideoDevice(uint cardid, QString& device);
+
+    static bool         IsDVB(uint cardid);
+    static DISEQC_TYPES GetDISEqCType(uint cardid);
+
+    static CARD_TYPES   GetDVBType(uint device, QString &name, QString &card_type);
+    static bool         HasDVBCRCBug(uint device);
+    static QString      GetDefaultInput(uint cardid);
+
+    static bool         IgnoreEncrypted(uint cardid, const QString &inputname);
+
+    static bool         hasV4L2(int videofd);
+    static InputNames   probeV4LInputs(int videofd, bool &ok);
+};
+
+class SourceUtil
+{
+  public:
+    static QString GetChannelSeparator(uint sourceid);
+    static QString GetChannelFormat(uint sourceid);
+    static uint    GetChannelCount(uint sourceid);
 };
 
 class VideoSource;
@@ -47,31 +106,6 @@ protected:
     const VideoSource& parent;
 };
 
-class RegionSelector: public ComboBoxSetting, public TransientStorage {
-    Q_OBJECT
-public:
-    RegionSelector() {
-        setLabel(QObject::tr("Region"));
-        fillSelections();
-    };
-
-public slots:
-    void fillSelections();
-};
-
-class ProviderSelector: public ComboBoxSetting, public TransientStorage {
-    Q_OBJECT
-public:
-    ProviderSelector(const QString& _grabber) :
-        grabber(_grabber) { setLabel(QObject::tr("Provider")); };
-
-public slots:
-    void fillSelections(const QString& location);
-
-protected:
-    QString grabber;
-};
-
 class FreqTableSelector: public ComboBoxSetting, public VSSetting {
     Q_OBJECT
 public:
@@ -103,53 +137,54 @@ public:
 class DataDirectUserID;
 class DataDirectPassword;
 
-class DataDirect_config: public VerticalConfigurationGroup {
+class DataDirect_config: public VerticalConfigurationGroup
+{
     Q_OBJECT
-public:
+  public:
     DataDirect_config(const VideoSource& _parent, int _source = DD_ZAP2IT); 
 
-    virtual void load();
+    virtual void load(void);
 
     QString getLineupID(void) const { return lineupselector->getValue(); };
 
-protected slots:
-    void fillDataDirectLineupSelector();
+  protected slots:
+    void fillDataDirectLineupSelector(void);
 
-protected:
-    const VideoSource& parent;
-    DataDirectUserID* userid;
-    DataDirectPassword* password;
-    DataDirectLineupSelector* lineupselector;
-    DataDirectButton* button;
+  protected:
+    const VideoSource        &parent;
+    DataDirectUserID         *userid;
+    DataDirectPassword       *password;
+    DataDirectButton         *button;
+    DataDirectLineupSelector *lineupselector;
     QString lastloadeduserid;
     QString lastloadedpassword;
     int source;
 };
 
-class XMLTV_uk_config: public VerticalConfigurationGroup {
-public:
-    XMLTV_uk_config(const VideoSource& _parent);
-
-    virtual void save();
-
-protected:
-    const VideoSource& parent;
-    RegionSelector* region;
-    ProviderSelector* provider;
-};
-
-class XMLTV_generic_config: public LabelSetting {
+class XMLTV_generic_config: public VerticalConfigurationGroup 
+{
 public:
     XMLTV_generic_config(const VideoSource& _parent, QString _grabber);
 
-    virtual void load() {};
     virtual void save();
+    virtual void save(QString) { save(); }
 
 protected:
     const VideoSource& parent;
     QString grabber;
 };
 
+class EITOnly_config: public VerticalConfigurationGroup
+{
+public:
+    EITOnly_config(const VideoSource& _parent);
+    virtual void save();
+    virtual void save(QString) { save(); }
+
+protected:
+    UseEIT *useeit;
+};
+
 class XMLTVConfig: public VerticalConfigurationGroup, 
                    public TriggeredConfigurationGroup {
 public:
@@ -173,6 +208,10 @@ public:
         if (name)
             ConfigurationWizard::save();
     };
+    virtual void save(QString destination) {
+        if (name)
+            ConfigurationWizard::save(destination);
+    };
 
 private:
     class ID: virtual public IntegerSetting,
@@ -249,6 +288,11 @@ public:
         settingValue = "";
         SimpleDBStorage::save();
     };
+    void save(QString destination) {
+        changed = true;
+        settingValue = "";
+        SimpleDBStorage::save(destination);
+    };
 };
 
 class DVBVbiDevice: public LineEditSetting, public CCSetting {
@@ -263,14 +307,20 @@ public:
         settingValue = "";
         SimpleDBStorage::save();
     };
+    void save(QString destination) {
+        changed = true;
+        settingValue = "";
+        SimpleDBStorage::save(destination);
+    };
 };
 
-class DVBDiseqcInputList
+class DVBDiSEqCInputList
 {
   public:
-    DVBDiseqcInputList() { clearValues(); }
-    DVBDiseqcInputList(const QString& in, const QString& prt, 
-                       const QString& pos) : input(in), port(prt), position(pos)
+    DVBDiSEqCInputList() { clearValues(); }
+    DVBDiSEqCInputList(const QString& in, const QString& prt, 
+                       const QString& pos)
+        : input(in), port(prt), position(pos)
     {}
 
     void clearValues() 
@@ -285,19 +335,6 @@ class DVBDiseqcInputList
     QString position;
 };
 
-class DVBDefaultInput: public ComboBoxSetting, public CCSetting {
-    Q_OBJECT
-public:
-
-    DVBDefaultInput(const CaptureCard& parent):
-        CCSetting(parent,"defaultinput") {
-        setLabel(QObject::tr("Default input"));
-    };
-
-public slots:
-    void fillSelections(const QString& type);
-};
-
 class CardType: public ComboBoxSetting, public CCSetting {
 public:
     CardType(const CaptureCard& parent);
@@ -313,21 +350,17 @@ class DVBConfigurationGroup: public VerticalConfigurationGroup {
 public:
     DVBConfigurationGroup(CaptureCard& a_parent);
 
-    void load() {
-        VerticalConfigurationGroup::load();
-    };
-
 public slots:
     void probeCard(const QString& cardNumber);
 
 private:
-    CaptureCard& parent;
+    CaptureCard        &parent;
 
-    DVBDefaultInput* defaultinput;
-    DVBCardName* cardname;
-    DVBCardType* cardtype;
-    DVBDiseqcType* diseqctype;
-    TransButtonSetting *advcfg;
+    TunerCardInput     *defaultinput;
+    DVBCardName        *cardname;
+    DVBCardType        *cardtype;
+    SignalTimeout      *signal_timeout;
+    ChannelTimeout     *channel_timeout;
 };
 
 class CaptureCardGroup: public VerticalConfigurationGroup,
@@ -360,9 +393,8 @@ public:
     };
 
 public slots:
-    void execDVBConfigMenu();
-    void execDiseqcWiz();
-    void execAdvConfigWiz();
+    void DiSEqCPanel();
+    void recorderOptionsPanel();
     void setDvbCard(const QString& card) { dvbCard = card; };
 
 private:
@@ -467,4 +499,84 @@ protected:
     vector<CardInput*> cardinputs;
 };
 
+class StartingChannel : public ComboBoxSetting, public CISetting
+{
+    Q_OBJECT
+  public:
+    StartingChannel(const CardInput &parent) :
+        ComboBoxSetting(false, 1), CISetting(parent, "startchan")
+    {
+        setLabel(QObject::tr("Starting channel"));
+        setHelpText(QObject::tr("Starting LiveTV channel.") + " " +
+                    QObject::tr("This is updated on every successful "
+                                "channel change."));
+    }
+    void fillSelections(void) {;}
+  public slots:
+    void SetSourceID(const QString &sourceid);
+};
+
+class CardID;
+class InputName;
+class SourceID;
+class DVBLNBChooser;
+class DiSEqCPos;
+class DiSEqCPort;
+class LNBLofSwitch;
+class LNBLofLo;
+class LNBLofHi;
+
+class CardInput: public ConfigurationWizard
+{
+    Q_OBJECT
+  public:
+    CardInput(bool is_dvb_card);
+
+    int getInputID(void) const { return id->intValue(); };
+
+    void loadByID(int id);
+    void loadByInput(int cardid, QString input);
+    QString getSourceName(void) const;
+
+    void fillDiseqcSettingsInput(QString _pos, QString _port);
+
+    virtual void save();
+    virtual void save(QString /*destination*/) { save(); }
+
+  public slots:
+    void channelScanner();
+    void sourceFetch();
+
+  private:
+    class ID: virtual public IntegerSetting,
+              public AutoIncrementStorage
+    {
+      public:
+        ID():
+            AutoIncrementStorage("cardinput", "cardid")
+        {
+            setVisible(false);
+            setName("CardInputID");
+        };
+        virtual QWidget* configWidget(ConfigurationGroup *cg, QWidget* parent,
+                                      const char* widgetName = 0)
+        {
+            (void)cg; (void)parent; (void)widgetName;
+            return NULL;
+        };
+    };
+
+    ID              *id;
+    CardID          *cardid;
+    InputName       *inputname;
+    SourceID        *sourceid;
+    DVBLNBChooser   *lnbsettings;
+    DiSEqCPos       *diseqcpos;
+    DiSEqCPort      *diseqcport;
+    LNBLofSwitch    *lnblofswitch;
+    LNBLofLo        *lnbloflo;
+    LNBLofHi        *lnblofhi;
+    StartingChannel *startchan;
+};
+
 #endif
