@@ -37,10 +37,24 @@ extern yuv2rgb_fun yuv2rgb;
 void yuv2rgb_init (int bpp, int mode);
 yuv2rgb_fun yuv2rgb_init_mmxext (int bpp, int mode);
 yuv2rgb_fun yuv2rgb_init_mmx (int bpp, int mode);
-yuv2rgb_fun yuv2rgb_init_mlib (int bpp, int mode);
+//yuv2rgb_fun yuv2rgb_init_mlib (int bpp, int mode);
 
 // actually does to i420
 void rgb32_to_yuv420p(unsigned char *lum, unsigned char *cb, unsigned char *cr,
                       unsigned char *alpha, unsigned char *src, 
                       int width, int height, int srcwidth);
 
+typedef void (* yuv2vuy_fun) (uint8_t * image, uint8_t * py,
+                              uint8_t * pu, uint8_t * pv,
+                              int h_size, int v_size,
+                              int vuy_stride, int y_stride, int uv_stride);
+
+yuv2vuy_fun yuv2vuy_init_altivec (void);
+
+typedef void (* vuy2yuv_fun) (uint8_t * image, uint8_t * py,
+                              uint8_t * pu, uint8_t * pv,
+                              int h_size, int v_size,
+                              int vuy_stride, int y_stride, int uv_stride);
+
+vuy2yuv_fun vuy2yuv_init_altivec (void);
+
