@@ -9,21 +9,20 @@
 #include "mythmainwindow.h"
 #include "mythcontext.h"
 #include "mythfontproperties.h"
+#include "mythuistatetype.h"
+#include "mythuibutton.h"
 
 MythListButton::MythListButton(MythUIType *parent, const char *name, 
                                const QRect& area, bool showArrow, 
                                bool showScrollArrows)
               : MythUIType(parent, name)
 {
-    m_rect             = area;
+    m_Area             = area;
 
     m_showArrow        = showArrow;
     m_showScrollArrows = showScrollArrows;
 
     m_active           = false;
-    m_visible          = true;
-    m_showUpArrow      = false;
-    m_showDnArrow      = false;
     m_drawFromBottom   = false;
 
     m_itemList.setAutoDelete(false);
@@ -42,62 +41,27 @@ MythListButton::MythListButton(MythUIType *parent, const char *name,
     m_itemHeight      = 0;
     m_itemsVisible    = 0;
 
-    m_drawoffset      = QPoint(0, 0);
+    m_upArrow = m_downArrow = NULL;
 
     m_textFlags = Qt::AlignLeft | Qt::AlignVCenter;
 
     SetItemRegColor(QColor("#505050"), QColor("#000000"), 100);
     SetItemSelColor(QColor("#52CA38"), QColor("#349838"), 255);
 
-    m_itemRegPix = NULL;
-    m_itemSelActPix = NULL;
-    m_itemSelInactPix = NULL;
-    m_upArrowRegPix = NULL;
-    m_dnArrowRegPix = NULL;
-    m_upArrowActPix = NULL;
-    m_dnArrowActPix = NULL;
-    m_arrowPix = NULL;
-    m_checkNonePix = NULL;
-    m_checkHalfPix = NULL;
-    m_checkFullPix = NULL;
-    
     m_fontActive = new MythFontProperties();
     m_fontInactive = new MythFontProperties();
 }
 
-    MythListButton::~MythListButton()
+MythListButton::~MythListButton()
 {    
     Reset();
     delete m_topIterator;
     delete m_selIterator;
 
-    if (m_itemRegPix)
-        delete m_itemRegPix;
-    if (m_itemSelActPix)
-        delete m_itemSelActPix;
-    if (m_itemSelInactPix)
-        delete m_itemSelInactPix;
-    if (m_upArrowRegPix)
-        delete m_upArrowRegPix;
-    if (m_dnArrowRegPix)
-        delete m_dnArrowRegPix;
-    if (m_upArrowActPix)
-        delete m_upArrowActPix;
-    if (m_dnArrowActPix)
-        delete m_dnArrowActPix;
-    if (m_arrowPix)
-        delete m_arrowPix;
-    if (m_checkNonePix)
-        delete m_checkNonePix;
-    if (m_checkHalfPix)
-        delete m_checkHalfPix;
-    if (m_checkFullPix)
-        delete m_checkFullPix;
     if (m_fontActive)
        delete m_fontActive; 
     if (m_fontInactive)
        delete m_fontInactive; 
-    
 }
 
 void MythListButton::SetItemRegColor(const QColor& beg, 
@@ -151,6 +115,8 @@ void MythListButton::SetTextFlags(int flags)
 void MythListButton::SetActive(bool active)
 {
     m_active = active;
+    if (m_initialized)
+        SetPositionArrowStates();
 }
 
 void MythListButton::Reset()
@@ -172,10 +138,61 @@ void MythListButton::Reset()
     m_selIterator->toFirst(); 
     m_topIterator->toFirst();
 
-    m_showUpArrow = false;
-    m_showDnArrow = false;
+    SetPositionArrowStates();
+}
 
-    SetRedraw();
+void MythListButton::SetPositionArrowStates(void)
+{
+    if (!m_initialized)
+        Init();
+
+    if (m_ButtonList.size() > 0)
+    {
+        int button = 0;
+    
+        if (m_drawFromBottom && m_itemCount < (int)m_itemsVisible)
+            button = m_itemsVisible - m_itemCount;
+
+        for (int i = 0; i < button; i++)
+             m_ButtonList[i]->SetVisible(false);
+
+        QPtrListIterator<MythListButtonItem> it = (*m_topIterator);
+        while (it.current() && button < (int)m_itemsVisible)
+        {
+            MythUIButton *realButton = m_ButtonList[button];
+            MythListButtonItem *buttonItem = it.current();
+
+            buttonItem->SetToRealButton(realButton, true);
+            realButton->SetVisible(true);
+
+            button++;
+            ++it;
+        }
+
+        for (; button < (int)m_itemsVisible; button++)
+            m_ButtonList[button]->SetVisible(false);
+    }
+
+    if (!m_showScrollArrows || !m_downArrow || !m_upArrow)
+        return;
+
+    if (m_itemCount == 0)
+    {
+        m_downArrow->DisplayState(MythUIStateType::Off);
+        m_upArrow->DisplayState(MythUIStateType::Off);
+    }
+    else
+    {
+        if (m_topItem != m_itemList.first())
+            m_upArrow->DisplayState(MythUIStateType::Full);
+        else
+            m_upArrow->DisplayState(MythUIStateType::Off);
+    
+        if (m_topPosition + (int)m_itemsVisible < m_itemCount)
+            m_downArrow->DisplayState(MythUIStateType::Full);
+        else
+            m_downArrow->DisplayState(MythUIStateType::Off);
+    }
 }
 
 void MythListButton::InsertItem(MythListButtonItem *item)
@@ -185,11 +202,6 @@ void MythListButton::InsertItem(MythListButtonItem *item)
 
     m_itemCount++;
 
-    if (m_showScrollArrows && m_itemCount > (int)m_itemsVisible)
-        m_showDnArrow = true;
-    else
-        m_showDnArrow = false;
-
     if (!lastItem) 
     {
         m_topItem = item;
@@ -200,7 +212,7 @@ void MythListButton::InsertItem(MythListButtonItem *item)
         emit itemSelected(item);
     }
 
-    SetRedraw();
+    SetPositionArrowStates();
 }
 
 void MythListButton::RemoveItem(MythListButtonItem *item)
@@ -258,17 +270,8 @@ void MythListButton::RemoveItem(MythListButtonItem *item)
     m_itemList.remove(item);
     m_itemCount--;
 
-    if (m_topItem != m_itemList.first())
-        m_showUpArrow = true;
-    else
-        m_showUpArrow = false;
+    SetPositionArrowStates();
 
-    if (m_topPosition + (int)m_itemsVisible < m_itemCount)
-        m_showDnArrow = true;
-    else
-        m_showDnArrow = false;
-
-    SetRedraw();
     if (m_selItem) 
         emit itemSelected(m_selItem);
 }
@@ -302,12 +305,8 @@ void MythListButton::SetItemCurrent(MythListButtonItem* item)
     m_selItem = item;
     (*m_topIterator) = (*m_selIterator);
 
-    if (m_showScrollArrows && m_itemCount > (int)m_itemsVisible)
-        m_showDnArrow = true;
-    else
-        m_showDnArrow = false;
+    SetPositionArrowStates();
 
-    SetRedraw();
     emit itemSelected(m_selItem);
 }
 
@@ -404,17 +403,8 @@ void MythListButton::MoveUp(MovementUnit unit)
         m_topPosition = m_selPosition;
     }
 
-    if (m_topItem != m_itemList.first())
-        m_showUpArrow = true;
-    else
-        m_showUpArrow = false;
+    SetPositionArrowStates();
 
-    if (m_topPosition + (int)m_itemsVisible < m_itemCount)
-        m_showDnArrow = true;
-    else
-        m_showDnArrow = false;
-
-    SetRedraw();
     emit itemSelected(m_selItem);
 }
 
@@ -463,28 +453,19 @@ void MythListButton::MoveDown(MovementUnit unit)
    
     m_topItem = m_topIterator->current();
 
-    if (m_topItem != m_itemList.first())
-        m_showUpArrow = true;
-    else
-        m_showUpArrow = false;
+    SetPositionArrowStates();
 
-    if (m_topPosition + (int)m_itemsVisible < m_itemCount)
-        m_showDnArrow = true;
-    else
-        m_showDnArrow = false;
-
-    SetRedraw(); 
     emit itemSelected(m_selItem);
 }
 
 bool MythListButton::MoveToNamedPosition(const QString &position_name)
 {
-    if(m_selPosition < 0)
+    if (m_selPosition < 0)
     {
         return false;
     }
 
-    if(!m_selIterator->toFirst())
+    if (!m_selIterator->toFirst())
     {
         return false;
     }
@@ -493,7 +474,7 @@ bool MythListButton::MoveToNamedPosition(const QString &position_name)
     bool found_it = false;
     while(m_selIterator->current())
     {
-        if(m_selIterator->current()->text() == position_name)
+        if (m_selIterator->current()->text() == position_name)
         {
             found_it = true;
             break;
@@ -502,7 +483,7 @@ bool MythListButton::MoveToNamedPosition(const QString &position_name)
         ++m_selPosition;
     }
 
-    if(!found_it)
+    if (!found_it)
     {
         m_selPosition = -1;
         return false;
@@ -518,17 +499,8 @@ bool MythListButton::MoveToNamedPosition(const QString &position_name)
    
     m_topItem = m_topIterator->current();
 
-    if (m_topItem != m_itemList.first())
-        m_showUpArrow = true;
-    else
-        m_showUpArrow = false;
+    SetPositionArrowStates();
 
-    if (m_topPosition + (int)m_itemsVisible < m_itemCount)
-        m_showDnArrow = true;
-    else
-        m_showDnArrow = false;
-
-    SetRedraw();
     return true;
 }
 
@@ -585,81 +557,6 @@ bool MythListButton::MoveItemUpDown(MythListButtonItem *item, bool flag)
     return true;
 }
 
-void MythListButton::Draw(MythPainter *p, int xoffset, int yoffset, 
-                          int alphaMod)
-{
-    Draw(p, xoffset, yoffset, true, alphaMod);
-}
-
-void MythListButton::Draw(MythPainter *p, int xoffset, int yoffset, 
-                          bool active_on, int alphaMod)
-{
-    if (!m_visible)
-        return;
-
-    if (!m_initialized)
-        Init();
-
-    int alpha = CalcAlpha(alphaMod);
-
-    MythFontProperties *font = (m_active) ? m_fontActive : m_fontInactive;
-    
-    if (!active_on)
-        font = m_fontInactive;
-    
-    int x = m_rect.x() + xoffset + m_drawoffset.x();
-    int y = m_rect.y() + yoffset + m_drawoffset.y();
-    int basey = y;
-
-    if (m_drawFromBottom)
-    {
-        if (m_itemCount < (int)m_itemsVisible)
-        {
-            y += (m_itemsVisible - m_itemCount) * 
-                 (m_itemHeight + m_itemSpacing);
-        }
-    }
-
-    QPtrListIterator<MythListButtonItem> it = (*m_topIterator);
-    while (it.current() && 
-           (y - basey) <= (m_contentsRect.height() - m_itemHeight)) 
-    {
-        if (active_on && it.current()->getOverrideInactive())
-        {
-            font = m_fontInactive;
-            it.current()->paint(p, font, x, y, active_on, alpha);
-            font = (m_active) ? m_fontActive : m_fontInactive;;
-        }
-        else
-            it.current()->paint(p, font, x, y, active_on, alpha);
-
-        y += m_itemHeight + m_itemSpacing;
-
-        ++it;
-    }
-
-    y = m_rect.y() + yoffset + m_drawoffset.y();
-
-    if (m_showScrollArrows) 
-    {
-        if (m_showUpArrow)
-            p->DrawImage(x + m_arrowsRect.x(), y + m_arrowsRect.y(),
-                         m_upArrowActPix, alpha);
-        else
-            p->DrawImage(x + m_arrowsRect.x(), y + m_arrowsRect.y(),
-                         m_upArrowRegPix, alpha);
-
-        if (m_showDnArrow)
-            p->DrawImage(x + m_arrowsRect.x() + m_upArrowRegPix->width() + 
-                         m_itemMargin, y + m_arrowsRect.y(),
-                         m_dnArrowActPix, alpha);
-        else
-            p->DrawImage(x + m_arrowsRect.x() + m_upArrowRegPix->width() + 
-                         m_itemMargin, y + m_arrowsRect.y(),
-                         m_dnArrowRegPix, alpha);
-    }
-}
-
 void MythListButton::Init()
 {
     if (m_initialized)
@@ -667,177 +564,199 @@ void MythListButton::Init()
 
     m_initialized = true;
 
-    MythPainter *painter = GetMythPainter();
-
     QFontMetrics fm(m_fontActive->face);
     QSize sz1 = fm.size(Qt::SingleLine, "XXXXX");
     fm = QFontMetrics(m_fontInactive->face);
     QSize sz2 = fm.size(Qt::SingleLine, "XXXXX");
     m_itemHeight = QMAX(sz1.height(), sz2.height()) + (int)(2 * m_itemMargin);
 
+    QRect arrowsRect;
     if (m_showScrollArrows) 
     {
-        LoadPixmap(&m_upArrowRegPix, "uparrow-reg");
-        LoadPixmap(&m_upArrowActPix, "uparrow-sel");
-        LoadPixmap(&m_dnArrowRegPix, "dnarrow-reg");
-        LoadPixmap(&m_dnArrowActPix, "dnarrow-sel");
+        m_upArrow = new MythUIStateType(this, "upscrollarrow");
+        m_downArrow = new MythUIStateType(this, "downscrollarrow");
 
-        m_arrowsRect = QRect(0, m_rect.height() - m_upArrowActPix->height() - 1,
-                             m_rect.width(), m_upArrowActPix->height());
+        MythImage *upReg, *upAct, *downReg, *downAct;
+        LoadArrowPixmaps(&upReg, &upAct, &downReg, &downAct);
+
+        m_upArrow->AddImage(MythUIStateType::Off, upReg);
+        m_upArrow->AddImage(MythUIStateType::Full, upAct);
+        m_downArrow->AddImage(MythUIStateType::Off, downReg);
+        m_downArrow->AddImage(MythUIStateType::Full, downAct);
+
+        arrowsRect = PlaceArrows(upAct->size());
+
+        upAct->DownRef();
+        upReg->DownRef();
+        downReg->DownRef();
+        downAct->DownRef();
     }
     else 
-        m_arrowsRect = QRect(0, 0, 0, 0);
-        
-    m_contentsRect = QRect(0, 0, m_rect.width(), m_rect.height() -
-                           m_arrowsRect.height() - 2 * m_itemMargin);
+        arrowsRect = QRect(0, 0, 0, 0);
+ 
+    m_contentsRect = CalculateContentsRect(arrowsRect);
 
-    m_itemsVisible = 0;
-    int y = 0;
+    CalculateVisibleItems();
 
-    while (y <= m_contentsRect.height() - m_itemHeight) 
+    MythImage *itemRegPix, *itemSelActPix, *itemSelInactPix;
+    MythImage *arrowPix, *checkNonePix, *checkHalfPix, *checkFullPix;
+
+    LoadPixmap(&checkNonePix, "check-empty");
+    LoadPixmap(&checkHalfPix, "check-half");
+    LoadPixmap(&checkFullPix, "check-full");
+
+    LoadPixmap(&arrowPix, "arrow");
+
+    QSize gradsize(ItemWidth(), m_itemHeight);
+    itemRegPix = MythImage::Gradient(gradsize, m_itemRegBeg, m_itemRegEnd,
+                                     m_itemRegAlpha);
+    itemSelActPix = MythImage::Gradient(gradsize, m_itemSelBeg, m_itemSelEnd,
+                                        m_itemSelAlpha);
+    itemSelInactPix = MythImage::Gradient(gradsize, m_itemSelBeg, m_itemSelEnd,
+                                          m_itemRegAlpha);
+
+    for (int i = 0; i < (int)m_itemsVisible; i++)
     {
-        y += m_itemHeight + m_itemSpacing;
-        m_itemsVisible++;
+        QString name = QString("buttonlist button %1").arg(i);
+        MythUIButton *button = new MythUIButton(this, name);
+        button->SetBackgroundImage(MythUIButton::Normal, itemRegPix);
+        button->SetBackgroundImage(MythUIButton::Active, itemRegPix);
+        button->SetBackgroundImage(MythUIButton::SelectedInactive,
+                                   itemSelInactPix);
+        button->SetBackgroundImage(MythUIButton::Selected, itemSelActPix);
+
+        button->SetPaddingMargin(m_itemMargin);
+
+        button->SetCheckImage(MythUIStateType::Off, checkNonePix);
+        button->SetCheckImage(MythUIStateType::Half, checkHalfPix);
+        button->SetCheckImage(MythUIStateType::Full, checkFullPix);
+
+        button->SetRightArrowImage(arrowPix);
+
+        button->SetFont(MythUIButton::Normal, *m_fontActive);
+        button->SetFont(MythUIButton::Disabled, *m_fontInactive);
+
+        button->SetPosition(GetButtonPosition(i));
+        m_ButtonList.push_back(button);
     }
 
-    LoadPixmap(&m_checkNonePix, "check-empty");
-    LoadPixmap(&m_checkHalfPix, "check-half");
-    LoadPixmap(&m_checkFullPix, "check-full");
-    
-    LoadPixmap(&m_arrowPix, "arrow");
+    itemRegPix->DownRef();
+    itemSelActPix->DownRef();
+    itemSelInactPix->DownRef();
+    arrowPix->DownRef();
+    checkNonePix->DownRef();
+    checkHalfPix->DownRef();
+    checkFullPix->DownRef();
 
-    QImage img(m_rect.width(), m_itemHeight, 32);
-    img.setAlphaBuffer(true);
+    SetPositionArrowStates();
+}
 
-    for (int y = 0; y < img.height(); y++)
-    {
-        for (int x = 0; x < img.width(); x++)
-        {
-            uint *p = (uint *)img.scanLine(y) + x;
-            *p = qRgba(0, 0, 0, m_itemRegAlpha);
-        }
-    }
-
-    qApp->lock();
-
-    {
-        float rstep = float(m_itemRegEnd.red() - m_itemRegBeg.red()) / 
-                      float(m_itemHeight);
-        float gstep = float(m_itemRegEnd.green() - m_itemRegBeg.green()) / 
-                      float(m_itemHeight);
-        float bstep = float(m_itemRegEnd.blue() - m_itemRegBeg.blue()) / 
-                      float(m_itemHeight);
-
-        QPixmap itemRegPix = QPixmap(img);
-        QPainter p(&itemRegPix);
-
-        float r = m_itemRegBeg.red();
-        float g = m_itemRegBeg.green();
-        float b = m_itemRegBeg.blue();
-        for (int y = 0; y < img.height(); y++) 
-        {
-            QColor c((int)r, (int)g, (int)b);
-            p.setPen(c);
-            p.drawLine(0, y, img.width(), y);
-            r += rstep;
-            g += gstep;
-            b += bstep;
-        }
-        p.setPen(black);
-        p.drawLine(0, 0, 0, img.height() - 1);
-        p.drawLine(0, 0, img.width() - 1, 0);
-        p.drawLine(0, img.height() - 1, img.width() - 1, img.height() - 1);
-        p.drawLine(img.width() - 1, 0, img.width() - 1, img.height() - 1);
-        p.end();
-
-        m_itemRegPix = painter->GetFormatImage();
-        m_itemRegPix->Assign(itemRegPix.convertToImage());
-    }   
-
-    {
-        float rstep = float(m_itemSelEnd.red() - m_itemSelBeg.red()) /
-                      float(m_itemHeight);
-        float gstep = float(m_itemSelEnd.green() - m_itemSelBeg.green()) / 
-                      float(m_itemHeight);
-        float bstep = float(m_itemSelEnd.blue() - m_itemSelBeg.blue()) /
-                      float(m_itemHeight);
-
-        QPixmap itemSelInactPix = QPixmap(img);
-        QPainter p(&itemSelInactPix);
-
-        float r = m_itemSelBeg.red();
-        float g = m_itemSelBeg.green();
-        float b = m_itemSelBeg.blue();
-        for (int y = 0; y < img.height(); y++) 
-        {
-            QColor c((int)r, (int)g, (int)b);
-            p.setPen(c);
-            p.drawLine(0, y, img.width(), y);
-            r += rstep;
-            g += gstep;
-            b += bstep;
-        }
-        p.setPen(black);
-        p.drawLine(0, 0, 0, img.height() - 1);
-        p.drawLine(0, 0, img.width() - 1, 0);
-        p.drawLine(0, img.height() - 1, img.width() - 1, img.height() - 1);
-        p.drawLine(img.width() - 1, 0, img.width() - 1, img.height() - 1);
-        p.end();
-
-        m_itemSelInactPix = painter->GetFormatImage();
-        m_itemSelInactPix->Assign(itemSelInactPix.convertToImage());
-
-        img.setAlphaBuffer(false);
-        
-        QPixmap itemSelActPix = QPixmap(img);
-        p.begin(&itemSelActPix);
-
-        r = m_itemSelBeg.red();
-        g = m_itemSelBeg.green();
-        b = m_itemSelBeg.blue();
-        for (int y = 0; y < img.height(); y++) 
-        {
-            QColor c((int)r, (int)g, (int)b);
-            p.setPen(c);
-            p.drawLine(0, y, img.width(), y);
-            r += rstep;
-            g += gstep;
-            b += bstep;
-        }
-        p.setPen(black);
-        p.drawLine(0, 0, 0, img.height() - 1);
-        p.drawLine(0, 0, img.width() - 1, 0);
-        p.drawLine(0, img.height() - 1, img.width() - 1, img.height() - 1);
-        p.drawLine(img.width() - 1, 0, img.width() - 1, img.height() - 1);
-        p.end();
-
-        m_itemSelActPix = painter->GetFormatImage();
-        m_itemSelActPix->Assign(itemSelActPix.convertToImage());
-    }
-
-    qApp->unlock();
-
-    if (m_itemList.count() > m_itemsVisible && m_showScrollArrows)
-        m_showDnArrow = true;
-    else
-        m_showDnArrow = false;
+QPoint MythListButton::GetButtonPosition(uint i) const
+{
+    return QPoint(0, i * (m_itemHeight + m_itemSpacing));
 }
 
 void MythListButton::LoadPixmap(MythImage **pix, const QString& fileName)
 {
     QString file = "lb-" + fileName + ".png";
-
-    *pix = GetMythPainter()->GetFormatImage(); 
     QImage *p = gContext->LoadScaleImage(file);
-    if (p) 
+    *pix = MythImage::FromQImage(&p);
+}
+
+void MythListButton::LoadArrowPixmaps(MythImage **PrevReg, MythImage **PrevAct,
+                                      MythImage **NextReg, MythImage **NextAct)
+{
+    LoadPixmap(PrevReg, "uparrow-reg");
+    LoadPixmap(PrevAct, "uparrow-sel");
+    LoadPixmap(NextReg, "dnarrow-reg");
+    LoadPixmap(NextAct, "dnarrow-sel");
+}
+
+const QRect MythListButton::PlaceArrows(const QSize &arrowSize)
+{
+    int y = m_Area.height() - arrowSize.height() - 1;
+    m_upArrow->SetPosition(QPoint(0, y));
+    m_downArrow->SetPosition(QPoint(arrowSize.width() + m_itemMargin, y));
+
+    return QRect(0, y, m_Area.width(), arrowSize.height());
+}
+
+QRect MythListButton::CalculateContentsRect(const QRect &arrowsRect) const
+{
+    return QRect(0, 0, GetArea().width(), GetArea().height() -
+                 arrowsRect.height() - 2 * m_itemMargin);
+}
+
+void MythListButton::CalculateVisibleItems(void)
+{
+    int y = 0;
+    m_itemsVisible = 0;
+    while (y <= m_contentsRect.height() - m_itemHeight) 
     {
-        (*pix)->Assign(*p);
-        delete p;
+        y += m_itemHeight + m_itemSpacing;
+        m_itemsVisible++;
     }
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
+MythHorizListButton::MythHorizListButton(MythUIType *parent,
+                                         const char *name, const QRect &area,
+                                         bool showArrow, bool showScrollArrows,
+                                         uint horizontalItems)
+    : MythListButton(parent, name, area, showArrow, showScrollArrows)
+{
+    m_itemsVisible = horizontalItems;
+}
+
+const QRect MythHorizListButton::PlaceArrows(const QSize &arrowSize)
+{
+    int x = GetArea().width() - arrowSize.width() - 1;
+    int ytop = GetArea().height() / 2 - m_itemMargin / 2 - arrowSize.height();
+    int ybottom = GetArea().height() / 2 + m_itemMargin / 2;
+
+    m_upArrow->SetPosition(QPoint(x, ybottom));
+    m_downArrow->SetPosition(QPoint(x, ytop));
+
+    /* rightmost rectangle (full height) */
+    QRect arrowsRect(x, 0, arrowSize.width() + 1, m_Area.height());
+
+    /* now calculate the item width */
+    int dx = GetArea().width() - arrowSize.width() - 1 - 2 * m_itemMargin -
+             (m_itemsVisible - 1) * m_itemSpacing;
+ 
+    /* set the item width */
+    m_itemWidth = dx / m_itemsVisible;
+
+    return QRect(x, 0, arrowSize.width() + 1, m_Area.height());
+}
+
+QRect MythHorizListButton::CalculateContentsRect(const QRect &arrowsRect) const
+{
+    return QRect(0, 0,
+                 GetArea().width() - arrowsRect.width() - 2 * m_itemMargin,
+                 GetArea().height());
+}
+
+QPoint MythHorizListButton::GetButtonPosition(uint i) const
+{
+    return QPoint(i * (ItemWidth() + m_itemSpacing),
+                  GetArea().height() / 2 - m_itemHeight / 2);
+}
+
+void MythHorizListButton::LoadArrowPixmaps(MythImage **PrevReg,
+                                           MythImage **PrevAct,
+                                           MythImage **NextReg,
+                                           MythImage **NextAct)
+{
+    LoadPixmap(PrevReg, "ltarrow-reg");
+    LoadPixmap(PrevAct, "ltarrow-sel");
+    LoadPixmap(NextReg, "rtarrow-reg");
+    LoadPixmap(NextAct, "rtarrow-sel");
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
 MythListButtonItem::MythListButtonItem(MythListButton* lbtype, 
                                        const QString& text,
                                        MythImage *image, bool checkable,
@@ -857,63 +776,9 @@ MythListButtonItem::MythListButtonItem(MythListButton* lbtype,
     if (state >= NotChecked)
         m_checkable = true;
 
-    CalcDimensions();
-
     m_parent->InsertItem(this);
 }
 
-void MythListButtonItem::CalcDimensions(void)
-{
-    if (!m_parent->m_initialized)
-        m_parent->Init();
-
-    int  margin    = m_parent->m_itemMargin;
-    int  width     = m_parent->m_rect.width();
-    int  height    = m_parent->m_itemHeight;
-    bool arrow = false;
-
-    if (m_parent->m_showArrow || m_showArrow)
-        arrow = true;
-
-    MythImage *checkPix = m_parent->m_checkNonePix;
-    MythImage *arrowPix = m_parent->m_arrowPix;
-    
-    int cw = checkPix->width();
-    int ch = checkPix->height();
-    int aw = arrowPix->width();
-    int ah = arrowPix->height();
-    int pw = m_image ? m_image->width() : 0;
-    int ph = m_image ? m_image->height() : 0;
-   
-    if (m_checkable) 
-        m_checkRect = QRect(margin, (height - ch)/2, cw, ch);
-    else
-        m_checkRect = QRect(0,0,0,0);
-
-    if (arrow) 
-        m_arrowRect = QRect(width - aw - margin, (height - ah)/2,
-                            aw, ah);
-    else
-        m_arrowRect = QRect(0,0,0,0);
-
-    if (m_image) 
-        m_imageRect = QRect(m_checkable ? (2*margin + m_checkRect.width()) :
-                            margin, (height - ph)/2,
-                            pw, ph);
-    else
-        m_imageRect = QRect(0,0,0,0);
-
-    m_textRect = QRect(margin +
-                       (m_checkable ? m_checkRect.width() + margin : 0) +
-                       (m_image    ? m_imageRect.width() + margin : 0),
-                       0,
-                       width - 2*margin -
-                       (m_checkable ? m_checkRect.width() + margin : 0) -
-                       (arrow ? m_arrowRect.width() + margin : 0) -
-                       (m_image ? m_imageRect.width() + margin : 0),
-                       height);
-}
-
 MythListButtonItem::~MythListButtonItem()
 {
     if (m_parent)
@@ -928,7 +793,6 @@ QString MythListButtonItem::text() const
 void MythListButtonItem::setText(const QString &text)
 {
     m_text = text;
-    CalcDimensions();
 }
 
 const MythImage* MythListButtonItem::image() const
@@ -939,7 +803,6 @@ const MythImage* MythListButtonItem::image() const
 void MythListButtonItem::setImage(MythImage *image)
 {
     m_image = image;
-    CalcDimensions();
 }
 
 bool MythListButtonItem::checkable() const
@@ -967,13 +830,11 @@ void MythListButtonItem::setChecked(MythListButtonItem::CheckState state)
 void MythListButtonItem::setCheckable(bool flag)
 {
     m_checkable = flag;
-    CalcDimensions();
 }
 
 void MythListButtonItem::setDrawArrow(bool flag)
 {
     m_showArrow = flag;
-    CalcDimensions();
 }
 
 void MythListButtonItem::setData(void *data)
@@ -1001,72 +862,38 @@ bool MythListButtonItem::moveUpDown(bool flag)
     return m_parent->MoveItemUpDown(this, flag);
 }
 
-void MythListButtonItem::paint(MythPainter *p, MythFontProperties *font, 
-                               int x, int y, bool active_on, int alpha)
+void MythListButtonItem::SetToRealButton(MythUIButton *button, bool active_on)
 {
+    button->SetText(m_text, m_parent->m_textFlags);
+    button->SetButtonImage(m_image);
+
+    if (m_state == NotChecked)
+        button->SetCheckState(MythUIStateType::Off);
+    else if (m_state == HalfChecked)
+        button->SetCheckState(MythUIStateType::Half);
+    else
+        button->SetCheckState(MythUIStateType::Full);
+
+    button->EnableCheck(m_checkable);
+
     if (this == m_parent->m_selItem)
     {
         if (m_parent->m_active && !m_overrideInactive && active_on)
-            p->DrawImage(x, y, m_parent->m_itemSelActPix, alpha);
+            button->SelectState(MythUIButton::Selected);
         else
         {
             if (active_on)
-                p->DrawImage(x, y, m_parent->m_itemSelInactPix, alpha);
+                button->SelectState(MythUIButton::SelectedInactive);
             else
-                p->DrawImage(x, y, m_parent->m_itemRegPix, alpha);
+                button->SelectState(MythUIButton::Active);
         }
 
-        if (m_parent->m_showArrow || m_showArrow)
-        {
-            QRect ar(m_arrowRect);
-            ar.moveBy(x, y);
-            p->DrawImage(ar.x(), ar.y(), m_parent->m_arrowPix, alpha);
-        }
+        button->EnableRightArrow(m_parent->m_showArrow || m_showArrow);
     }
     else
     {
-
-        p->DrawImage(x, y, m_parent->m_itemRegPix, alpha);
-        /*
-        
-            Don't understand this
-                    - thor
-                    
-        if (m_parent->m_active && !m_overrideInactive)
-        {
-            p->drawPixmap(x, y, m_parent->m_itemRegPix);
-        }
-        else
-        {
-            p->drawPixmap(x, y, m_parent->m_itemRegPix);
-        }
-        */
+        button->SelectState(MythUIButton::Normal);
+        button->EnableRightArrow(false);
     }
-
-    if (m_checkable)
-    {
-        QRect cr(m_checkRect);
-        cr.moveBy(x, y);
-        
-        if (m_state == HalfChecked)
-            p->DrawImage(cr.x(), cr.y(), m_parent->m_checkHalfPix, alpha);
-        else if (m_state == FullChecked)
-            p->DrawImage(cr.x(), cr.y(), m_parent->m_checkFullPix, alpha);
-        else
-            p->DrawImage(cr.x(), cr.y(), m_parent->m_checkNonePix, alpha);
-    }
-
-    if (m_image)
-    {
-        QRect pr(m_imageRect);
-        pr.moveBy(x, y);
-        p->DrawImage(pr.x(), pr.y(), m_image, alpha);
-    }
-
-    QRect tr(m_textRect);
-    tr.moveBy(x, y);
-    QString text = m_parent->cutDown(m_text, &(font->face), false,
-                                     tr.width(), tr.height());
-    p->DrawText(tr, text, m_parent->m_textFlags, *font, alpha);    
 }
 
