@@ -4,11 +4,14 @@
 #include <qtimer.h>
 #include <qpainter.h>
 #include <qpixmap.h>
-#include <qsqldatabase.h>
 #include <qkeysequence.h>
+#include <qpaintdevicemetrics.h>
 #ifdef QWS
 #include <qwindowsystem_qws.h>
 #endif
+#ifdef Q_WS_MACX
+#import <HIToolbox/Menus.h>   // For GetMBarHeight()
+#endif
 
 #ifdef USE_LIRC
 #include <pthread.h>
@@ -22,14 +25,18 @@
 #include "mythpainter_ogl.h"
 #include "mythpainter_qt.h"
 #include "mythcontext.h"
+#include "mythdbcon.h"
+#include "mythgesture.h"
 
 //#include "screensaver.h"
 
+#define GESTURE_TIMEOUT 1000
+
 #ifdef USE_LIRC
 static void *SpawnLirc(void *param)
 {
     MythMainWindow *main_window = (MythMainWindow *)param;
-    QString config_file = QDir::homeDirPath() + "/.mythtv/lircrc";
+    QString config_file = MythContext::GetConfDir() + "/lircrc";
     QString program("mythtv");
     LircClient *cl = new LircClient(main_window);
     if (!cl->Init(config_file, program))
@@ -89,7 +96,11 @@ class MythMainWindowPrivate
 
     float wmult, hmult;
     int screenwidth, screenheight;
+
+    QRect screenRect;
+
     int xbase, ybase;
+    bool does_fill_screen;
 
     bool ignore_lirc_keys;
 
@@ -118,6 +129,11 @@ class MythMainWindowPrivate
     MythPainter *painter;
 
     bool AllowInput;
+
+    QRegion repaintRegion;
+
+    MythGesture gesture;
+    QTimer *gestureTimer;
 };
 
 // Make keynum in QKeyEvent be equivalent to what's in QKeySequence
@@ -175,6 +191,7 @@ MythMainWindow::MythMainWindow()
     d->AllowInput = false;
 
     d->painter = new MythOpenGLPainter();
+    //d->painter = new MythQtPainter();
 
     Init();
 
@@ -186,6 +203,8 @@ MythMainWindow::MythMainWindow()
     d->escapekey = Key_Escape;
     d->mainStack = NULL;
 
+    installEventFilter(this);
+
 #ifdef USE_LIRC
     pthread_t lirc_tid;
     pthread_attr_t attr;
@@ -225,21 +244,22 @@ MythMainWindow::MythMainWindow()
     RegisterKey("Global", "8", "8", "8");
     RegisterKey("Global", "9", "9", "9");
 
-    qApp->setOverrideCursor(QCursor(Qt::BlankCursor));
-
     setAutoBufferSwap(false);
 
+    d->gestureTimer = new QTimer(this);
+    connect(d->gestureTimer, SIGNAL(timeout()), this, SLOT(mouseTimeout()));
+
     d->drawTimer = new QTimer(this);
     connect(d->drawTimer, SIGNAL(timeout()), this, SLOT(drawTimeout()));
     d->drawTimer->start(1000 / 70);
 
     d->AllowInput = true;
+
+    d->repaintRegion = QRegion(QRect(0,0,0,0));
 }
 
 MythMainWindow::~MythMainWindow()
 {
-    qApp->restoreOverrideCursor();
-
     while (!d->stackList.isEmpty())
     {
         delete d->stackList.back();
@@ -270,44 +290,60 @@ void MythMainWindow::drawTimeout(void)
 {
     bool redraw = false;
 
+    if (!d->repaintRegion.isEmpty())
+        redraw = true;
+
     QValueVector<MythScreenStack *>::Iterator it;
     for (it = d->stackList.begin(); it != d->stackList.end(); ++it)
     {
-         QValueVector<MythScreenType *> drawList;
-         (*it)->GetDrawOrder(drawList);
+        QValueVector<MythScreenType *> drawList;
+        (*it)->GetDrawOrder(drawList);
 
-         QValueVector<MythScreenType *>::Iterator screenit;
-         for (screenit = drawList.begin(); screenit != drawList.end();
-              ++screenit)
-         {
-             (*screenit)->Pulse();
-         }
+        QValueVector<MythScreenType *>::Iterator screenit;
+        for (screenit = drawList.begin(); screenit != drawList.end();
+             ++screenit)
+        {
+            (*screenit)->Pulse();
+        }
 
-         // Should we care if non-top level screens need redrawing?
-         MythScreenType *top = (*it)->GetTopScreen();
-         if (top && top->NeedsRedraw())
-             redraw = true;
+        // Should we care if non-top level screens need redrawing?
+        MythScreenType *top = (*it)->GetTopScreen();
+        if (top && top->NeedsRedraw())
+        {
+            QRect topDirty = top->GetDirtyArea();
+            d->repaintRegion = d->repaintRegion.unite(topDirty);
+            redraw = true;
+        }
     }
 
     if (!redraw)
+    {
         return;
+    }
+
+    if (!d->painter->SupportsClipping())
+        d->repaintRegion = d->repaintRegion.unite(d->screenRect);
 
     d->painter->Begin(this);
 
+    d->painter->SetClipRect(d->repaintRegion.boundingRect());
+
     for (it = d->stackList.begin(); it != d->stackList.end(); ++it)
     {
-         QValueVector<MythScreenType *> redrawList;
-         (*it)->GetDrawOrder(redrawList);
+        QValueVector<MythScreenType *> redrawList;
+        (*it)->GetDrawOrder(redrawList);
 
-         QValueVector<MythScreenType *>::Iterator screenit;
-         for (screenit = redrawList.begin(); screenit != redrawList.end(); 
-              ++screenit)
-         {
-             (*screenit)->Draw(d->painter, 0, 0);
-         }
+        QValueVector<MythScreenType *>::Iterator screenit;
+        for (screenit = redrawList.begin(); screenit != redrawList.end(); 
+             ++screenit)
+        {
+            (*screenit)->Draw(d->painter, 0, 0, 255, d->repaintRegion.boundingRect());
+        }
     }
 
     d->painter->End();
+    
+    d->repaintRegion = QRegion(QRect(0,0,0,0));
 }
 
 void MythMainWindow::closeEvent(QCloseEvent *e)
@@ -316,47 +352,83 @@ void MythMainWindow::closeEvent(QCloseEvent *e)
     qApp->quit();
 }
 
+void MythMainWindow::paintEvent(QPaintEvent *pe)
+{
+    d->repaintRegion = d->repaintRegion.unite(pe->region());
+}
+
 void MythMainWindow::Init(void)
 {
-
     gContext->GetScreenSettings(d->xbase, d->screenwidth, d->wmult,
                                 d->ybase, d->screenheight, d->hmult);
+
+    d->screenRect = QRect(d->xbase, d->ybase, d->screenwidth, d->screenheight);
+
     setGeometry(d->xbase, d->ybase, d->screenwidth, d->screenheight);
     setFixedSize(QSize(d->screenwidth, d->screenheight));
 
-/*
-    setFont(gContext->GetMediumFont());
-    setCursor(QCursor(Qt::BlankCursor));
-*/
-    setGeometry(100, 100, 800, 600);
-
-    if (d->painter->GetName() != "OpenGL")
-        setFixedSize(QSize(800, 600));
-
+    bool hideCursor = gContext->GetNumSetting("HideMouseCursor", 1);
 #ifdef QWS
 #if QT_VERSION >= 0x030300
-    QWSServer::setCursorVisible(false);
+    QWSServer::setCursorVisible(!hideCursor);
 #endif
 #endif
 
-    WFlags flags = getWFlags();
-    flags |= WRepaintNoErase;
-#ifdef QWS
-    flags |= WResizeNoErase;
+    if (gContext->GetNumSetting("RunFrontendInWindow", 0))
+        d->does_fill_screen = false;
+    else
+        d->does_fill_screen = true;
+
+    // Set window border based on fullscreen attribute
+    Qt::WFlags flags = 0;
+    if (d->does_fill_screen)
+        flags = Qt::WStyle_Customize  |
+                Qt::WStyle_NoBorder;
+    else
+        flags = Qt::WStyle_Customize | Qt::WStyle_NormalBorder;
+
+    // Workarounds for Qt/Mac bugs
+#ifdef Q_WS_MACX
+    if (d->does_fill_screen)
+    {
+  #if QT_VERSION >= 0x030303
+        flags = Qt::WStyle_Customize | Qt::WStyle_Splash;
+  #else
+        // Qt/Mac 3.3.2 and earlier have problems with input focus
+        // when a borderless window is created more than once,
+        // so we have to force windows to have borders
+        flags = Qt::WStyle_Customize | Qt::WStyle_DialogBorder;
+
+        // Move this window up enough to hide its title bar, which in
+        // all the OS X releases so far is the same height as the menubar
+        d->ybase -= GetMBarHeight();
+  #endif
+    }
 #endif
-    setWFlags(flags);
+
+    reparent(parentWidget(), flags, pos());
 
     Show();
+
+    // Set cursor call must come after Show() to work on some systems.
+    setCursor((hideCursor) ? (Qt::BlankCursor) : (Qt::ArrowCursor));
+
     move(d->xbase, d->ybase);
 }
 
 void MythMainWindow::Show(void)
 {
-    //if (gContext->GetNumSetting("RunFrontendInWindow", 0))
-        show();
-    //else
-    //    showFullScreen();
+    show();
+#ifdef Q_WS_MACX
+    if (d->does_fill_screen)
+        HideMenuBar();
+    else
+        ShowMenuBar();
+#endif
+
     setActiveWindow();
+    raise();
+    qApp->wakeUpGuiThread();    // ensures that setActiveWindow() occurs
 }
 
 void MythMainWindow::ExitToMainMenu(void)
@@ -449,37 +521,41 @@ void MythMainWindow::RegisterKey(const QString &context, const QString &action,
 {
     QString keybind = key;
 
-    QSqlDatabase *db = QSqlDatabase::database();
-
-    QString thequery = QString("SELECT keylist FROM keybindings WHERE "
-                               "context = \"%1\" AND action = \"%2\" AND "
-                               "hostname = \"%2\";")
-                              .arg(context).arg(action)
-                              .arg(gContext->GetHostName());
-
-    QSqlQuery query = db->exec(thequery);
-
-    if (query.isActive() && query.numRowsAffected() > 0)
+    MSqlQuery query(MSqlQuery::InitCon());
+    if (query.isConnected())
     {
-        query.next();
+        query.prepare("SELECT keylist FROM keybindings WHERE "
+                      "context = :CONTEXT AND action = :ACTION AND "
+                      "hostname = :HOSTNAME ;");
+        query.bindValue(":CONTEXT", context);
+        query.bindValue(":ACTION", action);
+        query.bindValue(":HOSTNAME", gContext->GetHostName());
 
-        keybind = query.value(0).toString();
-    }
-    else
-    {
-        QString inskey = keybind;
-        inskey.replace('\\', "\\\\");
-        inskey.replace('\"', "\\\"");
+        if (query.exec() && query.isActive() && query.size() > 0)
+        {
+            query.next();
+            keybind = query.value(0).toString();
+        }
+        else
+        {
+            QString inskey = keybind;
+            inskey.replace('\\', "\\\\");
 
-        thequery = QString("INSERT INTO keybindings (context, action, "
-                           "description, keylist, hostname) VALUES "
-                           "(\"%1\", \"%2\", \"%3\", \"%4\", \"%5\");")
-                           .arg(context).arg(action).arg(description)
-                           .arg(inskey).arg(gContext->GetHostName());
+            query.prepare("INSERT INTO keybindings (context, action, "
+                          "description, keylist, hostname) VALUES "
+                          "( :CONTEXT, :ACTION, :DESCRIPTION, :KEYLIST, "
+                          ":HOSTNAME );");
+            query.bindValue(":CONTEXT", context);
+            query.bindValue(":ACTION", action);
+            query.bindValue(":DESCRIPTION", description);
+            query.bindValue(":KEYLIST", inskey);
+            query.bindValue(":HOSTNAME", gContext->GetHostName());
 
-        query = db->exec(thequery);
-        if (!query.isActive())
-            MythContext::DBError("Insert Keybinding", query);
+            if (!query.exec() || !query.isActive())
+            {
+                MythContext::DBError("Insert Keybinding", query);
+            }
+        }
     }
 
     QKeySequence keyseq(keybind);
@@ -517,37 +593,40 @@ void MythMainWindow::RegisterJump(const QString &destination,
 {
     QString keybind = key;
 
-    QSqlDatabase *db = QSqlDatabase::database();
-
-    QString thequery = QString("SELECT keylist FROM jumppoints WHERE "
-                               "destination = \"%1\" and hostname = \"%2\";")
-                              .arg(destination).arg(gContext->GetHostName());
-
-    QSqlQuery query = db->exec(thequery);
-
-    if (query.isActive() && query.numRowsAffected() > 0)
+    MSqlQuery query(MSqlQuery::InitCon());
+    if (query.isConnected())
     {
-        query.next();
+        query.prepare("SELECT keylist FROM jumppoints WHERE "
+                      "destination = :DEST and hostname = :HOST ;");
+        query.bindValue(":DEST", destination);
+        query.bindValue(":HOST", gContext->GetHostName());
+
+        if (query.exec() && query.isActive() && query.size() > 0)
+        {
+            query.next();
+            keybind = query.value(0).toString();
+        }
+        else
+        {
+            QString inskey = keybind;
+            inskey.replace('\\', "\\\\");
+            inskey.replace('\"', "\\\"");
+
+            query.prepare("INSERT INTO jumppoints (destination, description, "
+                          "keylist, hostname) VALUES ( :DEST, :DESC, :KEYLIST, "
+                          ":HOST );");
+            query.bindValue(":DEST", destination);
+            query.bindValue(":DEDC", description);
+            query.bindValue(":KEYLIST", inskey);
+            query.bindValue(":HOST", gContext->GetHostName());
+
+            if (!query.exec() || !query.isActive())
+            {
+                MythContext::DBError("Insert Jump Point", query);
+            }
+        }
+    }
  
-        keybind = query.value(0).toString();
-    }
-    else
-    {
-        QString inskey = keybind;
-        inskey.replace('\\', "\\\\");
-        inskey.replace('\"', "\\\"");
-
-        thequery = QString("INSERT INTO jumppoints (destination, description, "
-                           "keylist, hostname) VALUES (\"%1\", \"%2\", \"%3\", "
-                           "\"%4\");").arg(destination).arg(description)
-                                      .arg(inskey)
-                                      .arg(gContext->GetHostName());
-
-        query = db->exec(thequery);
-        if (!query.isActive())
-            MythContext::DBError("Insert Jump Point", query);
-    }
-   
     JumpData jd = { callback, destination, description };
     d->destinationMap[destination] = jd;
  
@@ -656,25 +735,129 @@ void MythMainWindow::AllowInput(bool allow)
     d->AllowInput = allow;
 }
 
-void MythMainWindow::keyPressEvent(QKeyEvent *e)
+void MythMainWindow::mouseTimeout(void)
 {
-    if (!d->AllowInput)
-        return;
+    MythGestureEvent *e;
 
-    QValueVector<MythScreenStack *>::Iterator it;
-    for (it = d->stackList.begin(); it != d->stackList.end(); ++it)
+    /* complete the stroke if its our first timeout */
+    if (d->gesture.recording())
     {
-         MythScreenType *top = (*it)->GetTopScreen();
-         if (top)
-         {
-             if (top->keyPressEvent(e))
-                 break;
-         }
+        d->gesture.stop();
     }
+
+    /* get the last gesture */
+    e = d->gesture.gesture();
+
+    if (e->gesture() < MythGestureEvent::Click)
+        QApplication::postEvent(this, e);
 }
 
-void MythMainWindow::customEvent(QCustomEvent* /* ce */)
+bool MythMainWindow::eventFilter(QObject *, QEvent *e)
 {
+    MythGestureEvent *ge;
+
+    /* dont let anything through if input is disallowed. */
+    if (!d->AllowInput)
+        return true;
+
+    switch (e->type())
+    {
+        case QEvent::KeyPress:
+        {
+            QValueVector<MythScreenStack *>::Iterator it;
+            for (it = d->stackList.begin(); it != d->stackList.end(); ++it)
+            {
+                MythScreenType *top = (*it)->GetTopScreen();
+                if (top)
+                {
+                    if (top->keyPressEvent(dynamic_cast<QKeyEvent*>(e)))
+                        return true;
+                }
+            }
+            break;
+        }
+        case QEvent::MouseButtonPress:
+        {
+            if (!d->gesture.recording())
+            {
+                d->gesture.start();
+                d->gesture.record(dynamic_cast<QMouseEvent*>(e)->pos());
+
+                /* start a single shot timer */
+                d->gestureTimer->start(GESTURE_TIMEOUT);
+
+                return true;
+            }
+            break;
+        }
+        case QEvent::MouseButtonRelease:
+        {
+            if (d->gestureTimer->isActive())
+                d->gestureTimer->stop();
+
+            if (d->gesture.recording())
+            {
+                d->gesture.stop();
+                ge = d->gesture.gesture();
+
+                /* handle clicks separately */
+                if (ge->gesture() == MythGestureEvent::Click)
+                {            
+                    MythUIType *clicked;
+                    QValueVector<MythScreenStack *>::iterator it;
+                    QPoint p = dynamic_cast<QMouseEvent*>(e)->pos();
+
+                    delete ge;
+
+                    for (it = d->stackList.begin(); it != d->stackList.end(); 
+                         it++)
+                    {
+                        MythScreenType *screen = (*it)->GetTopScreen();
+                        if ((clicked = screen->GetChildAt(p)) != NULL)
+                        {
+                            cout << "UI Type: " << clicked->name() << endl;
+                            break;
+                        }
+                    }
+                }
+                else 
+                    QApplication::postEvent(this, ge);
+
+                return true;
+            }
+            break;
+        }
+        case QEvent::MouseMove:
+        {
+            if (d->gesture.recording())
+            {
+                /* reset the timer */
+                d->gestureTimer->stop();
+                d->gestureTimer->start(GESTURE_TIMEOUT);
+
+                d->gesture.record(dynamic_cast<QMouseEvent*>(e)->pos());
+                return true;
+            }
+            break;
+        }
+        default:
+            break;
+    }
+
+    return false;
+}
+
+void MythMainWindow::customEvent(QCustomEvent *ce)
+{
+    if (ce->type() == MythGestureEventType)
+    {
+        MythGestureEvent *ge = dynamic_cast<MythGestureEvent*>(ce);
+        if (ge != NULL)
+        {
+            cout << "Gesture: " << QString(*ge) << endl;
+        }
+    }
+
 #if 0
     if (ce->type() == kExitToMainMenuEventType && d->exitingtomain)
     {
@@ -707,7 +890,7 @@ void MythMainWindow::customEvent(QCustomEvent* /* ce */)
             {
                 if ((itr.data().MediaType & (int)pDev->getMediaType()))
                 {
-                    VERBOSE(VB_ALL, "Found a handler");
+                    VERBOSE(VB_IMPORTANT, "Found a handler");
                     d->exitingtomain = true;
                     d->exitmenumediadevicecallback = itr.data().callback;
                     d->mediadeviceforcallback = pDev;
@@ -816,3 +999,56 @@ QObject *MythMainWindow::getTarget(QKeyEvent &key)
     return key_target;
 }
 
+QFont MythMainWindow::CreateFont(const QString &face, int pointSize, 
+                                 int weight, bool italic)
+{
+    // Store w/h mult in MainWindow, and query here
+    //pointSize = pointSize * GetMythMainWindow()->GetHMult();
+
+    // cache this in MainWindow as well.
+    QPaintDeviceMetrics pdm(GetMythMainWindow());
+
+    // Maybe just a helper function in MyhtMainWindow to calc this all?
+    float desired = 100.0;
+    pointSize = (int)(pointSize * desired / pdm.logicalDpiY());
+    pointSize = (int)(pointSize * d->hmult);
+
+    return QFont(face, pointSize, weight, italic);
+}
+
+QRect MythMainWindow::NormRect(const QRect &rect)
+{
+    QRect ret;
+    ret.setWidth((int)(rect.width() * d->wmult));
+    ret.setHeight((int)(rect.height() * d->hmult));
+    ret.moveTopLeft(QPoint((int)(rect.x() * d->wmult),
+                           (int)(rect.y() * d->hmult)));
+    ret = ret.normalize();
+
+    return ret;
+}
+
+QPoint MythMainWindow::NormPoint(const QPoint &point)
+{
+    QPoint ret;
+    ret.setX((int)(point.x() * d->wmult));
+    ret.setY((int)(point.y() * d->hmult));
+
+    return ret;
+}
+
+int MythMainWindow::NormX(const int x)
+{
+    return (int)(x * d->wmult);
+}
+
+int MythMainWindow::NormY(const int y)
+{
+    return (int)(y * d->hmult);
+}
+
+QRect MythMainWindow::GetUIScreenRect(void)
+{
+    return QRect(0, 0, d->screenwidth, d->screenheight);
+}
+
