@@ -4,8 +4,8 @@
 #include "mythpainter.h"
 #include "mythimage.h"
 
-class StringImageCache;
-class IntImageCache;
+#include <list>
+using namespace std;
 
 class MythOpenGLPainter : public MythPainter
 {
@@ -16,6 +16,7 @@ class MythOpenGLPainter : public MythPainter
     virtual QString GetName(void) { return QString("OpenGL"); }
     virtual bool SupportsAnimation(void) { return true; }
     virtual bool SupportsAlpha(void) { return true; }
+    virtual bool SupportsClipping(void) { return false; }
 
     virtual void Begin(QWidget *parent);
     virtual void End();
@@ -40,9 +41,14 @@ class MythOpenGLPainter : public MythPainter
     MythImage *GetImageFromString(const QString &msg, int flags, const QRect &r,
                                   const MythFontProperties &font);
 
-    StringImageCache *m_StringImageCache;
-    IntImageCache *m_IntImageCache;
     QMap<MythImage *, unsigned int> m_ImageIntMap;
+    list<MythImage *> m_ImageExpireList;
+
+    QMap<QString, MythImage *> m_StringToImageMap;
+    list<QString> m_StringExpireList;
+
+    int q_gl_texture;
+    bool texture_rects;
 };
 
 #endif
