@@ -13,6 +13,7 @@ class MythFontProperties;
 class MythUIText : public MythUIType
 {
   public:
+    MythUIText(MythUIType *parent, const char *name);
     MythUIText(const QString &text, const MythFontProperties &font,
                QRect displayRect, QRect altDisplayRect,
                MythUIType *parent, const char *name);
@@ -30,17 +31,17 @@ class MythUIText : public MythUIType
     int GetJustification(void);
     void SetCutDown(bool cut);
 
-    void SetDisplayArea(const QRect &rect);
+    void SetArea(const QRect &rect);
 
     virtual void Pulse(void);
 
-    virtual void Draw(MythPainter *p, int xoffset, int yoffset, 
-                      int alphaMod = 255);
-
     void CycleColor(QColor startColor, QColor endColor, int numSteps);
     void StopCycling();
 
   protected:
+    virtual void DrawSelf(MythPainter *p, int xoffset, int yoffset,
+                          int alphaMod, QRect clipRect);
+
     int m_Justification;
     QRect m_OrigDisplayRect;
     QRect m_AltDisplayRect;
