@@ -13,8 +13,10 @@
 using namespace std;
 
 #include "housekeeper.h"
+#include "jobqueue.h"
 
 #include "libmyth/mythcontext.h"
+#include "libmyth/mythdbcon.h"
 
 bool HouseKeeper_filldb_running = false;
 
@@ -24,14 +26,15 @@ void reapChild(int /* sig */)
     HouseKeeper_filldb_running = false;
 }
 
-HouseKeeper::HouseKeeper(bool runthread, bool master, QSqlDatabase *ldb)
+HouseKeeper::HouseKeeper(bool runthread, bool master)
 {
-    db = ldb;
     isMaster = master;
 
     threadrunning = runthread;
     filldbRunning = false;
 
+    CleanupMyOldRecordings();
+
     if (runthread)
     {
         pthread_t hkthread;
@@ -48,22 +51,29 @@ bool HouseKeeper::wantToRun(const QString &dbTag, int period, int minhour,
 {
     bool runOK = false;
     unsigned int oneday = 60 * 60 * 24;
+    unsigned int longEnough = 0;
+
+    if (period)
+        longEnough = ((period * oneday) - 600);
+    else
+        longEnough = oneday / 8;
 
     QDateTime now = QDateTime::currentDateTime();
     QDateTime lastrun;
     lastrun.setTime_t(0);
-    if (db->isOpen())
+
+    MSqlQuery result(MSqlQuery::InitCon());
+    if (result.isConnected())
     {
-        gContext->KickDatabase(db);
-        QString query = QString("SELECT lastrun FROM housekeeping WHERE "
-                                "tag = \"%1\";").arg(dbTag);
-        QSqlQuery result = db->exec(query);
-        if (result.isActive() && result.numRowsAffected() > 0)
+        result.prepare("SELECT lastrun FROM housekeeping WHERE tag = :TAG ;");
+        result.bindValue(":TAG", dbTag);
+
+        if (result.exec() && result.isActive() && result.size() > 0)
         {
             result.next();
             lastrun = result.value(0).toDateTime();
 
-            if ((now.toTime_t() - lastrun.toTime_t()) > period * oneday)
+            if ((now.toTime_t() - lastrun.toTime_t()) > longEnough)
             {
                 int hour = now.toString(QString("h")).toInt();
                 if ((hour >= minhour) && (hour <= maxhour))
@@ -72,9 +82,10 @@ bool HouseKeeper::wantToRun(const QString &dbTag, int period, int minhour,
         }
         else
         {
-            query = QString("INSERT INTO housekeeping(tag,lastrun) "
-                            "values(\"%1\",now())").arg(dbTag);
-            db->exec(query);
+            result.prepare("INSERT INTO housekeeping(tag,lastrun) "
+                           "values(:TAG ,now());");
+            result.bindValue(":TAG", dbTag);
+            result.exec();
 
             runOK = true;
         }
@@ -85,28 +96,52 @@ bool HouseKeeper::wantToRun(const QString &dbTag, int period, int minhour,
 
 void HouseKeeper::updateLastrun(const QString &dbTag)
 {
-    if (db->isOpen())
+    MSqlQuery result(MSqlQuery::InitCon());
+    if (result.isConnected())
     {
-        gContext->KickDatabase(db);
-        QString query = QString("DELETE FROM housekeeping WHERE "
-                                "tag = \"%1\";").arg(dbTag);
-        QSqlQuery result = db->exec(query);
-        query = QString("INSERT INTO housekeeping(tag,lastrun) "
-                        "values(\"%1\",now())").arg(dbTag);
-        result = db->exec(query);
+        result.prepare("DELETE FROM housekeeping WHERE tag = :TAG ;");
+        result.bindValue(":TAG", dbTag);
+        result.exec();
+
+        result.prepare("INSERT INTO housekeeping(tag,lastrun) "
+                       "values(:TAG ,now()) ;");
+        result.bindValue(":TAG", dbTag);
+        result.exec();
     }
 }
 
+QDateTime HouseKeeper::getLastRun(const QString &dbTag)
+{
+    QDateTime lastRun;
+    MSqlQuery result(MSqlQuery::InitCon());
+
+    lastRun.setTime_t(0);
+
+    result.prepare("SELECT lastrun FROM housekeeping WHERE tag = :TAG ;");
+    result.bindValue(":TAG", dbTag);
+
+    if (result.exec() && result.isActive() && result.size() > 0)
+    {
+        result.next();
+        lastRun = result.value(0).toDateTime();
+    }
+
+    return lastRun;
+}
+
 void HouseKeeper::RunHouseKeeping(void)
 {
     int period, maxhr, minhr;
+    QString dbTag;
     // wait a little for main server to come up and things to settle down
     sleep(10);
 
     while (1)
     {
+        VERBOSE(VB_IMPORTANT, "Running HouseKeeping");
         gContext->LogEntry("mythbackend", LP_DEBUG,
                            "Running housekeeping thread", "");
+
         // These tasks are only done from the master backend
         if (isMaster)
         {
@@ -145,7 +180,44 @@ void HouseKeeper::RunHouseKeeping(void)
                         maxhr = gContext->GetNumSetting("MythFillMaxHour", 24);
                     }
 
-                    if (wantToRun("MythFillDB", period, minhr, maxhr))
+                    bool grabberSupportsNextTime = false;
+                    MSqlQuery result(MSqlQuery::InitCon());
+                    if (result.isConnected())
+                    {
+                        result.prepare("SELECT COUNT(*) FROM videosource "
+                                       "WHERE xmltvgrabber IN "
+                                           "( 'datadirect', 'technovera' );");
+
+                        if ((result.exec()) &&
+                            (result.isActive()) &&
+                            (result.size() > 0) &&
+                            (result.next()) &&
+                            (result.value(0).toInt() > 0))
+                            grabberSupportsNextTime = true;
+                    }
+
+                    bool runMythFill = false;
+                    if (grabberSupportsNextTime &&
+                        gContext->GetNumSetting("MythFillGrabberSuggestsTime", 1))
+                    {
+                        QDateTime nextRun = QDateTime::fromString(
+                            gContext->GetSetting("MythFillSuggestedRunTime",
+                            "1970-01-01T00:00:00"), Qt::ISODate);
+                        QDateTime lastRun = getLastRun("MythFillDB");
+                        QDateTime now = QDateTime::currentDateTime();
+                        int hour = now.toString(QString("h")).toInt();
+
+                        if ((nextRun < now) &&
+                            (lastRun.secsTo(now) > (3 * 60 * 60)) &&
+                            ((minhr <= hour) && (hour <= maxhr)))
+                            runMythFill = true;
+                    }
+                    else if (wantToRun("MythFillDB", period, minhr, maxhr))
+                    {
+                        runMythFill = true;
+                    }
+
+                    if (runMythFill)
                     {
                         QString msg = "Running mythfilldatabase";
                         gContext->LogEntry("mythbackend", LP_DEBUG, msg, "");
@@ -155,8 +227,27 @@ void HouseKeeper::RunHouseKeeping(void)
                     }
                 }
             }
+
+            if (wantToRun("JobQueueCleanup", 1, 0, 24))
+            {
+                JobQueue::CleanupOldJobsInQueue();
+                updateLastrun("JobQueueCleanup");
+            }
+
+            if (wantToRun("InUseProgramsCleanup", 1, 0, 24))
+            {
+                CleanupAllOldInUsePrograms();
+                updateLastrun("InUseProgramsCleanup");
+            }
         }
-    
+
+        dbTag = QString("JobQueueRecover-%1").arg(gContext->GetHostName());
+        if (wantToRun(dbTag, 1, 0, 24))
+        {
+            JobQueue::RecoverOldJobsInQueue();
+            updateLastrun(dbTag);
+        }
+
         sleep(300);
     }
 } 
@@ -171,19 +262,17 @@ void HouseKeeper::flushLogs()
     QDateTime max = QDateTime::currentDateTime();
     max = max.addDays(0 - maxdays);
 
-    if (db->isOpen())
+    MSqlQuery result(MSqlQuery::InitCon());
+    if (result.isConnected())
     {
-        gContext->KickDatabase(db);
-        QString dstring = days.toString(QString("yyyy-MM-dd hh:mm:ss"));
-        QString query = QString("DELETE FROM mythlog WHERE "
-                                "acknowledged=1 and logdate<\"%1\";")
-                                .arg(dstring);
-        QSqlQuery result = db->exec(query);
+        result.prepare("DELETE FROM mythlog WHERE "
+                       "acknowledged=1 and logdate < :DAYS ;");
+        result.bindValue(":DAYS", days);
+        result.exec();
 
-        dstring = max.toString(QString("yyyy-MM-dd hh:mm:ss"));
-        query = QString("DELETE FROM mythlog WHERE logdate<\"%1\";")
-                                .arg(dstring);
-        result = db->exec(query);
+        result.prepare("DELETE FROM mythlog WHERE logdate< :MAX ;");
+        result.bindValue(":MAX", max);
+        result.exec();
     }
 }
 
@@ -201,7 +290,7 @@ void HouseKeeper::runFillDatabase()
         command = QString("%1 %2").arg(mfpath).arg(mfarg);
     else
         command = QString("%1 %2 >>%3 2>&1").arg(mfpath).arg(mfarg).arg(mflog);
-    
+
     signal(SIGCHLD, &reapChild);
     HouseKeeper_filldb_running = true;
     if (fork() == 0)
@@ -209,10 +298,32 @@ void HouseKeeper::runFillDatabase()
         for(int i = 3; i < sysconf(_SC_OPEN_MAX) - 1; ++i)
             close(i);
         system(command.ascii());
-        _exit(0);
+        _exit(0); // this exit is ok, non-error exit from system command.
     }
 }
 
+void HouseKeeper::CleanupMyOldRecordings(void)
+{
+    MSqlQuery query(MSqlQuery::InitCon());
+
+    query.prepare("DELETE FROM inuseprograms "
+                  "WHERE hostname = :HOSTNAME AND "
+                    "( recusage = 'recorder' OR recusage LIKE 'Unknown %' );");
+    query.bindValue(":HOSTNAME", gContext->GetHostName());
+    query.exec();
+}
+
+void HouseKeeper::CleanupAllOldInUsePrograms(void)
+{
+    QDateTime fourHoursAgo = QDateTime::currentDateTime().addSecs(-4 * 60 * 60);
+    MSqlQuery query(MSqlQuery::InitCon());
+
+    query.prepare("DELETE FROM inuseprograms "
+                  "WHERE lastupdatetime > :FOURHOURSAGO ;");
+    query.bindValue(":FOURHOURSAGO", fourHoursAgo);
+    query.exec();
+}
+
 void *HouseKeeper::doHouseKeepingThread(void *param)
 {
     HouseKeeper *hkeeper = (HouseKeeper*)param;
@@ -220,3 +331,5 @@ void *HouseKeeper::doHouseKeepingThread(void *param)
  
     return NULL;
 }
+
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
