@@ -16,12 +16,6 @@ HttpStatus::HttpStatus(MainServer *parent, int port)
           : QServerSocket(port, 1)
 {
     m_parent = parent;
-
-    if (!ok())
-    { 
-        cerr << "Failed to bind to port " << port << endl;
-        exit(0);
-    }
 }
 
 void HttpStatus::newConnection(int socket)
@@ -56,8 +50,17 @@ void HttpStatus::readClient(void)
 
                 os << "HTTP/1.0 200 Ok\r\n"
                    << "Content-Type: text/xml; charset=\"UTF-8\"\r\n"
-                   << "\r\n"
-                   << doc.toString();
+                   << "\r\n";
+
+                // Work around a memory leak seen in Qt 3.3 when
+                // more than 510 characters are presented to
+                // QTextStream' <<. Use 200 character chunks to
+                // allow plenty of room for Unicode -> UTF8
+                // expansions.
+
+                QString xml(doc.toString());
+                for (unsigned i = 0; i < xml.length(); i += 200)
+                    os << xml.mid(i, 200);
             }
             else
                 PrintStatus( socket, &doc );
@@ -224,15 +227,6 @@ void HttpStatus::PrintStatus(QSocket *socket, QDomDocument *pDoc )
     if (!node.isNull())
         PrintMachineInfo( os, node.toElement());
 
-#if USING_DVB
-    // DVB Signal Information ------------------
-
-    node = docElem.namedItem( "DVBStatus" );
-
-    if (!node.isNull())
-        PrintDVBStatus( os, node.toElement());
-#endif
-
     os << "\r\n</body>\r\n</html>\r\n";
 }
 
@@ -471,9 +465,13 @@ int HttpStatus::PrintScheduled( QTextStream &os, QDomElement scheduled )
 
                     // Output HTML
 
-                os << "      <a href=\"#\">"
-                   << recStartTs.addSecs(-nPreRollSecs).toString("ddd") << " "
-                   << recStartTs.addSecs(-nPreRollSecs).toString(shortdateformat) << " "
+                os << "      <a href=\"#\">";
+                if (shortdateformat.find("ddd") == -1) {
+                    // If day-of-week not already present somewhere, prepend it.
+                    os << recStartTs.addSecs(-nPreRollSecs).toString("ddd")
+                        << " ";
+                }
+                os << recStartTs.addSecs(-nPreRollSecs).toString(shortdateformat) << " "
                    << recStartTs.addSecs(-nPreRollSecs).toString(timeformat) << " - ";
 
                 if (nEncoderId > 0)
@@ -713,10 +711,13 @@ int HttpStatus::PrintMachineInfo( QTextStream &os, QDomElement info )
 
         if (!e.isNull())
         {
+            QString datetimefmt = "yyyy-MM-dd hh:mm";
             int     nDays   = e.attribute( "guideDays", "0" ).toInt();
-            QString    sStatus = e.attribute( "status"   , ""  );
             QString sStart  = e.attribute( "start"    , ""  );
             QString sEnd    = e.attribute( "end"      , ""  );
+            QString sStatus = e.attribute( "status"   , ""  );
+            QDateTime next  = QDateTime::fromString( e.attribute( "next"     , ""  ), Qt::ISODate);
+            QString sNext   = next.isNull() ? "" : next.toString(datetimefmt);
             QString sMsg    = "";
 
             QDateTime thru  = QDateTime::fromString( e.attribute( "guideThru", ""  ), Qt::ISODate);
@@ -737,10 +738,16 @@ int HttpStatus::PrintMachineInfo( QTextStream &os, QDomElement info )
 
             os << sStatus << "<br />\r\n";    
 
+            if (!next.isNull() && sNext >= sStart)
+            {
+                os << "    Suggested next mythfilldatabase run: "
+                    << sNext << ".<br />\r\n";
+            }
+
             if (!thru.isNull())
             {
                 os << "    There's guide data until "
-                   << QDateTime( thru ).toString("yyyy-MM-dd hh:mm");
+                   << QDateTime( thru ).toString(datetimefmt);
 
                 if (nDays > 0)
                     os << " (" << nDays << " day" << (nDays == 1 ? "" : "s" ) << ")";
@@ -755,7 +762,7 @@ int HttpStatus::PrintMachineInfo( QTextStream &os, QDomElement info )
                    << "Have you run mythfilldatabase?";
 
             if (!sMsg.isNull() && !sMsg.isEmpty())
-                os << "<br />\r\nDataDirect Status: " << sMsg;
+                os << "<br />\r\n    DataDirect Status: " << sMsg;
 
             os << "\r\n  </div>\r\n";
         }
@@ -764,66 +771,4 @@ int HttpStatus::PrintMachineInfo( QTextStream &os, QDomElement info )
     return( 1 );
 }
 
-#if USING_DVB
-
-int HttpStatus::PrintDVBStatus( QTextStream &os, QDomElement status )
-{
-    QString timeDateFormat = gContext->GetSetting("DateFormat", "ddd MMMM d") +
-                       " " + gContext->GetSetting("TimeFormat", "h:mm AP");
-
-    int nCount = 0;
-
-    os << "\r\n  <div class=\"content\">\r\n" <<
-        "    <h2>DVB Signal Information</h2>\r\n" <<
-        "    Details of DVB error statistics for last 48 hours:<br />\r\n";
-
-
-    QDomNode node = status.firstChild();
-
-    while (!node.isNull())
-    {
-        QDomElement eStatus = node.toElement();
-        QDateTime   start   = GetDateTime( eStatus.attribute( "start" ,"" ));
-        QDateTime   end     = GetDateTime( eStatus.attribute( "end"   ,"" ));
-
-        QDomNode nodeInfo = node.firstChild();
-
-        if (!nodeInfo.isNull() )
-            os << "    <br />Recording period from " 
-               << start.toString(timeDateFormat)
-               << " to " << end.toString( timeDateFormat ) 
-               << "<br />\n";
-
-        while (!nodeInfo.isNull())
-        {
-            QDomElement e = nodeInfo.toElement();
-                
-            os << "    Encoder " << e.attribute( "cardId"   , "0").toInt()
-               << " Min SNR: "   << e.attribute( "minSNR"   , "0").toInt()
-               << " Avg SNR: "   << e.attribute( "avgSNR"   , "0").toInt()
-               << " Min BER: "   << e.attribute( "minBER"   , "0").toInt()
-               << " Avg BER: "   << e.attribute( "avgBER"   , "0").toInt()
-               << " Cont Errs: " << e.attribute( "contErrs" , "0").toInt()
-               << " Overflows: " << e.attribute( "overflows", "0").toInt()
-               << "<br />\r\n";
-
-            nCount++;
-
-            nodeInfo = nodeInfo.nextSibling();
-        }
-
-        node = node.nextSibling();
-    }
-
-    if (nCount == 0)
-    {
-        os << "    <br />There is no DVB signal quality data available to "
-            "display.<br />\r\n";
-    }
-
-    os << "  </div>\r\n";
-
-    return( nCount );
-}
-
-#endif
+// vim:set shiftwidth=4 tabstop=4 expandtab:
