@@ -8,6 +8,12 @@
 #include <fcntl.h>
 #include <libgen.h>
 #include <signal.h>
+#include <cerrno>
+
+#include "mythconfig.h"
+#ifdef CONFIG_DARWIN
+    #include <sys/aio.h>    // O_SYNC
+#endif
 
 #include <iostream>
 #include <fstream>
@@ -21,9 +27,10 @@ using namespace std;
 #include "remoteutil.h"
 #include "housekeeper.h"
 
-#include "libmythtv/programinfo.h"
 #include "libmyth/mythcontext.h"
 #include "libmyth/mythdbcon.h"
+#include "libmyth/exitcodes.h"
+#include "libmythtv/programinfo.h"
 #include "libmythtv/dbcheck.h"
 #include "libmythtv/jobqueue.h"
 
@@ -34,15 +41,27 @@ JobQueue *jobqueue = NULL;
 QString pidfile;
 QString lockfile_location;
 HouseKeeper *housekeeping = NULL;
+QString logfile = "";
 
-bool setupTVs(bool ismaster)
+bool setupTVs(bool ismaster, bool &error)
 {
+    error = false;
     QString localhostname = gContext->GetHostName();
 
     MSqlQuery query(MSqlQuery::InitCon());
 
     if (ismaster)
     {
+        // Hack to make sure recorded.basename gets set if the user
+        // downgrades to a prior version and creates new entries
+        // without it.
+        if (!query.exec("UPDATE recorded SET basename = CONCAT(chanid, '_', "
+                        "DATE_FORMAT(starttime, '%Y%m%d%H%i00'), '_', "
+                        "DATE_FORMAT(endtime, '%Y%m%d%H%i00'), '.nuv') "
+                        "WHERE basename = '';"))
+            MythContext::DBError("Updating record basename",
+                                 query.lastQuery());
+
         // Hack to make sure record.station gets set if the user
         // downgrades to a prior version and creates new entries
         // without it.
@@ -107,7 +126,7 @@ bool setupTVs(bool ismaster)
                 cerr << msg;
                 gContext->LogEntry("mythbackend", LP_CRITICAL,
                                    "Problem with capture cards", msg);
-                exit(2);
+                error = true;
             }
 
             if (!ismaster)
@@ -161,8 +180,42 @@ void cleanup(void)
         unlink(pidfile.ascii());
 
     unlink(lockfile_location.ascii());
+
+    signal(SIGHUP, SIG_DFL);
 }
-    
+
+int log_rotate(int report_error)
+{
+    /* http://www.gossamer-threads.com/lists/mythtv/dev/110113 */
+
+    int new_logfd = open(logfile, O_WRONLY|O_CREAT|O_APPEND|O_SYNC, 0664);
+    if (new_logfd < 0)
+    {
+        // If we can't open the new logfile, send data to /dev/null
+        if (report_error)
+        {
+            cerr << "cannot open logfile " << logfile << endl;
+            return -1;
+        }
+        new_logfd = open("/dev/null", O_WRONLY);
+        if (new_logfd < 0)
+        {
+            // There's not much we can do, so punt.
+            return -1;
+        }
+    }
+    while (dup2(new_logfd, 1) < 0 && errno == EINTR) ;
+    while (dup2(new_logfd, 2) < 0 && errno == EINTR) ;
+    while (close(new_logfd) < 0 && errno == EINTR) ;
+    return 0;
+}
+
+void log_rotate_handler(int)
+{
+    log_rotate(0);
+}
+
+
 int main(int argc, char **argv)
 {
     for(int i = 3; i < sysconf(_SC_OPEN_MAX) - 1; ++i)
@@ -170,15 +223,16 @@ int main(int argc, char **argv)
 
     QApplication a(argc, argv, false);
 
-    QString logfile = "";
+    
     QString binname = basename(a.argv()[0]);
-    QString verboseString = QString(" important general");
 
     bool daemonize = false;
     bool printsched = false;
     bool testsched = false;
     bool resched = false;
     bool nosched = false;
+    bool nojobqueue = false;
+    bool noexpirer = false;
     bool printexpire = false;
     for (int argpos = 1; argpos < a.argc(); ++argpos)
     {
@@ -191,7 +245,7 @@ int main(int argc, char **argv)
                 if (logfile.startsWith("-"))
                 {
                     cerr << "Invalid or missing argument to -l/--logfile option\n";
-                    return -1;
+                    return BACKEND_EXIT_INVALID_CMDLINE;
                 }
                 else
                 {
@@ -208,7 +262,7 @@ int main(int argc, char **argv)
                 if (pidfile.startsWith("-"))
                 {
                     cerr << "Invalid or missing argument to -p/--pidfile option\n";
-                    return -1;
+                    return BACKEND_EXIT_INVALID_CMDLINE;
                 } 
                 else
                 {
@@ -225,109 +279,18 @@ int main(int argc, char **argv)
         else if (!strcmp(a.argv()[argpos],"-v") ||
                  !strcmp(a.argv()[argpos],"--verbose"))
         {
-            if (a.argc() > argpos)
+            if (a.argc()-1 > argpos)
             {
-                QString temp = a.argv()[argpos+1];
-                if (temp.startsWith("-"))
-                {
-                    cerr << "Invalid or missing argument to -v/--verbose option\n";
-                    return -1;
-                } 
-                else
-                {
-                    QStringList verboseOpts;
-                    verboseOpts = QStringList::split(',',a.argv()[argpos+1]);
-                    ++argpos;
-                    for (QStringList::Iterator it = verboseOpts.begin(); 
-                         it != verboseOpts.end(); ++it )
-                    {
-                        if(!strcmp(*it,"none"))
-                        {
-                            print_verbose_messages = VB_NONE;
-                            verboseString = "";
-                        }
-                        else if(!strcmp(*it,"all"))
-                        {
-                            print_verbose_messages = VB_ALL;
-                            verboseString = "all";
-                        }
-                        else if(!strcmp(*it,"quiet"))
-                        {
-                            print_verbose_messages = VB_IMPORTANT;
-                            verboseString = "important";
-                        }
-                        else if(!strcmp(*it,"record"))
-                        {
-                            print_verbose_messages |= VB_RECORD;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"playback"))
-                        {
-                            print_verbose_messages |= VB_PLAYBACK;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"channel"))
-                        {
-                            print_verbose_messages |= VB_CHANNEL;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"osd"))
-                        {
-                            print_verbose_messages |= VB_OSD;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"file"))
-                        {
-                            print_verbose_messages |= VB_FILE;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"schedule"))
-                        {
-                            print_verbose_messages |= VB_SCHEDULE;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"network"))
-                        {
-                            print_verbose_messages |= VB_NETWORK;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"commflag"))
-                        {
-                            print_verbose_messages |= VB_COMMFLAG;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"jobqueue"))
-                        {
-                            print_verbose_messages |= VB_JOBQUEUE;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"audio"))
-                        {
-                            print_verbose_messages |= VB_AUDIO;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"libav"))
-                        {
-                            print_verbose_messages |= VB_LIBAV;
-                            verboseString += " " + *it;
-                        }
-                        else if(!strcmp(*it,"siparser"))
-                        {
-                            print_verbose_messages |= VB_SIPARSER;
-                            verboseString += " " + *it;
-                        }
-                        else
-                        {
-                            cerr << "Unknown argument for -v/--verbose: "
-                                 << *it << endl;;
-                        }
-                    }
-                }
+                if (parse_verbose_arg(a.argv()[argpos+1]) ==
+                        GENERIC_EXIT_INVALID_CMDLINE)
+                    return BACKEND_EXIT_INVALID_CMDLINE;
+
+                ++argpos;
             } 
             else
             {
                 cerr << "Missing argument to -v/--verbose option\n";
-                return -1;
+                return BACKEND_EXIT_INVALID_CMDLINE;
             }
         } 
         else if (!strcmp(a.argv()[argpos],"--printsched"))
@@ -346,14 +309,28 @@ int main(int argc, char **argv)
         {
             nosched = true;
         } 
+        else if (!strcmp(a.argv()[argpos],"--nojobqueue"))
+        {
+            nojobqueue = true;
+        } 
+        else if (!strcmp(a.argv()[argpos],"--noautoexpire"))
+        {
+            noexpirer = true;
+        } 
         else if (!strcmp(a.argv()[argpos],"--printexpire"))
         {
             printexpire = true;
         } 
         else if (!strcmp(a.argv()[argpos],"--version"))
         {
-            cout << MYTH_BINARY_VERSION << endl;
-            exit(0);
+            extern const char *myth_source_version;
+            cout << "Library API version: " << MYTH_BINARY_VERSION << endl;
+            cout << "Source code version: " << myth_source_version << endl;
+#ifdef MYTH_BUILD_CONFIG
+            cout << "Options compiled in:" <<endl;
+            cout << MYTH_BUILD_CONFIG << endl;
+#endif
+            return BACKEND_EXIT_OK;
         }
         else
         {
@@ -361,37 +338,30 @@ int main(int argc, char **argv)
                 !strcmp(a.argv()[argpos],"--help")))
                 cerr << "Invalid argument: " << a.argv()[argpos] << endl;
             cerr << "Valid options are: " << endl <<
+                    "-h or --help                   List valid command line parameters" << endl <<
                     "-l or --logfile filename       Writes STDERR and STDOUT messages to filename" << endl <<
                     "-p or --pidfile filename       Write PID of mythbackend " <<
                                                     "to filename" << endl <<
                     "-d or --daemon                 Runs mythbackend as a daemon" << endl <<
-                    "-v or --verbose debug-level    Prints more information" << endl <<
-                    "                               Accepts any combination (separated by comma)" << endl << 
-                    "                               of all,none,quiet,record,playback,channel," << endl <<
-                    "                               osd,file,schedule,network,commflag,audio," << endl <<
-                    "                               libav,jobqueue" << endl <<
+                    "-v or --verbose debug-level    Use '-v help' for level info" << endl <<
                     "--printexpire                  List of auto-expire programs" << endl <<
                     "--printsched                   Upcoming scheduled programs" << endl <<
                     "--testsched                    Test run scheduler (ignore existing schedule)" << endl <<
                     "--resched                      Force the scheduler to update" << endl <<
                     "--nosched                      Do not perform any scheduling" << endl <<
+                    "--nojobqueue                   Do not start the JobQueue" << endl <<
+                    "--noautoexpire                 Do not start the AutoExpire thread" << endl <<
                     "--version                      Version information" << endl;
-            return -1;
+            return BACKEND_EXIT_INVALID_CMDLINE;
         }
     }
 
-    int logfd = -1;
-
-    if (logfile != "")
+    if (logfile != "" )
     {
-        logfd = open(logfile.ascii(), O_WRONLY|O_CREAT|O_APPEND|O_SYNC, 0664);
-         
-        if (logfd < 0)
-        {
-            perror(logfile.ascii());
-            cerr << "Error opening logfile\n";
-            return -1;
-        }
+        if (log_rotate(1) < 0)
+            cerr << "cannot open logfile; using stdout/stderr" << endl;
+        else
+            signal(SIGHUP, &log_rotate_handler);
     }
     
     ofstream pidfs;
@@ -402,7 +372,7 @@ int main(int argc, char **argv)
         {
             perror(pidfile.ascii());
             cerr << "Error opening pidfile";
-            return -1;
+            return BACKEND_EXIT_OPENING_PIDFILE_ERROR;
         }
     }
 
@@ -413,7 +383,7 @@ int main(int argc, char **argv)
         if (daemon(0, 1) < 0)
         {
             perror("daemon");
-            return -1;
+            return BACKEND_EXIT_DEAMONIZING_ERROR;
         }
 
 
@@ -423,36 +393,28 @@ int main(int argc, char **argv)
         pidfs.close();
     }
 
-    if (logfd != -1)
-    {
-        // Send stdout and stderr to the logfile
-        dup2(logfd, 1);
-        dup2(logfd, 2);
-
-        // Close the unduplicated logfd
-        if (logfd != 1 && logfd != 2)
-            close(logfd);
-    }
-
     gContext = NULL;
     gContext = new MythContext(MYTH_BINARY_VERSION);
-    if(!gContext->Init(false))
+    if (!gContext->Init(false))
     {
         VERBOSE(VB_IMPORTANT, "Failed to init MythContext, exiting.");
-        return -1;
+        return BACKEND_EXIT_NO_MYTHCONTEXT;
     }
     gContext->SetBackend(true);
 
     close(0);
 
+    gContext->ActivateSettingsCache(false);
     if (!UpgradeTVDatabaseSchema())
     {
         VERBOSE(VB_IMPORTANT, "Couldn't upgrade database to new schema");
-        return -1;
+        return BACKEND_EXIT_DB_OUTOFDATE;
     }    
+    gContext->ActivateSettingsCache(true);
 
     if (printsched || testsched)
     {
+        gContext->SetBackend(false);
         sched = new Scheduler(false, &tvList);
         if (!testsched && gContext->ConnectToMasterServer())
         {
@@ -470,23 +432,25 @@ int main(int argc, char **argv)
         print_verbose_messages |= VB_SCHEDULE;
         sched->PrintList(true);
         cleanup();
-        exit(0);
+        return BACKEND_EXIT_OK;
     }
 
     if (resched)
     {
         gContext->SetBackend(false);
 
+        bool ok = false;
         if (gContext->ConnectToMasterServer())
         {
             VERBOSE(VB_IMPORTANT, "Connected to master for reschedule");
             ScheduledRecording::signalChange(-1);
+            ok = true;
         }
         else
             VERBOSE(VB_IMPORTANT, "Cannot connect to master for reschedule");
 
         cleanup();
-        exit(0);
+        return (ok) ? BACKEND_EXIT_OK : BACKEND_EXIT_NO_CONNECT;
     }
 
     if (printexpire)
@@ -495,7 +459,7 @@ int main(int argc, char **argv)
         expirer->FillExpireList();
         expirer->PrintExpireList();
         cleanup();
-        exit(0);
+        return BACKEND_EXIT_OK;
     }
 
     int port = gContext->GetNumSetting("BackendServerPort", 6543);
@@ -511,7 +475,7 @@ int main(int argc, char **argv)
         cerr << "No setting found for this machine's BackendServerIP.\n"
              << "Please run setup on this machine and modify the first page\n"
              << "of the general settings.\n";
-        exit(3);
+        return BACKEND_EXIT_NO_IP_ADDRESS;
     }
 
     if (masterip == myip)
@@ -522,8 +486,8 @@ int main(int argc, char **argv)
         ismaster = true;
 
         if (nosched)
-            cerr << "******** The Scheduler has been DISABLED with "
-                    "the --nosched option ********\n";
+            cerr << "********** The Scheduler has been DISABLED with "
+                    "the --nosched option **********\n";
     }
     else
     {
@@ -531,24 +495,36 @@ int main(int argc, char **argv)
         gContext->LogEntry("mythbackend", LP_INFO,
                            "MythBackend started as a slave backend", "");
     }
- 
-    bool runsched = setupTVs(ismaster);
+
+    // Get any initial housekeeping done before we fire up anything else
+    housekeeping = new HouseKeeper(true, ismaster);
+
+    bool fatal_error = false;
+    bool runsched = setupTVs(ismaster, fatal_error);
+    if (fatal_error)
+        return BACKEND_EXIT_CAP_CARD_SETUP_ERROR;
 
     if (ismaster && runsched && !nosched)
     {
         sched = new Scheduler(true, &tvList);
     }
 
-    expirer = new AutoExpire(true, ismaster);
+    if (noexpirer)
+        cerr << "********* Auto-Expire has been DISABLED with "
+                "the --noautoexpire option ********\n";
+    else
+        expirer = new AutoExpire(true, ismaster);
 
-    housekeeping = new HouseKeeper(true, ismaster);
+    if (nojobqueue)
+        cerr << "********* The JobQueue has been DISABLED with "
+                "the --nojobqueue option *********\n";
+    else
+        jobqueue = new JobQueue(ismaster);
 
-    jobqueue = new JobQueue(ismaster);
-
-    VERBOSE(VB_ALL, QString("%1 version: %2 www.mythtv.org")
+    VERBOSE(VB_IMPORTANT, QString("%1 version: %2 www.mythtv.org")
                             .arg(binname).arg(MYTH_BINARY_VERSION));
 
-    VERBOSE(VB_ALL, QString("Enabled verbose msgs :%1").arg(verboseString));
+    VERBOSE(VB_IMPORTANT, QString("Enabled verbose msgs: %1").arg(verboseString));
 
     lockfile_location = gContext->GetSetting("RecordFilePrefix") + "/nfslockfile.lock";
 
@@ -567,11 +543,11 @@ int main(int argc, char **argv)
                  << "Be sure that \'" << gContext->GetSetting("RecordFilePrefix")
                  << "\' exists and that both \nthe directory and that "
                  << "file are writeable by this user.\n";
-            return -1;
+            return BACKEND_EXIT_OPENING_VLOCKFILE_ERROR;
         }
     }
 
-    new MainServer(ismaster, port, statusport, &tvList, sched);
+    new MainServer(ismaster, port, statusport, &tvList, sched, expirer);
 
     if (ismaster)
     {
@@ -579,7 +555,7 @@ int main(int argc, char **argv)
             = gContext->GetSetting("WOLslaveBackendsCommand","");
         if (!WOLslaveBackends.isEmpty())
         {
-            VERBOSE(VB_ALL, "Waking slave Backends now.");
+            VERBOSE(VB_IMPORTANT, "Waking slave Backends now.");
             system(WOLslaveBackends.ascii());
         }
     }
@@ -589,5 +565,5 @@ int main(int argc, char **argv)
     gContext->LogEntry("mythbackend", LP_INFO, "MythBackend exiting", "");
     cleanup();
 
-    return 0;
+    return BACKEND_EXIT_OK;
 }
