@@ -18,17 +18,16 @@ using namespace std;
 #include "channelrecpriority.h"
 #include "tv.h"
 
+#include "exitcodes.h"
 #include "dialogbox.h"
 #include "mythcontext.h"
+#include "mythdbcon.h"
 #include "scheduledrecording.h"
 #include "infostructs.h"
 
-ChannelRecPriority::ChannelRecPriority(QSqlDatabase *ldb, 
-                                       MythMainWindow *parent, const char *name)
+ChannelRecPriority::ChannelRecPriority(MythMainWindow *parent, const char *name)
                   : MythDialog(parent, name)
 {
-    db = ldb;
-
     curitem = NULL;
     pageDowner = false;
     bgTransBackup = NULL;
@@ -69,8 +68,9 @@ ChannelRecPriority::ChannelRecPriority(QSqlDatabase *ldb,
     }
     else
     {
-        cerr << "MythFrontEnd: ChannelRecPriority - Failed to get selector.\n";
-        exit(19);
+        VERBOSE(VB_IMPORTANT, "MythFrontEnd: ChannelRecPriority - "
+                "Failed to get selector.");
+        exit(FRONTEND_BUGGY_EXIT_NO_SELECTOR);
     }
 
     bgTransBackup = gContext->LoadScalePixmap("trans-backup.png");
@@ -92,11 +92,13 @@ ChannelRecPriority::ChannelRecPriority(QSqlDatabase *ldb,
         gContext->GetSetting("LongChannelFormat", "<num> <name>");
 
     gContext->addListener(this);
+    gContext->addCurrentLocation("ChannelRecPriority");
 }
 
 ChannelRecPriority::~ChannelRecPriority()
 {
     gContext->removeListener(this);
+    gContext->removeCurrentLocation();
     delete theme;
     if (bgTransBackup)
         delete bgTransBackup;
@@ -123,11 +125,11 @@ void ChannelRecPriority::keyPressEvent(QKeyEvent *e)
                 pageUp();
             else if (action == "PAGEDOWN")
                 pageDown();
-            else if (action == "RIGHT")
+            else if (action == "RANKINC")
                 changeRecPriority(1);
-            else if (action == "LEFT")
+            else if (action == "RANKDEC")
                 changeRecPriority(-1);
-            else if (action == "ESCAPE")
+            else if (action == "ESCAPE" || action == "LEFT")
             {
                 saveRecPriority(); 
                 gContext->SaveSetting("ChannelRecPrioritySorting",
@@ -188,8 +190,8 @@ void ChannelRecPriority::LoadWindow(QDomElement &element)
             }
             else
             {
-                cerr << "Unknown element: " << e.tagName() << endl;
-                exit(20);
+                VERBOSE(VB_IMPORTANT, QString("Unknown child element: %1. Ignoring.")
+                        .arg(e.tagName()));
             }
         }
     }
@@ -371,15 +373,16 @@ void ChannelRecPriority::changeRecPriority(int howMuch)
     }
 }
 
-void ChannelRecPriority::applyChannelRecPriorityChange(QSqlDatabase *db, 
-                                                       QString chanid, 
+void ChannelRecPriority::applyChannelRecPriorityChange(QString chanid, 
                                                  const QString &newrecpriority) 
 {
-    QString query = QString("UPDATE channel SET recpriority = '%1' "
-                            "WHERE chanid = '%2';").arg(newrecpriority)
-                                                   .arg(chanid);
-    QSqlQuery result = db->exec(query);
-    if (!result.isActive())
+    MSqlQuery query(MSqlQuery::InitCon());
+    query.prepare("UPDATE channel SET recpriority = :RECPRI "
+                            "WHERE chanid = :CHANID ;");
+    query.bindValue(":RECPRI", newrecpriority);
+    query.bindValue(":CHANID", chanid);
+
+    if (!query.exec() || !query.isActive())
         MythContext::DBError("Save recpriority update", query);
 }
 
@@ -395,11 +398,10 @@ void ChannelRecPriority::saveRecPriority(void)
         // if this channel's recording priority changed from when we entered
         // save new value out to db
         if (chanInfo->recpriority != origRecPriorityData[key])
-            applyChannelRecPriorityChange(db, 
-                                   QString::number(chanInfo->chanid), 
-                                   chanInfo->recpriority);
+            applyChannelRecPriorityChange(QString::number(chanInfo->chanid),
+                                          chanInfo->recpriority);
     }
-    ScheduledRecording::signalChange(db);
+    ScheduledRecording::signalChange(0);
 }
 
 void ChannelRecPriority::FillList(void)
@@ -408,21 +410,21 @@ void ChannelRecPriority::FillList(void)
 
     channelData.clear();
 
-    QString query = QString("SELECT chanid, channum, callsign, icon, "
-                            "recpriority, name FROM channel;");
+    MSqlQuery result(MSqlQuery::InitCon());
+    result.prepare("SELECT chanid, channum, sourceid, callsign, "
+                            "icon, recpriority, name FROM channel;");
 
-    QSqlQuery result = db->exec(query);
-
-    if (result.isActive() && result.numRowsAffected() > 0)
+    if (result.exec() && result.isActive() && result.size() > 0)
     {
         while (result.next()) {
             ChannelInfo *chaninfo = new ChannelInfo;
             chaninfo->chanid = result.value(0).toInt();
             chaninfo->chanstr = result.value(1).toString();
-            chaninfo->callsign = QString::fromUtf8(result.value(2).toString());
-            chaninfo->iconpath = result.value(3).toString();
-            chaninfo->recpriority = result.value(4).toString();
-            chaninfo->channame = QString::fromUtf8(result.value(5).toString());
+            chaninfo->sourceid = result.value(2).toInt();
+            chaninfo->callsign = QString::fromUtf8(result.value(3).toString());
+            chaninfo->iconpath = result.value(4).toString();
+            chaninfo->recpriority = result.value(5).toString();
+            chaninfo->channame = QString::fromUtf8(result.value(6).toString());
             channelData[QString::number(cnt)] = *chaninfo;
 
             // save recording priority value in map so we don't have to save 
@@ -435,7 +437,7 @@ void ChannelRecPriority::FillList(void)
         }
     }
     else if (!result.isActive())
-        MythContext::DBError("Get channel recording priorities query", query);
+        MythContext::DBError("Get channel recording priorities query", result);
 }
 
 typedef struct RecPriorityInfo 
@@ -490,12 +492,15 @@ void ChannelRecPriority::SortList()
 
     switch(sortType) 
     {
-        case byChannel : sort(sortedList.begin(), sortedList.end(), 
-                              channelSort());
-                         break;
-        case byRecPriority : sort(sortedList.begin(), sortedList.end(), 
-                           channelRecPrioritySort());
-                      break;
+        case byRecPriority:
+            sort(sortedList.begin(), sortedList.end(), 
+            channelRecPrioritySort());
+            break;
+        case byChannel:
+        default:
+            sort(sortedList.begin(), sortedList.end(), 
+            channelSort());
+            break;
     }
 
     channelData.clear();
@@ -654,6 +659,10 @@ void ChannelRecPriority::updateInfo(QPainter *p)
             if (type)
                 type->SetText(curitem->Text(longchannelformat));
  
+            type = (UITextType *)container->GetType("source");
+            if (type)
+                type->SetText(QString("%1").arg(curitem->sourceid));
+
             type = (UITextType *)container->GetType("recpriority");
             if (type) {
                 if (curitem->recpriority.toInt() > 0)
