@@ -17,10 +17,10 @@ using namespace std;
 
 #include "programrecpriority.h"
 #include "scheduledrecording.h"
-#include "infodialog.h"
 #include "proglist.h"
 #include "tv.h"
 
+#include "exitcodes.h"
 #include "dialogbox.h"
 #include "mythcontext.h"
 #include "mythdbcon.h"
@@ -69,6 +69,7 @@ ProgramRecPriorityInfo& ProgramRecPriorityInfo::operator=(const ProgramInfo &oth
     dupin = other.dupin;
     dupmethod = other.dupmethod;
     recgroup = other.recgroup;
+    playgroup = other.playgroup;
     chancommfree = other.chancommfree;
 
     sourceid = other.sourceid;
@@ -130,8 +131,9 @@ ProgramRecPriority::ProgramRecPriority(MythMainWindow *parent,
     }
     else
     {
-        cerr << "MythFrontEnd: ProgramRecPriority - Failed to get selector object.\n";
-        exit(27);
+        VERBOSE(VB_IMPORTANT, "MythFrontEnd::ProgramRecPriority(): "
+                "Failed to get selector object.");
+        exit(FRONTEND_BUGGY_EXIT_NO_SELECTOR);
     }
 
     bgTransBackup = gContext->LoadScalePixmap("trans-backup.png");
@@ -150,11 +152,13 @@ ProgramRecPriority::ProgramRecPriority(MythMainWindow *parent,
     setNoErase();
 
     gContext->addListener(this);
+    gContext->addCurrentLocation("ProgramRecPriority");
 }
 
 ProgramRecPriority::~ProgramRecPriority()
 {
     gContext->removeListener(this);
+    gContext->removeCurrentLocation();
     delete theme;
     if (bgTransBackup)
         delete bgTransBackup;
@@ -288,8 +292,9 @@ void ProgramRecPriority::LoadWindow(QDomElement &element)
             }
             else
             {
-                cerr << "Unknown element: " << e.tagName() << endl;
-                exit(28);
+                VERBOSE(VB_IMPORTANT,
+                        QString("ProgramRecPriority: Unknown child element: "
+                                "%1. Ignoring.").arg(e.tagName()));
             }
         }
     }
@@ -490,27 +495,37 @@ void ProgramRecPriority::edit(void)
                      cnt++, ++it);
                 progInfo = &(it.data());
            
-                int rtRecPriors[10];
-                rtRecPriors[0] = gContext->GetNumSetting("SingleRecordRecPrior", 0);
-                rtRecPriors[1] = gContext->GetNumSetting("TimeslotRecordRecPrior", 0);
-                rtRecPriors[2] = gContext->GetNumSetting("ChannelRecordRecPrior", 0);
-                rtRecPriors[3] = gContext->GetNumSetting("AllRecordRecPrior", 0);
-                rtRecPriors[4] = gContext->GetNumSetting("WeekslotRecordRecPrior", 0);
-                rtRecPriors[5] = gContext->GetNumSetting("FindOneRecordRecPrior", 0);
-                rtRecPriors[6] = gContext->GetNumSetting("OverrideRecordRecPrior", 0);
-                rtRecPriors[7] = gContext->GetNumSetting("OverrideRecordRecPrior", 0);
-                rtRecPriors[8] = gContext->GetNumSetting("FindOneRecordRecPrior", 0);
-                rtRecPriors[9] = gContext->GetNumSetting("FindOneRecordRecPrior", 0);
+                int rtRecPriors[11];
+                rtRecPriors[0] = 0;
+                rtRecPriors[kSingleRecord] = 
+                    gContext->GetNumSetting("SingleRecordRecPriority", 1);
+                rtRecPriors[kTimeslotRecord] = 
+                    gContext->GetNumSetting("TimeslotRecordRecPriority", 0);
+                rtRecPriors[kChannelRecord] = 
+                    gContext->GetNumSetting("ChannelRecordRecPriority", 0);
+                rtRecPriors[kAllRecord] = 
+                    gContext->GetNumSetting("AllRecordRecPriority", 0);
+                rtRecPriors[kWeekslotRecord] = 
+                    gContext->GetNumSetting("WeekslotRecordRecPriority", 0);
+                rtRecPriors[kFindOneRecord] = 
+                    gContext->GetNumSetting("FindOneRecordRecPriority", -1);
+                rtRecPriors[kOverrideRecord] = 
+                    gContext->GetNumSetting("OverrideRecordRecPriority", 0);
+                rtRecPriors[kDontRecord] = 
+                    gContext->GetNumSetting("OverrideRecordRecPriority", 0);
+                rtRecPriors[kFindDailyRecord] = 
+                    gContext->GetNumSetting("FindOneRecordRecPriority", -1);
+                rtRecPriors[kFindWeeklyRecord] = 
+                    gContext->GetNumSetting("FindOneRecordRecPriority", -1);
 
                 // set the recording priorities of that program 
                 progInfo->recpriority = recPriority;
                 progInfo->recType = (RecordingType)rectype;
-                progInfo->recTypeRecPriority = rtRecPriors[progInfo->recType-1];
+                progInfo->recTypeRecPriority = rtRecPriors[progInfo->recType];
                 // also set the origRecPriorityData with new recording 
                 // priority so we don't save to db again when we exit
-                QString key = progInfo->MakeUniqueKey(); 
-                origRecPriorityData[key] = progInfo->recpriority;
-
+                origRecPriorityData[progInfo->recordid] = 
+                                                        progInfo->recpriority;
                 // also set the active/inactive state
                 progInfo->recstatus = inactive ? rsInactive : rsWillRecord;
 
@@ -646,7 +661,7 @@ void ProgramRecPriority::saveRecPriority(void)
     for (it = programData.begin(); it != programData.end(); ++it) 
     {
         ProgramRecPriorityInfo *progInfo = &(it.data());
-        QString key = progInfo->MakeUniqueKey(); 
+        int key = progInfo->recordid; 
 
         // if this program's recording priority changed from when we entered
         // save new value out to db
@@ -657,7 +672,7 @@ void ProgramRecPriority::saveRecPriority(void)
 
 void ProgramRecPriority::FillList(void)
 {
-    int cnt = 999, rtRecPriors[10];
+    int cnt = 999, rtRecPriors[11];
     vector<ProgramInfo *> recordinglist;
 
     programData.clear();
@@ -672,8 +687,7 @@ void ProgramRecPriority::FillList(void)
 
         // save recording priority value in map so we don't have to 
         // save all program's recording priority values when we exit
-        QString key = (*pgiter)->MakeUniqueKey();
-        origRecPriorityData[key] = (*pgiter)->recpriority;
+        origRecPriorityData[(*pgiter)->recordid] = (*pgiter)->recpriority;
 
         delete (*pgiter);
         cnt--;
@@ -684,16 +698,27 @@ void ProgramRecPriority::FillList(void)
 //    cerr << " programs" << endl;
 
     // get all the recording type recording priority values
-    rtRecPriors[0] = gContext->GetNumSetting("SingleRecordRecPriority", 0);
-    rtRecPriors[1] = gContext->GetNumSetting("TimeslotRecordRecPriority", 0);
-    rtRecPriors[2] = gContext->GetNumSetting("ChannelRecordRecPriority", 0);
-    rtRecPriors[3] = gContext->GetNumSetting("AllRecordRecPriority", 0);
-    rtRecPriors[4] = gContext->GetNumSetting("WeekslotRecordRecPriority", 0);
-    rtRecPriors[5] = gContext->GetNumSetting("FindOneRecordRecPriority", 0);
-    rtRecPriors[6] = gContext->GetNumSetting("OverrideRecordRecPriority", 0);
-    rtRecPriors[7] = gContext->GetNumSetting("OverrideRecordRecPriority", 0);
-    rtRecPriors[8] = gContext->GetNumSetting("FindOneRecordRecPriority", 0);
-    rtRecPriors[9] = gContext->GetNumSetting("FindOneRecordRecPriority", 0);
+    rtRecPriors[0] = 0;
+    rtRecPriors[kSingleRecord] = 
+        gContext->GetNumSetting("SingleRecordRecPriority", 1);
+    rtRecPriors[kTimeslotRecord] = 
+        gContext->GetNumSetting("TimeslotRecordRecPriority", 0);
+    rtRecPriors[kChannelRecord] = 
+        gContext->GetNumSetting("ChannelRecordRecPriority", 0);
+    rtRecPriors[kAllRecord] = 
+        gContext->GetNumSetting("AllRecordRecPriority", 0);
+    rtRecPriors[kWeekslotRecord] = 
+        gContext->GetNumSetting("WeekslotRecordRecPriority", 0);
+    rtRecPriors[kFindOneRecord] = 
+        gContext->GetNumSetting("FindOneRecordRecPriority", -1);
+    rtRecPriors[kOverrideRecord] = 
+        gContext->GetNumSetting("OverrideRecordRecPriority", 0);
+    rtRecPriors[kDontRecord] = 
+        gContext->GetNumSetting("OverrideRecordRecPriority", 0);
+    rtRecPriors[kFindDailyRecord] = 
+        gContext->GetNumSetting("FindOneRecordRecPriority", -1);
+    rtRecPriors[kFindWeeklyRecord] = 
+        gContext->GetNumSetting("FindOneRecordRecPriority", -1);
     
     // get channel recording priorities and recording types associated with each
     // program from db
@@ -722,7 +747,7 @@ void ProgramRecPriority::FillList(void)
             QString tempDate = result.value(4).toString();
             RecordingType recType = (RecordingType)result.value(5).toInt();
             int channelRecPriority = result.value(6).toInt();
-            int recTypeRecPriority = rtRecPriors[recType-1];
+            int recTypeRecPriority = rtRecPriors[recType];
             int inactive = result.value(7).toInt();
 
             if (recType == kAllRecord || recType == kFindOneRecord ||
@@ -754,6 +779,8 @@ void ProgramRecPriority::FillList(void)
     else
         MythContext::DBError("Get program recording priorities query", result);
 
+    conMatch.clear();
+    nowMatch.clear();
     recMatch.clear();
     ProgramList schedList;
     schedList.FromScheduler();
@@ -765,8 +792,15 @@ void ProgramRecPriority::FillList(void)
         if (s->recendts > now)
         {
             listMatch[s->recordid]++;
-            if (s->recstatus == rsWillRecord || s->recstatus == rsRecording)
+            if (s->recstatus == rsConflict)
+                conMatch[s->recordid]++;
+            else if (s->recstatus == rsWillRecord)
                 recMatch[s->recordid]++;
+            else if (s->recstatus == rsRecording)
+            {
+                nowMatch[s->recordid]++;
+                recMatch[s->recordid]++;
+            }
         }
     }
 }
@@ -1018,13 +1052,15 @@ void ProgramRecPriority::updateList(QPainter *p)
                         ltype->SetItemText(cnt, 6, 
                                 QString::number(abs(finalRecPriority)));
 
-                        if (progInfo->recType == kDontRecord ||
-                            progInfo->recstatus == rsInactive)
-                            ltype->EnableForcedFont(cnt, "inactive");
+                        if (conMatch[progInfo->recordid] > 0)
+                            ltype->EnableForcedFont(cnt, "conflicting");
+                        else if (nowMatch[progInfo->recordid] > 0)
+                            ltype->EnableForcedFont(cnt, "recording");
                         else if (recMatch[progInfo->recordid] > 0)
                             ltype->EnableForcedFont(cnt, "record");
-                        // else if (listMatch[progInfo->recordid] < 1)
-                        //    ltype->EnableForcedFont(cnt, "dormant");
+                        else if (progInfo->recType == kDontRecord ||
+                            progInfo->recstatus == rsInactive)
+                            ltype->EnableForcedFont(cnt, "inactive");
 
                         cnt++;
                         listCount++;
