@@ -10,7 +10,6 @@
 #include "programinfo.h"
 #include "scheduledrecording.h"
 
-class QSqlDatabase;
 
 class ProgramRecPriorityInfo : public ProgramInfo
 {
@@ -36,8 +35,7 @@ class ProgramRecPriority : public MythDialog
         byRecType
     };
 
-    ProgramRecPriority(QSqlDatabase *ldb, MythMainWindow *parent, 
-                 const char *name = 0);
+    ProgramRecPriority(MythMainWindow *parent, const char *name = 0);
     ~ProgramRecPriority();
 
   protected slots:
@@ -59,7 +57,9 @@ class ProgramRecPriority : public MythDialog
     void FillList(void);
     void SortList();
     QMap<QString, ProgramRecPriorityInfo> programData;
-    QMap<QString, int> origRecPriorityData;
+    QMap<int, int> origRecPriorityData;
+    QMap<int, int> conMatch;
+    QMap<int, int> nowMatch;
     QMap<int, int> recMatch;
     QMap<int, int> listMatch;
 
@@ -77,8 +77,6 @@ class ProgramRecPriority : public MythDialog
 
     ProgramRecPriorityInfo *curitem;
 
-    QSqlDatabase *db;
-
     QPixmap myBackground;
     QPixmap *bgTransBackup;
 
