@@ -9,19 +9,6 @@
 
 #include <unistd.h>
 
-#ifdef __linux__
-#include <sys/statvfs.h>
-#include <sys/sysinfo.h>
-#endif
-#if defined(__FreeBSD__) || defined(CONFIG_DARWIN)
-#include <sys/sysctl.h>
-#include <sys/param.h>
-#include <sys/mount.h>
-#endif
-#ifdef CONFIG_DARWIN
-#include <mach/mach.h>
-#endif
-
 #include <iostream>
 #include <cerrno>
 using namespace std;
@@ -36,8 +23,17 @@ using namespace std;
 #include "util.h"
 #include "mythdbcon.h"
 
+/** \class StatusBox
+ *  \brief Reports on various status items.
+ *
+ *  StatusBox reports on the listing status, that is how far
+ *  into the future program listings exits. It also reports
+ *  on the status of each tuner, the log entries, the status
+ *  of the job queue, and the machine status.
+ */
+
 StatusBox::StatusBox(MythMainWindow *parent, const char *name)
-         : MythDialog(parent, name)
+    : MythDialog(parent, name), errored(false)
 {
     // Set this value to the number of items in icon_list
     // to prevent scrolling off the bottom
@@ -48,12 +44,15 @@ StatusBox::StatusBox(MythMainWindow *parent, const char *name)
 
     setNoErase();
     LoadTheme();
+    if (IsErrored())
+        return;
   
     icon_list->SetItemText(item_count++, QObject::tr("Listings Status"));
     icon_list->SetItemText(item_count++, QObject::tr("Tuner Status"));
     icon_list->SetItemText(item_count++, QObject::tr("Log Entries"));
     icon_list->SetItemText(item_count++, QObject::tr("Job Queue"));
     icon_list->SetItemText(item_count++, QObject::tr("Machine Status"));
+    icon_list->SetItemText(item_count++, QObject::tr("AutoExpire List"));
     icon_list->SetItemCurrent(0);
     icon_list->SetActive(true);
 
@@ -81,10 +80,13 @@ StatusBox::StatusBox(MythMainWindow *parent, const char *name)
     min_level = gContext->GetNumSetting("LogDefaultView",1);
     my_parent = parent;
     clicked();
+
+    gContext->addCurrentLocation("StatusBox");
 }
 
 StatusBox::~StatusBox(void)
 {
+    gContext->removeCurrentLocation();
 }
 
 void StatusBox::paintEvent(QPaintEvent *e)
@@ -113,54 +115,45 @@ void StatusBox::updateContent()
         contentSize = contentTotalLines;
     contentMid = contentSize / 2;
 
-    // In order to maintain a center scrolling highlight, we need to determine
-    // the point at which to start the content list so that the current item
-    // is in the center
-    int startPos = 0; 
-    if (contentPos >= contentMid)
+    int startPos = 0;
+    int highlightPos = 0;
+
+    if (contentPos < contentMid)
+    {
+        startPos = 0;
+        highlightPos = contentPos;
+    }
+    else if (contentPos >= (contentTotalLines - contentMid))
+    {
+        startPos = contentTotalLines - contentSize;
+        highlightPos = contentSize - (contentTotalLines - contentPos);
+    }
+    else if (contentPos >= contentMid)
+    {
         startPos = contentPos - contentMid;
+        highlightPos = contentMid;
+    }
  
     if (content  == NULL) return;
     LayerSet *container = content;
 
-    // If the content position is beyond the midpoint and before the end
-    // or if the content is first being displayed, write the content using
-    // the offset we determined above.  If we are before the midpoint or 
-    // close to the end, we stop moving the content up or down to let the 
-    // hightlight move up and down with fixed content
-    if (((contentPos >= contentMid) &&
-         (contentPos < (contentTotalLines - contentMid))) ||
-        (contentPos == 0))
+    list_area->ResetList();
+    for (int x = startPos; (x - startPos) <= contentSize; x++)
     {
-        list_area->ResetList();
-        for (int x = startPos; (x - startPos) <= contentSize; x++)
-            if (contentLines.contains(x))
-            {
-                list_area->SetItemText(x - startPos, contentLines[x]);
-                if (contentFont.contains(x))
-                    list_area->EnableForcedFont(x - startPos, contentFont[x]);
-            }
+        if (contentLines.contains(x))
+        {
+            list_area->SetItemText(x - startPos, contentLines[x]);
+            if (contentFont.contains(x))
+                list_area->EnableForcedFont(x - startPos, contentFont[x]);
+        }
     }
 
-    // If we are scrolling, the determine the item to highlight
-    if (doScroll)
+    list_area->SetItemCurrent(highlightPos);
+
+    if (inContent)
     {
-        int newPos = 0;
-
-        if (contentPos < contentMid)
-            newPos = contentPos;
-        else if (contentPos >= (contentTotalLines - contentMid))
-            newPos = contentSize - (contentTotalLines - contentPos);
-        else
-            newPos = contentMid;
-
-        list_area->SetItemCurrent(newPos);
-
-        if (inContent)
-        {
-            helptext->SetText(contentDetail[contentPos]);
-            update(TopRect);
-        }
+        helptext->SetText(contentDetail[contentPos]);
+        update(TopRect);
     }
 
     list_area->SetUpArrow((startPos > 0) && (contentSize < contentTotalLines));
@@ -229,7 +222,12 @@ void StatusBox::LoadTheme()
     theme = new XMLParse();
     theme->SetWMult(wmult);
     theme->SetHMult(hmult);
-    theme->LoadTheme(xmldata, "status", "status-");
+    if (!theme->LoadTheme(xmldata, "status", "status-"))
+    {
+        VERBOSE(VB_IMPORTANT, "StatusBox: Unable to load theme.");
+        errored = true;
+        return;
+    }
 
     for (QDomNode child = xmldata.firstChild(); !child.isNull();
          child = child.nextSibling()) {
@@ -253,53 +251,63 @@ void StatusBox::LoadTheme()
                     ContentRect = area;
             }
             else {
-                cerr << "Unknown element: " << e.tagName()
-                          << endl;
-                exit(-1);
+                QString msg =
+                    QString(tr("The theme you are using contains an "
+                               "unknown element ('%1').  It will be ignored"))
+                    .arg(e.tagName());
+                VERBOSE(VB_IMPORTANT, msg);
+                errored = true;
             }
         }
     }
 
     selector = theme->GetSet("selector");
-    if (!selector) {
-        cerr << "StatusBox: Failed to get selector container." << endl;
-        exit(-1);
+    if (!selector)
+    {
+        VERBOSE(VB_IMPORTANT, "StatusBox: Failed to get selector container.");
+        errored = true;
     }
 
     icon_list = (UIListType*)selector->GetType("icon_list");
-    if (!icon_list) {
-        cerr << "StatusBox: Failed to get icon list area." << endl;
-        exit(-1);
+    if (!icon_list)
+    {
+        VERBOSE(VB_IMPORTANT, "StatusBox: Failed to get icon list area.");
+        errored = true;
     }
 
     content = theme->GetSet("content");
-    if (!content) {
-        cerr << "StatusBox: Failed to get content container." << endl;
-        exit(-1);
+    if (!content)
+    {
+        VERBOSE(VB_IMPORTANT, "StatusBox: Failed to get content container.");
+        errored = true;
     }
 
     list_area = (UIListType*)content->GetType("list_area");
-    if (!list_area) {
-        cerr << "StatusBox: Failed to get list area." << endl;
-        exit(-1);
+    if (!list_area)
+    {
+        VERBOSE(VB_IMPORTANT, "StatusBox: Failed to get list area.");
+        errored = true;
     }
 
     topbar = theme->GetSet("topbar");
-    if (!topbar) {
-        cerr << "StatusBox: Failed to get topbar container." << endl;
-        exit(-1);
+    if (!topbar)
+    {
+        VERBOSE(VB_IMPORTANT, "StatusBox: Failed to get topbar container.");
+        errored = true;
     }
 
     heading = (UITextType*)topbar->GetType("heading");
-    if (!heading) {
-        cerr << "StatusBox: Failed to get heading area." << endl;
-        exit(-1);
+    if (!heading)
+    {
+        VERBOSE(VB_IMPORTANT, "StatusBox: Failed to get heading area.");
+        errored = true;
     }
 
     helptext = (UITextType*)topbar->GetType("helptext");
-    if (!helptext) {
-        cerr << "StatusBox: Failed to get helptext area." << endl;
-        exit(-1);
+    if (!helptext)
+    {
+        VERBOSE(VB_IMPORTANT, "StatusBox: Failed to get helptext area.");
+        errored = true;
     }
 }
 
@@ -342,6 +350,11 @@ void StatusBox::keyPressEvent(QKeyEvent *e)
                     doLogEntries();
                 }
             }
+            else if ((inContent) &&
+                     (currentItem == QObject::tr("Job Queue")))
+            {
+                clicked();
+            }
         }
         else if (action == "UP")
         {
@@ -378,6 +391,20 @@ void StatusBox::keyPressEvent(QKeyEvent *e)
                 update(SelectRect);
             }
         }
+        else if (action == "PAGEUP" && inContent) 
+        {
+            contentPos -= contentSize;
+            if (contentPos < 0)
+                contentPos = 0;
+            update(ContentRect);
+        }
+        else if (action == "PAGEDOWN" && inContent)
+        {
+            contentPos += contentSize;
+            if (contentPos > (contentTotalLines - 1))
+                contentPos = contentTotalLines - 1;
+            update(ContentRect);
+        }
         else if ((action == "RIGHT") &&
                  (!inContent) &&
                  ((contentTotalLines > contentSize) ||
@@ -470,6 +497,12 @@ void StatusBox::setHelpText()
 
             helptext->SetText(machineStr);
         }
+
+        if (currentItem == QObject::tr("AutoExpire List"))
+            helptext->SetText(QObject::tr("The AutoExpire List shows all "
+                "recordings which may be expired and the order of their "
+                "expiration. Recordings at the top of the list will be "
+                "expired first."));
     }
     update(TopRect);
 }
@@ -600,12 +633,14 @@ void StatusBox::clicked()
         doJobQueueStatus();
     else if (currentItem == QObject::tr("Machine Status"))
         doMachineStatus();
+    else if (currentItem == QObject::tr("AutoExpire List"))
+        doAutoExpireList();
 }
 
 void StatusBox::doListingsStatus()
 {
-    QString mfdLastRunStart, mfdLastRunEnd, mfdLastRunStatus, Status;
-    QString querytext, DataDirectMessage;
+    QString mfdLastRunStart, mfdLastRunEnd, mfdLastRunStatus, mfdNextRunStart;
+    QString querytext, Status, DataDirectMessage;
     int DaysOfData;
     QDateTime qdtNow, GuideDataThrough;
     int count = 0;
@@ -618,10 +653,10 @@ void StatusBox::doListingsStatus()
     qdtNow = QDateTime::currentDateTime();
 
     MSqlQuery query(MSqlQuery::InitCon());
-    query.prepare("SELECT max(endtime) FROM program;");
+    query.prepare("SELECT max(endtime) FROM program WHERE manualid=0;");
     query.exec();
 
-    if (query.isActive() && query.numRowsAffected())
+    if (query.isActive() && query.size())
     {
         query.next();
         GuideDataThrough = QDateTime::fromString(query.value(0).toString(),
@@ -631,8 +666,11 @@ void StatusBox::doListingsStatus()
     mfdLastRunStart = gContext->GetSetting("mythfilldatabaseLastRunStart");
     mfdLastRunEnd = gContext->GetSetting("mythfilldatabaseLastRunEnd");
     mfdLastRunStatus = gContext->GetSetting("mythfilldatabaseLastRunStatus");
+    mfdNextRunStart = gContext->GetSetting("MythFillSuggestedRunTime");
     DataDirectMessage = gContext->GetSetting("DataDirectMessage");
 
+    mfdNextRunStart.replace("T", " ");
+
     contentLines[count++] = QObject::tr("Myth version:") + " " +
                                         MYTH_BINARY_VERSION;
     contentLines[count++] = QObject::tr("Last mythfilldatabase guide update:");
@@ -643,6 +681,11 @@ void StatusBox::doListingsStatus()
 
     contentLines[count++] = QObject::tr("Result: ") + mfdLastRunStatus;
 
+
+    if (mfdNextRunStart >= mfdLastRunStart) 
+        contentLines[count++] = QObject::tr("Suggested Next: ") + 
+                                mfdNextRunStart;
+
     DaysOfData = qdtNow.daysTo(GuideDataThrough);
 
     if (GuideDataThrough.isNull())
@@ -653,7 +696,6 @@ void StatusBox::doListingsStatus()
     }
     else
     {
-        contentLines[count++] = "";
         contentLines[count++] = QObject::tr("There is guide data until ") + 
                                 QDateTime(GuideDataThrough)
                                           .toString("yyyy-MM-dd hh:mm");
@@ -699,7 +741,7 @@ void StatusBox::doTunerStatus()
     contentDetail.clear();
     contentFont.clear();
 
-    if (query.isActive() && query.numRowsAffected())
+    if (query.isActive() && query.size())
     {
         while(query.next())
         {
@@ -884,55 +926,83 @@ void StatusBox::doJobQueueStatus()
 
 // Some helper routines for doMachineStatus() that format the output strings
 
-#define MB (1024*1024)
-
-static const QString usageStr(float total, float used, float free)
+/** \fn sm_str(long long, int)
+ *  \brief Returns a short string describing an amount of space, choosing
+ *         one of a number of useful units, "TB", "GB", "MB", or "KB".
+ *  \param sizeKB Number of kilobytes.
+ *  \param prec   Precision to use if we have less than ten of whatever
+ *                unit is chosen.
+ */
+static const QString sm_str(long long sizeKB, int prec=1)
 {
-    char  buff1[10];       // QString::arg(double) doesn't allow %0.2f,
-    char  buff2[10];       // so we need to use sprintf() with a buffer
-    char *units  = "MB";
-    char *format = "%0.0f";
-
-    if (total == 0.0)
-        return "unknown";
-
-    if (total > 1024)
-        units = "GB", format = "%0.2f",
-        total /= 1024, used /= 1024, free /= 1024;
-
-    sprintf(buff1, format, total);
-
-    QString s = QString("%1 %2").arg(buff1).arg(units);
-    //QString s = QString("%1 %2 %3")
-                //.arg(buff1).arg(units).arg(QObject::tr("total"));
-
-
-    // If, some some reason, we didn't get the usage stats, shorten output:
-
-    if (used == 0 && free == 0)
-        return s;
-
-
-    sprintf(buff1, format, used);
-    sprintf(buff2, format, free);
-
-    return s + ", " +
-           QString("%1 %2 %3")
-           .arg(buff1).arg(units).arg(QObject::tr("used"))
-           + ", " +
-           QString("%1 %2 %3 (%4%)")
-           .arg(buff2).arg(units).arg(QObject::tr("free"))
-           .arg((int)(100*free/total));
+    if (sizeKB>1024*1024*1024) // Terabytes
+    {
+        double sizeGB = sizeKB/(1024*1024*1024.0);
+        return QString("%1 TB").arg(sizeGB, 0, 'f', (sizeGB>10)?0:prec);
+    }
+    else if (sizeKB>1024*1024) // Gigabytes
+    {
+        double sizeGB = sizeKB/(1024*1024.0);
+        return QString("%1 GB").arg(sizeGB, 0, 'f', (sizeGB>10)?0:prec);
+    }
+    else if (sizeKB>1024) // Megabytes
+    {
+        double sizeMB = sizeKB/1024.0;
+        return QString("%1 MB").arg(sizeMB, 0, 'f', (sizeMB>10)?0:prec);
+    }
+    // Kilobytes
+    return QString("%1 KB").arg(sizeKB);
 }
 
-static const QString diskUsageStr(const char *path)
+static const QString usage_str_kb(long long total,
+                                  long long used,
+                                  long long free)
 {
-    double total, free, used;
+    QString ret = QObject::tr("Unknown");
+    if (total > 0.0 && free > 0.0)
+    {
+        double percent = (100.0*free)/total;
+        ret = QObject::tr("%1 total, %2 used, %3 (or %4%) free.")
+            .arg(sm_str(total)).arg(sm_str(used))
+            .arg(sm_str(free)).arg(percent, 0, 'f', (percent >= 10.0) ? 0 : 2);
+    }
+    return ret;
+}
 
-    if (diskUsage(path, total, used, free))
-        return usageStr (total, used, free);
-    else
-        return QString("%1 - %2").arg(path).arg(strerror(errno));
+static const QString usage_str_mb(float total, float used, float free)
+{
+    return usage_str_kb((long long)(total*1024), (long long)(used*1024),
+                        (long long)(free*1024));
+}
+
+static void disk_usage_with_rec_time_kb(QStringList& out, long long total,
+                                        long long used, long long free,
+                                        const recprof2bps_t& prof2bps)
+{
+    const QString tail = QObject::tr(", using your %1 rate of %2 Kb/sec");
+
+    out<<usage_str_kb(total, used, free);
+    if (free<0)
+        return;
+
+    recprof2bps_t::const_iterator it = prof2bps.begin();
+    for (; it != prof2bps.end(); ++it)
+    {
+        const QString pro =
+                tail.arg(it.key()).arg((int)((float)it.data() / 1024.0));
+        
+        long long bytesPerMin = (it.data() >> 1) * 15;
+        uint minLeft = ((free<<5)/bytesPerMin)<<5;
+        minLeft = (minLeft/15)*15;
+        uint hoursLeft = minLeft/60;
+        if (hoursLeft > 3)
+            out<<QObject::tr("%1 hours left").arg(hoursLeft) + pro;
+        else if (minLeft > 90)
+            out<<QObject::tr("%1 hours and %2 minutes left")
+                .arg(hoursLeft).arg(minLeft%60) + pro;
+        else
+            out<<QObject::tr("%1 minutes left").arg(minLeft) + pro;
+    }
 }
 
 static const QString uptimeStr(time_t uptime)
@@ -976,22 +1046,81 @@ static const QString uptimeStr(time_t uptime)
     }
 }
 
+/** \fn StatusBox::getActualRecordedBPS(QString hostnames)
+ *  \brief Fills in recordingProfilesBPS w/ average bitrate from recorded table
+ */
+void StatusBox::getActualRecordedBPS(QString hostnames)
+{
+    recordingProfilesBPS.clear();
+
+    QString querystr;
+    MSqlQuery query(MSqlQuery::InitCon());
+
+    querystr =
+        "SELECT sum(filesize) * 8 / "
+            "sum(((unix_timestamp(endtime) - unix_timestamp(starttime)))) "
+            "AS avg_bitrate "
+        "FROM recorded WHERE hostname in (%1) "
+            "AND (unix_timestamp(endtime) - unix_timestamp(starttime)) > 300;";
+
+    query.prepare(querystr.arg(hostnames));
+
+    if (query.exec() && query.isActive() && query.size() > 0 && query.next() &&
+        query.value(0).toDouble() > 0)
+    {
+        recordingProfilesBPS[QObject::tr("average")] =
+            (int)(query.value(0).toDouble());
+    }
+
+    querystr =
+        "SELECT max(filesize * 8 / "
+            "(unix_timestamp(endtime) - unix_timestamp(starttime))) "
+            "AS max_bitrate "
+        "FROM recorded WHERE hostname in (%1) "
+            "AND (unix_timestamp(endtime) - unix_timestamp(starttime)) > 300;";
+
+    query.prepare(querystr.arg(hostnames));
+
+    if (query.exec() && query.isActive() && query.size() > 0 && query.next() &&
+        query.value(0).toDouble() > 0)
+    {
+        recordingProfilesBPS[QObject::tr("maximum")] =
+            (int)(query.value(0).toDouble());
+    }
+}
+
+/** \fn StatusBox::doMachineStatus()
+ *  \brief Show machine status.
+ *  
+ *   This returns statisics for master backend when using
+ *   a frontend only machine. And returns info on the current
+ *   system if frontend is running on a backend machine.
+ *  \bug We should report on all backends and the current frontend.
+ */
 void StatusBox::doMachineStatus()
 {
     int           count(0);
     int           totalM, usedM, freeM;    // Physical memory
     int           totalS, usedS, freeS;    // Virtual  memory (swap)
     time_t        uptime;
+    int           detailBegin;
+    QString       detailString;
+    int           detailLoop;
 
     contentLines.clear();
     contentDetail.clear();
     contentFont.clear();
-    //doScroll = false;??
+    doScroll = true;
+
+    detailBegin = count;
+    detailString = "";
 
     if (isBackend)
-        contentLines[count++] = QObject::tr("System") + ":";
+        contentLines[count] = QObject::tr("System") + ":";
     else
-        contentLines[count++] = QObject::tr("This machine") + ":";
+        contentLines[count] = QObject::tr("This machine") + ":";
+    detailString += contentLines[count] + "\n";
+    count++;
 
     // uptime
     if (!getUptime(uptime))
@@ -1012,6 +1141,7 @@ void StatusBox::doMachineStatus()
         sprintf(buff, "%0.2lf, %0.2lf, %0.2lf", loads[0], loads[1], loads[2]);
         contentLines[count].append(QString(buff));
     }
+    detailString += contentLines[count] + "\n";
     count++;
 
 
@@ -1020,18 +1150,33 @@ void StatusBox::doMachineStatus()
     {
         usedM = totalM - freeM;
         if (totalM > 0)
-            contentLines[count++] = "   " + QObject::tr("RAM") +
-                                    ": "  + usageStr(totalM, usedM, freeM);
+        {
+            contentLines[count] = "   " + QObject::tr("RAM") +
+                                  ": "  + usage_str_mb(totalM, usedM, freeM);
+            detailString += contentLines[count] + "\n";
+            count++;
+        }
         usedS = totalS - freeS;
         if (totalS > 0)
-            contentLines[count++] = "   " +
-                                    QObject::tr("Swap") +
-                                    ": "  + usageStr(totalS, usedS, freeS);
+        {
+            contentLines[count] = "   " + QObject::tr("Swap") +
+                                  ": "  + usage_str_mb(totalS, usedS, freeS);
+            detailString += contentLines[count] + "\n";
+            count++;
+        }
     }
 
+    for (detailLoop = detailBegin; detailLoop < count; detailLoop++)
+        contentDetail[detailLoop] = detailString;
+
+    detailBegin = count;
+    detailString = "";
+
     if (!isBackend)
     {
-        contentLines[count++] = QObject::tr("MythTV server") + ":";
+        contentLines[count] = QObject::tr("MythTV server") + ":";
+        detailString += contentLines[count] + "\n";
+        count++;
 
         // uptime
         if (!RemoteGetUptime(uptime))
@@ -1049,77 +1194,152 @@ void StatusBox::doMachineStatus()
             contentLines[count].append(QString(buff));
         }
         else
-            contentLines[count++].append(QObject::tr("unknown"));
+            contentLines[count].append(QObject::tr("unknown"));
+
+        detailString += contentLines[count] + "\n";
+        count++;
 
         // memory usage
         if (RemoteGetMemStats(totalM, freeM, totalS, freeS))
         {
             usedM = totalM - freeM;
             if (totalM > 0)
-                contentLines[count++] = "   " + QObject::tr("RAM") +
-                                        ": "  + usageStr(totalM, usedM, freeM);
+            {
+                contentLines[count] = "   " + QObject::tr("RAM") +
+                                      ": "  + usage_str_mb(totalM, usedM, freeM);
+                detailString += contentLines[count] + "\n";
+                count++;
+            }
+
             usedS = totalS - freeS;
             if (totalS > 0)
-                contentLines[count++] = "   " + QObject::tr("Swap") +
-                                        ": "  + usageStr(totalS, usedS, freeS);
+            {
+                contentLines[count] = "   " + QObject::tr("Swap") +
+                                      ": "  + usage_str_mb(totalS, usedS, freeS);
+                detailString += contentLines[count] + "\n";
+                count++;
+            }
         }
     }
 
+    for (detailLoop = detailBegin; detailLoop < count; detailLoop++)
+        contentDetail[detailLoop] = detailString;
+
+    detailBegin = count;
+    detailString = "";
+
     // get free disk space
-    contentLines[count++] = QString(QObject::tr("Disk space:"));
-
-    if (isBackend)
+    QString hostnames;
+    
+    vector<FileSystemInfo> fsInfos = RemoteGetFreeSpace();
+    for (uint i=0; i<fsInfos.size(); i++)
     {
-        QString str;
+        hostnames = QString("\"%1\"").arg(fsInfos[i].hostname);
+        hostnames.replace(QRegExp(" "), "");
+        hostnames.replace(QRegExp(","), "\",\"");
 
-        contentLines[count] = "   " + QString(QObject::tr("Recordings")) + ": ";
+        getActualRecordedBPS(hostnames);
 
-        // Perform the database queries
-        MSqlQuery query(MSqlQuery::InitCon());
-        query.prepare("SELECT * FROM settings "
-                      "WHERE value=\"RecordFilePrefix\" "
-                      "AND hostname = :HOSTNAME ;");
-        query.bindValue(":HOSTNAME", gContext->GetHostName());
-        query.exec();
+        QStringList list;
+        disk_usage_with_rec_time_kb(list,
+            fsInfos[i].totalSpaceKB, fsInfos[i].usedSpaceKB,
+            fsInfos[i].totalSpaceKB - fsInfos[i].usedSpaceKB,
+            recordingProfilesBPS);
 
-        // did we get results?
-        if (query.isActive() && query.numRowsAffected())
+        contentLines[count] = 
+            QObject::tr("Disk usage on %1:").arg(fsInfos[i].hostname);
+        detailString += contentLines[count] + "\n";
+        count++;
+
+        QStringList::iterator it = list.begin();
+        for (;it != list.end(); ++it)
         {
-            // skip to first result
-            query.next();
-            str = diskUsageStr(query.value(1).toCString());
+            contentLines[count] =  QString("   ") + (*it);
+            detailString += contentLines[count] + "\n";
+            count++;
         }
-        else
-            str = QString(QObject::tr("DB error - RecordFilePrefix unknown"));
-        contentLines[count++].append(str);
 
-        // Ditto for the second path
-        contentLines[count] = "   " + QString(QObject::tr("TV buffer")) + ": ";
+        for (detailLoop = detailBegin; detailLoop < count; detailLoop++)
+            contentDetail[detailLoop] = detailString;
 
-        query.prepare("SELECT * FROM settings "
-                      "WHERE value=\"LiveBufferDir\" "
-                      "AND hostname = :HOSTNAME ;");
-        query.bindValue(":HOSTNAME", gContext->GetHostName());
-        query.exec();
-        if (query.isActive() && query.numRowsAffected())
-        {
-            query.next();
-            str = diskUsageStr(query.value(1).toCString());
-        }
-        else
-            str = QString(QObject::tr("DB error - LiveBufferDir unknown"));
-        contentLines[count++].append(str);
-    }
-    else
-    {
-        int  total, used;
-        RemoteGetFreeSpace(total, used);
-        if (total)
-            contentLines[count++] = "   " + usageStr(total, used, total - used);
-        else
-            contentLines[count++] = "   " + QString(QObject::tr("Unknown"));
+        detailBegin = count;
+        detailString = "";
     }
 
     contentTotalLines = count;
     update(ContentRect);
 }
+
+/** \fn StatusBox::doAutoExpireList()
+ *  \brief Show list of recordings which may AutoExpire
+ */
+void StatusBox::doAutoExpireList()
+{
+    int                   count(0);
+    vector<ProgramInfo *> expList;
+    ProgramInfo*          pginfo;
+    QString               contentLine;
+    QString               detailInfo;
+    QString               staticInfo;
+    long long             totalSize(0);
+    long long             liveTVSize(0);
+    int                   liveTVCount(0);
+
+    contentLines.clear();
+    contentDetail.clear();
+    contentFont.clear();
+    doScroll = true;
+
+    RemoteGetAllExpiringRecordings(expList);
+
+    vector<ProgramInfo *>::iterator it;
+    for (it = expList.begin(); it != expList.end(); it++)
+    {
+        pginfo = *it;
+       
+        totalSize += pginfo->filesize;
+        if (pginfo->recgroup == "LiveTV")
+        {
+            liveTVSize += pginfo->filesize;
+            liveTVCount++;
+        }
+    }
+
+    staticInfo = tr("%1 recordings consuming %2 are allowed to expire")
+                    .arg(expList.size()).arg(sm_str(totalSize / 1024)) + "\n";
+
+    if (liveTVCount)
+        staticInfo += tr("%1 of these are LiveTV and consume %2")
+                        .arg(liveTVCount).arg(sm_str(liveTVSize / 1024)) + "\n";
+    else
+        staticInfo += "\n";
+
+    for (it = expList.begin(); it != expList.end(); it++)
+    {
+        pginfo = *it;
+        contentLine = pginfo->recstartts.toString(dateFormat) + " - " +
+                      pginfo->title + " (" + sm_str(pginfo->filesize / 1024) +
+                      ")";
+        detailInfo = staticInfo + pginfo->title;
+
+        if (pginfo->subtitle != "")
+            detailInfo += " - " + pginfo->subtitle + "";
+
+        detailInfo += " (" + sm_str(pginfo->filesize / 1024) + ")\n";
+
+        if (pginfo->recgroup == "LiveTV")
+            contentLine += " (" + tr("LiveTV") + ")";
+
+        detailInfo += pginfo->recstartts.toString(timeDateFormat) + " - " +
+                      pginfo->recendts.toString(timeDateFormat) + "\n";
+
+        contentLines[count] = contentLine;
+        contentDetail[count] = detailInfo;
+        count++;
+    }
+
+    contentTotalLines = count;
+    update(ContentRect);
+}
+
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
