@@ -14,20 +14,17 @@ using namespace std;
 
 #include "viewscheduled.h"
 #include "scheduledrecording.h"
-#include "infodialog.h"
 #include "proglist.h"
 #include "tv.h"
 
+#include "exitcodes.h"
 #include "dialogbox.h"
 #include "mythcontext.h"
 #include "remoteutil.h"
 
-ViewScheduled::ViewScheduled(QSqlDatabase *ldb, MythMainWindow *parent, 
-                             const char *name)
+ViewScheduled::ViewScheduled(MythMainWindow *parent, const char *name)
              : MythDialog(parent, name)
 {
-    db = ldb;
-
     dateformat = gContext->GetSetting("ShortDateFormat", "M/d");
     timeformat = gContext->GetSetting("TimeFormat", "h:mm AP");
     channelFormat = gContext->GetSetting("ChannelFormat", "<num> <sign>");
@@ -55,8 +52,9 @@ ViewScheduled::ViewScheduled(QSqlDatabase *ldb, MythMainWindow *parent,
     }
     else
     {
-        cerr << "MythFrontEnd: ViewSchedule - Failed to get selector object.\n";
-        exit(29);
+        VERBOSE(VB_IMPORTANT, "ViewScheduled::ViewScheduled(): "
+                "Failed to get selector object.");
+        exit(FRONTEND_BUGGY_EXIT_NO_SELECTOR);
     }
 
     updateBackground();
@@ -71,12 +69,14 @@ ViewScheduled::ViewScheduled(QSqlDatabase *ldb, MythMainWindow *parent,
     setNoErase();
 
     gContext->addListener(this);
+    gContext->addCurrentLocation("ViewScheduled");
 }
 
 ViewScheduled::~ViewScheduled()
 {
     gContext->SaveSetting("ViewSchedShowLevel", !showAll);
     gContext->removeListener(this);
+    gContext->removeCurrentLocation();
     delete theme;
 }
 
@@ -104,7 +104,7 @@ void ViewScheduled::keyPressEvent(QKeyEvent *e)
                 upcoming();
             else if (action == "DETAILS")
                 details();
-            else if (action == "ESCAPE")
+            else if (action == "ESCAPE" || action == "LEFT")
                 done(MythDialog::Accepted);
             else if (action == "UP")
                 cursorUp();
@@ -156,8 +156,9 @@ void ViewScheduled::LoadWindow(QDomElement &element)
                 parseContainer(e);
             else
             {
-                cerr << "Unknown element: " << e.tagName() << endl;
-                exit(30);
+                VERBOSE(VB_IMPORTANT,
+                        QString("ViewScheduled: Unknown child element: %1. "
+                                "Ignoring.").arg(e.tagName()));
             }
         }
     }
@@ -261,9 +262,10 @@ void ViewScheduled::FillList(void)
     ProgramInfo *p = recList.first();
     while (p)
     {
-        if ((p->recstatus <= rsWillRecord || 
-             showAll || p->recstatus > rsEarlierShowing) &&
-            p->recendts >= now)
+        if ((p->recendts >= now || p->endts >= now) && 
+            (showAll || p->recstatus <= rsWillRecord || 
+             p->recstatus == rsDontRecord ||
+             (p->recstatus > rsEarlierShowing && p->recstatus != rsRepeat)))
             p = recList.next();
         else
         {
@@ -350,11 +352,15 @@ void ViewScheduled::updateList(QPainter *p)
 
                 if (p->recstatus == rsRecording)
                     ltype->EnableForcedFont(i, "recording");
-                else if (p->recstatus == rsConflict)
+                else if (p->recstatus == rsConflict ||
+                         p->recstatus == rsAborted)
                     ltype->EnableForcedFont(i, "conflictingrecording");
-                else if (p->recstatus > rsWillRecord || 
-                         p->recstatus < rsRecording)
-                    ltype->EnableForcedFont(i, "disabledrecording");
+                else if (p->recstatus == rsWillRecord)
+                    ltype->EnableForcedFont(i, "record");
+                else if (p->recstatus == rsRepeat ||
+                         (p->recstatus != rsDontRecord &&
+                          p->recstatus <= rsEarlierShowing))
+                    ltype->EnableForcedFont(i, "disabledrecording"); 
             }
         }
     }
@@ -459,8 +465,7 @@ void ViewScheduled::updateInfo(QPainter *p)
         ProgramInfo *p = recList[listPos];
         if (p)
         {
-            QSqlDatabase *m_db = QSqlDatabase::database();
-            p->ToMap(m_db, infoMap);
+            p->ToMap(infoMap);
             container->ClearAllText();
             container->SetText(infoMap);
         }
@@ -496,8 +501,7 @@ void ViewScheduled::updateRecStatus(QPainter *p)
         ProgramInfo *p = recList[listPos];
         if (p)
         {
-            QSqlDatabase *m_db = QSqlDatabase::database();
-            p->ToMap(m_db, infoMap);
+            p->ToMap(infoMap);
             container->ClearAllText();
             container->SetText(infoMap);
         }
@@ -522,7 +526,7 @@ void ViewScheduled::edit()
     if (!p)
         return;
 
-    p->EditScheduled(db);
+    p->EditScheduled();
 }
 
 void ViewScheduled::upcoming()
@@ -532,8 +536,7 @@ void ViewScheduled::upcoming()
     if (!p)
         return;
 
-    ProgLister *pl = new ProgLister(plTitle, p->title,
-                                   QSqlDatabase::database(),
+    ProgLister *pl = new ProgLister(plTitle, p->title, "",
                                    gContext->GetMainWindow(), "proglist");
     pl->exec();
     delete pl;
@@ -544,7 +547,7 @@ void ViewScheduled::details()
     ProgramInfo *p = recList[listPos];
 
     if (p)
-        p->showDetails(QSqlDatabase::database());
+        p->showDetails();
 }
 
 void ViewScheduled::selected()
@@ -553,7 +556,7 @@ void ViewScheduled::selected()
     if (!p)
         return;
 
-    p->EditRecording(db);
+    p->EditRecording();
 }
 
 void ViewScheduled::customEvent(QCustomEvent *e)
