@@ -19,6 +19,11 @@ using namespace std;
 #include "osdtypes.h"
 #include "remoteutil.h"
 #include "mythcontext.h"
+#include "jobqueue.h"
+
+extern "C" {
+#include "../libavcodec/avcodec.h"
+}
 
 // This class is to act as a fake audio output device to store the data
 // for reencoding.
@@ -62,7 +67,7 @@ class AudioReencodeBuffer : public AudioOutput
     }
 
     // timecode is in milliseconds.
-    virtual void AddSamples(char *buffer, int samples, long long timecode)
+    virtual bool AddSamples(char *buffer, int samples, long long timecode)
     {
         int freebuf = bufsize - audiobuffer_len;
 
@@ -80,9 +85,11 @@ class AudioReencodeBuffer : public AudioOutput
         audiobuffer_len += samples * bytes_per_sample;
         // last_audiotime is at the end of the sample
         last_audiotime = timecode + samples * 1000 / eff_audiorate;
+
+        return true;
     }
 
-    virtual void AddSamples(char *buffers[], int samples, long long timecode)
+    virtual bool AddSamples(char *buffers[], int samples, long long timecode)
     {
         int audio_bytes = bits / 8;
         int freebuf = bufsize - audiobuffer_len;
@@ -108,6 +115,8 @@ class AudioReencodeBuffer : public AudioOutput
 
         // last_audiotime is at the end of the sample
         last_audiotime = timecode + samples * 1000 / eff_audiorate;
+
+        return true;
     }
 
     virtual void SetTimecode(long long timecode)
@@ -122,25 +131,77 @@ class AudioReencodeBuffer : public AudioOutput
     {
         (void)paused;
     }
+    virtual void Drain(void)
+    {
+        // Do nothing
+        return;
+    }
 
     virtual int GetAudiotime(void)
     {
         return last_audiotime;
     }
 
+    virtual int GetVolumeChannel(int)
+    { 
+        // Do nothing
+        return 100;
+    }
+    virtual void SetVolumeChannel(int, int) 
+    {
+        // Do nothing
+    }
+    virtual void SetVolumeAll(int)
+    {
+        // Do nothing
+    }
+    
+
+    virtual int GetCurrentVolume(void)
+    { 
+        // Do nothing
+        return 100;
+    }
+    virtual void SetCurrentVolume(int) 
+    {
+        // Do nothing
+    }
+    virtual void AdjustCurrentVolume(int) 
+    {
+        // Do nothing
+    }
+    virtual void SetMute(bool) 
+    {
+        // Do nothing
+    }
+    virtual void ToggleMute(void) 
+    {
+        // Do nothing
+    }
+    virtual kMuteState GetMute(void) 
+    {
+        // Do nothing
+        return MUTE_OFF;
+    }
+    virtual kMuteState IterateMutedChannels(void) 
+    {
+        // Do nothing
+        return MUTE_OFF;
+    }
+
+    //  These are pure virtual in AudioOutput, but we don't need them here
+    virtual void bufferOutputData(bool){ return; }
+    virtual int readOutputData(unsigned char*, int ){ return 0; }
+
     int bufsize;
     unsigned char *audiobuffer;
     int audiobuffer_len, channels, bits, bytes_per_sample, eff_audiorate;
     long long last_audiotime;
 };
 
-Transcode::Transcode(QSqlDatabase *db, ProgramInfo *pginfo)
+Transcode::Transcode(ProgramInfo *pginfo)
 {
-    QString dbname = QString("transcode%1%2").arg(getpid()).arg(rand());
-    nvpdb = new MythSqlDatabase(dbname);
-
     m_proginfo = pginfo;
-    m_db = db;
     nvr = NULL;
     nvp = NULL;
     inRingBuffer = NULL;
@@ -170,8 +231,6 @@ Transcode::~Transcode()
         }
         delete kfa_table;
     }
-    if (nvpdb)
-        delete nvpdb;
 }
 void Transcode::ReencoderAddKFA(long curframe, long lastkey, long num_keyframes)
 {
@@ -191,13 +250,16 @@ bool Transcode::GetProfile(QString profileName, QString encodingType)
     {
         bool result = false;
         if (encodingType == "MPEG-2")
-            result = profile.loadByGroup(m_db, "MPEG2", "Transcoders");
+            result = profile.loadByGroup("MPEG2", "Transcoders");
         if (encodingType == "MPEG-4" || encodingType == "RTjpeg")
-            result = profile.loadByGroup(m_db, "RTjpeg/MPEG4",
+            result = profile.loadByGroup("RTjpeg/MPEG4",
                                          "Transcoders");
         if (! result)
         {
-            cerr << "Couldn't find profile for : " << encodingType << endl;
+            VERBOSE(VB_IMPORTANT,
+                    QString("Transcode: Couldn't find profile for : %1").
+                    arg(encodingType));
+
             return false;
         }
     }
@@ -208,10 +270,11 @@ bool Transcode::GetProfile(QString profileName, QString encodingType)
         profileID = profileName.toInt(&isNum);
         // If a bad profile is specified, there will be trouble
         if (isNum && profileID > 0)
-            profile.loadByID(m_db, profileID);
+            profile.loadByID(profileID);
         else
         {
-            cerr << "Couldn't find profile #: " << profileName << endl;
+            VERBOSE(VB_IMPORTANT, QString("Couldn't find profile #: %1").
+                    arg(profileName));
             return false;
         }
     }
@@ -232,21 +295,28 @@ void TranscodeWriteText(void *ptr, unsigned char *buf, int len, int timecode, in
 int Transcode::TranscodeFile(char *inputname, char *outputname,
                               QString profileName,
                               bool honorCutList, bool framecontrol,
-                              bool chkTranscodeDB, QString fifodir)
+                              int jobID, QString fifodir)
 { 
     int audioframesize;
     int audioFrame = 0;
 
-    nvp = new NuppelVideoPlayer(nvpdb, m_proginfo);
-    nvp->SetNoVideo();
+    if (jobID >= 0)
+        JobQueue::ChangeJobComment(jobID, "0% " + QObject::tr("Completed"));
+
+    nvp = new NuppelVideoPlayer("transcoder", m_proginfo);
+    nvp->SetNullVideo();
 
     QDateTime curtime = QDateTime::currentDateTime();
     QDateTime statustime = curtime;
     if (honorCutList && m_proginfo)
     {
-        if (m_proginfo->IsEditing(m_db) || m_proginfo->IsCommProcessing(m_db))
+        if ((m_proginfo->IsEditing()) ||
+            (JobQueue::IsJobRunning(JOB_COMMFLAG, m_proginfo)))
+        {
+            VERBOSE(VB_IMPORTANT, "Transcoding aborted, cutlist changed");
             return REENCODE_CUTLIST_CHANGE;
-        m_proginfo->SetMarkupFlag(MARK_UPDATED_CUT, false, m_db);
+        }
+        m_proginfo->SetMarkupFlag(MARK_UPDATED_CUT, false);
         curtime = curtime.addSecs(60);
     }
 
@@ -255,16 +325,20 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
         statustime = statustime.addSecs(5);
     }
     // Input setup
-    nvr = new NuppelVideoRecorder(NULL);
+    nvr = new NuppelVideoRecorder(NULL, NULL);
     inRingBuffer = new RingBuffer(inputname, false, false);
     nvp->SetRingBuffer(inRingBuffer);
 
     AudioOutput *audioOutput = new AudioReencodeBuffer(0, 0);
     AudioReencodeBuffer *arb = ((AudioReencodeBuffer*)audioOutput);
     nvp->SetAudioOutput(audioOutput);
+    nvp->SetTranscoding(true);
 
     if (nvp->OpenFile(false) < 0)
+    {
+        VERBOSE(VB_IMPORTANT, "Transcoding aborted, error opening file.");
         return REENCODE_ERROR;
+    }
 
     nvp->ReinitAudio();
     QString encodingType = nvp->GetEncodingType();
@@ -274,26 +348,61 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
 
     int video_width = nvp->GetVideoWidth();
     int video_height = nvp->GetVideoHeight();
+    float video_aspect = nvp->GetVideoAspect();
     float video_frame_rate = nvp->GetFrameRate();
     long long total_frame_count = nvp->GetTotalFrameCount();
+    int newWidth = video_width;
+    int newHeight = video_height;
 
     kfa_table = new QPtrList<struct kfatable_entry>;
 
     if (fifodir == NULL)
     {
         if (!GetProfile(profileName, encodingType)) {
+            VERBOSE(VB_IMPORTANT, "Transcoding aborted, no profile found.");
             return REENCODE_ERROR;
         }
         vidsetting = profile.byName("videocodec")->getValue();
-        if (vidsetting == "MPEG-2")
+        audsetting = profile.byName("audiocodec")->getValue();
+
+        if (encodingType == "MPEG-2" &&
+            profile.byName("transcodelossless")->getValue().toInt())
+        {
+            VERBOSE(VB_IMPORTANT, "Switching to MPEG-2 transcoder.");
             return REENCODE_MPEG2TRANS;
+        }
+
         // Recorder setup
-        nvr->SetFrameRate(video_frame_rate);
+        if (profile.byName("transcodelossless")->getValue().toInt())
+        {
+            vidsetting = encodingType;
+            audsetting = "MP3";
+        }
+        else if (profile.byName("transcoderesize")->getValue().toInt())
+        {
+            newWidth = profile.byName("width")->getValue().toInt();
+            newHeight = profile.byName("height")->getValue().toInt();
+
+            if (profile.byName("transcodepreserveaspect")->getValue().toInt())
+                newHeight = (int)(1.0 * newWidth * video_height / video_width);
+
+            if (encodingType.left(4).lower() == "mpeg")
+            {
+                // make sure dimensions are valid for MPEG codecs
+                newHeight = (newHeight + 15) & ~0xF;
+                newWidth  = (newWidth  + 15) & ~0xF;
+            }
+
+            VERBOSE(VB_IMPORTANT, QString("Resizing from %1x%2 to %3x%4")
+                    .arg(video_width).arg(video_height)
+                    .arg(newWidth).arg(newHeight));
+        }
 
         // this is ripped from tv_rec SetupRecording. It'd be nice to merge
         nvr->SetOption("inpixfmt", FMT_YV12);
-        nvr->SetOption("width", video_width);
-        nvr->SetOption("height", video_height);
+
+        nvr->SetOption("width", newWidth);
+        nvr->SetOption("height", newHeight);
 
         nvr->SetOption("tvformat", gContext->GetSetting("TVFormat"));
         nvr->SetOption("vbiformat", gContext->GetSetting("VbiFormat"));
@@ -321,10 +430,10 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
         }
         else
         {
-            cerr << "Unknown video codec: " << vidsetting << endl;
+            VERBOSE(VB_IMPORTANT, QString("Unknown video codec: %1").arg(vidsetting));
+            return REENCODE_ERROR;
         }
 
-        audsetting = profile.byName("audiocodec")->getValue();
         nvr->SetOption("samplerate", arb->eff_audiorate);
         if (audsetting == "MP3")
         {
@@ -338,11 +447,15 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
         }
         else
         {
-            cerr << "Unknown audio codec: " << audsetting << endl;
+            VERBOSE(VB_IMPORTANT, QString("Unknown audio codec: %1").arg(audsetting));
         }
 
         nvr->AudioInit(true);
 
+        nvr->SetFrameRate(video_frame_rate);
+        nvr->SetVideoAspect(video_aspect);
+        nvr->SetTranscoding(true);
+
         outRingBuffer = new RingBuffer(outputname, true, false);
         nvr->SetRingBuffer(outRingBuffer);
         nvr->WriteHeader();
@@ -350,7 +463,8 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
     }
 
     if (vidsetting == encodingType && !framecontrol &&
-        fifodir == NULL && honorCutList)
+        fifodir == NULL && honorCutList &&
+        video_width == newWidth && video_height == newHeight)
     {
         copyvideo = true;
         VERBOSE(VB_GENERAL, "Reencoding video in 'raw' mode");
@@ -358,12 +472,18 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
 
     keyframedist = 30;
     nvp->InitForTranscode(copyaudio, copyvideo);
+    if (nvp->IsErrored())
+    {
+        VERBOSE(VB_IMPORTANT, "Unable to initialize NuppelVideoPlayer for Transcode");
+        delete nvp;
+        return REENCODE_ERROR;
+    }
 
     VideoFrame frame;
     frame.codec = FMT_YV12;
-    frame.width = video_width;
-    frame.height = video_height;
-    frame.size = video_width * video_height * 3 / 2;
+    frame.width = newWidth;
+    frame.height = newHeight;
+    frame.size = newWidth * newHeight * 3 / 2;
 
     if (fifodir != NULL)
     {
@@ -378,9 +498,9 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
         if (!fifow->FIFOInit(0, QString("video"), vidfifo, frame.size, 50) ||
             !fifow->FIFOInit(1, QString("audio"), audfifo, audio_size, 25))
         {
-           cerr << "Error initializing fifo writer.  Aborting" << endl;
-           unlink(outputname);
-           return REENCODE_ERROR;
+            VERBOSE(VB_IMPORTANT, "Error initializing fifo writer.  Aborting");
+            unlink(outputname);
+            return REENCODE_ERROR;
         }
         VERBOSE(VB_GENERAL, QString("Video %1x%2@%3fps Audio rate: %4")
                                    .arg(video_width).arg(video_height)
@@ -404,6 +524,7 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
     long totalAudio = 0;
     int dropvideo = 0;
     long long lasttimecode = 0;
+    long long timecodeOffset = 0;
 
     float rateTimeConv = arb->eff_audiorate * arb->bytes_per_sample / 1000.0;
     float vidFrameTime = 1000.0 / video_frame_rate;
@@ -411,6 +532,17 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
     VideoOutput *videoOutput = nvp->getVideoOutput();
     bool is_key = 0;
     bool first_loop = true;
+    unsigned char *newFrame = new unsigned char[newWidth * newHeight * 3 / 2];
+
+    frame.buf = newFrame;
+    AVPicture imageIn, imageOut;
+    ImgReSampleContext *scontext;
+
+    if (video_width != newWidth || video_height != newHeight)
+        VERBOSE(VB_GENERAL, QString("Resizing video from %1x%2 to %3x%4")
+                                    .arg(video_width).arg(video_height)
+                                    .arg(newWidth).arg(newHeight));
+
     while (nvp->TranscodeGetNextFrame(dm_iter, &did_ff, &is_key, honorCutList))
     {
         if (first_loop)
@@ -419,15 +551,15 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
             first_loop = false;
         }
         VideoFrame *lastDecode = videoOutput->GetLastDecodedFrame();
-        frame.buf = lastDecode->buf;
+
         frame.timecode = lastDecode->timecode;
 
         if (frame.timecode < lasttimecode)
             frame.timecode = (long long)(lasttimecode + vidFrameTime);
-        lasttimecode = frame.timecode;
 
         if (fifow)
         {
+            frame.buf = lastDecode->buf;
             totalAudio += arb->audiobuffer_len;
             int audbufTime = (int)(totalAudio / rateTimeConv);
             int auddelta = arb->last_audiotime - audbufTime;
@@ -501,6 +633,7 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
             videoOutput->DoneDisplayingFrame();
             audioOutput->Reset();
             nvp->FlushTxtBuffers();
+            lasttimecode = frame.timecode;
         }
         else if (copyaudio)
         {
@@ -510,6 +643,9 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
             {
                 // The Raw state changed during decode.  This is not good
                 unlink(outputname);
+                delete newFrame;
+                VERBOSE(VB_IMPORTANT, "Transcoding aborted, NuppelVideoPlayer "
+                                      "is not in raw audio mode.");
                 return REENCODE_ERROR;
             }
 
@@ -538,7 +674,16 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
                 }
             }
 
-            if (! nvp->WriteStoredData(outRingBuffer, (did_ff == 0)))
+            if (did_ff == 1)
+            {
+                timecodeOffset +=
+                    (frame.timecode - lasttimecode - (int)vidFrameTime);
+            }
+            lasttimecode = frame.timecode;
+            frame.timecode -= timecodeOffset;
+
+            if (! nvp->WriteStoredData(outRingBuffer, (did_ff == 0),
+                                       timecodeOffset))
             {
                 if (did_ff == 1)
                 {
@@ -546,6 +691,23 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
                   did_ff = 2;
                   writekeyframe = true;
                 }
+
+                if ((video_width == newWidth) && (video_height == newHeight))
+                {
+                    frame.buf = lastDecode->buf;
+                }
+                else
+                {
+                    avpicture_fill(&imageIn, lastDecode->buf, PIX_FMT_YUV420P,
+                                   video_width, video_height);
+                    avpicture_fill(&imageOut, frame.buf, PIX_FMT_YUV420P,
+                                   newWidth, newHeight);
+                    scontext = img_resample_init(newWidth, newHeight,
+                                                 video_width, video_height);
+                    img_resample(scontext, &imageOut, &imageIn);
+                    img_resample_close(scontext);
+                }
+
                 nvr->WriteVideo(&frame, true, writekeyframe);
             }
             audioOutput->Reset();
@@ -553,17 +715,50 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
         } 
         else 
         {
+            if (did_ff == 1)
+            {
+                did_ff = 2;
+                timecodeOffset +=
+                    (frame.timecode - lasttimecode - (int)vidFrameTime);
+            }
+
+            if ((video_width == newWidth) && (video_height == newHeight))
+            {
+                frame.buf = lastDecode->buf;
+            }
+            else
+            {
+                avpicture_fill(&imageIn, lastDecode->buf, PIX_FMT_YUV420P,
+                               video_width, video_height);
+                avpicture_fill(&imageOut, frame.buf, PIX_FMT_YUV420P,
+                               newWidth, newHeight);
+                scontext = img_resample_init(newWidth, newHeight,
+                                             video_width, video_height);
+                img_resample(scontext, &imageOut, &imageIn);
+                img_resample_close(scontext);
+            }
+
             // audio is fully decoded, so we need to reencode it
             audioframesize = arb->audiobuffer_len;
             if (audioframesize > 0)
             {
                 nvr->SetOption("audioframesize", audioframesize);
                 int starttime = audioOutput->GetAudiotime();
-                nvr->WriteAudio(arb->audiobuffer, audioFrame++, starttime);
+                nvr->WriteAudio(arb->audiobuffer, audioFrame++,
+                                starttime - timecodeOffset);
+                if (nvr->IsErrored()) {
+                    VERBOSE(VB_IMPORTANT, "Transcode: Encountered "
+                            "irrecoverable error in NVR::WriteAudio");
+                    delete newFrame;
+                    return REENCODE_ERROR;
+                }
                 arb->audiobuffer_len = 0;
             }
             nvp->TranscodeWriteText(&TranscodeWriteText, (void *)(nvr));
 
+            lasttimecode = frame.timecode;
+            frame.timecode -= timecodeOffset;
+
             if (forceKeyFrames)
                 nvr->WriteVideo(&frame, true, true);
             else
@@ -571,41 +766,39 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
         }
         if (showprogress && QDateTime::currentDateTime() > statustime)
         {
-            cerr << "Processed: " << curFrameNum << " of " << total_frame_count
-                 << " frames (" << (long)(curFrameNum / video_frame_rate)
-                 << " seconds)\r";
+            VERBOSE(VB_IMPORTANT, QString("Processed: %1 of %2 frames(%3 seconds)").
+                    arg((long)curFrameNum).arg((long)total_frame_count).
+                    arg((long)(curFrameNum / video_frame_rate)));
             statustime = QDateTime::currentDateTime();
             statustime = statustime.addSecs(5);
         }
         if (QDateTime::currentDateTime() > curtime) 
         {
-            if (honorCutList && 
-                m_proginfo->CheckMarkupFlag(MARK_UPDATED_CUT, m_db)) 
+            if (honorCutList && m_proginfo &&
+                m_proginfo->CheckMarkupFlag(MARK_UPDATED_CUT))
             {
                 unlink(outputname);
+                delete newFrame;
+                VERBOSE(VB_IMPORTANT, "Transcoding aborted, cutlist updated");
                 return REENCODE_CUTLIST_CHANGE;
             }
 
-            if (chkTranscodeDB)
+            if (jobID >= 0)
             {
-                QString query = QString("SELECT * FROM transcoding WHERE "
-                        "chanid = '%1' AND starttime = '%2' AND "
-                        "((status & %3) = %4) AND hostname = '%5';")
-                        .arg(m_proginfo->chanid)
-                        .arg(m_proginfo->startts.toString("yyyyMMddhhmmss"))
-                        .arg(TRANSCODE_STOP)
-                        .arg(TRANSCODE_STOP)
-                        .arg(gContext->GetHostName());
-                MythContext::KickDatabase(m_db);
-                QSqlQuery result = m_db->exec(query);
-                if (result.isActive() && result.numRowsAffected() > 0)
+                if (JobQueue::GetJobCmd(jobID) == JOB_STOP)
                 {
                     unlink(outputname);
-                    return REENCODE_ERROR;
+                    delete newFrame;
+                    VERBOSE(VB_IMPORTANT, "Transcoding STOPped by JobQueue");
+                    return REENCODE_STOPPED;
                 }
+                int percentage = curFrameNum * 100 / total_frame_count;
+                JobQueue::ChangeJobComment(jobID,
+                                           QString("%1% ").arg(percentage) + 
+                                           QObject::tr("Completed"));
             }
             curtime = QDateTime::currentDateTime();
-            curtime = curtime.addSecs(60);
+            curtime = curtime.addSecs(20);
         }
 
         curFrameNum++;
@@ -614,12 +807,22 @@ int Transcode::TranscodeFile(char *inputname, char *outputname,
 
     if (! fifow)
     {
-      nvr->WriteSeekTable();
-      if (!kfa_table->isEmpty())
-          nvr->WriteKeyFrameAdjustTable(kfa_table);
+        if (m_proginfo)
+        {
+            m_proginfo->ClearPositionMap(MARK_KEYFRAME);
+            m_proginfo->ClearPositionMap(MARK_GOP_START);
+            m_proginfo->ClearPositionMap(MARK_GOP_BYFRAME);
+        }
+
+        nvr->WriteSeekTable();
+        if (!kfa_table->isEmpty())
+            nvr->WriteKeyFrameAdjustTable(kfa_table);
     } else {
         fifow->FIFODrain();
     }
+    delete newFrame;
     return REENCODE_OK;
 }
 
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
+
