@@ -5,7 +5,9 @@
 #include "tv.h"
 #include "programinfo.h"
 
+#include "libmyth/exitcodes.h"
 #include "libmyth/mythcontext.h"
+#include "libmyth/mythdbcon.h"
 #include "libmyth/mythdialogs.h"
 
 #include <iostream>
@@ -18,9 +20,13 @@ static void *run_priv_thread(void *data)
     (void)data;
     while (true) 
     {
-        if (gContext->hasPrivRequest()) 
+        gContext->waitPrivRequest();
+        
+        for (MythPrivRequest req = gContext->popPrivRequest(); 
+             true; req = gContext->popPrivRequest()) 
         {
-            MythPrivRequest req = gContext->popPrivRequest();
+            bool done = false;
+
             switch (req.getType()) 
             {
             case MythPrivRequest::MythRealtime:
@@ -32,16 +38,17 @@ static void *run_priv_thread(void *data)
                     {
                         int status = pthread_setschedparam(
                             *target_thread, SCHED_FIFO, &sp);
-                        if (status) {
-                            perror("pthread_setschedparam");
-                            VERBOSE(VB_GENERAL, "Running as SUID root would allow "
-                                    "some threads to run with realtime priority, "
-                                    "improving video smoothness.");
+                        if (status) 
+                        {
+                            // perror("pthread_setschedparam");
+                            VERBOSE(VB_GENERAL, "Realtime priority would require SUID as root.");
                         }
+                        else
+                            VERBOSE(VB_GENERAL, "Using realtime priority.");
                     }
                     else
                     {
-                        VERBOSE(VB_IMPORTANT, "Something sus here. NULL thread ptr "
+                        VERBOSE(VB_IMPORTANT, "Unexpected NULL thread ptr "
                                 "for MythPrivRequest::MythRealtime");
                     }
                 }
@@ -50,10 +57,12 @@ static void *run_priv_thread(void *data)
                 pthread_exit(NULL);
                 break;
             case MythPrivRequest::PrivEnd:
+                done = true; // queue is empty
                 break;
             }
+            if (done)
+                break; // from processing the queue
         }
-        sleep(1);
     }
     return NULL; // will never happen
 }
@@ -62,7 +71,13 @@ int main(int argc, char *argv[])
 {
     QApplication a(argc, argv);
 
+    gContext = NULL;
     gContext = new MythContext(MYTH_BINARY_VERSION);
+    if (!gContext->Init())
+    {
+        VERBOSE(VB_IMPORTANT, "Failed to init MythContext, exiting.");
+        return TV_EXIT_NO_MYTHCONTEXT;
+    }
 
     // Create priveleged thread, then drop privs
     pthread_t priv_thread;
@@ -79,33 +94,24 @@ int main(int argc, char *argv[])
     QString themedir = gContext->FindThemeDir(themename);
     if (themedir == "")
     {   
-        cerr << "Couldn't find theme " << themename << endl;
-        return 44; // exit(44)
+        QString msg = QString("Fatal Error: Couldn't find theme '%1'.")
+            .arg(themename);
+        VERBOSE(VB_IMPORTANT, msg);
+        return TV_EXIT_NO_THEME;
     }
     
     gContext->LoadQtConfig();
 
-    QSqlDatabase *db = QSqlDatabase::addDatabase("QMYSQL3");
-    if (!db)
-    {
-        printf("Couldn't connect to database\n");
-        return 45; // exit(45)
-    }       
-    if (!gContext->OpenDatabase(db))
-    {
-        printf("couldn't open db\n");
-        return 46; // exit(46)
-    }
-
-    gContext->LoadQtConfig();
-
     QString auddevice = gContext->GetSetting("AudioOutputDevice");
     if (auddevice == "" || auddevice == QString::null)
     {
-        cerr << "You need to run 'mythfrontend', not 'mythtv'.\n";
-        return 47; // exit(47)
+        VERBOSE(VB_IMPORTANT, "Fatal Error: Audio not configured, you need "
+                "to run 'mythfrontend', not 'mythtv'.");
+        return TV_EXIT_NO_AUDIO;
     }
 
+    print_verbose_messages |= VB_PLAYBACK | VB_LIBAV;// | VB_AUDIO;
+
     MythMainWindow *mainWindow = new MythMainWindow();
     mainWindow->Init();
     gContext->SetMainWindow(mainWindow);
@@ -113,7 +119,11 @@ int main(int argc, char *argv[])
     TV::InitKeys();
 
     TV *tv = new TV();
-    tv->Init();
+    if (!tv->Init())
+    {
+        VERBOSE(VB_IMPORTANT, "Fatal Error: Could not initializing TV class.");
+        return TV_EXIT_NO_TV;
+    }
 
     if (a.argc() > 1)
     {
@@ -137,6 +147,7 @@ int main(int argc, char *argv[])
 
     sleep(1);
     delete tv;
+
     if (priv_thread != 0) 
     {
         void *value;
@@ -145,5 +156,5 @@ int main(int argc, char *argv[])
     }
     delete gContext;
 
-    return 0; // exit(0)
+    return TV_EXIT_OK;
 }
