@@ -1,6 +1,9 @@
+#include <cstdio>
+
 #include "backendsettings.h"
 #include "libmyth/mythcontext.h"
 #include "libmyth/settings.h"
+#include <unistd.h>
 
 
 static HostLineEdit *LocalServerIP()
@@ -70,17 +73,6 @@ static HostLineEdit *RecordFilePrefix()
     return gc;
 };
 
-static HostLineEdit *LiveBufferPrefix()
-{
-    HostLineEdit *gc = new HostLineEdit("LiveBufferDir");
-    gc->setLabel(QObject::tr("Directory to hold the Live-TV buffers"));
-    gc->setValue("/mnt/store/");
-    gc->setHelpText(QObject::tr("All Live-TV buffers will get stored in this "
-                    "directory. These buffers are used to allow you to pause, "
-                    "fast forward and rewind through live TV."));
-    return gc;
-};
-
 static GlobalComboBox *TVFormat()
 {
     GlobalComboBox *gc = new GlobalComboBox("TVFormat");
@@ -92,6 +84,10 @@ static GlobalComboBox *TVFormat()
     gc->addSelection("PAL-NC");
     gc->addSelection("PAL-M");
     gc->addSelection("PAL-N");
+    gc->addSelection("PAL-BG");
+    gc->addSelection("PAL-DK");
+    gc->addSelection("PAL-I");
+    gc->addSelection("PAL-60");
     gc->addSelection("NTSC-JP");
     gc->setHelpText(QObject::tr("The TV standard to use for viewing TV."));
     return gc;
@@ -117,6 +113,7 @@ static GlobalComboBox *FreqTable()
     gc->addSelection("us-cable");
     gc->addSelection("us-bcast");
     gc->addSelection("us-cable-hrc");
+    gc->addSelection("us-cable-irc");
     gc->addSelection("japan-bcast");
     gc->addSelection("japan-cable");
     gc->addSelection("europe-west");
@@ -136,33 +133,13 @@ static GlobalComboBox *FreqTable()
     return gc;
 };
 
-static HostSlider *BufferSize()
-{
-    HostSlider* gc = new HostSlider("BufferSize", 1, 100, 1);
-    gc->setLabel(QObject::tr("Live TV buffer (GB)"));
-    gc->setValue(5);
-    gc->setHelpText(QObject::tr("How large the live TV buffer is allowed to "
-                    "grow."));
-    return gc;
-};
-
-static HostSlider *MaxBufferFill()
-{
-    HostSlider *gc = new HostSlider("MaxBufferFill", 1, 100, 1);
-    gc->setLabel(QObject::tr("Minimum free Live TV buffer (MB)"));
-    gc->setValue(50);
-    gc->setHelpText(QObject::tr("How full the live TV buffer is allowed to "
-                    "grow before forcing an unpause."));
-    return gc;
-};
-
 static GlobalCheckBox *SaveTranscoding()
 {
     GlobalCheckBox *gc = new GlobalCheckBox("SaveTranscoding");
-    gc->setLabel(QObject::tr("Save original files after transcoding"));
+    gc->setLabel(QObject::tr("Save original files after transcoding (globally)"));
     gc->setValue(false);
     gc->setHelpText(QObject::tr("When set and the transcoder is active, the "
-                    "original nuv files will be renamed to nuv.old once the "
+                    "original files will be renamed to .old once the "
                     "transcoding is complete."));
     return gc;
 };
@@ -230,14 +207,25 @@ static GlobalComboBox *TimeOffset()
     gc->addSelection("-0130");
     gc->addSelection("-0100");
     gc->addSelection("-0030");
-    gc->setHelpText(QObject::tr("If your local timezone does not match the "
-                    "timezone returned by XMLTV, use this setting to have "
-                    "mythfilldatabase adjust the program start and end times."
-                    "None disables this feature, Auto automatically "
-                    "detects your local timezone"));
+    gc->setHelpText(QObject::tr("Adjust the relative timezone of the XMLTV EPG data read "
+                    "by mythfilldatabase.  'Auto' converts the XMLTV time to local time "
+                    "using your computer's timezone.  'None' ignores the "
+                    "XMLTV timezone, interpreting times as local."));
     return gc;
 };
 
+static GlobalSpinBox *EITTransportTimeout()
+{
+    GlobalSpinBox *gc = new GlobalSpinBox("EITTransportTimeout", 1, 15, 1);
+    gc->setLabel(QObject::tr("EIT Transport Timeout (mins)"));
+    gc->setValue(5);
+    QString helpText = QObject::tr(
+        "Maximum time to spend waiting for listings data on one DTV channel "
+        "before checking for new listings data on the next channel.");
+    gc->setHelpText(helpText);
+    return gc;
+}
+
 static GlobalCheckBox *MasterBackendOverride()
 {
     GlobalCheckBox *gc = new GlobalCheckBox("MasterBackendOverride");
@@ -364,7 +352,7 @@ static GlobalLineEdit *preSDWUCheckCommand()
                     "shutdown. The return value determines if "
                     "the backend can shutdown. 0 - yes, "
                     "1 - restart idleing, "
-                    "2 - reset the backend to wait for frontend"));
+                    "2 - reset the backend to wait for a frontend."));
     return gc;
 };
 
@@ -392,7 +380,7 @@ static GlobalLineEdit *startupCommand()
 
 static HostSpinBox *JobQueueMaxSimultaneousJobs()
 {
-    HostSpinBox *gc = new HostSpinBox("JobQueueMaxSimultaneousJobs", 1, 5, 1);
+    HostSpinBox *gc = new HostSpinBox("JobQueueMaxSimultaneousJobs", 1, 10, 1);
     gc->setLabel(QObject::tr("Maximum simultaneous jobs on this backend"));
     gc->setHelpText(QObject::tr("The Job Queue will be limited to running "
                     "this many simultaneous jobs on this backend."));
@@ -402,7 +390,7 @@ static HostSpinBox *JobQueueMaxSimultaneousJobs()
 
 static HostSpinBox *JobQueueCheckFrequency()
 {
-    HostSpinBox *gc = new HostSpinBox("JobQueueCheckFrequency", 10, 300, 5);
+    HostSpinBox *gc = new HostSpinBox("JobQueueCheckFrequency", 5, 300, 5);
     gc->setLabel(QObject::tr("Job Queue Check frequency (in seconds)"));
     gc->setHelpText(QObject::tr("When looking for new jobs to process, the "
                     "Job Queue will wait this long between checks."));
@@ -476,7 +464,7 @@ static GlobalCheckBox *AutoCommflagWhileRecording()
     gc->setHelpText(QObject::tr("If set and Auto Commercial Flagging is ON for "
                                 "a recording, the flagging job will be started "
                                 "as soon as the recording starts.  NOT "
-                                "recommended on underpowered systems"));
+                                "recommended on underpowered systems."));
     return gc;
 };
 
@@ -566,6 +554,16 @@ static HostCheckBox *JobAllowCommFlag()
     return gc;
 };
 
+static HostCheckBox *JobAllowTranscode()
+{
+    HostCheckBox *gc = new HostCheckBox("JobAllowTranscode");
+    gc->setLabel(QObject::tr("Allow Transcoding jobs"));
+    gc->setValue(true);
+    gc->setHelpText(QObject::tr("Allow jobs of this type to run on this "
+                                "backend."));
+    return gc;
+};
+
 static HostCheckBox *JobAllowUserJob1()
 {
     HostCheckBox *gc = new HostCheckBox("JobAllowUserJob1");
@@ -620,9 +618,6 @@ BackendSettings::BackendSettings() {
     VerticalConfigurationGroup* group1 = new VerticalConfigurationGroup(false);
     group1->setLabel(QObject::tr("Host-specific Backend Setup"));
     group1->addChild(RecordFilePrefix());
-    group1->addChild(LiveBufferPrefix());
-    group1->addChild(BufferSize());
-    group1->addChild(MaxBufferFill());
     group1->addChild(SaveTranscoding());
     addChild(group1);
 
@@ -632,6 +627,7 @@ BackendSettings::BackendSettings() {
     group2->addChild(VbiFormat());
     group2->addChild(FreqTable());
     group2->addChild(TimeOffset());
+    group2->addChild(EITTransportTimeout());
     group2->addChild(MasterBackendOverride());
     group2->addChild(DeletesFollowLinks());
     addChild(group2);
@@ -675,6 +671,7 @@ BackendSettings::BackendSettings() {
     group5a1->addChild(JobQueueWindowEnd());
     group5a1->addChild(JobQueueCPU());
     group5a1->addChild(JobAllowCommFlag());
+    group5a1->addChild(JobAllowTranscode());
     group5a->addChild(group5a1);
 
     VerticalConfigurationGroup* group5a2 =
@@ -706,4 +703,3 @@ BackendSettings::BackendSettings() {
     group7->addChild(UserJob4());
     addChild(group7);    
 }
-
