@@ -19,36 +19,19 @@
 #include "libmyth/mythdbcon.h"
 #include "libmyth/langsettings.h"
 #include "libmyth/dialogbox.h"
-#include "libmythtv/videosource.h"
-#include "libmythtv/channeleditor.h"
+#include "libmyth/exitcodes.h"
 #include "libmyth/themedmenu.h"
-#include "backendsettings.h"
+#include "libmyth/util.h"
 
 #include "libmythtv/dbcheck.h"
+#include "libmythtv/videosource.h"
+#include "libmythtv/channeleditor.h"
+#include "libmythtv/remoteutil.h"
+#include "backendsettings.h"
+#include "checksetup.h"
 
 using namespace std;
-
-void clearCardDB(void)
-{
-    MSqlQuery query(MSqlQuery::InitCon());
-
-    query.exec("TRUNCATE TABLE capturecard;");
-    query.exec("TRUNCATE TABLE cardinput;");
-}
-
-void clearAllDB(void)
-{
-    MSqlQuery query(MSqlQuery::InitCon());
-
-    query.exec("TRUNCATE TABLE channel;");
-    query.exec("TRUNCATE TABLE program;");
-    query.exec("TRUNCATE TABLE videosource;");
-    query.exec("TRUNCATE TABLE credits;");
-    query.exec("TRUNCATE TABLE programrating;");
-    query.exec("TRUNCATE TABLE programgenres;");
-    query.exec("TRUNCATE TABLE dtv_multiplex;");
-    query.exec("TRUNCATE TABLE cardinput;");
-}
+bool is_backend_running(void);
 
 void SetupMenuCallback(void* data, QString& selection) {
     (void)data;
@@ -95,6 +78,8 @@ void SetupMenu(void)
 int main(int argc, char *argv[])
 {
     QString geometry = "";
+    QString verboseString = QString(" important general");
+
 #ifdef Q_WS_X11
     // Remember any -geometry argument which QApplication init will remove
     for(int argpos = 1; argpos + 1 < argc; ++argpos)
@@ -130,18 +115,37 @@ int main(int argc, char *argv[])
                 return -1;
             }            
         }
+        else if (!strcmp(a.argv()[argpos],"-v") ||
+                  !strcmp(a.argv()[argpos],"--verbose"))
+        {
+            if (a.argc()-1 > argpos)
+            {
+                if (parse_verbose_arg(a.argv()[argpos+1]) ==
+                        GENERIC_EXIT_INVALID_CMDLINE)
+                    return BACKEND_EXIT_INVALID_CMDLINE;                        
+                ++argpos;
+            } 
+            else
+            {
+                cerr << "Missing argument to -v/--verbose option\n";
+                return BACKEND_EXIT_INVALID_CMDLINE;
+            }
+        } 
         else
         {
-            if (!(!strcmp(a.argv()[argpos],"-h") ||    
+            if (!(!strcmp(a.argv()[argpos],"-h") ||
                 !strcmp(a.argv()[argpos],"--help") ||
                 !strcmp(a.argv()[argpos],"--usage")))
                 cerr << "Invalid argument: " << a.argv()[argpos] << endl;
-            cerr << "Valid options are: \n" << 
+
+            cerr << "Valid options are: "<<endl
 #ifdef Q_WS_X11 
-                    "-display X-server     Create GUI on X-server, not localhost\n" <<
+                 <<"-display X-server              Create GUI on X-server, not localhost"<<endl
 #endif          
-                    "--geometry WxH        Override window size settings\n" <<
-                    "-geometry WxH+X+Y     Override window size and position\n";
+                 <<"--geometry WxH                 Override window size settings"<<endl
+                 <<"-geometry WxH+X+Y              Override window size and position"<<endl
+                 <<"-v or --verbose debug-level    Use '-v help' for level info"<<endl 
+                 <<endl;
             return -1;
         }
     }
@@ -149,17 +153,17 @@ int main(int argc, char *argv[])
     gContext = NULL;
     gContext = new MythContext(MYTH_BINARY_VERSION);
 
-    if (geometry != "")
-        if (!gContext->ParseGeometryOverride(geometry))
-            cerr << "Illegal -geometry argument '"
-                 << geometry << "' (ignored)\n";
-
     if (!gContext->Init(true))
     {
         VERBOSE(VB_IMPORTANT, "Failed to init MythContext, exiting.");
         return -1;
     }
 
+    if (geometry != "")
+        if (!gContext->ParseGeometryOverride(geometry))
+            cerr << "Illegal -geometry argument '"
+                 << geometry << "' (ignored)\n";
+
     if (!MSqlQuery::testDBConnection())
     {
         cerr << "Unable to open database.\n";
@@ -189,36 +193,94 @@ int main(int argc, char *argv[])
     LanguageSettings::prompt();
     LanguageSettings::load("mythfrontend");
 
-    DialogBox dboxCard(mainWindow, QObject::tr("Would you like to clear all "
-                                   "capture card settings before starting "
-                                   "configuration?"));
-    dboxCard.AddButton(QObject::tr("No, leave my card settings alone"));
-    dboxCard.AddButton(QObject::tr("Yes, delete my card settings"));
-    if (dboxCard.exec() == 2)
-        clearCardDB();
-    
-    // Give the user time to realize the first dialog is gone
-    // before we bring up a similar-looking one
-    usleep(750000);
-    
-    DialogBox dboxProg(mainWindow, QObject::tr("Would you like to clear all "
-                                   "program data and channel settings before "
-                                   "starting configuration? This will not "
-                                   "affect any existing recordings."));
-    dboxProg.AddButton(QObject::tr("No, leave my channel settings alone"));
-    dboxProg.AddButton(QObject::tr("Yes, delete my channel settings"));
-    if (dboxProg.exec() == 2)
-        clearAllDB();
+    QString warn = 
+        QObject::tr("WARNING") + ": " +
+        QObject::tr("MythTV has detected that the backend is running.")+"\n\n"+
+        QObject::tr("Changing existing card inputs, deleting anything, "
+                    "or scanning for channels may not work.");
+
+    bool backendIsRunning = false;
+
+    if (is_backend_running())
+    {
+        int val = MythPopupBox::show2ButtonPopup(
+            gContext->GetMainWindow(), QObject::tr("WARNING"),
+            warn,
+            QObject::tr("Continue"),
+            QObject::tr("Exit"), 1);
+        if (1 == val)
+            return 0;
+
+        backendIsRunning = true;
+    }
 
     REG_KEY("qt", "DELETE", "Delete", "D");
     REG_KEY("qt", "EDIT", "Edit", "E");
 
-    SetupMenu();
+    DialogBox *dia = NULL;
+    bool haveProblems = false;
+    do
+    {
+        // Let the user select buttons, type values, scan for channels, et c.
+        SetupMenu();
 
-    cout << "If this is the master backend server:\n";
-    cout << "Now, please run 'mythfilldatabase' to populate the database\n";
-    cout << "with channel information.\n";
-    cout << endl;
+        // Look for common problems
+        QString *problems = new QString("");
+        haveProblems = CheckSetup(problems);
+
+        if (haveProblems)
+        {
+            QString prompt;
+
+            if (problems->contains("\n") > 1)
+                prompt = QObject::tr("Do you want to fix these problems?");
+            else
+                prompt = QObject::tr("Do you want to fix this problem?");
+
+            dia = new DialogBox(mainWindow, problems->append("\n" + prompt));
+            dia->AddButton(QObject::tr("Yes please"));
+            dia->AddButton(QObject::tr("No, I know what I am doing"));
+
+            if (dia->exec() == 2)
+                haveProblems = false;
+            delete dia;
+        }
+
+        delete problems;
+
+    // Execute UI again until there are no more problems:
+    }
+    while (haveProblems);
+
+    dia = new DialogBox(mainWindow,
+                        QObject::tr("If this is the master backend server, "
+                                    "please run 'mythfilldatabase' "
+                                    "to populate the database "
+                                    "with channel information."));
+    dia->AddButton(QObject::tr("OK"));
+    dia->exec();
+    delete dia;
+
+    if (backendIsRunning)
+        RemoteSendMessage("CLEAR_SETTINGS_CACHE");
 
     return 0;
 }
+
+bool is_backend_running(void)
+{
+    QString tmp_file = "/tmp/backendrunning";
+    myth_system("ps -ae | grep mythbackend > " + tmp_file,
+                MYTH_SYSTEM_DONT_BLOCK_LIRC |
+                MYTH_SYSTEM_DONT_BLOCK_JOYSTICK_MENU);
+
+    FILE *fptr = fopen(tmp_file, "r");
+    if (!fptr)
+        return false;
+    char buf[1024];
+    int read_bytes = fread(buf, 1, 1024, fptr);
+    fclose(fptr);
+    unlink(tmp_file);
+
+    return read_bytes != 0;
+}
